@@ -1,4 +1,4 @@
-// $Id: CglFlowCover.cpp 1123 2013-04-06 20:47:24Z stefan $
+// $Id: CglFlowCover.cpp 1391 2017-11-28 11:56:42Z forrest $
 //-----------------------------------------------------------------------------
 // name:     Cgl Lifted Simple Generalized Flow Cover Cut Generator
 // author:   Yan Xu                email: yan.xu@sas.com
@@ -24,8 +24,8 @@
 
 // added #define to get rid of warnings (so uncomment if =true)
 //#define CGLFLOW_DEBUG2
-static bool CGLFLOW_DEBUG=false;
-static bool doLift=true;
+//static bool CGLFLOW_DEBUG=false;
+#define CGLFLOW_DEBUG 0
 #include <iomanip>
 //-------------------------------------------------------------------
 // Overloaded operator<< for printing VUB and VLB.
@@ -36,9 +36,6 @@ std::ostream& operator<<( std::ostream& os, const CglFlowVUB &v )
   return os; 
 }
 
-// Initialize static memeber
-int CglFlowCover::numFlowCuts_ = 0;
-
 //-------------------------------------------------------------------
 // Determine row types. Find the VUBS and VLBS. 
 //-------------------------------------------------------------------  
@@ -229,11 +226,9 @@ CglFlowCover::flowPreprocess(const OsiSolverInterface& si)
 void CglFlowCover::generateCuts(const OsiSolverInterface & si, OsiCuts & cs,
 				const CglTreeInfo info)
 {
-  static int count=0;
   if (getMaxNumCuts() <= 0) return;
     
   if (getNumFlowCuts() >= getMaxNumCuts()) return;
-  ++count;
 
 #if 0
   bool preInit = false;
@@ -350,7 +345,7 @@ void CglFlowCover::generateCuts(const OsiSolverInterface & si, OsiCuts & cs,
 CglFlowCover::CglFlowCover()
   :
   CglCutGenerator(),
-  maxNumCuts_(2000),
+  maxNumCuts_(2000000),
   EPSILON_(1.0e-6),
   UNDEFINED_(-1),
   INFTY_(1.0e30),
@@ -358,6 +353,7 @@ CglFlowCover::CglFlowCover()
   firstProcess_(true),
   numRows_(0),
   numCols_(0),
+  numFlowCuts_(0),
   doneInitPre_(false),
   vubs_(0),
   vlbs_(0),
@@ -486,18 +482,26 @@ CglFlowCover::generateOneFlowCut( const OsiSolverInterface & si,
     
   CglFlowVLB VLB;
   CglFlowVUB VUB;
-  static int count=0;
-  ++count;
-  CGLFLOW_DEBUG=false;
-  doLift=true;
+  //CGLFLOW_DEBUG=false;
+  bool doLift=true;
   // Get integer types
   const char * columnType = si.getColType ();
   for (i = 0; i < rowLen; ++i) {
     if ( xlp[ind[i]] - floor(xlp[ind[i]]) > EPSILON_ && ceil(xlp[ind[i]]) - xlp[ind[i]] > EPSILON_ )
       break;
   }
+  if (i != rowLen) {
+    for (int j = 0; j < rowLen; ++j) {
+      if ( fabs(coef[j])<=EPSILON_) {
+	doLift = false;
+	break;
+      }
+    }
+  } else {
+    doLift = false;
+  }
 
-  if (i == rowLen)  {
+  if (!doLift)  {
     delete [] sign;
     delete [] up; 
     delete [] x;   
@@ -556,7 +560,7 @@ CglFlowCover::generateOneFlowCut( const OsiSolverInterface & si,
 
     if ( columnType[ind[i]]==1 ) {   // Binary variable
       value = coef[i];
-      if (value > EPSILON_)
+      if (value > 0.0)
 	sign[i] = CGLFLOW_COL_BINPOS;
       else {
 	sign[i] = CGLFLOW_COL_BINNEG;
@@ -568,7 +572,7 @@ CglFlowCover::generateOneFlowCut( const OsiSolverInterface & si,
     }
     else {   
       value = coef[i];
-      if (value > EPSILON_)
+      if (value > 0.0)
 	sign[i] = CGLFLOW_COL_CONTPOS;
       else {
 	sign[i] = CGLFLOW_COL_CONTNEG;
@@ -1151,10 +1155,10 @@ CglFlowCover::generateOneFlowCut( const OsiSolverInterface & si,
 	if ( fabs(lastCoef) >= EPSILON_ ) {
 	  cutCoef[j]=lastCoef;
 	  cutInd[j++]=lastInd;
-	  lastCoef = cutCoef[i];
-	  if (i<cutLen)
-	    lastInd=cutInd[i];
 	}
+	lastCoef = cutCoef[i];
+	if (i<cutLen)
+	  lastInd=cutInd[i];
       } else {
 	lastCoef += cutCoef[i];
       }
@@ -1182,14 +1186,17 @@ CglFlowCover::generateOneFlowCut( const OsiSolverInterface & si,
 #endif
     cutLen = j;
     // Skip if no elements ? - bug somewhere
-    assert (cutLen);
+    if (cutLen == 0)
+        return false;
         
     // Recheck the violation.
+    double saveViolation = violation;
     violation = 0.0;
     for (i = 0; i < cutLen; ++i) 
       violation += cutCoef[i] * xlp[cutInd[i]];
     
     violation -= cutRHS;
+    assert (fabs(violation-saveViolation)<1.0e-2);
 
     if ( violation > TOLERANCE_ ) {
       flowCut.setRow(cutLen, cutInd, cutCoef);
@@ -1263,6 +1270,8 @@ CglFlowCover::determineOneRowType(const OsiSolverInterface& si,
 {
   if (rowLen == 0) 
     return CGLFLOW_ROW_UNDEFINED;
+  if (sense == 'R')
+    return CGLFLOW_ROW_UNINTERSTED; // Could be fixed
     
   CglFlowRowType rowType = CGLFLOW_ROW_UNDEFINED;
   // Get integer types
