@@ -6,7 +6,9 @@
 
 #include "core/fpdfapi/cmaps/GB1/cmaps_gb1.h"
 
-const uint16_t kFXCMAP_GBK_EUC_V_2[20 * 3] = {
+namespace fxcmap {
+
+const uint16_t kGBK_EUC_V_2[20 * 3] = {
     0xA1A2, 0xA1A2, 0x023F, 0xA1A3, 0xA1A3, 0x023E, 0xA1AA, 0xA1AA, 0x0256,
     0xA1AB, 0xA1AC, 0x1E18, 0xA1AD, 0xA1AD, 0x0257, 0xA1B2, 0xA1BF, 0x0246,
     0xA1FE, 0xA1FE, 0x1E1A, 0xA3A1, 0xA3A1, 0x0242, 0xA3A8, 0xA3A9, 0x0244,
@@ -15,3 +17,5 @@ const uint16_t kFXCMAP_GBK_EUC_V_2[20 * 3] = {
     0xA3DD, 0xA3DD, 0x1E1E, 0xA3DF, 0xA3DF, 0x0258, 0xA3FB, 0xA3FB, 0x0254,
     0xA3FD, 0xA3FD, 0x0255, 0xA3FE, 0xA3FE, 0x1E1F,
 };
+
+}  // namespace fxcmap
