@@ -6,7 +6,9 @@
 
 #include "core/fpdfapi/cmaps/Japan1/cmaps_japan1.h"
 
-const uint16_t kFXCMAP_90ms_RKSJ_H_2[171 * 3] = {
+namespace fxcmap {
+
+const uint16_t k90ms_RKSJ_H_2[171 * 3] = {
     0x0020, 0x007D, 0x00E7, 0x007E, 0x007E, 0x0277, 0x00A0, 0x00DF, 0x0146,
     0x8140, 0x817E, 0x0279, 0x8180, 0x81AC, 0x02B8, 0x81B8, 0x81BF, 0x02E5,
     0x81C8, 0x81CE, 0x02ED, 0x81DA, 0x81E8, 0x02F4, 0x81F0, 0x81F7, 0x0303,
@@ -65,3 +67,5 @@ const uint16_t kFXCMAP_90ms_RKSJ_H_2[171 * 3] = {
     0xFA80, 0xFACF, 0x20CA, 0xFAD0, 0xFAD0, 0x07C9, 0xFAD1, 0xFAFC, 0x211A,
     0xFB40, 0xFB7E, 0x2146, 0xFB80, 0xFBFC, 0x2185, 0xFC40, 0xFC4B, 0x2202,
 };
+
+}  // namespace fxcmap
