@@ -10,19 +10,21 @@
 #include <string.h>
 
 #include <algorithm>
+#include <array>
 #include <iterator>
 #include <utility>
 
 #include "core/fxcodec/cfx_codec_memory.h"
+#include "core/fxcrt/byteorder.h"
+#include "core/fxcrt/compiler_specific.h"
 #include "core/fxcrt/data_vector.h"
-#include "core/fxcrt/fx_system.h"
 #include "core/fxcrt/stl_util.h"
 
 namespace fxcodec {
 
 namespace {
 
-constexpr int32_t kGifInterlaceStep[4] = {8, 8, 4, 2};
+constexpr std::array<const int32_t, 4> kGifInterlaceStep = {{8, 8, 4, 2}};
 
 }  // namespace
 
@@ -41,13 +43,12 @@ bool CFX_GifContext::GetRecordPosition(uint32_t cur_pos,
                                        int32_t top,
                                        int32_t width,
                                        int32_t height,
-                                       int32_t pal_num,
-                                       CFX_GifPalette* pal,
+                                       pdfium::span<CFX_GifPalette> pal,
                                        int32_t trans_index,
                                        bool interlace) {
   return delegate_->GifInputRecordPositionBuf(
-      cur_pos, FX_RECT(left, top, left + width, top + height), pal_num, pal,
-      trans_index, interlace);
+      cur_pos, FX_RECT(left, top, left + width, top + height), pal, trans_index,
+      interlace);
 }
 
 GifDecoder::Status CFX_GifContext::ReadHeader() {
@@ -65,9 +66,9 @@ GifDecoder::Status CFX_GifContext::GetFrame() {
         return GifDecoder::Status::kSuccess;
       case GIF_D_STATUS_SIG: {
         uint8_t signature;
-        if (!ReadAllOrNone(&signature, sizeof(signature)))
+        if (!ReadAllOrNone(pdfium::byte_span_from_ref(signature))) {
           return GifDecoder::Status::kUnfinished;
-
+        }
         switch (signature) {
           case GIF_SIG_EXTENSION:
             SaveDecodingStatus(GIF_D_STATUS_EXT);
@@ -90,9 +91,9 @@ GifDecoder::Status CFX_GifContext::GetFrame() {
       }
       case GIF_D_STATUS_EXT: {
         uint8_t extension;
-        if (!ReadAllOrNone(&extension, sizeof(extension)))
+        if (!ReadAllOrNone(pdfium::byte_span_from_ref(extension))) {
           return GifDecoder::Status::kUnfinished;
-
+        }
         switch (extension) {
           case GIF_BLOCK_CE:
             SaveDecodingStatus(GIF_D_STATUS_EXT_CE);
@@ -123,10 +124,9 @@ GifDecoder::Status CFX_GifContext::GetFrame() {
       case GIF_D_STATUS_IMG_DATA: {
         uint8_t img_data_size;
         size_t read_marker = input_buffer_->GetPosition();
-
-        if (!ReadAllOrNone(&img_data_size, sizeof(img_data_size)))
+        if (!ReadAllOrNone(pdfium::byte_span_from_ref(img_data_size))) {
           return GifDecoder::Status::kUnfinished;
-
+        }
         while (img_data_size != GIF_BLOCK_TERMINAL) {
           if (!input_buffer_->Seek(input_buffer_->GetPosition() +
                                    img_data_size)) {
@@ -138,10 +138,10 @@ GifDecoder::Status CFX_GifContext::GetFrame() {
           // ScanForTerminalMarker() cannot be used here.
           SaveDecodingStatus(GIF_D_STATUS_IMG_DATA);
           read_marker = input_buffer_->GetPosition();
-          if (!ReadAllOrNone(&img_data_size, sizeof(img_data_size)))
+          if (!ReadAllOrNone(pdfium::byte_span_from_ref(img_data_size))) {
             return GifDecoder::Status::kUnfinished;
+          }
         }
-
         SaveDecodingStatus(GIF_D_STATUS_SIG);
         continue;
       }
@@ -170,18 +170,12 @@ GifDecoder::Status CFX_GifContext::LoadFrame(size_t frame_num) {
   if (decode_status_ == GIF_D_STATUS_TAIL) {
     gif_image->row_buffer.resize(gif_img_row_bytes);
     CFX_GifGraphicControlExtension* gif_img_gce = gif_image->image_GCE.get();
-    int32_t loc_pal_num =
-        gif_image->image_info.local_flags.local_pal
-            ? (2 << gif_image->image_info.local_flags.pal_bits)
-            : 0;
-    CFX_GifPalette* pLocalPalette = gif_image->local_palettes.empty()
-                                        ? nullptr
-                                        : gif_image->local_palettes.data();
+    pdfium::span<CFX_GifPalette> pLocalPalette = gif_image->local_palettes;
     if (!gif_img_gce) {
       bool bRes = GetRecordPosition(
           gif_image->data_pos, gif_image->image_info.left,
           gif_image->image_info.top, gif_image->image_info.width,
-          gif_image->image_info.height, loc_pal_num, pLocalPalette, -1,
+          gif_image->image_info.height, pLocalPalette, -1,
           gif_image->image_info.local_flags.interlace);
       if (!bRes) {
         gif_image->row_buffer.clear();
@@ -191,7 +185,7 @@ GifDecoder::Status CFX_GifContext::LoadFrame(size_t frame_num) {
       bool bRes = GetRecordPosition(
           gif_image->data_pos, gif_image->image_info.left,
           gif_image->image_info.top, gif_image->image_info.width,
-          gif_image->image_info.height, loc_pal_num, pLocalPalette,
+          gif_image->image_info.height, pLocalPalette,
           gif_image->image_GCE->gce_flags.transparency
               ? static_cast<int32_t>(gif_image->image_GCE->trans_index)
               : -1,
@@ -221,16 +215,15 @@ GifDecoder::Status CFX_GifContext::LoadFrame(size_t frame_num) {
   // TODO(crbug.com/pdfium/1793): This logic can be simplified a lot, but it
   // probably makes more sense to switch to a different GIF decoder altogether.
   if (decode_status_ == GIF_D_STATUS_IMG_DATA) {
-    if (!ReadAllOrNone(&img_data_size, sizeof(img_data_size)))
+    if (!ReadAllOrNone(pdfium::byte_span_from_ref(img_data_size))) {
       return GifDecoder::Status::kUnfinished;
-
+    }
     if (img_data_size != GIF_BLOCK_TERMINAL) {
       img_data.resize(img_data_size);
-      if (!ReadAllOrNone(img_data.data(), img_data_size)) {
+      if (!ReadAllOrNone(img_data)) {
         input_buffer_->Seek(read_marker);
         return GifDecoder::Status::kUnfinished;
       }
-
       if (!lzw_decompressor_) {
         lzw_decompressor_ = LZWDecompressor::Create(GetPaletteExp(gif_image),
                                                     gif_image->code_exp);
@@ -239,13 +232,15 @@ GifDecoder::Status CFX_GifContext::LoadFrame(size_t frame_num) {
           return GifDecoder::Status::kError;
         }
       }
-      lzw_decompressor_->SetSource(img_data.data(), img_data_size);
+      lzw_decompressor_->SetSource(img_data);
 
       SaveDecodingStatus(GIF_D_STATUS_IMG_DATA);
       img_row_offset_ += img_row_avail_size_;
       img_row_avail_size_ = gif_img_row_bytes - img_row_offset_;
-      LZWDecompressor::Status ret = lzw_decompressor_->Decode(
-          gif_image->row_buffer.data() + img_row_offset_, &img_row_avail_size_);
+      auto img_row_span = pdfium::make_span(gif_image->row_buffer)
+                              .subspan(img_row_offset_, img_row_avail_size_);
+      LZWDecompressor::Status ret = UNSAFE_TODO(
+          lzw_decompressor_->Decode(img_row_span.data(), &img_row_avail_size_));
       if (ret == LZWDecompressor::Status::kError) {
         DecodingFailureAtTailCleanup(gif_image);
         return GifDecoder::Status::kError;
@@ -261,24 +256,24 @@ GifDecoder::Status CFX_GifContext::LoadFrame(size_t frame_num) {
 
         if (ret == LZWDecompressor::Status::kUnfinished) {
           read_marker = input_buffer_->GetPosition();
-          if (!ReadAllOrNone(&img_data_size, sizeof(img_data_size)))
+          if (!ReadAllOrNone(pdfium::byte_span_from_ref(img_data_size))) {
             return GifDecoder::Status::kUnfinished;
-
+          }
           if (img_data_size != GIF_BLOCK_TERMINAL) {
             img_data.resize(img_data_size);
-            if (!ReadAllOrNone(img_data.data(), img_data_size)) {
+            if (!ReadAllOrNone(img_data)) {
               input_buffer_->Seek(read_marker);
               return GifDecoder::Status::kUnfinished;
             }
-
-            lzw_decompressor_->SetSource(img_data.data(), img_data_size);
+            lzw_decompressor_->SetSource(img_data);
 
             SaveDecodingStatus(GIF_D_STATUS_IMG_DATA);
             img_row_offset_ += img_row_avail_size_;
             img_row_avail_size_ = gif_img_row_bytes - img_row_offset_;
-            ret = lzw_decompressor_->Decode(
-                gif_image->row_buffer.data() + img_row_offset_,
-                &img_row_avail_size_);
+            img_row_span = pdfium::make_span(gif_image->row_buffer)
+                               .subspan(img_row_offset_, img_row_avail_size_);
+            ret = UNSAFE_TODO(lzw_decompressor_->Decode(img_row_span.data(),
+                                                        &img_row_avail_size_));
           }
         }
 
@@ -301,9 +296,10 @@ GifDecoder::Status CFX_GifContext::LoadFrame(size_t frame_num) {
 
           img_row_offset_ = 0;
           img_row_avail_size_ = gif_img_row_bytes;
-          ret = lzw_decompressor_->Decode(
-              gif_image->row_buffer.data() + img_row_offset_,
-              &img_row_avail_size_);
+          img_row_span = pdfium::make_span(gif_image->row_buffer)
+                             .subspan(img_row_offset_, img_row_avail_size_);
+          ret = UNSAFE_TODO(lzw_decompressor_->Decode(img_row_span.data(),
+                                                      &img_row_avail_size_));
         }
 
         if (ret == LZWDecompressor::Status::kError) {
@@ -325,54 +321,49 @@ uint32_t CFX_GifContext::GetAvailInput() const {
   if (!input_buffer_)
     return 0;
 
-  return pdfium::base::checked_cast<uint32_t>(input_buffer_->GetSize() -
-                                              input_buffer_->GetPosition());
+  return pdfium::checked_cast<uint32_t>(input_buffer_->GetSize() -
+                                        input_buffer_->GetPosition());
 }
 
-bool CFX_GifContext::ReadAllOrNone(uint8_t* dest, uint32_t size) {
-  if (!input_buffer_ || !dest)
+bool CFX_GifContext::ReadAllOrNone(pdfium::span<uint8_t> dest) {
+  if (!input_buffer_ || dest.empty()) {
     return false;
-
+  }
   size_t read_marker = input_buffer_->GetPosition();
-  size_t read = input_buffer_->ReadBlock({dest, size});
-  if (read < size) {
+  size_t read = input_buffer_->ReadBlock(dest);
+  if (read < dest.size()) {
     input_buffer_->Seek(read_marker);
     return false;
   }
-
   return true;
 }
 
 GifDecoder::Status CFX_GifContext::ReadGifSignature() {
   CFX_GifHeader header;
-  if (!ReadAllOrNone(reinterpret_cast<uint8_t*>(&header), 6))
+  if (!ReadAllOrNone(pdfium::byte_span_from_ref(header))) {
     return GifDecoder::Status::kUnfinished;
-
-  if (strncmp(header.signature, kGifSignature87, 6) != 0 &&
-      strncmp(header.signature, kGifSignature89, 6) != 0) {
+  }
+  if (UNSAFE_TODO(strncmp(header.signature, kGifSignature87, 6)) != 0 &&
+      UNSAFE_TODO(strncmp(header.signature, kGifSignature89, 6)) != 0) {
     return GifDecoder::Status::kError;
   }
-
   return GifDecoder::Status::kSuccess;
 }
 
 GifDecoder::Status CFX_GifContext::ReadLogicalScreenDescriptor() {
   CFX_GifLocalScreenDescriptor lsd;
   size_t read_marker = input_buffer_->GetPosition();
-
-  if (!ReadAllOrNone(reinterpret_cast<uint8_t*>(&lsd), sizeof(lsd)))
+  if (!ReadAllOrNone(pdfium::byte_span_from_ref(lsd))) {
     return GifDecoder::Status::kUnfinished;
-
+  }
   if (lsd.global_flags.global_pal) {
     uint32_t palette_count = unsigned(2 << lsd.global_flags.pal_bits);
     if (lsd.bc_index >= palette_count)
       return GifDecoder::Status::kError;
     bc_index_ = lsd.bc_index;
 
-    uint32_t palette_size = palette_count * sizeof(CFX_GifPalette);
     std::vector<CFX_GifPalette> palette(palette_count);
-    if (!ReadAllOrNone(reinterpret_cast<uint8_t*>(palette.data()),
-                       palette_size)) {
+    if (!ReadAllOrNone(pdfium::as_writable_byte_span(palette))) {
       // Roll back the read for the LSD
       input_buffer_->Seek(read_marker);
       return GifDecoder::Status::kUnfinished;
@@ -384,10 +375,8 @@ GifDecoder::Status CFX_GifContext::ReadLogicalScreenDescriptor() {
     std::swap(global_palette_, palette);
   }
 
-  width_ = static_cast<int>(
-      FXSYS_UINT16_GET_LSBFIRST(reinterpret_cast<uint8_t*>(&lsd.width)));
-  height_ = static_cast<int>(
-      FXSYS_UINT16_GET_LSBFIRST(reinterpret_cast<uint8_t*>(&lsd.height)));
+  width_ = fxcrt::FromLE16(lsd.width);
+  height_ = fxcrt::FromLE16(lsd.height);
 
   return GifDecoder::Status::kSuccess;
 }
@@ -409,9 +398,9 @@ GifDecoder::Status CFX_GifContext::DecodeExtension() {
     }
     case GIF_D_STATUS_EXT_PTE: {
       CFX_GifPlainTextExtension gif_pte;
-      if (!ReadAllOrNone(reinterpret_cast<uint8_t*>(&gif_pte), sizeof(gif_pte)))
+      if (!ReadAllOrNone(pdfium::byte_span_from_ref(gif_pte))) {
         return GifDecoder::Status::kUnfinished;
-
+      }
       graphic_control_extension_ = nullptr;
       if (!ScanForTerminalMarker()) {
         input_buffer_->Seek(read_marker);
@@ -421,16 +410,17 @@ GifDecoder::Status CFX_GifContext::DecodeExtension() {
     }
     case GIF_D_STATUS_EXT_GCE: {
       CFX_GifGraphicControlExtension gif_gce;
-      if (!ReadAllOrNone(reinterpret_cast<uint8_t*>(&gif_gce), sizeof(gif_gce)))
+      if (!ReadAllOrNone(pdfium::byte_span_from_ref(gif_gce))) {
         return GifDecoder::Status::kUnfinished;
-
-      if (!graphic_control_extension_.get())
+      }
+      if (!graphic_control_extension_.get()) {
         graphic_control_extension_ =
             std::make_unique<CFX_GifGraphicControlExtension>();
+      }
       graphic_control_extension_->block_size = gif_gce.block_size;
       graphic_control_extension_->gce_flags = gif_gce.gce_flags;
-      graphic_control_extension_->delay_time = FXSYS_UINT16_GET_LSBFIRST(
-          reinterpret_cast<uint8_t*>(&gif_gce.delay_time));
+      graphic_control_extension_->delay_time =
+          fxcrt::FromLE16(gif_gce.delay_time);
       graphic_control_extension_->trans_index = gif_gce.trans_index;
       break;
     }
@@ -454,18 +444,14 @@ GifDecoder::Status CFX_GifContext::DecodeImageInfo() {
 
   size_t read_marker = input_buffer_->GetPosition();
   CFX_GifImageInfo img_info;
-  if (!ReadAllOrNone(reinterpret_cast<uint8_t*>(&img_info), sizeof(img_info)))
+  if (!ReadAllOrNone(pdfium::byte_span_from_ref(img_info))) {
     return GifDecoder::Status::kUnfinished;
-
+  }
   auto gif_image = std::make_unique<CFX_GifImage>();
-  gif_image->image_info.left =
-      FXSYS_UINT16_GET_LSBFIRST(reinterpret_cast<uint8_t*>(&img_info.left));
-  gif_image->image_info.top =
-      FXSYS_UINT16_GET_LSBFIRST(reinterpret_cast<uint8_t*>(&img_info.top));
-  gif_image->image_info.width =
-      FXSYS_UINT16_GET_LSBFIRST(reinterpret_cast<uint8_t*>(&img_info.width));
-  gif_image->image_info.height =
-      FXSYS_UINT16_GET_LSBFIRST(reinterpret_cast<uint8_t*>(&img_info.height));
+  gif_image->image_info.left = fxcrt::FromLE16(img_info.left);
+  gif_image->image_info.top = fxcrt::FromLE16(img_info.top);
+  gif_image->image_info.width = fxcrt::FromLE16(img_info.width);
+  gif_image->image_info.height = fxcrt::FromLE16(img_info.height);
   gif_image->image_info.local_flags = img_info.local_flags;
   if (gif_image->image_info.left + gif_image->image_info.width > width_ ||
       gif_image->image_info.top + gif_image->image_info.height > height_)
@@ -476,17 +462,15 @@ GifDecoder::Status CFX_GifContext::DecodeImageInfo() {
     gif_image->local_palette_exp = gif_img_info_lf->pal_bits;
     uint32_t loc_pal_count = unsigned(2 << gif_img_info_lf->pal_bits);
     std::vector<CFX_GifPalette> loc_pal(loc_pal_count);
-    if (!ReadAllOrNone(reinterpret_cast<uint8_t*>(loc_pal.data()),
-                       loc_pal_count * sizeof(CFX_GifPalette))) {
+    if (!ReadAllOrNone(pdfium::as_writable_byte_span(loc_pal))) {
       input_buffer_->Seek(read_marker);
       return GifDecoder::Status::kUnfinished;
     }
-
     gif_image->local_palettes = std::move(loc_pal);
   }
 
   uint8_t code_size;
-  if (!ReadAllOrNone(&code_size, sizeof(code_size))) {
+  if (!ReadAllOrNone(pdfium::span_from_ref(code_size))) {
     input_buffer_->Seek(read_marker);
     return GifDecoder::Status::kUnfinished;
   }
@@ -519,17 +503,15 @@ void CFX_GifContext::DecodingFailureAtTailCleanup(CFX_GifImage* gif_image) {
 
 bool CFX_GifContext::ScanForTerminalMarker() {
   uint8_t data_size;
-
-  if (!ReadAllOrNone(&data_size, sizeof(data_size)))
+  if (!ReadAllOrNone(pdfium::span_from_ref(data_size))) {
     return false;
-
+  }
   while (data_size != GIF_BLOCK_TERMINAL) {
     if (!input_buffer_->Seek(input_buffer_->GetPosition() + data_size) ||
-        !ReadAllOrNone(&data_size, sizeof(data_size))) {
+        !ReadAllOrNone(pdfium::span_from_ref(data_size))) {
       return false;
     }
   }
-
   return true;
 }
 
