@@ -1,4 +1,4 @@
-// Copyright 2017 PDFium Authors. All rights reserved.
+// Copyright 2017 The PDFium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,7 +7,7 @@
 #ifndef CORE_FXCRT_CSS_CFX_CSSCUSTOMPROPERTY_H_
 #define CORE_FXCRT_CSS_CFX_CSSCUSTOMPROPERTY_H_
 
-#include "core/fxcrt/fx_string.h"
+#include "core/fxcrt/widestring.h"
 
 class CFX_CSSCustomProperty {
  public:
