@@ -1,4 +1,4 @@
-// Copyright 2017 PDFium Authors. All rights reserved.
+// Copyright 2017 The PDFium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,10 +7,8 @@
 #ifndef CORE_FXCRT_CSS_CFX_CSSOUTPUTTEXTBUF_H_
 #define CORE_FXCRT_CSS_CFX_CSSOUTPUTTEXTBUF_H_
 
-#include <vector>
-
-#include "core/fxcrt/fx_memory_wrappers.h"
-#include "core/fxcrt/fx_string.h"
+#include "core/fxcrt/data_vector.h"
+#include "core/fxcrt/widestring.h"
 
 class CFX_CSSOutputTextBuf {
  public:
@@ -23,7 +21,7 @@ class CFX_CSSOutputTextBuf {
   WideStringView GetTrailingBlankTrimmedString() const;
 
  protected:
-  std::vector<wchar_t, FxAllocAllocator<wchar_t>> m_Buffer;
+  DataVector<wchar_t> m_Buffer;
 };
 
 #endif  // CORE_FXCRT_CSS_CFX_CSSOUTPUTTEXTBUF_H_
