@@ -8,7 +8,7 @@
 
 #include <algorithm>
 
-#include "third_party/base/notreached.h"
+#include "core/fxcrt/notreached.h"
 
 // Color types are ordered by increasing number of components so we can choose
 // a best color type during some conversions.
@@ -87,7 +87,7 @@ CFX_Color CFX_Color::ConvertColorType(Type nConvertColorType) const {
         case CFX_Color::Type::kTransparent:
           break;
         case CFX_Color::Type::kGray:
-          NOTREACHED_NORETURN();
+          NOTREACHED();
         case CFX_Color::Type::kRGB:
           ret = ConvertGRAY2RGB(fColor1);
           break;
@@ -104,7 +104,7 @@ CFX_Color CFX_Color::ConvertColorType(Type nConvertColorType) const {
           ret = ConvertRGB2GRAY(fColor1, fColor2, fColor3);
           break;
         case CFX_Color::Type::kRGB:
-          NOTREACHED_NORETURN();
+          NOTREACHED();
         case CFX_Color::Type::kCMYK:
           ret = ConvertRGB2CMYK(fColor1, fColor2, fColor3);
           break;
@@ -121,7 +121,7 @@ CFX_Color CFX_Color::ConvertColorType(Type nConvertColorType) const {
           ret = ConvertCMYK2RGB(fColor1, fColor2, fColor3, fColor4);
           break;
         case CFX_Color::Type::kCMYK:
-          NOTREACHED_NORETURN();
+          NOTREACHED();
       }
       break;
   }
