@@ -19,6 +19,7 @@ class DetectorResult : public ZXing::DetectorResult
 	int _nbLayers = 0;
 	bool _readerInit = false;
 	bool _isMirrored = false;
+	int _runeValue = -1;
 
 	DetectorResult(const DetectorResult&) = delete;
 	DetectorResult& operator=(const DetectorResult&) = delete;
@@ -28,13 +29,14 @@ public:
 	DetectorResult(DetectorResult&&) noexcept = default;
 	DetectorResult& operator=(DetectorResult&&) noexcept = default;
 
-	DetectorResult(ZXing::DetectorResult&& result, bool isCompact, int nbDatablocks, int nbLayers, bool readerInit, bool isMirrored)
+	DetectorResult(ZXing::DetectorResult&& result, bool isCompact, int nbDatablocks, int nbLayers, bool readerInit, bool isMirrored, int runeValue)
 		: ZXing::DetectorResult{std::move(result)},
 		  _compact(isCompact),
 		  _nbDatablocks(nbDatablocks),
 		  _nbLayers(nbLayers),
 		  _readerInit(readerInit),
-		  _isMirrored(isMirrored)
+		  _isMirrored(isMirrored),
+		  _runeValue(runeValue)
 	{}
 
 	bool isCompact() const { return _compact; }
@@ -42,6 +44,9 @@ public:
 	int nbLayers() const { return _nbLayers; }
 	bool readerInit() const { return _readerInit; }
 	bool isMirrored() const { return _isMirrored; }
+
+	// Only meaningful is nbDatablocks == 0
+	int runeValue() const { return _runeValue; }
 };
 
 } // namespace ZXing::Aztec
