@@ -15,8 +15,8 @@ class Reader : public ZXing::Reader
 public:
 	using ZXing::Reader::Reader;
 
-	Result decode(const BinaryBitmap& image) const override;
-	Results decode(const BinaryBitmap& image, int maxSymbols) const override;
+	Barcode decode(const BinaryBitmap& image) const override;
+	Barcodes decode(const BinaryBitmap& image, int maxSymbols) const override;
 };
 
 } // namespace ZXing::Aztec
