@@ -15,7 +15,7 @@ class Code128Reader : public RowReader
 public:
 	using RowReader::RowReader;
 
-	Result decodePattern(int rowNumber, PatternView& next, std::unique_ptr<DecodingState>&) const override;
+	Barcode decodePattern(int rowNumber, PatternView& next, std::unique_ptr<DecodingState>&) const override;
 };
 
 } // namespace ZXing::OneD
