@@ -9,25 +9,32 @@
 #define DMSrcSink_DEFINED
 
 #include "gm/gm.h"
-#include "include/core/SkBBHFactory.h"
 #include "include/core/SkBitmap.h"
 #include "include/core/SkCanvas.h"
-#include "include/core/SkData.h"
 #include "include/core/SkPicture.h"
-#include "src/utils/SkMultiPictureDocument.h"
+#include "include/docs/SkMultiPictureDocument.h"
+#include "include/gpu/graphite/PrecompileContext.h"
 #include "tools/flags/CommonFlagsConfig.h"
 #include "tools/gpu/MemoryCache.h"
 
 #include <functional>
 
+#if !defined (SK_DISABLE_LEGACY_TESTS)
+    #include "include/gpu/graphite/ContextOptions.h"
+    #include "tools/graphite/TestOptions.h"
+#endif
+
 //#define TEST_VIA_SVG
 
-namespace skiagm {
-namespace verifiers {
+namespace skgpu {
+class UniqueKey;
+}
+namespace skiagm::verifiers {
 class VerifierList;
-}  // namespace verifiers
-}  // namespace skiagm
-
+}
+namespace skiatools::graphite {
+    class PipelineCallBackHandler;
+}
 namespace DM {
 
 // This is just convenience.  It lets you use either return "foo" or return SkStringPrintf(...).
@@ -91,32 +98,31 @@ struct SinkFlags {
 };
 
 struct Src {
+    using GraphiteTestContext = skiatest::graphite::GraphiteTestContext;
+
     virtual ~Src() {}
-    virtual Result SK_WARN_UNUSED_RESULT draw(SkCanvas* canvas) const = 0;
+    [[nodiscard]] virtual Result draw(SkCanvas* canvas, GraphiteTestContext*) const = 0;
     virtual SkISize size() const = 0;
     virtual Name name() const = 0;
-    virtual void modifyGrContextOptions(GrContextOptions* options) const {}
+    virtual void modifyGrContextOptions(GrContextOptions*) const  {}
+    virtual void modifyGraphiteContextOptions(skgpu::graphite::ContextOptions*) const {}
     virtual bool veto(SinkFlags) const { return false; }
 
     virtual int pageCount() const { return 1; }
-    virtual Result SK_WARN_UNUSED_RESULT draw([[maybe_unused]] int page, SkCanvas* canvas) const {
-        return this->draw(canvas);
+    [[nodiscard]] virtual Result draw([[maybe_unused]] int page,
+                                      SkCanvas* canvas,
+                                      GraphiteTestContext* graphiteTestContext) const {
+        return this->draw(canvas, graphiteTestContext);
     }
     virtual SkISize size([[maybe_unused]] int page) const { return this->size(); }
     // Force Tasks using this Src to run on the main thread?
     virtual bool serial() const { return false; }
-
-    /** Return a list of verifiers for the src, or null if no verifiers should be run .*/
-    virtual std::unique_ptr<skiagm::verifiers::VerifierList> getVerifiers() const {
-        return nullptr;
-    }
 };
 
 struct Sink {
     virtual ~Sink() {}
     // You may write to either the bitmap or stream.  If you write to log, we'll print that out.
-    virtual Result SK_WARN_UNUSED_RESULT draw(const Src&, SkBitmap*, SkWStream*, SkString* log)
-        const = 0;
+    [[nodiscard]] virtual Result draw(const Src&, SkBitmap*, SkWStream*, SkString* log) const = 0;
 
     // Override the color space of this Sink, after creation
     virtual void setColorSpace(sk_sp<SkColorSpace>) {}
@@ -139,12 +145,13 @@ class GMSrc : public Src {
 public:
     explicit GMSrc(skiagm::GMFactory);
 
-    Result draw(SkCanvas*) const override;
+    Result draw(SkCanvas*, GraphiteTestContext*) const override;
     SkISize size() const override;
     Name name() const override;
     void modifyGrContextOptions(GrContextOptions* options) const override;
-
-    std::unique_ptr<skiagm::verifiers::VerifierList> getVerifiers() const override;
+#if defined(SK_GRAPHITE)
+    void modifyGraphiteContextOptions(skgpu::graphite::ContextOptions*) const override;
+#endif
 
 private:
     skiagm::GMFactory fFactory;
@@ -171,7 +178,7 @@ public:
     };
     CodecSrc(Path, Mode, DstColorType, SkAlphaType, float);
 
-    Result draw(SkCanvas*) const override;
+    Result draw(SkCanvas*, GraphiteTestContext*) const override;
     SkISize size() const override;
     Name name() const override;
     bool veto(SinkFlags) const override;
@@ -189,7 +196,7 @@ class AndroidCodecSrc : public Src {
 public:
     AndroidCodecSrc(Path, CodecSrc::DstColorType, SkAlphaType, int sampleSize);
 
-    Result draw(SkCanvas*) const override;
+    Result draw(SkCanvas*, GraphiteTestContext*) const override;
     SkISize size() const override;
     Name name() const override;
     bool veto(SinkFlags) const override;
@@ -218,7 +225,7 @@ public:
 
     BRDSrc(Path, Mode, CodecSrc::DstColorType, uint32_t);
 
-    Result draw(SkCanvas*) const override;
+    Result draw(SkCanvas*, GraphiteTestContext*) const override;
     SkISize size() const override;
     Name name() const override;
     bool veto(SinkFlags) const override;
@@ -238,7 +245,7 @@ public:
     };
     ImageGenSrc(Path, Mode, SkAlphaType, bool);
 
-    Result draw(SkCanvas*) const override;
+    Result draw(SkCanvas*, GraphiteTestContext*) const override;
     SkISize size() const override;
     Name name() const override;
     bool veto(SinkFlags) const override;
@@ -255,7 +262,7 @@ class ColorCodecSrc : public Src {
 public:
     ColorCodecSrc(Path, bool decode_to_dst);
 
-    Result draw(SkCanvas*) const override;
+    Result draw(SkCanvas*, GraphiteTestContext*) const override;
     SkISize size() const override;
     Name name() const override;
     bool veto(SinkFlags) const override;
@@ -268,7 +275,7 @@ class SKPSrc : public Src {
 public:
     explicit SKPSrc(Path path);
 
-    Result draw(SkCanvas*) const override;
+    Result draw(SkCanvas*, GraphiteTestContext*) const override;
     SkISize size() const override;
     Name name() const override;
 private:
@@ -282,7 +289,7 @@ class BisectSrc : public SKPSrc {
 public:
     explicit BisectSrc(Path path, const char* trail);
 
-    Result draw(SkCanvas*) const override;
+    Result draw(SkCanvas*, GraphiteTestContext*) const override;
 
 private:
     SkString fTrail;
@@ -295,7 +302,7 @@ class SkottieSrc final : public Src {
 public:
     explicit SkottieSrc(Path path);
 
-    Result draw(SkCanvas*) const override;
+    Result draw(SkCanvas*, GraphiteTestContext*) const override;
     SkISize size() const override;
     Name name() const override;
     bool veto(SinkFlags) const override;
@@ -323,7 +330,7 @@ class SVGSrc : public Src {
 public:
     explicit SVGSrc(Path path);
 
-    Result draw(SkCanvas*) const override;
+    Result draw(SkCanvas*, GraphiteTestContext*) const override;
     SkISize size() const override;
     Name name() const override;
     bool veto(SinkFlags) const override;
@@ -343,8 +350,8 @@ public:
     explicit MSKPSrc(Path path);
 
     int pageCount() const override;
-    Result draw(SkCanvas* c) const override;
-    Result draw(int, SkCanvas*) const override;
+    Result draw(SkCanvas* c, GraphiteTestContext*) const override;
+    Result draw(int, SkCanvas*, GraphiteTestContext*) const override;
     SkISize size() const override;
     SkISize size(int) const override;
     Name name() const override;
@@ -375,7 +382,7 @@ public:
                   std::function<void(GrDirectContext*)> initContext = nullptr,
                   std::function<SkCanvas*(SkCanvas*)> wrapCanvas = nullptr) const;
 
-    sk_gpu_test::GrContextFactory::ContextType contextType() const { return fContextType; }
+    skgpu::ContextType contextType() const { return fContextType; }
     const sk_gpu_test::GrContextFactory::ContextOverrides& contextOverrides() const {
         return fContextOverrides;
     }
@@ -398,7 +405,7 @@ protected:
     bool readBack(SkSurface*, SkBitmap* dst) const;
 
 private:
-    sk_gpu_test::GrContextFactory::ContextType        fContextType;
+    skgpu::ContextType                                fContextType;
     sk_gpu_test::GrContextFactory::ContextOverrides   fContextOverrides;
     SkCommandLineConfigGpu::SurfType                  fSurfType;
     int                                               fSampleCount;
@@ -433,23 +440,6 @@ public:
     Result draw(const Src&, SkBitmap*, SkWStream*, SkString*) const override;
 };
 
-class GPUThreadTestingSink : public GPUSink {
-public:
-    GPUThreadTestingSink(const SkCommandLineConfigGpu*, const GrContextOptions&);
-
-    Result draw(const Src&, SkBitmap*, SkWStream*, SkString*) const override;
-
-    const char* fileExtension() const override {
-        // Suppress writing out results from this config - we just want to do our matching test
-        return nullptr;
-    }
-
-private:
-    std::unique_ptr<SkExecutor> fExecutor;
-
-    using INHERITED = GPUSink;
-};
-
 class GPUPersistentCacheTestingSink : public GPUSink {
 public:
     GPUPersistentCacheTestingSink(const SkCommandLineConfigGpu*, const GrContextOptions&);
@@ -467,9 +457,9 @@ private:
     using INHERITED = GPUSink;
 };
 
-class GPUPrecompileTestingSink : public GPUSink {
+class GaneshPrecompileTestingSink : public GPUSink {
 public:
-    GPUPrecompileTestingSink(const SkCommandLineConfigGpu*, const GrContextOptions&);
+    GaneshPrecompileTestingSink(const SkCommandLineConfigGpu*, const GrContextOptions&);
 
     Result draw(const Src&, SkBitmap*, SkWStream*, SkString*) const override;
 
@@ -584,28 +574,75 @@ private:
 
 class GraphiteSink : public Sink {
 public:
-    using ContextType = sk_gpu_test::GrContextFactory::ContextType;
-
-    GraphiteSink(const SkCommandLineConfigGraphite*);
+    GraphiteSink(const SkCommandLineConfigGraphite*, const skiatest::graphite::TestOptions&);
 
     Result draw(const Src&, SkBitmap*, SkWStream*, SkString*) const override;
     bool serial() const override { return true; }
     const char* fileExtension() const override { return "png"; }
-    SinkFlags flags() const override { return SinkFlags{ SinkFlags::kGPU, SinkFlags::kDirect }; }
+    SinkFlags flags() const override { return SinkFlags{SinkFlags::kGPU, SinkFlags::kDirect}; }
     void setColorSpace(sk_sp<SkColorSpace> colorSpace) override { fColorSpace = colorSpace; }
     SkColorInfo colorInfo() const override {
         return SkColorInfo(fColorType, fAlphaType, fColorSpace);
     }
 
-private:
-    ContextType fContextType;
+protected:
+    sk_sp<SkSurface> makeSurface(skgpu::graphite::Recorder*, SkISize) const;
+
+    skiatest::graphite::TestOptions fOptions;
+    skgpu::ContextType fContextType;
     SkColorType fColorType;
     SkAlphaType fAlphaType;
     sk_sp<SkColorSpace> fColorSpace;
 };
 
+#if defined(SK_ENABLE_PRECOMPILE)
+// In general this sink:
+//   renders a gm, skp or svg (in drawSrc)
+//   collects all the UniqueKeys                  |
+//   clears the pipeline cache                    | (in resetAndRecreatePipelines)
+//   recreates the pipelines from the UniqueKeys  |
+//   renders a second time (in drawSrc)
+//   asserts that no new pipelines were created
+class GraphitePrecompileTestingSink : public GraphiteSink {
+public:
+    GraphitePrecompileTestingSink(const SkCommandLineConfigGraphite*,
+                                  const skiatest::graphite::TestOptions&);
+    ~GraphitePrecompileTestingSink() override;
+
+    Result draw(const Src&, SkBitmap*, SkWStream*, SkString*) const override;
+
+    const char* fileExtension() const override {
+        // Suppress writing out results from this config - we just want to check that
+        // the precompilation API is expressive enough and prepopulates the cache.
+        // If desired, this could be updated to save the result of the precompiled rendering.
+        // However; if all the keys match, as is expected, the images should always match.
+        return nullptr;
+    }
+
+private:
+
+    Result drawSrc(const Src&,
+                   skgpu::graphite::Context*,
+                   skiatest::graphite::GraphiteTestContext*,
+                   skgpu::graphite::Recorder*) const;
+    Result resetAndRecreatePipelines(skiatools::graphite::PipelineCallBackHandler*,
+                                     skgpu::graphite::PrecompileContext*) const;
+
+#ifdef SK_DEBUG
+    static void LogMissingKey(skgpu::graphite::PrecompileContext*,
+                              const skgpu::UniqueKey& missingKey,
+                              const char* missingKeyName,
+                              const std::vector<skgpu::UniqueKey>& pool,
+                              const char* poolName);
 #endif
 
+    static void CompareKeys(skgpu::graphite::PrecompileContext*,
+                            const std::vector<skgpu::UniqueKey>& vA, const char* aName,
+                            const std::vector<skgpu::UniqueKey>& vB, const char* bName);
+};
+#endif // SK_ENABLE_PRECOMPILE
+#endif // SK_GRAPHITE
+
 /*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*/
 
 class Via : public Sink {
