@@ -1,7 +1,6 @@
 // Copyright 2019 Google LLC.
 // Use of this source code is governed by a BSD-style license that can be found in the LICENSE file.
 #include "tools/fiddle/examples.h"
-// HASH=935c8f8b9782d297a73d7186f6ef7945
 REG_FIDDLE(Canvas_drawTextRSXform, 256, 256, false, 0) {
 void draw(SkCanvas* canvas) {
     const int iterations = 26;
@@ -18,7 +17,7 @@ void draw(SkCanvas* canvas) {
         alphabet[i] = 'A' + i;
     }
     SkPaint paint;
-    SkFont font(nullptr, 20);
+    SkFont font(fontMgr->matchFamilyStyle(nullptr, {}), 20);
     auto spiral = SkTextBlob::MakeFromRSXform(alphabet, sizeof(alphabet), transforms, font);
     canvas->drawTextBlob(spiral, 110, 138, paint);
 }
