@@ -1,29 +1,21 @@
-/** 
- * XML Security Library example: Verifying a simple SAML response with X509 certificate
+/**
+ * XML Security Library example: Verifying a file signed with X509 certificate
  *
- * Verifies a simple SAML response. In addition to refular verification
- * we ensure that the signature has only one <dsig:Reference/> element
- * with an empty or NULL URI attribute and one enveloped signature transform
- * as it is required by SAML specification.
- * 
- * This example was developed and tested with OpenSSL crypto library. The 
+ * Verifies a file signed with X509 certificate.
+ *
+ * This example was developed and tested with OpenSSL crypto library. The
  * certificates management policies for another crypto library may break it.
  *
- * Usage: 
- *      verify4 <signed-file> <trusted-cert-pem-file1> [<trusted-cert-pem-file2> [...]]
+ * Usage:
+ *      verify4 <signed-file> <id-attribute-to-verify> <trusted-cert-pem-file1> [<trusted-cert-pem-file2> [...]]
  *
- * Example (sucecess):
- *      ./verify4 verify4-res.xml ca2cert.pem cacert.pem
- *
- * Example (failure):
- *      ./verify4 verify4-bad-res.xml ca2cert.pem cacert.pem
- * In the same time, verify3 example successfuly verifies this signature:
- *      ./verify3 verify4-bad-res.xml ca2cert.pem cacert.pem
+ * Example:
+ *      ./verify4 sign4-res.xml "data" ca2cert.pem cacert.pem
  *
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
- * 
- * Copyright (C) 2002-2016 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ *
+ * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 #include <stdlib.h>
 #include <string.h>
@@ -44,20 +36,21 @@
 #include <xmlsec/crypto.h>
 
 xmlSecKeysMngrPtr load_trusted_certs(char** files, int files_size);
-int verify_file(xmlSecKeysMngrPtr mngr, const char* xml_file);
+int verify_file(xmlSecKeysMngrPtr mngr, const char* xml_file, const char* id_attr);
+int verify_signature_results(xmlSecDSigCtxPtr dsigCtx, const char* id_attr);
 
-int 
+int
 main(int argc, char **argv) {
 #ifndef XMLSEC_NO_XSLT
     xsltSecurityPrefsPtr xsltSecPrefs = NULL;
 #endif /* XMLSEC_NO_XSLT */
     xmlSecKeysMngrPtr mngr;
-    
+
     assert(argv);
 
-    if(argc < 3) {
+    if(argc < 4) {
         fprintf(stderr, "Error: wrong number of arguments.\n");
-        fprintf(stderr, "Usage: %s <xml-file> <cert-file1> [<cert-file2> [...]]\n", argv[0]);
+        fprintf(stderr, "Usage: %s <xml-file> <id-attribute-to-verify> <cert-file1> [<cert-file2> [...]]\n", argv[0]);
         return(1);
     }
 
@@ -66,22 +59,19 @@ main(int argc, char **argv) {
     LIBXML_TEST_VERSION
     xmlLoadExtDtdDefaultValue = XML_DETECT_IDS | XML_COMPLETE_ATTRS;
     xmlSubstituteEntitiesDefault(1);
-#ifndef XMLSEC_NO_XSLT
-    xmlIndentTreeOutput = 1; 
-#endif /* XMLSEC_NO_XSLT */
 
     /* Init libxslt */
 #ifndef XMLSEC_NO_XSLT
     /* disable everything */
-    xsltSecPrefs = xsltNewSecurityPrefs(); 
+    xsltSecPrefs = xsltNewSecurityPrefs();
     xsltSetSecurityPrefs(xsltSecPrefs,  XSLT_SECPREF_READ_FILE,        xsltSecurityForbid);
     xsltSetSecurityPrefs(xsltSecPrefs,  XSLT_SECPREF_WRITE_FILE,       xsltSecurityForbid);
     xsltSetSecurityPrefs(xsltSecPrefs,  XSLT_SECPREF_CREATE_DIRECTORY, xsltSecurityForbid);
     xsltSetSecurityPrefs(xsltSecPrefs,  XSLT_SECPREF_READ_NETWORK,     xsltSecurityForbid);
     xsltSetSecurityPrefs(xsltSecPrefs,  XSLT_SECPREF_WRITE_NETWORK,    xsltSecurityForbid);
-    xsltSetDefaultSecurityPrefs(xsltSecPrefs); 
+    xsltSetDefaultSecurityPrefs(xsltSecPrefs);
 #endif /* XMLSEC_NO_XSLT */
-                
+
     /* Init xmlsec library */
     if(xmlSecInit() < 0) {
         fprintf(stderr, "Error: xmlsec initialization failed.\n");
@@ -96,15 +86,15 @@ main(int argc, char **argv) {
 
     /* Load default crypto engine if we are supporting dynamic
      * loading for xmlsec-crypto libraries. Use the crypto library
-     * name ("openssl", "nss", etc.) to load corresponding 
+     * name ("openssl", "nss", etc.) to load corresponding
      * xmlsec-crypto library.
      */
 #ifdef XMLSEC_CRYPTO_DYNAMIC_LOADING
     if(xmlSecCryptoDLLoadLibrary(NULL) < 0) {
         fprintf(stderr, "Error: unable to load default xmlsec-crypto library. Make sure\n"
                         "that you have it installed and check shared libraries path\n"
-                        "(LD_LIBRARY_PATH and/or LTDL_LIBRARY_PATH) envornment variables.\n");
-        return(-1);     
+                        "(LD_LIBRARY_PATH and/or LTDL_LIBRARY_PATH) environment variables.\n");
+        return(-1);
     }
 #endif /* XMLSEC_CRYPTO_DYNAMIC_LOADING */
 
@@ -121,36 +111,36 @@ main(int argc, char **argv) {
     }
 
     /* create keys manager and load trusted certificates */
-    mngr = load_trusted_certs(&(argv[2]), argc - 2);
+    mngr = load_trusted_certs(&(argv[3]), argc - 3);
     if(mngr == NULL) {
         return(-1);
     }
-    
+
     /* verify file */
-    if(verify_file(mngr, argv[1]) < 0) {
-        xmlSecKeysMngrDestroy(mngr);    
+    if(verify_file(mngr, argv[1], argv[2]) < 0) {
+        xmlSecKeysMngrDestroy(mngr);
         return(-1);
-    }    
-    
+    }
+
     /* destroy keys manager */
     xmlSecKeysMngrDestroy(mngr);
-    
+
     /* Shutdown xmlsec-crypto library */
     xmlSecCryptoShutdown();
-    
+
     /* Shutdown crypto library */
     xmlSecCryptoAppShutdown();
-    
+
     /* Shutdown xmlsec library */
     xmlSecShutdown();
 
     /* Shutdown libxslt/libxml */
 #ifndef XMLSEC_NO_XSLT
     xsltFreeSecurityPrefs(xsltSecPrefs);
-    xsltCleanupGlobals();            
+    xsltCleanupGlobals();
 #endif /* XMLSEC_NO_XSLT */
     xmlCleanupParser();
-    
+
     return(0);
 }
 
@@ -160,23 +150,23 @@ main(int argc, char **argv) {
  * @files_size:         the number of filenames in #files.
  *
  * Creates simple keys manager and load trusted certificates from PEM #files.
- * The caller is responsible for destroing returned keys manager using
+ * The caller is responsible for destroying returned keys manager using
  * @xmlSecKeysMngrDestroy.
  *
  * Returns the pointer to newly created keys manager or NULL if an error
  * occurs.
  */
-xmlSecKeysMngrPtr 
+xmlSecKeysMngrPtr
 load_trusted_certs(char** files, int files_size) {
     xmlSecKeysMngrPtr mngr;
     int i;
-        
+
     assert(files);
     assert(files_size > 0);
-    
+
     /* create and initialize keys manager, we use a simple list based
      * keys manager, implement your own xmlSecKeysStore klass if you need
-     * something more sophisticated 
+     * something more sophisticated
      */
     mngr = xmlSecKeysMngrCreate();
     if(mngr == NULL) {
@@ -187,8 +177,8 @@ load_trusted_certs(char** files, int files_size) {
         fprintf(stderr, "Error: failed to initialize keys manager.\n");
         xmlSecKeysMngrDestroy(mngr);
         return(NULL);
-    }    
-    
+    }
+
     for(i = 0; i < files_size; ++i) {
         assert(files[i]);
 
@@ -203,37 +193,43 @@ load_trusted_certs(char** files, int files_size) {
     return(mngr);
 }
 
-/** 
+/**
  * verify_file:
  * @mngr:               the pointer to keys manager.
  * @xml_file:           the signed XML file name.
+ * @id__attr:           the expected ID attribute for the node that was signed
  *
  * Verifies XML signature in #xml_file.
  *
  * Returns 0 on success or a negative value if an error occurs.
  */
-int 
-verify_file(xmlSecKeysMngrPtr mngr, const char* xml_file) {
+int
+verify_file(xmlSecKeysMngrPtr mngr, const char* xml_file, const char* id_attr) {
+    const xmlChar* id_attributes[] = { BAD_CAST "id", BAD_CAST "ID", NULL };
     xmlDocPtr doc = NULL;
     xmlNodePtr node = NULL;
     xmlSecDSigCtxPtr dsigCtx = NULL;
     int res = -1;
-    
+
     assert(mngr);
     assert(xml_file);
+    assert(id_attr);
 
     /* load file */
-    doc = xmlParseFile(xml_file);
+    doc = xmlReadFile(xml_file, NULL, XML_PARSE_PEDANTIC | XML_PARSE_NONET);
     if ((doc == NULL) || (xmlDocGetRootElement(doc) == NULL)){
         fprintf(stderr, "Error: unable to parse file \"%s\"\n", xml_file);
-        goto done;      
+        goto done;
     }
-    
+
+    /* add ID attributes to the doc context since we don't have DTDs */
+    xmlSecAddIDs(doc, NULL, id_attributes);
+
     /* find start node */
     node = xmlSecFindNode(xmlDocGetRootElement(doc), xmlSecNodeSignature, xmlSecDSigNs);
     if(node == NULL) {
         fprintf(stderr, "Error: start node not found in \"%s\"\n", xml_file);
-        goto done;      
+        goto done;
     }
 
     /* create signature context */
@@ -243,67 +239,94 @@ verify_file(xmlSecKeysMngrPtr mngr, const char* xml_file) {
         goto done;
     }
 
-    /* limit the Reference URI attributes to empty or NULL */
-    dsigCtx->enabledReferenceUris = xmlSecTransformUriTypeEmpty;
-    
-    /* limit allowed transforms for signature and reference processing */
-    if((xmlSecDSigCtxEnableSignatureTransform(dsigCtx, xmlSecTransformInclC14NId) < 0) ||
-       (xmlSecDSigCtxEnableSignatureTransform(dsigCtx, xmlSecTransformExclC14NId) < 0) ||
-       (xmlSecDSigCtxEnableSignatureTransform(dsigCtx, xmlSecTransformSha1Id) < 0) ||
-       (xmlSecDSigCtxEnableSignatureTransform(dsigCtx, xmlSecTransformRsaSha1Id) < 0)) {
-
-        fprintf(stderr,"Error: failed to limit allowed signature transforms\n");
-        goto done;
-    }
-    if((xmlSecDSigCtxEnableReferenceTransform(dsigCtx, xmlSecTransformInclC14NId) < 0) ||
-       (xmlSecDSigCtxEnableReferenceTransform(dsigCtx, xmlSecTransformExclC14NId) < 0) ||
-       (xmlSecDSigCtxEnableReferenceTransform(dsigCtx, xmlSecTransformSha1Id) < 0) ||
-       (xmlSecDSigCtxEnableReferenceTransform(dsigCtx, xmlSecTransformEnvelopedId) < 0)) {
-
-        fprintf(stderr,"Error: failed to limit allowed reference transforms\n");
-        goto done;
-    }
-
-    /* in addition, limit possible key data to valid X509 certificates only */
-    if(xmlSecPtrListAdd(&(dsigCtx->keyInfoReadCtx.enabledKeyData), BAD_CAST xmlSecKeyDataX509Id) < 0) {
-        fprintf(stderr,"Error: failed to limit allowed key data\n");
-        goto done;
-    }
-    
     /* Verify signature */
     if(xmlSecDSigCtxVerify(dsigCtx, node) < 0) {
         fprintf(stderr,"Error: signature verify\n");
         goto done;
     }
 
-    /* check that we have only one Reference */
-    if((dsigCtx->status == xmlSecDSigStatusSucceeded) && 
-        (xmlSecPtrListGetSize(&(dsigCtx->signedInfoReferences)) != 1)) {
-        
-        fprintf(stderr,"Error: only one reference is allowed\n");
-        goto done;
-    }
-        
-    /* print verification result to stdout */
-    if(dsigCtx->status == xmlSecDSigStatusSucceeded) {
+    /* verif results and print outcome to stdout */
+    if(verify_signature_results(dsigCtx, id_attr) == 0) {
         fprintf(stdout, "Signature is OK\n");
     } else {
         fprintf(stdout, "Signature is INVALID\n");
-    }    
+    }
 
     /* success */
     res = 0;
 
-done:    
+done:
     /* cleanup */
     if(dsigCtx != NULL) {
         xmlSecDSigCtxDestroy(dsigCtx);
     }
-    
+
     if(doc != NULL) {
-        xmlFreeDoc(doc); 
+        xmlFreeDoc(doc);
     }
     return(res);
 }
 
+/**
+ * verify_signature_results:
+ * @dsigCtx:            the XMLDSig context
+ * @id__attr:           the expected ID attribute for the node that was signed
+ *
+ * Verifies XML signature results to ensure that signature was applied
+ * to the expected data.
+ *
+ * Returns 0 on success or a negative value if an error occurs.
+ */
+int
+verify_signature_results(xmlSecDSigCtxPtr dsigCtx, const char* id_attr) {
+    xmlSecDSigReferenceCtxPtr dsigRefCtx;
+    xmlSecTransformPtr transform;
+    char uri[1024];
+
+    assert(dsigCtx);
+    assert(id_attr);
+
+    /* check that signature verification succeeded */
+    if(dsigCtx->status != xmlSecDSigStatusSucceeded) {
+        fprintf(stderr,"Error: Signature verificaton result is not SUCCESS\n");
+        return(-1);
+    }
+
+    /* in this example we expect exactly ONE reference with URI="" and
+    *  exactly ONE enveloped signature transform (i.e. the whole document is signed)*/
+    if(xmlSecPtrListGetSize(&(dsigCtx->signedInfoReferences)) != 1) {
+        fprintf(stderr,"Error: Exactly one Reference is expected\n");
+        return(-1);
+    }
+    dsigRefCtx = (xmlSecDSigReferenceCtxPtr)xmlSecPtrListGetItem(&(dsigCtx->signedInfoReferences), 0);
+    if((dsigRefCtx == NULL) || (dsigRefCtx->status != xmlSecDSigStatusSucceeded)) {
+        fprintf(stderr,"Error: Reference verification result is not SUCCESS\n");
+        return(-1);
+    }
+
+    /* check URI */
+    if (strlen(id_attr) + 2 > sizeof(uri)) {
+        fprintf(stderr, "Error: id attribute is too long\n");
+        return(-1);
+    }
+    sprintf(uri, "#%s", id_attr);
+
+    if(!xmlStrEqual(dsigRefCtx->uri, BAD_CAST uri)) {
+        fprintf(stderr,"Error: Reference URI value doesn't match expected one\n");
+        return(-1);
+    }
+
+    /* check transforms: all transforms should be inserted by XMLSec */
+    transform = dsigRefCtx->transformCtx.first;
+    while(transform != NULL) {
+        if((transform->flags & XMLSEC_TRANSFORM_FLAGS_USER_SPECIFIED) != 0) {
+            fprintf(stderr,"Error: Found unexpected Transform name '%s'\n", transform->id->name);
+            return(-1);
+        }
+        transform = transform->next;
+    }
+
+    /* all good! */
+    return(0);
+}
 
