@@ -21,12 +21,7 @@ class CBC_OnedEAN13Writer final : public CBC_OneDimEANWriter {
   ~CBC_OnedEAN13Writer() override;
 
   // CBC_OneDimEANWriter:
-  uint8_t* EncodeWithHint(const ByteString& contents,
-                          BC_TYPE format,
-                          int32_t& outWidth,
-                          int32_t& outHeight,
-                          int32_t hints) override;
-  uint8_t* EncodeImpl(const ByteString& contents, int32_t& outLength) override;
+  DataVector<uint8_t> Encode(const ByteString& contents) override;
   bool CheckContentValidity(WideStringView contents) override;
   WideString FilterContents(WideStringView contents) override;
   int32_t CalcChecksum(const ByteString& contents) override;
@@ -35,8 +30,7 @@ class CBC_OnedEAN13Writer final : public CBC_OneDimEANWriter {
   bool ShowChars(WideStringView contents,
                  CFX_RenderDevice* device,
                  const CFX_Matrix& matrix,
-                 int32_t barWidth,
-                 int32_t multiple) override;
+                 int32_t barWidth) override;
 
   int32_t m_codeWidth;
 };
