@@ -1,4 +1,4 @@
-// Copyright 2014 PDFium Authors. All rights reserved.
+// Copyright 2014 The PDFium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -14,7 +14,10 @@
 #include <vector>
 
 #include "build/build_config.h"
+#include "core/fxcrt/check_op.h"
+#include "core/fxcrt/compiler_specific.h"
 #include "core/fxcrt/fx_extension.h"
+#include "core/fxcrt/span.h"
 #include "fxjs/cjs_event_context.h"
 #include "fxjs/cjs_object.h"
 #include "fxjs/cjs_publicmethods.h"
@@ -23,11 +26,9 @@
 #include "fxjs/fxv8.h"
 #include "fxjs/js_define.h"
 #include "fxjs/js_resources.h"
-#include "third_party/base/check_op.h"
-#include "third_party/base/cxx17_backports.h"
 #include "v8/include/v8-date.h"
 
-#if defined(OS_ANDROID)
+#if BUILDFLAG(IS_ANDROID)
 #include <ctype.h>
 #endif
 
@@ -47,12 +48,12 @@ struct TbConvertAdditional {
   int value;
 };
 
-const TbConvert TbConvertTable[] = {
+const TbConvert kTbConvertTable[] = {
     {L"mmmm", L"%B"}, {L"mmm", L"%b"}, {L"mm", L"%m"},   {L"dddd", L"%A"},
     {L"ddd", L"%a"},  {L"dd", L"%d"},  {L"yyyy", L"%Y"}, {L"yy", L"%y"},
     {L"HH", L"%H"},   {L"hh", L"%I"},  {L"MM", L"%M"},   {L"ss", L"%S"},
     {L"TT", L"%p"},
-#if defined(OS_WIN)
+#if BUILDFLAG(IS_WIN)
     {L"tt", L"%p"},   {L"h", L"%#I"},
 #else
     {L"tt", L"%P"},   {L"h", L"%l"},
@@ -99,7 +100,7 @@ CJS_Util::CJS_Util(v8::Local<v8::Object> pObject, CJS_Runtime* pRuntime)
 CJS_Util::~CJS_Util() = default;
 
 CJS_Result CJS_Util::printf(CJS_Runtime* pRuntime,
-                            const std::vector<v8::Local<v8::Value>>& params) {
+                            pdfium::span<v8::Local<v8::Value>> params) {
   const size_t num_params = params.size();
   if (num_params < 1)
     return CJS_Result::Failure(JSMessage::kParamError);
@@ -112,7 +113,7 @@ CJS_Result CJS_Util::printf(CJS_Runtime* pRuntime,
   {
     size_t offset = 0;
     while (true) {
-      absl::optional<size_t> offset_end =
+      std::optional<size_t> offset_end =
           unsafe_fmt_string.Find(L"%", offset + 1);
       if (!offset_end.has_value()) {
         unsafe_conversion_specifiers.push_back(
@@ -162,7 +163,7 @@ CJS_Result CJS_Util::printf(CJS_Runtime* pRuntime,
 }
 
 CJS_Result CJS_Util::printd(CJS_Runtime* pRuntime,
-                            const std::vector<v8::Local<v8::Value>>& params) {
+                            pdfium::span<v8::Local<v8::Value>> params) {
   const size_t iSize = params.size();
   if (iSize < 2)
     return CJS_Result::Failure(JSMessage::kParamError);
@@ -217,38 +218,40 @@ CJS_Result CJS_Util::printd(CJS_Runtime* pRuntime,
   cFormat.erase(std::remove(cFormat.begin(), cFormat.end(), '%'),
                 cFormat.end());
 
-  for (size_t i = 0; i < pdfium::size(TbConvertTable); ++i) {
-    int iStart = 0;
-    int iEnd;
-    while ((iEnd = cFormat.find(TbConvertTable[i].lpszJSMark, iStart)) != -1) {
-      cFormat.replace(iEnd, wcslen(TbConvertTable[i].lpszJSMark),
-                      TbConvertTable[i].lpszCppMark);
-      iStart = iEnd;
+  for (const auto& conversion : kTbConvertTable) {
+    size_t nFound = 0;
+    while (true) {
+      nFound = cFormat.find(conversion.lpszJSMark, nFound);
+      if (nFound == std::wstring::npos) {
+        break;
+      }
+      cFormat.replace(nFound, UNSAFE_TODO(wcslen(conversion.lpszJSMark)),
+                      conversion.lpszCppMark);
     }
   }
 
   if (year < 0)
     return CJS_Result::Failure(JSMessage::kValueError);
 
-  const TbConvertAdditional cTableAd[] = {
+  const TbConvertAdditional table_additional[] = {
       {L'm', month}, {L'd', day},
       {L'H', hour},  {L'h', hour > 12 ? hour - 12 : hour},
       {L'M', min},   {L's', sec},
   };
 
-  for (size_t i = 0; i < pdfium::size(cTableAd); ++i) {
-    int iStart = 0;
-    int iEnd;
-    while ((iEnd = cFormat.find(cTableAd[i].js_mark, iStart)) != -1) {
-      if (iEnd > 0) {
-        if (cFormat[iEnd - 1] == L'%') {
-          iStart = iEnd + 1;
-          continue;
-        }
+  for (const auto& conversion : table_additional) {
+    size_t nFound = 0;
+    while (true) {
+      nFound = cFormat.find(conversion.js_mark, nFound);
+      if (nFound == std::wstring::npos) {
+        break;
       }
-      cFormat.replace(iEnd, 1,
-                      WideString::Format(L"%d", cTableAd[i].value).c_str());
-      iStart = iEnd;
+      if (nFound != 0 && cFormat[nFound - 1] == L'%') {
+        ++nFound;
+        continue;
+      }
+      cFormat.replace(nFound, 1,
+                      WideString::FormatInteger(conversion.value).c_str());
     }
   }
 
@@ -261,13 +264,13 @@ CJS_Result CJS_Util::printd(CJS_Runtime* pRuntime,
   time.tm_sec = sec;
 
   wchar_t buf[64] = {};
-  FXSYS_wcsftime(buf, 64, cFormat.c_str(), &time);
+  UNSAFE_TODO(FXSYS_wcsftime(buf, 64, cFormat.c_str(), &time));
   cFormat = buf;
   return CJS_Result::Success(pRuntime->NewString(cFormat.c_str()));
 }
 
 CJS_Result CJS_Util::printx(CJS_Runtime* pRuntime,
-                            const std::vector<v8::Local<v8::Value>>& params) {
+                            pdfium::span<v8::Local<v8::Value>> params) {
   if (params.size() < 2)
     return CJS_Result::Failure(JSMessage::kParamError);
 
@@ -368,7 +371,7 @@ WideString CJS_Util::StringPrintx(const WideString& wsFormat,
 }
 
 CJS_Result CJS_Util::scand(CJS_Runtime* pRuntime,
-                           const std::vector<v8::Local<v8::Value>>& params) {
+                           pdfium::span<v8::Local<v8::Value>> params) {
   if (params.size() < 2)
     return CJS_Result::Failure(JSMessage::kParamError);
 
@@ -384,9 +387,8 @@ CJS_Result CJS_Util::scand(CJS_Runtime* pRuntime,
   return CJS_Result::Success(pRuntime->NewDate(dDate));
 }
 
-CJS_Result CJS_Util::byteToChar(
-    CJS_Runtime* pRuntime,
-    const std::vector<v8::Local<v8::Value>>& params) {
+CJS_Result CJS_Util::byteToChar(CJS_Runtime* pRuntime,
+                                pdfium::span<v8::Local<v8::Value>> params) {
   if (params.size() < 1)
     return CJS_Result::Failure(JSMessage::kParamError);
 
