@@ -1,4 +1,4 @@
-// Copyright 2016 PDFium Authors. All rights reserved.
+// Copyright 2016 The PDFium Authors
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,25 +10,26 @@
 #include <memory>
 
 #include "fxjs/cfxjs_engine.h"
-#include "v8/include/v8.h"
+#include "v8/include/v8-forward.h"
+#include "v8/include/v8-persistent-handle.h"
 
-class CFXJSE_RuntimeData : public FXJS_PerIsolateData::ExtensionIface {
+class CFXJSE_RuntimeData final : public CFXJS_PerIsolateData::ExtensionIface {
  public:
-  ~CFXJSE_RuntimeData() override;
-
   static CFXJSE_RuntimeData* Get(v8::Isolate* pIsolate);
 
-  v8::Global<v8::FunctionTemplate> m_hRootContextGlobalTemplate;
-  v8::Global<v8::Context> m_hRootContext;
-
- protected:
-  CFXJSE_RuntimeData();
-
-  static std::unique_ptr<CFXJSE_RuntimeData> Create(v8::Isolate* pIsolate);
-
- private:
   CFXJSE_RuntimeData(const CFXJSE_RuntimeData&) = delete;
   CFXJSE_RuntimeData& operator=(const CFXJSE_RuntimeData&) = delete;
+  ~CFXJSE_RuntimeData() override;
+
+  v8::Local<v8::Context> GetRootContext(v8::Isolate* pIsolate);
+
+ private:
+  static std::unique_ptr<CFXJSE_RuntimeData> Create(v8::Isolate* pIsolate);
+
+  CFXJSE_RuntimeData();
+
+  v8::Global<v8::FunctionTemplate> root_context_global_template_;
+  v8::Global<v8::Context> root_context_;
 };
 
 #endif  // FXJS_XFA_CFXJSE_RUNTIMEDATA_H_
