@@ -40,9 +40,9 @@
 
 #include "dataprovider.h"
 
-#include "context.h"
-#include "decryptionresult.h"
-#include "data.h"
+#include <gpgme++/context.h>
+#include <gpgme++/decryptionresult.h>
+#include <gpgme++/data.h>
 
 #include <QBuffer>
 
@@ -122,15 +122,7 @@ GpgME::DecryptionResult QGpgME::QGpgMEDecryptJob::exec(const QByteArray &cipherT
 {
     const result_type r = decrypt_qba(context(), cipherText);
     plainText = std::get<1>(r);
-    resultHook(r);
-    return mResult;
-}
-
-//PENDING(marc) implement showErrorDialog()
-
-void QGpgMEDecryptJob::resultHook(const result_type &tuple)
-{
-    mResult = std::get<0>(tuple);
+    return std::get<0>(r);
 }
 
 #include "qgpgmedecryptjob.moc"
