@@ -63,7 +63,7 @@ public:
                             const skjson::ObjectValue* jobject,
                             SkV2* v, float* orientation);
 
-    bool isStatic() const { return fAnimators.empty(); }
+    bool isStatic() const { return fAnimators.empty() && !fHasSlotID; }
 
 protected:
     friend class skottie::SlotManager;
@@ -80,6 +80,7 @@ private:
 
     std::vector<sk_sp<Animator>> fAnimators;
     bool                         fHasSynced = false;
+    bool                         fHasSlotID = false;
 };
 
 } // namespace internal
