@@ -1,18 +1,21 @@
-                                           /*
+/*
 * Copyright 2022 Google Inc.
 *
 * Use of this source code is governed by a BSD-style license that can be
 * found in the LICENSE file.
 */
+#include "modules/skunicode/include/SkUnicode_client.h"
+
 #include "include/core/SkSpan.h"
 #include "include/core/SkString.h"
 #include "include/core/SkTypes.h"
-#include "include/private/SkBitmaskEnum.h"
 #include "include/private/base/SkTArray.h"
 #include "include/private/base/SkTo.h"
 #include "modules/skunicode/include/SkUnicode.h"
-#include "modules/skunicode/src/SkUnicode_client.h"
+#include "modules/skunicode/src/SkBidiFactory_icu_subset.h"
+#include "modules/skunicode/src/SkUnicode_hardcoded.h"
 #include "modules/skunicode/src/SkUnicode_icu_bidi.h"
+#include "src/base/SkBitmaskEnum.h"
 #include "src/base/SkUTF.h"
 
 #include <algorithm>
@@ -33,42 +36,7 @@
 
 using namespace skia_private;
 
-#ifndef SK_UNICODE_ICU_IMPLEMENTATION
-
-const char* SkUnicode_IcuBidi::errorName(UErrorCode status) {
-    return u_errorName_skia(status);
-}
-void SkUnicode_IcuBidi::bidi_close(UBiDi* bidi) {
-    ubidi_close_skia(bidi);
-}
-UBiDiDirection SkUnicode_IcuBidi::bidi_getDirection(const UBiDi* bidi) {
-    return ubidi_getDirection_skia(bidi);
-}
-SkBidiIterator::Position SkUnicode_IcuBidi::bidi_getLength(const UBiDi* bidi) {
-    return ubidi_getLength_skia(bidi);
-}
-SkBidiIterator::Level SkUnicode_IcuBidi::bidi_getLevelAt(const UBiDi* bidi, int pos) {
-    return ubidi_getLevelAt_skia(bidi, pos);
-}
-UBiDi* SkUnicode_IcuBidi::bidi_openSized(int32_t maxLength, int32_t maxRunCount, UErrorCode* pErrorCode) {
-    return ubidi_openSized_skia(maxLength, maxRunCount, pErrorCode);
-}
-void SkUnicode_IcuBidi::bidi_setPara(UBiDi* bidi,
-                         const UChar* text,
-                         int32_t length,
-                         UBiDiLevel paraLevel,
-                         UBiDiLevel* embeddingLevels,
-                         UErrorCode* status) {
-    return ubidi_setPara_skia(bidi, text, length, paraLevel, embeddingLevels, status);
-}
-void SkUnicode_IcuBidi::bidi_reorderVisual(const SkUnicode::BidiLevel runLevels[],
-                               int levelsCount,
-                               int32_t logicalFromVisual[]) {
-    ubidi_reorderVisual_skia(runLevels, levelsCount, logicalFromVisual);
-}
-#endif
-
-class SkUnicode_client : public SkUnicode {
+class SkUnicode_client : public SkUnicodeHardCodedCharProperties {
 public:
     struct Data {
         SkSpan<const char> fText8;
@@ -103,13 +71,6 @@ public:
                                            std::move(words),
                                            std::move(graphemeBreaks),
                                            std::move(lineBreaks))) { }
-    SkUnicode_client(const SkUnicode_client* origin)
-            : fData(origin->fData) {}
-
-
-    std::unique_ptr<SkUnicode> copy() override {
-        return std::make_unique<SkUnicode_client>(this);
-    }
 
     ~SkUnicode_client() override = default;
 
@@ -128,82 +89,23 @@ public:
                         int utf8Units,
                         TextDirection dir,
                         std::vector<BidiRegion>* results) override {
-        return SkUnicode::extractBidi(utf8, utf8Units, dir, results);
+        return fBidiFact->ExtractBidi(utf8, utf8Units, dir, results);
     }
 
-    // TODO: Take if from the Client or hard code here?
-    static bool isControl(SkUnichar utf8) {
-        return (utf8 < ' ') || (utf8 >= 0x7f && utf8 <= 0x9f) ||
-               (utf8 >= 0x200D && utf8 <= 0x200F) ||
-               (utf8 >= 0x202A && utf8 <= 0x202E);
+    bool getUtf8Words(const char utf8[],
+                      int utf8Units,
+                      const char* locale,
+                      std::vector<Position>* results) override {
+        SkDEBUGF("Method 'getUtf8Words' is not implemented\n");
+        return false;
     }
 
-    static bool isWhitespace(SkUnichar unichar) {
-        static constexpr std::array<SkUnichar, 21> whitespaces {
-                0x0009, // character tabulation
-                0x000A, // line feed
-                0x000B, // line tabulation
-                0x000C, // form feed
-                0x000D, // carriage return
-                0x0020, // space
-              //0x0085, // next line
-              //0x00A0, // no-break space
-                0x1680, // ogham space mark
-                0x2000, // en quad
-                0x2001, // em quad
-                0x2002, // en space
-                0x2003, // em space
-                0x2004, // three-per-em space
-                0x2005, // four-per-em space
-                0x2006, // six-per-em space
-              //0x2007, // figure space
-                0x2008, // punctuation space
-                0x2009, // thin space
-                0x200A, // hair space
-                0x2028, // line separator
-                0x2029, // paragraph separator
-              //0x202F, // narrow no-break space
-                0x205F, // medium mathematical space
-                0x3000};// ideographic space
-        return std::find(whitespaces.begin(), whitespaces.end(), unichar) != whitespaces.end();
-    }
-
-    static bool isSpace(SkUnichar unichar) {
-        static constexpr std::array<SkUnichar, 25> spaces {
-                0x0009, // character tabulation
-                0x000A, // line feed
-                0x000B, // line tabulation
-                0x000C, // form feed
-                0x000D, // carriage return
-                0x0020, // space
-                0x0085, // next line
-                0x00A0, // no-break space
-                0x1680, // ogham space mark
-                0x2000, // en quad
-                0x2001, // em quad
-                0x2002, // en space
-                0x2003, // em space
-                0x2004, // three-per-em space
-                0x2005, // four-per-em space
-                0x2006, // six-per-em space
-                0x2007, // figure space
-                0x2008, // punctuation space
-                0x2009, // thin space
-                0x200A, // hair space
-                0x2028, // line separator
-                0x2029, // paragraph separator
-                0x202F, // narrow no-break space
-                0x205F, // medium mathematical space
-                0x3000}; // ideographic space
-        return std::find(spaces.begin(), spaces.end(), unichar) != spaces.end();
-    }
-
-    static bool isTabulation(SkUnichar utf8) {
-        return utf8 == '\t';
-    }
-
-    static bool isHardBreak(SkUnichar utf8) {
-        return utf8 == '\n';
+    bool getSentences(const char utf8[],
+                      int utf8Units,
+                      const char* locale,
+                      std::vector<SkUnicode::Position>* results) override {
+        SkDEBUGF("Method 'getSentences' is not implemented\n");
+        return false;
     }
 
     bool computeCodeUnitFlags(char utf8[],
@@ -228,7 +130,7 @@ public:
             SkUnichar unichar = SkUTF::NextUTF8(&current, end);
             if (unichar < 0) unichar = 0xFFFD;
             auto after = current - utf8;
-            if (replaceTabs && SkUnicode_client::isTabulation(unichar)) {
+            if (replaceTabs && this->isTabulation(unichar)) {
                 results->at(before) |= SkUnicode::kTabulation;
                 if (replaceTabs) {
                     unichar = ' ';
@@ -236,15 +138,18 @@ public:
                 }
             }
             for (auto i = before; i < after; ++i) {
-                if (SkUnicode_client::isSpace(unichar)) {
+                if (this->isSpace(unichar)) {
                     results->at(i) |= SkUnicode::kPartOfIntraWordBreak;
                 }
-                if (SkUnicode_client::isWhitespace(unichar)) {
+                if (this->isWhitespace(unichar)) {
                     results->at(i) |= SkUnicode::kPartOfWhiteSpaceBreak;
                 }
-                if (SkUnicode_client::isControl(unichar)) {
+                if (this->isControl(unichar)) {
                     results->at(i) |= SkUnicode::kControl;
                 }
+                if (this->isIdeographic(unichar)) {
+                    results->at(i) |= SkUnicode::kIdeographic;
+                }
             }
         }
         return true;
@@ -263,6 +168,21 @@ public:
         for (auto& grapheme : fData->fGraphemeBreaks) {
             (*results)[grapheme] |= CodeUnitFlags::kGraphemeStart;
         }
+        for (auto i = 0; i < utf16Units; ++i) {
+            auto unichar = utf16[i];
+            if (this->isSpace(unichar)) {
+                results->at(i) |= SkUnicode::kPartOfIntraWordBreak;
+            }
+            if (this->isWhitespace(unichar)) {
+                results->at(i) |= SkUnicode::kPartOfWhiteSpaceBreak;
+            }
+            if (this->isControl(unichar)) {
+                results->at(i) |= SkUnicode::kControl;
+            }
+            if (this->isIdeographic(unichar)) {
+                results->at(i) |= SkUnicode::kIdeographic;
+            }
+        }
         return true;
     }
 
@@ -272,19 +192,23 @@ public:
     }
 
     SkString toUpper(const SkString& str) override {
-        SkASSERT(false);
+        return this->toUpper(str, nullptr);
+    }
+
+    SkString toUpper(const SkString& str, const char* locale) override {
         return SkString(fData->fText8.data(), fData->fText8.size());
     }
 
     void reorderVisual(const BidiLevel runLevels[],
                        int levelsCount,
                        int32_t logicalFromVisual[]) override {
-        SkUnicode_IcuBidi::bidi_reorderVisual(runLevels, levelsCount, logicalFromVisual);
+        fBidiFact->bidi_reorderVisual(runLevels, levelsCount, logicalFromVisual);
     }
 private:
     friend class SkBreakIterator_client;
 
     std::shared_ptr<Data> fData;
+    sk_sp<SkBidiFactory> fBidiFact = sk_make_sp<SkBidiSubsetFactory>();
 };
 
 class SkBreakIterator_client: public SkBreakIterator {
@@ -326,12 +250,12 @@ public:
 };
 std::unique_ptr<SkBidiIterator> SkUnicode_client::makeBidiIterator(const uint16_t text[], int count,
                                                  SkBidiIterator::Direction dir) {
-    return SkUnicode::makeBidiIterator(text, count, dir);
+    return fBidiFact->MakeIterator(text, count, dir);
 }
 std::unique_ptr<SkBidiIterator> SkUnicode_client::makeBidiIterator(const char text[],
                                                  int count,
                                                  SkBidiIterator::Direction dir) {
-    return SkUnicode::makeBidiIterator(text, count, dir);
+    return fBidiFact->MakeIterator(text, count, dir);
 }
 std::unique_ptr<SkBreakIterator> SkUnicode_client::makeBreakIterator(const char locale[],
                                                    BreakType breakType) {
@@ -341,11 +265,17 @@ std::unique_ptr<SkBreakIterator> SkUnicode_client::makeBreakIterator(BreakType b
     return std::make_unique<SkBreakIterator_client>(fData);
 }
 
-std::unique_ptr<SkUnicode> SkUnicode::MakeClientBasedUnicode(
+namespace SkUnicodes::Client {
+sk_sp<SkUnicode> Make(
         SkSpan<char> text,
         std::vector<SkUnicode::Position> words,
         std::vector<SkUnicode::Position> graphemeBreaks,
         std::vector<SkUnicode::LineBreakBefore> lineBreaks) {
-    return std::make_unique<SkUnicode_client>(text, words, graphemeBreaks, lineBreaks);
+    return sk_make_sp<SkUnicode_client>(text,
+                                        std::move(words),
+                                        std::move(graphemeBreaks),
+                                        std::move(lineBreaks));
+    }
 }
 
+
