@@ -54,7 +54,7 @@ static sk_sp<SkImage> LoadImage(const sk_sp<skresources::ResourceProvider>& rp,
             break;
         }
         default:
-            SkDebugf("error loading image: unhandled iri type %d\n", (int)href.type());
+            SkDEBUGF("error loading image: unhandled iri type %d\n", (int)href.type());
             return nullptr;
     }
 
@@ -90,7 +90,7 @@ void SkSVGImage::onRender(const SkSVGRenderContext& ctx) const {
 
     const auto imgInfo = LoadImage(ctx.resourceProvider(), fHref, viewPort, fPreserveAspectRatio);
     if (!imgInfo.fImage) {
-        SkDebugf("can't render image: load image failed\n");
+        SkDEBUGF("can't render image: load image failed\n");
         return;
     }
 
@@ -101,7 +101,7 @@ void SkSVGImage::onRender(const SkSVGRenderContext& ctx) const {
 
 SkPath SkSVGImage::onAsPath(const SkSVGRenderContext&) const { return {}; }
 
-SkRect SkSVGImage::onObjectBoundingBox(const SkSVGRenderContext& ctx) const {
+SkRect SkSVGImage::onTransformableObjectBoundingBox(const SkSVGRenderContext& ctx) const {
     const SkSVGLengthContext& lctx = ctx.lengthContext();
     return lctx.resolveRect(fX, fY, fWidth, fHeight);
 }
