@@ -6,16 +6,17 @@
 #include <mysql/client_plugin.h>
 
 typedef struct st_mysql_client_plugin_AUTHENTICATION auth_plugin_t;
-static int client_mpvio_write_packet(struct st_plugin_vio*, const uchar*, size_t);
+static int client_mpvio_write_packet(struct st_plugin_vio*, const uchar*, int);
 static int native_password_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql);
 static int dummy_fallback_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql __attribute__((unused)));
 extern void read_user_name(char *name);
 extern char *ma_send_connect_attr(MYSQL *mysql, unsigned char *buffer);
 extern int ma_read_ok_packet(MYSQL *mysql, uchar *pos, ulong length);
+extern unsigned char *mysql_net_store_length(unsigned char *packet, ulonglong length);
 
 typedef struct {
   int (*read_packet)(struct st_plugin_vio *vio, uchar **buf);
-  int (*write_packet)(struct st_plugin_vio *vio, const uchar *pkt, size_t pkt_len);
+  int (*write_packet)(struct st_plugin_vio *vio, const uchar *pkt, int pkt_len);
   void (*info)(struct st_plugin_vio *vio, struct st_plugin_vio_info *info);
   /* -= end of MYSQL_PLUGIN_VIO =- */
   MYSQL *mysql;
@@ -84,6 +85,7 @@ static int native_password_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql)
   if (mysql && mysql->passwd[0])
   {
     char scrambled[SCRAMBLE_LENGTH + 1];
+    memset(scrambled, 0, SCRAMBLE_LENGTH + 1);
     ma_scramble_41((uchar *)scrambled, (char*)pkt, mysql->passwd);
     if (vio->write_packet(vio, (uchar*)scrambled, SCRAMBLE_LENGTH))
       return CR_ERROR;
@@ -118,7 +120,7 @@ static int dummy_fallback_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql __attr
   unsigned int i, last_errno= ((MCPVIO_EXT *)vio)->mysql->net.last_errno;
   if (last_errno)
   {
-    strncpy(last_error, ((MCPVIO_EXT *)vio)->mysql->net.last_error,
+    memcpy(last_error, ((MCPVIO_EXT *)vio)->mysql->net.last_error,
             sizeof(last_error) - 1);
     last_error[sizeof(last_error) - 1]= 0;
   }
@@ -135,7 +137,7 @@ static int dummy_fallback_auth_client(MYSQL_PLUGIN_VIO *vio, MYSQL *mysql __attr
   if (last_errno)
   {
     MYSQL *mysql= ((MCPVIO_EXT *)vio)->mysql;
-    strncpy(mysql->net.last_error, last_error,
+    memcpy(mysql->net.last_error, last_error,
             sizeof(mysql->net.last_error) - 1);
     mysql->net.last_error[sizeof(mysql->net.last_error) - 1]= 0;
   }
@@ -198,8 +200,6 @@ error:
   return res;
 }
 
-
-
 static int send_client_reply_packet(MCPVIO_EXT *mpvio,
                                     const uchar *data, int data_len)
 {
@@ -223,19 +223,33 @@ static int send_client_reply_packet(MCPVIO_EXT *mpvio,
   if (mysql->options.ssl_key || mysql->options.ssl_cert ||
       mysql->options.ssl_ca || mysql->options.ssl_capath ||
       mysql->options.ssl_cipher || mysql->options.use_ssl ||
-      (mysql->options.client_flag & CLIENT_SSL_VERIFY_SERVER_CERT))
+      mysql->options.extension->tls_verify_server_cert)
     mysql->options.use_ssl= 1;
   if (mysql->options.use_ssl)
     mysql->client_flag|= CLIENT_SSL;
 #endif /* HAVE_TLS && !EMBEDDED_LIBRARY*/
   if (mpvio->db)
     mysql->client_flag|= CLIENT_CONNECT_WITH_DB;
+  else
+    /* See CONC-490: If no database was specified, we need
+       to unset CLIENT_CONNECT_WITH_DB flag */
+    mysql->client_flag&= ~CLIENT_CONNECT_WITH_DB;
+
+  /* CONC-635: For connections via named pipe or shared memory the server
+               indicates the capability for secure connections (TLS), but
+               doesn't support it. */
+  if ((mysql->server_capabilities & CLIENT_SSL) &&
+      (mysql->net.pvio->type == PVIO_TYPE_NAMEDPIPE ||
+       mysql->net.pvio->type == PVIO_TYPE_SHAREDMEM))
+  {
+    mysql->server_capabilities &= ~(CLIENT_SSL);
+  }
 
   /* if server doesn't support SSL and verification of server certificate
      was set to mandatory, we need to return an error */
   if (mysql->options.use_ssl && !(mysql->server_capabilities & CLIENT_SSL))
   {
-    if ((mysql->client_flag & CLIENT_SSL_VERIFY_SERVER_CERT) ||
+    if (mysql->options.extension->tls_verify_server_cert ||
         (mysql->options.extension && (mysql->options.extension->tls_fp || 
                                       mysql->options.extension->tls_fp_list)))
     {
@@ -249,12 +263,37 @@ static int send_client_reply_packet(MCPVIO_EXT *mpvio,
 
   /* Remove options that server doesn't support */
   mysql->client_flag= mysql->client_flag &
-                       (~(CLIENT_COMPRESS | CLIENT_SSL | CLIENT_PROTOCOL_41) 
+                       (~(CLIENT_COMPRESS | CLIENT_ZSTD_COMPRESSION | CLIENT_SSL | CLIENT_PROTOCOL_41) 
                        | mysql->server_capabilities);
 
-#ifndef HAVE_COMPRESS
-  mysql->client_flag&= ~CLIENT_COMPRESS;
-#endif
+  /* save compress for reconnect */
+  if (mysql->client_flag & CLIENT_COMPRESS)
+    mysql->options.compress= 1;
+
+  if (mysql->options.compress && (mysql->server_capabilities & CLIENT_COMPRESS))
+  {
+    /* For MySQL 8.0 we will use zstd compression */
+    if (mysql->server_capabilities & CLIENT_ZSTD_COMPRESSION)
+    {
+      if ((compression_plugin(net) = (MARIADB_COMPRESSION_PLUGIN *)mysql_client_find_plugin(mysql, 
+                                    _mariadb_compression_algorithm_str(COMPRESSION_ZSTD),
+                                    MARIADB_CLIENT_COMPRESSION_PLUGIN)))
+      {
+        mysql->client_flag|= CLIENT_ZSTD_COMPRESSION;
+        mysql->client_flag&= ~CLIENT_COMPRESS;
+      }
+    }
+    /* load zlib compression as default */
+    if (!compression_plugin(net))
+    {
+      if ((compression_plugin(net) = (MARIADB_COMPRESSION_PLUGIN *)mysql_client_find_plugin(mysql, 
+                                    _mariadb_compression_algorithm_str(COMPRESSION_ZLIB),
+                                    MARIADB_CLIENT_COMPRESSION_PLUGIN)))
+      {
+        mysql->client_flag|= CLIENT_COMPRESS;
+      }
+    }
+  }
 
   if (mysql->client_flag & CLIENT_PROTOCOL_41)
   {
@@ -267,7 +306,10 @@ static int send_client_reply_packet(MCPVIO_EXT *mpvio,
     memset(buff + 9, 0, 32-9);
     if (!(mysql->server_capabilities & CLIENT_MYSQL))
     {
-      mysql->extension->mariadb_client_flag = MARIADB_CLIENT_SUPPORTED_FLAGS >> 32;
+      uint server_extended_cap= mysql->extension->mariadb_server_capabilities;
+      uint client_extended_cap= (uint)(MARIADB_CLIENT_SUPPORTED_FLAGS >> 32);
+      mysql->extension->mariadb_client_flag=
+          server_extended_cap & client_extended_cap;
       int4store(buff + 28, mysql->extension->mariadb_client_flag);
     }
     end= buff+32;
@@ -323,7 +365,17 @@ static int send_client_reply_packet(MCPVIO_EXT *mpvio,
   {
     if (mysql->server_capabilities & CLIENT_SECURE_CONNECTION)
     {
-      *end++= data_len;
+      if (mysql->server_capabilities & CLIENT_PLUGIN_AUTH_LENENC_CLIENT_DATA)
+      {
+        end= (char *)mysql_net_store_length((uchar *)end, data_len);
+      }
+      else {
+        /* Without CLIENT_PLUGIN_AUTH_LENENC_CLIENT_DATA capability password
+           length is limited up to 255 chars */
+        if (data_len > 0xFF)
+          goto error;
+        *end++= data_len;
+      }
       memcpy(end, data, data_len);
       end+= data_len;
     }
@@ -349,6 +401,22 @@ static int send_client_reply_packet(MCPVIO_EXT *mpvio,
 
   end= ma_send_connect_attr(mysql, (unsigned char *)end);
 
+  /* MySQL 8.0: 
+     If zstd compresson was specified, the server expects
+     1 byte for compression level
+  */
+  if (mysql->client_flag & CLIENT_ZSTD_COMPRESSION)
+  {
+    uchar compression_level= 3;
+    if (mysql->options.extension &&
+        mysql->options.extension->zstd_compression_level >= 1 &&
+        mysql->options.extension->zstd_compression_level <= 20)
+    {
+        compression_level= mysql->options.extension->zstd_compression_level;
+    }
+    *end++= compression_level;
+  }
+
   /* Write authentication package */
   if (ma_net_write(net, (unsigned char *)buff, (size_t) (end-buff)) || ma_net_flush(net))
   {
@@ -438,7 +506,7 @@ static int client_mpvio_read_packet(struct st_plugin_vio *mpv, uchar **buf)
 */
 
 static int client_mpvio_write_packet(struct st_plugin_vio *mpv,
-                                     const uchar *pkt, size_t pkt_len)
+                                     const uchar *pkt, int pkt_len)
 {
   int res;
   MCPVIO_EXT *mpvio= (MCPVIO_EXT*)mpv;
@@ -446,9 +514,9 @@ static int client_mpvio_write_packet(struct st_plugin_vio *mpv,
   if (mpvio->packets_written == 0)
   {
     if (mpvio->mysql_change_user)
-      res= send_change_user_packet(mpvio, pkt, (int)pkt_len);
+      res= send_change_user_packet(mpvio, pkt, pkt_len);
     else
-      res= send_client_reply_packet(mpvio, pkt, (int)pkt_len);
+      res= send_client_reply_packet(mpvio, pkt, pkt_len);
   }
   else
   {
@@ -547,39 +615,38 @@ static void client_mpvio_info(MYSQL_PLUGIN_VIO *vio,
 int run_plugin_auth(MYSQL *mysql, char *data, uint data_len,
                     const char *data_plugin, const char *db)
 {
-  const char    *auth_plugin_name;
+  const char    *auth_plugin_name= NULL;
   auth_plugin_t *auth_plugin;
   MCPVIO_EXT    mpvio;
   ulong		pkt_length;
   int           res;
 
+
   /* determine the default/initial plugin to use */
-  if (mysql->options.extension && mysql->options.extension->default_auth &&
-      mysql->server_capabilities & CLIENT_PLUGIN_AUTH)
+  if (mysql->server_capabilities & CLIENT_PLUGIN_AUTH)
   {
-    auth_plugin_name= mysql->options.extension->default_auth;
-    if (!(auth_plugin= (auth_plugin_t*) mysql_client_find_plugin(mysql,
-                       auth_plugin_name, MYSQL_CLIENT_AUTHENTICATION_PLUGIN)))
-      auth_plugin= &dummy_fallback_client_plugin;
+    if (mysql->options.extension && mysql->options.extension->default_auth)
+      auth_plugin_name= mysql->options.extension->default_auth;
+    else if (data_plugin)
+      auth_plugin_name= data_plugin;
   }
-  else
+  if (!auth_plugin_name)
   {
     if (mysql->server_capabilities & CLIENT_PROTOCOL_41)
-      auth_plugin= &mysql_native_password_client_plugin;
+       auth_plugin_name= native_password_plugin_name;
     else
-    {
-      if (!(auth_plugin= (auth_plugin_t*)mysql_client_find_plugin(mysql,
-                         "mysql_old_password", MYSQL_CLIENT_AUTHENTICATION_PLUGIN)))
-        auth_plugin= &dummy_fallback_client_plugin;
-    }
-    auth_plugin_name= auth_plugin->name;
+       auth_plugin_name= "mysql_old_password";
   }
+  if (!(auth_plugin= (auth_plugin_t*) mysql_client_find_plugin(mysql,
+                     auth_plugin_name, MYSQL_CLIENT_AUTHENTICATION_PLUGIN)))
+    auth_plugin= &dummy_fallback_client_plugin;
 
   mysql->net.last_errno= 0; /* just in case */
 
   if (data_plugin && strcmp(data_plugin, auth_plugin_name))
   {
-    /* data was prepared for a different plugin, don't show it to this one */
+    /* data was prepared for a different plugin, so we don't
+       send any data */
     data= 0;
     data_len= 0;
   }
@@ -597,6 +664,17 @@ int run_plugin_auth(MYSQL *mysql, char *data, uint data_len,
 retry:
   mpvio.plugin= auth_plugin;
 
+  if (auth_plugin_name &&
+     mysql->options.extension &&
+     mysql->options.extension->restricted_auth)
+  {
+    if (!strstr(mysql->options.extension->restricted_auth, auth_plugin_name))
+    {
+      my_set_error(mysql, CR_PLUGIN_NOT_ALLOWED, SQLSTATE_UNKNOWN, 0, data_plugin);
+      return 1;
+    }
+  }
+
   mysql->net.read_pos[0]= 0;
   res= auth_plugin->authenticate_user((struct st_plugin_vio *)&mpvio, mysql);
 
