@@ -14,7 +14,7 @@
 // under GPL version 2 or later
 //
 // Copyright (C) 2008 Koji Otani <sho@bbr.jp>
-// Copyright (C) 2009, 2018-2020, 2022, 2024 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2009, 2018-2020, 2022, 2024, 2025 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2012, 2017 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
 //
@@ -46,15 +46,15 @@ class CMap
 public:
     // Parse a CMap from <obj>, which can be a name or a stream.  Sets
     // the initial reference count to 1.  Returns NULL on failure.
-    static std::shared_ptr<CMap> parse(CMapCache *cache, const GooString &collectionA, Object *obj);
+    static std::shared_ptr<CMap> parse(CMapCache *cache, const std::string &collectionA, Object *obj);
 
     // Create the CMap specified by <collection> and <cMapName>.  Sets
     // the initial reference count to 1.  Returns NULL on failure.
-    static std::shared_ptr<CMap> parse(CMapCache *cache, const GooString &collectionA, const GooString &cMapNameA);
+    static std::shared_ptr<CMap> parse(CMapCache *cache, const std::string &collectionA, const std::string &cMapNameA);
 
     // Parse a CMap from <str>.  Sets the initial reference count to 1.
     // Returns NULL on failure.
-    static std::shared_ptr<CMap> parse(CMapCache *cache, const GooString &collectionA, Stream *str);
+    static std::shared_ptr<CMap> parse(CMapCache *cache, const std::string &collectionA, Stream *str);
 
     ~CMap();
 
@@ -68,7 +68,7 @@ public:
 
     // Return true if this CMap matches the specified <collectionA>, and
     // <cMapNameA>.
-    bool match(const GooString &collectionA, const GooString &cMapNameA);
+    bool match(const std::string &collectionA, const std::string &cMapNameA);
 
     // Return the CID corresponding to the character code starting at
     // <s>, which contains <len> bytes.  Sets *<c> to the char code, and
@@ -119,7 +119,7 @@ public:
     // Stream is a stream containing the CMap, can be NULL and
     // this means the CMap will be searched in the CMap files
     // Returns NULL on failure.
-    std::shared_ptr<CMap> getCMap(const GooString &collection, const GooString &cMapName);
+    std::shared_ptr<CMap> getCMap(const std::string &collection, const std::string &cMapName);
 
 private:
     std::array<std::shared_ptr<CMap>, cMapCacheSize> cache;
