@@ -55,11 +55,11 @@ Function::~Function() = default;
 
 std::unique_ptr<Function> Function::parse(Object *funcObj)
 {
-    std::set<int> usedParents;
-    return parse(funcObj, &usedParents);
+    RefRecursionChecker usedParents;
+    return parse(funcObj, usedParents);
 }
 
-std::unique_ptr<Function> Function::parse(Object *funcObj, std::set<int> *usedParents)
+std::unique_ptr<Function> Function::parse(Object *funcObj, RefRecursionChecker &usedParents)
 {
     Dict *dict;
 
@@ -664,7 +664,7 @@ void ExponentialFunction::transform(const double *in, double *out) const
 // StitchingFunction
 //------------------------------------------------------------------------
 
-StitchingFunction::StitchingFunction(Object *funcObj, Dict *dict, std::set<int> *usedParents)
+StitchingFunction::StitchingFunction(Object *funcObj, Dict *dict, RefRecursionChecker &usedParents)
 {
     Object obj1;
     int i;
@@ -689,23 +689,19 @@ StitchingFunction::StitchingFunction(Object *funcObj, Dict *dict, std::set<int>
         error(errSyntaxError, -1, "Missing 'Functions' entry in stitching function");
         return;
     }
-    k = obj1.arrayGetLength();
+    const int k = obj1.arrayGetLength();
     funcs.resize(k);
     bounds = (double *)gmallocn(k + 1, sizeof(double));
     encode = (double *)gmallocn(2 * k, sizeof(double));
     scale = (double *)gmallocn(k, sizeof(double));
     for (i = 0; i < k; ++i) {
-        std::set<int> usedParentsAux = *usedParents;
         Ref ref;
         Object obj2 = obj1.getArray()->get(i, &ref);
-        if (ref != Ref::INVALID()) {
-            if (usedParentsAux.find(ref.num) == usedParentsAux.end()) {
-                usedParentsAux.insert(ref.num);
-            } else {
-                return;
-            }
+        if (!usedParents.insert(ref)) {
+            return;
         }
-        if (!(funcs[i] = Function::parse(&obj2, &usedParentsAux))) {
+        const RefRecursionCheckerRemover remover(usedParents, ref);
+        if (!(funcs[i] = Function::parse(&obj2, usedParents))) {
             return;
         }
         if (funcs[i]->getInputSize() != 1 || (i > 0 && funcs[i]->getOutputSize() != funcs[0]->getOutputSize())) {
@@ -763,13 +759,12 @@ StitchingFunction::StitchingFunction(Object *funcObj, Dict *dict, std::set<int>
 
 StitchingFunction::StitchingFunction(const StitchingFunction *func, PrivateTag) : Function(func)
 {
-    k = func->k;
-
-    funcs.resize(func->funcs.size());
-    for (int i = 0; i < k; ++i) {
-        funcs[i] = func->funcs[i]->copy();
+    funcs.reserve(func->funcs.size());
+    for (const std::unique_ptr<Function> &f : func->funcs) {
+        funcs.push_back(f->copy());
     }
 
+    const int k = funcs.size();
     bounds = (double *)gmallocn(k + 1, sizeof(double));
     memcpy(bounds, func->bounds, (k + 1) * sizeof(double));
 
@@ -801,6 +796,7 @@ void StitchingFunction::transform(const double *in, double *out) const
     } else {
         x = in[0];
     }
+    const int k = funcs.size();
     for (i = 0; i < k - 1; ++i) {
         if (x < bounds[i + 1]) {
             break;
