@@ -4,8 +4,10 @@
 //
 // This file is licensed under the GPLv2 or later
 //
-// Copyright 2023, 2024 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright 2023-2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 //========================================================================
+#ifndef GPGME_CRYPTO_SIGN_BACKEND_H
+#define GPGME_CRYPTO_SIGN_BACKEND_H
 
 #include "CryptoSignBackend.h"
 
@@ -32,19 +34,20 @@ public:
     GpgSignatureCreation(const std::string &certId);
     void addData(unsigned char *dataBlock, int dataLen) final;
     std::unique_ptr<X509CertificateInfo> getCertificateInfo() const final;
-    std::variant<std::vector<unsigned char>, CryptoSign::SigningError> signDetached(const std::string &password) final;
+    std::variant<std::vector<unsigned char>, CryptoSign::SigningErrorMessage> signDetached(const std::string &password) final;
     CryptoSign::SignatureType signatureType() const final;
 
 private:
     std::unique_ptr<GpgME::Context> gpgContext;
     GpgME::Data gpgData;
     std::optional<GpgME::Key> key;
+    GpgME::Protocol protocol;
 };
 
 class GpgSignatureVerification : public CryptoSign::VerificationInterface
 {
 public:
-    explicit GpgSignatureVerification(const std::vector<unsigned char> &pkcs7data);
+    explicit GpgSignatureVerification(const std::vector<unsigned char> &pkcs7data, GpgME::Protocol protocol);
     SignatureValidationStatus validateSignature() final;
     void addData(unsigned char *dataBlock, int dataLen) final;
     std::chrono::system_clock::time_point getSigningTime() const final;
@@ -62,7 +65,10 @@ private:
     std::optional<GpgME::VerificationResult> gpgResult;
     std::future<CertificateValidationStatus> validationStatus;
     std::optional<CertificateValidationStatus> cachedValidationStatus;
+    GpgME::Protocol protocol;
 #if DUMP_SIGNATURE_DATA
     std::unique_ptr<std::ofstream> debugSignedData;
 #endif
 };
+
+#endif // GPGME_CRYPTO_SIGN_BACKEND_H
