@@ -8,6 +8,7 @@
  * Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
  * Copyright (C) 2018 Adam Reichold <adam.reichold@t-online.de>
  * Copyright (C) 2019, 2020 Oliver Sander <oliver.sander@tu-dresden.de>
+ * Copyright (C) 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -152,7 +153,7 @@ QSet<OptContentItem *> OptContentItem::recurseListChildren(bool includeMe) const
     return ret;
 }
 
-OptContentModelPrivate::OptContentModelPrivate(OptContentModel *qq, OCGs *optContent) : q(qq)
+OptContentModelPrivate::OptContentModelPrivate(OptContentModel *qq, const OCGs *optContent) : q(qq)
 {
     m_rootNode = new OptContentItem();
     const auto &ocgs = optContent->getOCGs();
@@ -184,7 +185,7 @@ OptContentModelPrivate::~OptContentModelPrivate()
     delete m_rootNode;
 }
 
-void OptContentModelPrivate::parseOrderArray(OptContentItem *parentNode, Array *orderArray)
+void OptContentModelPrivate::parseOrderArray(OptContentItem *parentNode, const Array *orderArray)
 {
     OptContentItem *lastItem = parentNode;
     for (int i = 0; i < orderArray->getLength(); ++i) {
@@ -215,7 +216,7 @@ void OptContentModelPrivate::parseOrderArray(OptContentItem *parentNode, Array *
     }
 }
 
-void OptContentModelPrivate::parseRBGroupsArray(Array *rBGroupArray)
+void OptContentModelPrivate::parseRBGroupsArray(const Array *rBGroupArray)
 {
     if (!rBGroupArray) {
         return;
@@ -233,7 +234,7 @@ void OptContentModelPrivate::parseRBGroupsArray(Array *rBGroupArray)
     }
 }
 
-OptContentModel::OptContentModel(OCGs *optContent, QObject *parent) : QAbstractItemModel(parent)
+OptContentModel::OptContentModel(const OCGs *optContent, QObject *parent) : QAbstractItemModel(parent)
 {
     d = new OptContentModelPrivate(this, optContent);
 }
@@ -352,7 +353,7 @@ bool OptContentModel::setData(const QModelIndex &index, const QVariant &value, i
             }
             std::stable_sort(indexes.begin(), indexes.end()); // NOLINT(modernize-use-ranges) QModelIndexList doesn't support ranges requirements
             Q_FOREACH (const QModelIndex &changedIndex, indexes) {
-                emit dataChanged(changedIndex, changedIndex);
+                Q_EMIT dataChanged(changedIndex, changedIndex);
             }
             return true;
         }
@@ -413,7 +414,7 @@ void OptContentModel::applyLink(LinkOCGState *link)
         }
         std::stable_sort(indexes.begin(), indexes.end()); // NOLINT(modernize-use-ranges) Qt5 QModelIndexList doesn't support ranges requirements
         Q_FOREACH (const QModelIndex &changedIndex, indexes) {
-            emit dataChanged(changedIndex, changedIndex);
+            Q_EMIT dataChanged(changedIndex, changedIndex);
         }
     }
 }
