@@ -29,6 +29,7 @@
  * Copyright (C) 2023 Kevin Ottens <kevin.ottens@enioka.com>. Work sponsored by De Bortoli Wines
  * Copyright (C) 2024 Stefan Brüns <stefan.bruens@rwth-aachen.de>
  * Copyright (C) 2024 Pratham Gandhi <ppg.1382@gmail.com>
+ * Copyright (C) 2024 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -228,7 +229,7 @@ std::unique_ptr<Link> PageData::convertLinkActionToLink(::LinkAction *a, Documen
     case actionLaunch: {
         LinkLaunch *e = (LinkLaunch *)a;
         const GooString *p = e->getParams();
-        popplerLink = std::make_unique<LinkExecute>(linkArea, e->getFileName()->c_str(), p ? p->c_str() : nullptr);
+        popplerLink = std::make_unique<LinkExecute>(linkArea, UnicodeParsedString(e->getFileName()), p ? QString::fromStdString(p->toStr()) : QString {});
     } break;
 
     case actionNamed: {
@@ -268,7 +269,7 @@ std::unique_ptr<Link> PageData::convertLinkActionToLink(::LinkAction *a, Documen
     } break;
 
     case actionURI: {
-        popplerLink = std::make_unique<LinkBrowse>(linkArea, ((LinkURI *)a)->getURI().c_str());
+        popplerLink = std::make_unique<LinkBrowse>(linkArea, QString::fromStdString(((LinkURI *)a)->getURI()));
     } break;
 
     case actionSound: {
@@ -346,6 +347,30 @@ std::unique_ptr<Link> PageData::convertLinkActionToLink(::LinkAction *a, Documen
         popplerLink = std::make_unique<LinkResetForm>(lrfp);
     } break;
 
+    case actionSubmitForm: {
+        ::LinkSubmitForm *lsf = (::LinkSubmitForm *)a;
+        const std::vector<std::string> &stdStringFields = lsf->getFields();
+        QVector<int> fieldIds;
+        fieldIds.reserve(stdStringFields.size());
+        Form *form = parentDoc->doc->getCatalog()->getForm();
+        for (const std::string &fieldStr : stdStringFields) {
+            ::FormField *field = form->findFieldByFullyQualifiedNameOrRef(fieldStr);
+            if (field && !field->getNoExport()) {
+                int numWidgets = field->getNumWidgets();
+                for (int i = 0; i < numWidgets; i++) {
+                    ::FormWidget *widget = field->getWidget(i);
+                    if (widget) {
+                        fieldIds.append(field->getWidget(i)->getID());
+                    }
+                }
+            }
+        }
+        QString qStringUrl = QString::fromStdString(lsf->getUrl());
+        LinkSubmitForm::SubmitFormFlags qFlags = static_cast<LinkSubmitForm::SubmitFormFlags>(lsf->getFlags());
+        LinkSubmitFormPrivate *lsfp = new LinkSubmitFormPrivate(linkArea, fieldIds, qStringUrl, qFlags);
+        popplerLink = std::make_unique<LinkSubmitForm>(lsfp);
+    } break;
+
     case actionUnknown:
         break;
     }
@@ -654,7 +679,7 @@ QImage Page::thumbnail() const
 QString Page::text(const QRectF &r, TextLayout textLayout) const
 {
     TextOutputDev *output_dev;
-    GooString *s;
+    GooString s;
     QString result;
 
     const bool rawOrder = textLayout == RawOrderLayout;
@@ -671,10 +696,9 @@ QString Page::text(const QRectF &r, TextLayout textLayout) const
         s = output_dev->getText(r.left(), r.top(), r.right(), r.bottom());
     }
 
-    result = QString::fromUtf8(s->c_str());
+    result = QString::fromStdString(s.toStr());
 
     delete output_dev;
-    delete s;
     return result;
 }
 
