@@ -2,7 +2,7 @@
 /* $OpenLDAP$ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
- * Copyright 1998-2017 The OpenLDAP Foundation.
+ * Copyright 1998-2024 The OpenLDAP Foundation.
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -118,7 +118,7 @@ entry_schema_check(
 
 			Debug( LDAP_DEBUG_ANY,
 			    "Entry (%s), %s\n",
-			    e->e_dn, textbuf, 0 );
+			    e->e_dn, textbuf );
 
 			return LDAP_CONSTRAINT_VIOLATION;
 		}
@@ -127,7 +127,7 @@ entry_schema_check(
 	/* check the object class attribute */
 	if ( aoc == NULL ) {
 		Debug( LDAP_DEBUG_ANY, "No objectClass for entry (%s)\n",
-		    e->e_dn, 0, 0 );
+		    e->e_dn );
 
 		*text = "no objectClass attribute";
 		return LDAP_OBJECT_CLASS_VIOLATION;
@@ -140,7 +140,7 @@ entry_schema_check(
 	if ( asc == NULL && !add ) {
 		Debug( LDAP_DEBUG_ANY,
 			"No structuralObjectClass for entry (%s)\n",
-		    e->e_dn, 0, 0 );
+		    e->e_dn );
 
 		*text = "no structuralObjectClass operational attribute";
 		return LDAP_OTHER;
@@ -171,7 +171,7 @@ entry_schema_check(
 
 		Debug( LDAP_DEBUG_ANY,
 			"entry_check_schema(%s): %s\n",
-			e->e_dn, textbuf, 0 );
+			e->e_dn, textbuf );
 
 		rc = LDAP_OBJECT_CLASS_VIOLATION;
 		goto done;
@@ -184,7 +184,7 @@ entry_schema_check(
 
 		Debug( LDAP_DEBUG_ANY,
 			"entry_check_schema(%s): %s\n",
-			e->e_dn, textbuf, 0 );
+			e->e_dn, textbuf );
 
 		rc = LDAP_OTHER;
 		goto done;
@@ -198,7 +198,7 @@ got_soc:
 
 		Debug( LDAP_DEBUG_ANY,
 			"entry_check_schema(%s): %s\n",
-			e->e_dn, textbuf, 0 );
+			e->e_dn, textbuf );
 
 		rc = LDAP_OBJECT_CLASS_VIOLATION;
 		goto done;
@@ -262,7 +262,7 @@ got_soc:
 
 			Debug( LDAP_DEBUG_ANY,
 				"Entry (%s): %s\n",
-				e->e_dn, textbuf, 0 );
+				e->e_dn, textbuf );
 
 			rc = LDAP_OBJECT_CLASS_VIOLATION;
 			goto done;
@@ -286,7 +286,7 @@ got_soc:
 
 				Debug( LDAP_DEBUG_ANY,
 					"Entry (%s): %s\n",
-					e->e_dn, textbuf, 0 );
+					e->e_dn, textbuf );
 
 				rc = LDAP_OBJECT_CLASS_VIOLATION;
 				goto done;
@@ -311,7 +311,7 @@ got_soc:
 
 				Debug( LDAP_DEBUG_ANY,
 					"Entry (%s): %s\n",
-					e->e_dn, textbuf, 0 );
+					e->e_dn, textbuf );
 
 				rc = LDAP_OBJECT_CLASS_VIOLATION;
 				goto done;
@@ -330,7 +330,7 @@ got_soc:
 
 			Debug( LDAP_DEBUG_ANY,
 				"entry_check_schema(%s): %s\n",
-				e->e_dn, textbuf, 0 );
+				e->e_dn, textbuf );
 
 			rc = LDAP_OBJECT_CLASS_VIOLATION;
 			goto done;
@@ -377,7 +377,7 @@ got_soc:
 
 					Debug( LDAP_DEBUG_ANY,
 						"entry_check_schema(%s): %s\n",
-						e->e_dn, textbuf, 0 );
+						e->e_dn, textbuf );
 
 					rc = LDAP_OBJECT_CLASS_VIOLATION;
 					goto done;
@@ -420,7 +420,7 @@ got_soc:
 				if( k == -1 ) {
 					Debug( LDAP_DEBUG_ANY,
 						"Entry (%s): %s\n",
-						e->e_dn, textbuf, 0 );
+						e->e_dn, textbuf );
 
 					rc = LDAP_OBJECT_CLASS_VIOLATION;
 					goto done;
@@ -435,7 +435,7 @@ got_soc:
 
 				Debug( LDAP_DEBUG_ANY,
 					"Entry (%s): %s\n",
-					e->e_dn, textbuf, 0 );
+					e->e_dn, textbuf );
 
 				rc = LDAP_OBJECT_CLASS_VIOLATION;
 				goto done;
@@ -489,7 +489,7 @@ got_soc:
 
 			Debug( LDAP_DEBUG_ANY,
 			    "Entry (%s), %s\n",
-			    e->e_dn, textbuf, 0 );
+			    e->e_dn, textbuf );
 
 			goto done;
 		}
@@ -513,7 +513,7 @@ oc_check_required(
 
 	Debug( LDAP_DEBUG_TRACE,
 		"oc_check_required entry (%s), objectClass \"%s\"\n",
-		e->e_dn, ocname->bv_val, 0 );
+		e->e_dn?e->e_dn:"(null)", ocname->bv_val );
 
 
 	/* check for empty oc_required */
@@ -548,7 +548,7 @@ int oc_check_allowed(
 
 	Debug( LDAP_DEBUG_TRACE,
 		"oc_check_allowed type \"%s\"\n",
-		at->sat_cname.bv_val, 0, 0 );
+		at->sat_cname.bv_val );
 
 	/* always allow objectClass attribute */
 	if ( strcasecmp( at->sat_cname.bv_val, "objectClass" ) == 0 ) {
