@@ -132,7 +132,7 @@ uint8_t SkReadBuffer::peekByte() {
         fError = true;
         return 0;
     }
-    return *((uint8_t*)fCurr);
+    return *((const uint8_t*)fCurr);
 }
 
 bool SkReadBuffer::readPad32(void* buffer, size_t bytes) {
@@ -332,22 +332,33 @@ uint32_t SkReadBuffer::getArrayCount() {
     if (!this->validate(IsPtrAlign4(fCurr) && this->isAvailable(inc))) {
         return 0;
     }
-    return *((uint32_t*)fCurr);
+    return *((const uint32_t*)fCurr);
 }
 
 static sk_sp<SkImage> deserialize_image(sk_sp<SkData> data, SkDeserialProcs dProcs,
                                         std::optional<SkAlphaType> alphaType) {
     sk_sp<SkImage> image;
-    if (dProcs.fImageProc) {
+    if (dProcs.fImageDataProc) {
+        image = dProcs.fImageDataProc(data, alphaType, dProcs.fImageCtx);
+    } else if (dProcs.fImageProc) {
+#if !defined(SK_LEGACY_DESERIAL_IMAGE_PROC)
         image = dProcs.fImageProc(data->data(), data->size(), dProcs.fImageCtx);
+#else
+        image = dProcs.fImageProc(data->data(), data->size(), alphaType, dProcs.fImageCtx);
+#endif
     }
     if (image) {
         return image;
     }
+#if !defined(SK_DISABLE_LEGACY_IMAGE_READBUFFER)
     // The default implementation will encode to PNG unless the input SkImages came from
     // a codec that was built-in (e.g. JPEG/WEBP). Thus, we should be sure to try all
     // available codecs when reading images out of an SKP.
     return SkImages::DeferredFromEncodedData(std::move(data), alphaType);
+#else
+    SkDEBUGFAIL("Need to set image proc in SkDeserialProcs");
+    return nullptr;
+#endif
 }
 
 static sk_sp<SkImage> add_mipmaps(sk_sp<SkImage> img, sk_sp<SkData> data,
@@ -365,6 +376,9 @@ static sk_sp<SkImage> add_mipmaps(sk_sp<SkImage> img, sk_sp<SkData> data,
         if (!ptr) {
             return img;
         }
+        // This use of SkData::MakeWithoutCopy is safe because the image goes
+        // out of scope after we read the pixels from it, so we are sure the
+        // data (from buffer) outlives the image.
         sk_sp<SkImage> mip = deserialize_image(SkData::MakeWithoutCopy(ptr, size), dProcs,
                                                alphaType);
         if (!mip) {
@@ -383,6 +397,9 @@ static sk_sp<SkImage> add_mipmaps(sk_sp<SkImage> img, sk_sp<SkData> data,
         return img;
     }
     sk_sp<SkImage> raster = img->makeRasterImage();
+    if (!raster) {
+        return img;
+    }
     sk_sp<SkImage> rasterWithMips = builder.attachTo(raster);
     SkASSERT(rasterWithMips); // attachTo should never return null
     return rasterWithMips;
@@ -424,7 +441,7 @@ sk_sp<SkImage> SkReadBuffer::readImage() {
             return nullptr;
         }
         if (image) {
-            image = add_mipmaps(image, data, fProcs, alphaType);
+            image = add_mipmaps(image, std::move(data), fProcs, alphaType);
         }
     }
     return image ? image : MakeEmptyImage(1, 1);
@@ -432,7 +449,7 @@ sk_sp<SkImage> SkReadBuffer::readImage() {
 
 sk_sp<SkTypeface> SkReadBuffer::readTypeface() {
     // Read 32 bits (signed)
-    //   0 -- return null (default font)
+    //   0 -- return null (empty font)
     //  >0 -- index
     //  <0 -- custom (serial procs) : negative size in bytes
 
