@@ -20,11 +20,14 @@
 #include "include/encode/SkEncoder.h"
 #include "include/encode/SkJpegEncoder.h"
 #include "include/private/base/SkAssert.h"
+#include "include/private/base/SkDebug.h"
 #include "include/private/base/SkNoncopyable.h"
 #include "include/private/base/SkTemplates.h"
 #include "src/base/SkMSAN.h"
 #include "src/codec/SkJpegConstants.h"
 #include "src/codec/SkJpegPriv.h"
+#include "src/core/SkConvertPixels.h"
+#include "src/core/SkImageInfoPriv.h"
 #include "src/encode/SkImageEncoderFns.h"
 #include "src/encode/SkImageEncoderPriv.h"
 #include "src/encode/SkJPEGWriteUtility.h"
@@ -65,12 +68,13 @@ public:
 
     skjpeg_error_mgr* errorMgr() { return &fErrMgr; }
 
-    transform_scanline_proc proc() const { return fProc; }
+    bool shouldUseColorXform() { return fUseColorXform; }
+    bool colorTransformProc(void* dst, const void* src, int width);
 
     ~SkJpegEncoderMgr() { jpeg_destroy_compress(&fCInfo); }
 
 private:
-    SkJpegEncoderMgr(SkWStream* stream) : fDstMgr(stream), fProc(nullptr) {
+    SkJpegEncoderMgr(SkWStream* stream) : fDstMgr(stream) {
         fCInfo.err = jpeg_std_error(&fErrMgr);
         fErrMgr.error_exit = skjpeg_error_exit;
         jpeg_create_compress(&fCInfo);
@@ -81,65 +85,78 @@ private:
     jpeg_compress_struct fCInfo;
     skjpeg_error_mgr fErrMgr;
     skjpeg_destination_mgr fDstMgr;
-    transform_scanline_proc fProc;
+
+    std::optional<SkImageInfo> fSrcInfo;
+    std::optional<SkImageInfo> fDstInfo;
+    bool fUseColorXform = false;
 };
 
+// This function should only be called if fUseColorXform is true and thus fSrcInfo
+// and fDstInfo have value. fSrcInfo, fDstInfo, and width must all have the same width.
+bool SkJpegEncoderMgr::colorTransformProc(void* dst, const void* src, const int width) {
+   SkASSERT(fUseColorXform);
+   SkASSERT(fSrcInfo && fDstInfo);
+   SkASSERT(width == fSrcInfo->width());
+   return SkConvertPixels(fDstInfo.value(), dst, fDstInfo->minRowBytes(),
+                          fSrcInfo.value(), src, fSrcInfo->minRowBytes());
+}
+
 bool SkJpegEncoderMgr::initializeRGB(const SkImageInfo& srcInfo,
                                      const SkJpegEncoder::Options& options,
                                      const SkJpegMetadataEncoder::SegmentList& metadataSegments) {
-    auto chooseProc8888 = [&]() {
-        if (kUnpremul_SkAlphaType == srcInfo.alphaType() &&
-            options.fAlphaOption == SkJpegEncoder::AlphaOption::kBlendOnBlack) {
-            return transform_scanline_to_premul_legacy;
-        }
-        return (transform_scanline_proc) nullptr;
-    };
-
     J_COLOR_SPACE jpegColorType = JCS_EXT_RGBA;
     int numComponents = 0;
-    switch (srcInfo.colorType()) {
-        case kRGBA_8888_SkColorType:
-            fProc = chooseProc8888();
-            jpegColorType = JCS_EXT_RGBA;
-            numComponents = 4;
-            break;
-        case kBGRA_8888_SkColorType:
-            fProc = chooseProc8888();
-            jpegColorType = JCS_EXT_BGRA;
-            numComponents = 4;
-            break;
-        case kRGB_565_SkColorType:
-            fProc = transform_scanline_565;
-            jpegColorType = JCS_RGB;
-            numComponents = 3;
-            break;
-        case kARGB_4444_SkColorType:
-            if (SkJpegEncoder::AlphaOption::kBlendOnBlack == options.fAlphaOption) {
-                return false;
-            }
+    SkImageInfo dstInfo;
+    fUseColorXform = false;
 
-            fProc = transform_scanline_444;
-            jpegColorType = JCS_RGB;
-            numComponents = 3;
-            break;
-        case kGray_8_SkColorType:
-        case kAlpha_8_SkColorType:
-        case kR8_unorm_SkColorType:
-            jpegColorType = JCS_GRAYSCALE;
-            numComponents = 1;
-            break;
-        case kRGBA_F16_SkColorType:
-            if (kUnpremul_SkAlphaType == srcInfo.alphaType() &&
-                options.fAlphaOption == SkJpegEncoder::AlphaOption::kBlendOnBlack) {
-                fProc = transform_scanline_F16_to_premul_8888;
-            } else {
-                fProc = transform_scanline_F16_to_8888;
-            }
+    SkColorType srcCT = srcInfo.colorType();
+    const bool applyPremul = SkJpegEncoder::AlphaOption::kBlendOnBlack == options.fAlphaOption
+                              && srcInfo.alphaType() == kUnpremul_SkAlphaType;
+    if (srcCT == kRGB_888x_SkColorType) {
+        jpegColorType = JCS_EXT_RGBX;
+        numComponents = 4;
+    } else if (!applyPremul && srcCT == kRGBA_8888_SkColorType){
+        jpegColorType = JCS_EXT_RGBA;
+        numComponents = 4;
+    } else if (!applyPremul && srcCT == kBGRA_8888_SkColorType) {
+        jpegColorType = JCS_EXT_BGRA;
+        numComponents = 4;
+    } else {
+      // Color type conversion is needed.
+      switch(SkColorTypeNumChannels(srcCT)) {
+        case 1:
+          // We support encoding kAlpha_8_SkColorType pixmaps as JCS_GRAYSCALE as
+          // this come up often. Otherwise we have no sensible way to encode alpha
+          // images.
+          if (SkColorTypeIsAlphaOnly(srcCT) && srcCT != kAlpha_8_SkColorType) {
+            return false;
+          }
+          jpegColorType = JCS_GRAYSCALE;
+          numComponents = 1;
+          break;
+        case 3:
+          jpegColorType = JCS_EXT_RGBX;
+          numComponents = 4;
+          dstInfo = SkImageInfo::Make(srcInfo.width(), 1, kRGB_888x_SkColorType, kUnpremul_SkAlphaType);
+          fUseColorXform = true;
+          break;
+        case 4: {
+            SkAlphaType dstAT = applyPremul ? kPremul_SkAlphaType : srcInfo.alphaType();
             jpegColorType = JCS_EXT_RGBA;
             numComponents = 4;
+            dstInfo = SkImageInfo::Make(srcInfo.width(), 1, kRGBA_8888_SkColorType, dstAT);
+            fUseColorXform = true;
             break;
+        }
         default:
             return false;
+      }
+    }
+    SkASSERT(numComponents != 0);
+
+    if (fUseColorXform) {
+        fSrcInfo = srcInfo.makeWH(srcInfo.width(), 1);
+        fDstInfo = dstInfo;
     }
 
     fCInfo.image_width = srcInfo.width();
@@ -324,7 +341,7 @@ std::unique_ptr<SkEncoder> SkJpegEncoderImpl::MakeRGB(
 SkJpegEncoderImpl::SkJpegEncoderImpl(std::unique_ptr<SkJpegEncoderMgr> encoderMgr,
                                      const SkPixmap& src)
         : SkEncoder(src,
-                    encoderMgr->proc() ? encoderMgr->cinfo()->input_components * src.width() : 0)
+                    encoderMgr->shouldUseColorXform() ? encoderMgr->cinfo()->input_components * src.width() : 0)
         , fEncoderMgr(std::move(encoderMgr)) {}
 
 SkJpegEncoderImpl::SkJpegEncoderImpl(std::unique_ptr<SkJpegEncoderMgr> encoderMgr,
@@ -354,12 +371,11 @@ bool SkJpegEncoderImpl::onEncodeRows(int numRows) {
         const void* srcRow = fSrc.addr(0, fCurrRow);
         for (int i = 0; i < numRows; i++) {
             JSAMPLE* jpegSrcRow = (JSAMPLE*)(const_cast<void*>(srcRow));
-            if (fEncoderMgr->proc()) {
+            if (fEncoderMgr->shouldUseColorXform()) {
                 sk_msan_assert_initialized(srcRow, SkTAddOffset<const void>(srcRow, srcBytes));
-                fEncoderMgr->proc()((char*)fStorage.get(),
-                                    (const char*)srcRow,
-                                    fSrc.width(),
-                                    fEncoderMgr->cinfo()->input_components);
+                if (!fEncoderMgr->colorTransformProc((void*)fStorage.get(), srcRow, fSrc.width())) {
+                    return false;
+                }
                 jpegSrcRow = fStorage.get();
                 sk_msan_assert_initialized(jpegSrcRow,
                                            SkTAddOffset<const void>(jpegSrcRow, jpegSrcBytes));
@@ -417,6 +433,9 @@ std::unique_ptr<SkEncoder> Make(SkWStream* dst, const SkPixmap& src, const Optio
     SkJpegMetadataEncoder::SegmentList metadataSegments;
     SkJpegMetadataEncoder::AppendXMPStandard(metadataSegments, options.xmpMetadata);
     SkJpegMetadataEncoder::AppendICC(metadataSegments, options, src.colorSpace());
+    if (options.fOrigin.has_value()) {
+      SkJpegMetadataEncoder::AppendOrigin(metadataSegments, options.fOrigin.value());
+    }
     return SkJpegEncoderImpl::MakeRGB(dst, src, options, metadataSegments);
 }
 
@@ -427,6 +446,9 @@ std::unique_ptr<SkEncoder> Make(SkWStream* dst,
     SkJpegMetadataEncoder::SegmentList metadataSegments;
     SkJpegMetadataEncoder::AppendXMPStandard(metadataSegments, options.xmpMetadata);
     SkJpegMetadataEncoder::AppendICC(metadataSegments, options, srcColorSpace);
+    if (options.fOrigin.has_value()) {
+      SkJpegMetadataEncoder::AppendOrigin(metadataSegments, options.fOrigin.value());
+    }
     return SkJpegEncoderImpl::MakeYUV(dst, src, srcColorSpace, options, metadataSegments);
 }
 
@@ -465,4 +487,20 @@ void AppendXMPStandard(SegmentList& segmentList, const SkData* xmpMetadata) {
     segmentList.emplace_back(kXMPMarker, s.detachAsData());
 }
 
+void AppendOrigin(SegmentList& segmentList, SkEncodedOrigin origin) {
+    if (origin < kDefault_SkEncodedOrigin || origin > kLast_SkEncodedOrigin) {
+      SkDebugf("Origin is not a valid value.\n");
+      return;
+    }
+    sk_sp<SkData> exif = exif_from_origin(origin);
+    if (!exif) {
+      return;
+    }
+    SkDynamicMemoryWStream s;
+    s.write(kExifSig, sizeof(kExifSig));
+    s.write8(0);
+    s.write(exif->data(), exif->size());
+    segmentList.emplace_back(kExifMarker, s.detachAsData());
+}
+
 }  // namespace SkJpegMetadataEncoder
