@@ -6,7 +6,7 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
  * SECTION:crypto
@@ -182,6 +182,9 @@ xmlSecGCryptAsymKeyDataAdoptKey(xmlSecKeyDataPtr data, gcry_sexp_t key_pair) {
     pub_key = NULL; /* data owns it now */
     priv_key = NULL; /* data owns it now */
 
+   /* Adopt functions assume ownership on success thus the caller would expect this to be released */
+    gcry_sexp_release(key_pair);
+
     /* success */
     res = 0;
 
@@ -731,7 +734,7 @@ xmlSecGCryptKeyDataDsaFinalize(xmlSecKeyDataPtr data) {
 }
 
 static int
-xmlSecGCryptKeyDataDsaGenerate(xmlSecKeyDataPtr data, xmlSecSize sizeBits, xmlSecKeyDataType type ATTRIBUTE_UNUSED) {
+xmlSecGCryptKeyDataDsaGenerate(xmlSecKeyDataPtr data, xmlSecSize sizeBits, xmlSecKeyDataType type XMLSEC_ATTRIBUTE_UNUSED) {
     xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecGCryptKeyDataDsaId), -1);
     xmlSecAssert2(sizeBits > 0, -1);
 
@@ -1245,7 +1248,7 @@ xmlSecGCryptKeyDataRsaFinalize(xmlSecKeyDataPtr data) {
 }
 
 static int
-xmlSecGCryptKeyDataRsaGenerate(xmlSecKeyDataPtr data, xmlSecSize sizeBits, xmlSecKeyDataType type ATTRIBUTE_UNUSED) {
+xmlSecGCryptKeyDataRsaGenerate(xmlSecKeyDataPtr data, xmlSecSize sizeBits, xmlSecKeyDataType type XMLSEC_ATTRIBUTE_UNUSED) {
     xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecGCryptKeyDataRsaId), -1);
     xmlSecAssert2(sizeBits > 0, -1);
 
@@ -1638,73 +1641,6 @@ xmlSecGCryptKeyDataEcGetPrivateKey(xmlSecKeyDataPtr data) {
     return(xmlSecGCryptAsymKeyDataGetPrivateKey(data));
 }
 
-/**
- * xmlSecGCryptKeyDataEcdsaGetKlass:
- *
- * Deprecated. The GCrypt EC key data klass.
- *
- * Returns: pointer to GCrypt EC key data klass.
- */
-xmlSecKeyDataId
-xmlSecGCryptKeyDataEcdsaGetKlass(void) {
-    return(xmlSecGCryptkeyDataEcGetKlass());
-}
-
-/**
- * xmlSecGCryptKeyDataEcdsaAdoptKey:
- * @data:               the pointer to EC key data.
- * @ec_key:            the pointer to GCrypt EC key.
- *
- * Deprecated. Sets the value of EC key data.
- *
- * Returns: 0 on success or a negative value otherwise.
- */
-int
-xmlSecGCryptKeyDataEcdsaAdoptKey(xmlSecKeyDataPtr data, gcry_sexp_t ec_key) {
-    return(xmlSecGCryptKeyDataEcAdoptKey(data, ec_key));
-}
-
-/**
- * xmlSecGCryptKeyDataEcdsaAdoptKeyPair:
- * @data:               the pointer to EC key data.
- * @pub_key:            the pointer to GCrypt EC pub key.
- * @priv_key:           the pointer to GCrypt EC priv key.
- *
- * Deprecated. Sets the value of EC key data.
- *
- * Returns: 0 on success or a negative value otherwise.
- */
-int
-xmlSecGCryptKeyDataEcdsaAdoptKeyPair(xmlSecKeyDataPtr data, gcry_sexp_t pub_key, gcry_sexp_t priv_key) {
-    return(xmlSecGCryptKeyDataEcAdoptKeyPair(data, pub_key, priv_key));
-}
-
-/**
- * xmlSecGCryptKeyDataEcdsaGetPublicKey:
- * @data:               the pointer to EC key data.
- *
- * Deprecated. Gets the GCrypt EC public key from EC key data.
- *
- * Returns: pointer to GCrypt public EC key or NULL if an error occurs.
- */
-gcry_sexp_t
-xmlSecGCryptKeyDataEcdsaGetPublicKey(xmlSecKeyDataPtr data) {
-    return(xmlSecGCryptKeyDataEcGetPublicKey(data));
-}
-
-/**
- * xmlSecGCryptKeyDataEcdsaGetPrivateKey:
- * @data:               the pointer to EC key data.
- *
- * Deprecated. Gets the GCrypt EC private key from EC key data.
- *
- * Returns: pointer to GCrypt private EC key or NULL if an error occurs.
- */
-gcry_sexp_t
-xmlSecGCryptKeyDataEcdsaGetPrivateKey(xmlSecKeyDataPtr data) {
-    return(xmlSecGCryptKeyDataEcGetPrivateKey(data));
-}
-
 static int
 xmlSecGCryptKeyDataEcInitialize(xmlSecKeyDataPtr data) {
     xmlSecAssert2(xmlSecKeyDataCheckId(data, xmlSecGCryptKeyDataEcId), -1);
