@@ -4,29 +4,45 @@
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  */
+#include "src/gpu/ganesh/GrBackendTextureImageGenerator.h"
 
 #include "include/core/SkColorSpace.h"
-#include "include/gpu/GrDirectContext.h"
-#include "include/gpu/GrRecordingContext.h"
-#include "src/core/SkMessageBus.h"
-#include "src/gpu/ganesh/GrBackendTextureImageGenerator.h"
+#include "include/core/SkImageInfo.h"
+#include "include/core/SkRect.h"
+#include "include/core/SkSize.h"
+#include "include/gpu/GpuTypes.h"
+#include "include/gpu/ganesh/GrDirectContext.h"
+#include "include/gpu/ganesh/GrRecordingContext.h"
+#include "include/gpu/ganesh/GrTypes.h"
+#include "include/private/base/SkAssert.h"
+#include "include/private/gpu/ganesh/GrTypesPriv.h"
+#include "src/gpu/RefCntedCallback.h"
+#include "src/gpu/SkBackingFit.h"
+#include "src/gpu/Swizzle.h"
+#include "src/gpu/ganesh/GrCaps.h"
 #include "src/gpu/ganesh/GrDirectContextPriv.h"
 #include "src/gpu/ganesh/GrGpu.h"
+#include "src/gpu/ganesh/GrGpuResourcePriv.h"
 #include "src/gpu/ganesh/GrProxyProvider.h"
 #include "src/gpu/ganesh/GrRecordingContextPriv.h"
 #include "src/gpu/ganesh/GrResourceCache.h"
 #include "src/gpu/ganesh/GrResourceProvider.h"
 #include "src/gpu/ganesh/GrResourceProviderPriv.h"
 #include "src/gpu/ganesh/GrSemaphore.h"
+#include "src/gpu/ganesh/GrSurface.h"
+#include "src/gpu/ganesh/GrSurfaceProxy.h"
 #include "src/gpu/ganesh/GrTexture.h"
-#include "src/gpu/ganesh/GrTextureProxyPriv.h"
+#include "src/gpu/ganesh/GrTextureProxy.h"
 #include "src/gpu/ganesh/SkGr.h"
 
+#include <functional>
+#include <utility>
+
 GrBackendTextureImageGenerator::RefHelper::RefHelper(
         sk_sp<GrTexture> texture,
         GrDirectContext::DirectContextID owningContextID,
         std::unique_ptr<GrSemaphore> semaphore)
-        : fOriginalTexture(texture)
+        : fOriginalTexture(std::move(texture))
         , fOwningContextID(owningContextID)
         , fBorrowingContextReleaseProc(nullptr)
         , fSemaphore(std::move(semaphore)) {}
@@ -39,7 +55,7 @@ GrBackendTextureImageGenerator::RefHelper::~RefHelper() {
 }
 
 std::unique_ptr<GrTextureGenerator>
-GrBackendTextureImageGenerator::Make(sk_sp<GrTexture> texture,
+GrBackendTextureImageGenerator::Make(const sk_sp<GrTexture>& texture,
                                      GrSurfaceOrigin origin,
                                      std::unique_ptr<GrSemaphore> semaphore,
                                      SkColorType colorType,
@@ -55,7 +71,7 @@ GrBackendTextureImageGenerator::Make(sk_sp<GrTexture> texture,
     SkColorInfo info(colorType, alphaType, std::move(colorSpace));
     return std::unique_ptr<GrTextureGenerator>(new GrBackendTextureImageGenerator(
             info,
-            std::move(texture),
+            texture,
             origin,
             dContext->directContextID(),
             std::move(semaphore)));
@@ -63,7 +79,7 @@ GrBackendTextureImageGenerator::Make(sk_sp<GrTexture> texture,
 
 GrBackendTextureImageGenerator::GrBackendTextureImageGenerator(
         const SkColorInfo& info,
-        sk_sp<GrTexture> texture,
+        const sk_sp<GrTexture>& texture,
         GrSurfaceOrigin origin,
         GrDirectContext::DirectContextID owningContextID,
         std::unique_ptr<GrSemaphore> semaphore)
@@ -76,6 +92,10 @@ GrBackendTextureImageGenerator::~GrBackendTextureImageGenerator() {
     fRefHelper->unref();
 }
 
+bool GrBackendTextureImageGenerator::onIsProtected() const {
+    return fBackendTexture.isProtected();
+}
+
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 
 void GrBackendTextureImageGenerator::ReleaseRefHelper_TextureReleaseProc(void* ctx) {
@@ -90,7 +110,7 @@ void GrBackendTextureImageGenerator::ReleaseRefHelper_TextureReleaseProc(void* c
 GrSurfaceProxyView GrBackendTextureImageGenerator::onGenerateTexture(
         GrRecordingContext* rContext,
         const SkImageInfo& info,
-        GrMipmapped mipmapped,
+        skgpu::Mipmapped mipmapped,
         GrImageTexGenPolicy texGenPolicy) {
     SkASSERT(rContext);
     SkASSERT_RELEASE(info.dimensions() == fBackendTexture.dimensions());
@@ -150,8 +170,8 @@ GrSurfaceProxyView GrBackendTextureImageGenerator::onGenerateTexture(
 
     GrColorType grColorType = SkColorTypeToGrColorType(info.colorType());
 
-    GrMipmapped textureIsMipMapped = fBackendTexture.hasMipmaps() ? GrMipmapped::kYes
-                                                                  : GrMipmapped::kNo;
+    skgpu::Mipmapped textureIsMipMapped =
+            fBackendTexture.hasMipmaps() ? skgpu::Mipmapped::kYes : skgpu::Mipmapped::kNo;
 
     // Ganesh assumes that, when wrapping a mipmapped backend texture from a client, that its
     // mipmaps are fully fleshed out.
@@ -220,7 +240,7 @@ GrSurfaceProxyView GrBackendTextureImageGenerator::onGenerateTexture(
     }
 
     if (texGenPolicy == GrImageTexGenPolicy::kDraw &&
-        (mipmapped == GrMipmapped::kNo || proxy->mipmapped() == GrMipmapped::kYes)) {
+        (mipmapped == skgpu::Mipmapped::kNo || proxy->mipmapped() == skgpu::Mipmapped::kYes)) {
         // If we have the correct mip support, we're done
         return GrSurfaceProxyView(std::move(proxy), fSurfaceOrigin, readSwizzle);
     } else {
