@@ -7,17 +7,32 @@
 
 #include "src/gpu/graphite/render/TessellateWedgesRenderStep.h"
 
-#include "src/sksl/SkSLString.h"
-
+#include "include/core/SkPath.h"
+#include "include/core/SkPathTypes.h"
+#include "include/private/base/SkAssert.h"
+#include "include/private/base/SkDebug.h"
+#include "include/private/base/SkPoint_impl.h"
+#include "include/private/base/SkSpan_impl.h"
+#include "src/core/SkSLTypeShared.h"
+#include "src/gpu/BufferWriter.h"
+#include "src/gpu/graphite/Attribute.h"
 #include "src/gpu/graphite/BufferManager.h"
+#include "src/gpu/graphite/DrawOrder.h"
 #include "src/gpu/graphite/DrawParams.h"
-#include "src/gpu/graphite/DrawWriter.h"
+#include "src/gpu/graphite/DrawTypes.h"
 #include "src/gpu/graphite/PipelineData.h"
+#include "src/gpu/graphite/geom/Geometry.h"
+#include "src/gpu/graphite/geom/Shape.h"
+#include "src/gpu/graphite/geom/Transform.h"
 #include "src/gpu/graphite/render/DynamicInstancesPatchAllocator.h"
-
 #include "src/gpu/tessellate/FixedCountBufferUtils.h"
 #include "src/gpu/tessellate/MidpointContourParser.h"
 #include "src/gpu/tessellate/PatchWriter.h"
+#include "src/gpu/tessellate/Tessellation.h"
+#include "src/gpu/tessellate/WangsFormula.h"
+#include "src/sksl/SkSLString.h"
+
+#include <cstddef>
 
 namespace skgpu::graphite {
 
@@ -42,13 +57,13 @@ using Writer = PatchWriter<DynamicInstancesPatchAllocator<FixedCountWedges>,
 
 // The order of the attribute declarations must match the order used by
 // PatchWriter::emitPatchAttribs, i.e.:
-//     join << fanPoint << stroke << color << depth << curveType << ssboIndex
+//     join << fanPoint << stroke << color << depth << curveType << ssboIndices
 static constexpr Attribute kBaseAttributes[] = {
         {"p01", VertexAttribType::kFloat4, SkSLType::kFloat4},
         {"p23", VertexAttribType::kFloat4, SkSLType::kFloat4},
         {"fanPointAttrib", VertexAttribType::kFloat2, SkSLType::kFloat2},
         {"depth", VertexAttribType::kFloat, SkSLType::kFloat},
-        {"ssboIndex", VertexAttribType::kInt, SkSLType::kInt}};
+        {"ssboIndices", VertexAttribType::kUInt2, SkSLType::kUInt2}};
 
 static constexpr Attribute kAttributesWithCurveType[] = {
         {"p01", VertexAttribType::kFloat4, SkSLType::kFloat4},
@@ -56,19 +71,18 @@ static constexpr Attribute kAttributesWithCurveType[] = {
         {"fanPointAttrib", VertexAttribType::kFloat2, SkSLType::kFloat2},
         {"depth", VertexAttribType::kFloat, SkSLType::kFloat},
         {"curveType", VertexAttribType::kFloat, SkSLType::kFloat},
-        {"ssboIndex", VertexAttribType::kInt, SkSLType::kInt}};
+        {"ssboIndices", VertexAttribType::kUInt2, SkSLType::kUInt2}};
 
 static constexpr SkSpan<const Attribute> kAttributes[2] = {kAttributesWithCurveType,
                                                            kBaseAttributes};
 
 }  // namespace
 
-TessellateWedgesRenderStep::TessellateWedgesRenderStep(std::string_view variantName,
+TessellateWedgesRenderStep::TessellateWedgesRenderStep(RenderStepID renderStepID,
                                                        bool infinitySupport,
                                                        DepthStencilSettings depthStencilSettings,
                                                        StaticBufferManager* bufferManager)
-        : RenderStep("TessellateWedgesRenderStep",
-                     variantName,
+        : RenderStep(renderStepID,
                      Flags::kRequiresMSAA |
                      (depthStencilSettings.fDepthWriteEnabled ? Flags::kPerformsShading
                                                               : Flags::kNone),
@@ -87,40 +101,42 @@ TessellateWedgesRenderStep::TessellateWedgesRenderStep(std::string_view variantN
     // problems, we can modify StaticBufferManager to de-duplicate requests.
     const size_t vertexSize = FixedCountWedges::VertexBufferSize();
     auto vertexData = bufferManager->getVertexWriter(vertexSize, &fVertexBuffer);
-    FixedCountWedges::WriteVertexBuffer(std::move(vertexData), vertexSize);
+    if (vertexData) {
+        FixedCountWedges::WriteVertexBuffer(std::move(vertexData), vertexSize);
+    } // otherwise static buffer creation failed, so do nothing; Context initialization will fail.
 
     const size_t indexSize = FixedCountWedges::IndexBufferSize();
     auto indexData = bufferManager->getIndexWriter(indexSize, &fIndexBuffer);
-    FixedCountWedges::WriteIndexBuffer(std::move(indexData), indexSize);
+    if (indexData) {
+        FixedCountWedges::WriteIndexBuffer(std::move(indexData), indexSize);
+    } // otherwise static buffer creation failed, so do nothing; Context initialization will fail.
 }
 
 TessellateWedgesRenderStep::~TessellateWedgesRenderStep() {}
 
 std::string TessellateWedgesRenderStep::vertexSkSL() const {
     return SkSL::String::printf(
-            R"(
-                float2 localCoord;
-                if (resolveLevel_and_idx.x < 0) {
-                    // A negative resolve level means this is the fan point.
-                    localCoord = fanPointAttrib;
-                } else {
-                    // TODO: Approximate perspective scaling to match how PatchWriter is configured
-                    // (or provide explicit tessellation level in instance data instead of
-                    // replicating work)
-                    float2x2 vectorXform = float2x2(localToDevice[0].xy, localToDevice[1].xy);
-                    localCoord = tessellate_filled_curve(
-                        vectorXform, resolveLevel_and_idx.x, resolveLevel_and_idx.y, p01, p23, %s);
-                }
-                float4 devPosition = localToDevice * float4(localCoord, 0.0, 1.0);
-                devPosition.z = depth;
-                stepLocalCoords = localCoord;
-            )",
+            "float2 localCoord;\n"
+            "if (resolveLevel_and_idx.x < 0) {\n"
+                // A negative resolve level means this is the fan point.
+                "localCoord = fanPointAttrib;\n"
+            "} else {\n"
+                // TODO: Approximate perspective scaling to match how PatchWriter is configured
+                // (or provide explicit tessellation level in instance data instead of
+                // replicating work)
+                "float2x2 vectorXform = float2x2(localToDevice[0].xy, localToDevice[1].xy);\n"
+                "localCoord = tessellate_filled_curve("
+                    "vectorXform, resolveLevel_and_idx.x, resolveLevel_and_idx.y, p01, p23, %s);\n"
+            "}\n"
+            "float4 devPosition = localToDevice * float4(localCoord, 0.0, 1.0);\n"
+            "devPosition.z = depth;\n"
+            "stepLocalCoords = localCoord;\n",
             fInfinitySupport ? "curve_type_using_inf_support(p23)" : "curveType");
 }
 
 void TessellateWedgesRenderStep::writeVertices(DrawWriter* dw,
                                                const DrawParams& params,
-                                               int ssboIndex) const {
+                                               skvx::uint2 ssboIndices) const {
     SkPath path = params.geometry().shape().asPath(); // TODO: Iterate the Shape directly
 
     int patchReserveCount = FixedCountWedges::PreallocCount(path.countVerbs());
@@ -130,13 +146,13 @@ void TessellateWedgesRenderStep::writeVertices(DrawWriter* dw,
                   fIndexBuffer,
                   patchReserveCount};
     writer.updatePaintDepthAttrib(params.order().depthAsFloat());
-    writer.updateSsboIndexAttrib(ssboIndex);
+    writer.updateSsboIndexAttrib(ssboIndices);
 
     // The vector xform approximates how the control points are transformed by the shader to
     // more accurately compute how many *parametric* segments are needed.
     // TODO: This doesn't account for perspective division yet, which will require updating the
     // approximate transform based on each verb's control points' bounding box.
-    SkASSERT(params.transform().type() < Transform::Type::kProjection);
+    SkASSERT(params.transform().type() < Transform::Type::kPerspective);
     writer.setShaderTransform(wangs_formula::VectorXform{params.transform().matrix()},
                               params.transform().maxScaleFactor());
 
