@@ -161,6 +161,9 @@ JrdStatement::JrdStatement(thread_db* tdbb, MemoryPool* p, CompilerScratch* csb)
 			if (!tail->csb_fields && !(tail->csb_flags & csb_update))
 				 rpb->rpb_stream_flags |= RPB_s_no_data;
 
+			if (tail->csb_flags & csb_unstable)
+				rpb->rpb_stream_flags |= RPB_s_unstable;
+
 			rpb->rpb_relation = tail->csb_relation;
 
 			delete tail->csb_fields;
@@ -202,9 +205,9 @@ JrdStatement* JrdStatement::makeStatement(thread_db* tdbb, CompilerScratch* csb,
 
 		DmlNode::doPass1(tdbb, csb, &csb->csb_node);
 
-		// CVC: I'm going to allocate the map before the loop to avoid alloc/dealloc calls.
-		AutoPtr<StreamType, ArrayDelete<StreamType> > localMap(FB_NEW_POOL(*tdbb->getDefaultPool())
-			StreamType[STREAM_MAP_LENGTH]);
+		// CVC: I'm going to preallocate the map before the loop to avoid alloc/dealloc calls.
+		StreamMap localMap;
+		StreamType* const map = localMap.getBuffer(STREAM_MAP_LENGTH);
 
 		// Copy and compile (pass1) domains DEFAULT and constraints.
 		MapFieldInfo::Accessor accessor(&csb->csb_map_field_info);
@@ -212,18 +215,17 @@ JrdStatement* JrdStatement::makeStatement(thread_db* tdbb, CompilerScratch* csb,
 		for (bool found = accessor.getFirst(); found; found = accessor.getNext())
 		{
 			FieldInfo& fieldInfo = accessor.current()->second;
-			//StreamType local_map[MAP_LENGTH];
 
 			AutoSetRestore<USHORT> autoRemapVariable(&csb->csb_remap_variable,
 				(csb->csb_variables ? csb->csb_variables->count() : 0) + 1);
 
-			fieldInfo.defaultValue = NodeCopier::copy(tdbb, csb, fieldInfo.defaultValue, localMap);
+			fieldInfo.defaultValue = NodeCopier::copy(tdbb, csb, fieldInfo.defaultValue, map);
 
 			csb->csb_remap_variable = (csb->csb_variables ? csb->csb_variables->count() : 0) + 1;
 
 			if (fieldInfo.validationExpr)
 			{
-				NodeCopier copier(csb, localMap);
+				NodeCopier copier(csb, map);
 				fieldInfo.validationExpr = copier.copy(tdbb, fieldInfo.validationExpr);
 			}
 
@@ -406,11 +408,23 @@ void JrdStatement::verifyAccess(thread_db* tdbb)
 		if (item->exa_action == ExternalAccess::exa_procedure)
 		{
 			routine = MET_lookup_procedure_id(tdbb, item->exa_prc_id, false, false, 0);
+			if (!routine)
+			{
+				string name;
+				name.printf("id %d", item->exa_prc_id);
+				ERR_post(Arg::Gds(isc_prcnotdef) << name);
+			}
 			aclType = id_procedure;
 		}
 		else if (item->exa_action == ExternalAccess::exa_function)
 		{
 			routine = Function::lookup(tdbb, item->exa_fun_id, false, false, 0);
+			if (!routine)
+			{
+				string name;
+				name.printf("id %d", item->exa_fun_id);
+				ERR_post(Arg::Gds(isc_funnotdef) << name);
+			}
 			aclType = id_function;
 		}
 		else
@@ -588,7 +602,7 @@ void JrdStatement::release(thread_db* tdbb)
 
 // Check that we have enough rights to access all resources this list of triggers touches.
 void JrdStatement::verifyTriggerAccess(thread_db* tdbb, jrd_rel* ownerRelation,
-	trig_vec* triggers, jrd_rel* view)
+	TrigVector* triggers, jrd_rel* view)
 {
 	if (!triggers)
 		return;
@@ -640,7 +654,7 @@ void JrdStatement::verifyTriggerAccess(thread_db* tdbb, jrd_rel* ownerRelation,
 
 // Invoke buildExternalAccess for triggers in vector
 inline void JrdStatement::triggersExternalAccess(thread_db* tdbb, ExternalAccessList& list,
-	trig_vec* tvec)
+	TrigVector* tvec)
 {
 	if (!tvec)
 		return;
@@ -687,7 +701,7 @@ void JrdStatement::buildExternalAccess(thread_db* tdbb, ExternalAccessList& list
 			if (!relation)
 				continue;
 
-			trig_vec *vec1, *vec2;
+			RefPtr<TrigVector> vec1, vec2;
 
 			switch (item->exa_action)
 			{
@@ -726,25 +740,10 @@ template <typename T> static void makeSubRoutines(thread_db* tdbb, JrdStatement*
 		Routine* subRoutine = subNode->routine;
 		CompilerScratch*& subCsb = subNode->subCsb;
 
-		JrdStatement* subStatement = JrdStatement::makeStatement(tdbb, subCsb, true);
+		JrdStatement* subStatement = JrdStatement::makeStatement(tdbb, subCsb, false);
 		subStatement->parentStatement = statement;
 		subRoutine->setStatement(subStatement);
 
-		switch (subRoutine->getObjectType())
-		{
-		case obj_procedure:
-			subStatement->procedure = static_cast<jrd_prc*>(subRoutine);
-			break;
-
-		case obj_udf:
-			subStatement->function = static_cast<Function*>(subRoutine);
-			break;
-
-		default:
-			fb_assert(false);
-			break;
-		}
-
 		// Move dependencies and permissions from the sub routine to the parent.
 
 		for (CompilerScratch::Dependency* dependency = subCsb->csb_dependencies.begin();
