@@ -185,7 +185,7 @@ void LotusChart::cleanState()
 std::map<std::string,int> LotusChart::getNameToChartIdMap() const
 {
 	std::map<std::string,int> res;
-	for (auto it : m_state->m_idChartMap)
+	for (auto const &it : m_state->m_idChartMap)
 	{
 		auto const &chart = it.second;
 		if (chart || !chart->m_name.empty())
@@ -205,7 +205,7 @@ void LotusChart::updateState()
 	   In mac's wk3 files, all used files are named, so this must be ok
 	 */
 	bool removeNoName=version()==1 && m_state->m_idChartMap.size()>1;
-	for (auto &it : m_state->m_idChartMap)
+	for (auto const &it : m_state->m_idChartMap)
 	{
 		auto id=it.first;
 		auto chart=it.second;
@@ -797,12 +797,12 @@ bool LotusChart::readMacAxis(std::shared_ptr<WPSStream> stream, long endPos)
 	}
 	auto chart=m_state->getChart(m_state->m_chartId,*this,stream);
 	auto id=int(libwps::readU8(input));
-	if (id<0||id>=3)
+	if (id>=3)
 	{
 		WPS_DEBUG_MSG(("LotusChart::readMacAxis: the id seems bad\n"));
 		f << "###";
 	}
-	auto &axis=chart->getAxis((id<0||id>=3) ? 4 : id);
+	auto &axis=chart->getAxis(id>=3 ? 4 : id);
 	f << "id[axis]=" << id << ",";
 	auto format=int(libwps::readU8(input));
 	if ((format&0x20)==0)
@@ -1019,7 +1019,7 @@ bool LotusChart::readMacPlacement(std::shared_ptr<WPSStream> stream, long endPos
 		f << "title,";
 		if ((val&0x10)==0)
 		{
-			WKSChart::TextZone::Type allTypes[]= {WKSChart::TextZone::T_Title, WKSChart::TextZone::T_SubTitle};
+			WKSChart::TextZone::Type const allTypes[]= {WKSChart::TextZone::T_Title, WKSChart::TextZone::T_SubTitle};
 			for (auto type : allTypes)
 				chart->getTextZone(type,true)->m_show=false;
 		}
