@@ -141,7 +141,7 @@ bool MWAWGraphicStyle::Pattern::getBinary(MWAWEmbeddedObject &picture) const
    */
   MWAWPictBitmapIndexed bitmap(m_dim);
   std::vector<MWAWColor> colors;
-  for (auto color : m_colors)
+  for (auto const &color : m_colors)
     colors.push_back(color);
   bitmap.setColors(colors);
   int numBytesByLines = m_dim[0]/8;
