@@ -1124,6 +1124,7 @@ void MWAWTextListener::insertTextBox
   librevenge::RVNGPropertyList propList;
   if (!frameStyle.m_frameNextName.empty())
     propList.insert("librevenge:next-frame-name",frameStyle.m_frameNextName.c_str());
+  // todo check style.m_rotate and style.m_flip
   m_documentInterface->openTextBox(propList);
   handleSubDocument(subDocument, libmwaw::DOC_TEXT_BOX);
   m_documentInterface->closeTextBox();
@@ -1673,7 +1674,7 @@ void MWAWTextListener::handleSubDocument(MWAWSubDocumentPtr const &subDocument,
 
   // Check whether the document is calling itself
   bool sendDoc = true;
-  for (auto doc : m_ds->m_subDocuments) {
+  for (auto const &doc : m_ds->m_subDocuments) {
     if (!subDocument)
       break;
     if (!doc)
