@@ -103,6 +103,8 @@ struct Field {
       o << "title,";
       break;
     case MWAWField::None:
+    case MWAWField::BookmarkStart:
+    case MWAWField::BookmarkEnd:
     case MWAWField::PageCount:
     case MWAWField::Database:
 #if !defined(__clang__)
@@ -385,13 +387,13 @@ struct ZoneInfo {
   }
 
   //! update the line list id
-  void updateListId(ListProperties &prop, MWAWListManager &listManager);
+  void updateListId(ListProperties const &prop, MWAWListManager &listManager);
 
   //! the list of line information
   std::vector<LineInfo> m_linesList;
 };
 
-void ZoneInfo::updateListId(ListProperties &prop, MWAWListManager &listManager)
+void ZoneInfo::updateListId(ListProperties const &prop, MWAWListManager &listManager)
 {
   bool headingStyle=prop.m_useHeadingStyle && prop.m_headingStyle>=1 && prop.m_headingStyle<=4;
 
@@ -466,10 +468,10 @@ void ZoneInfo::updateListId(ListProperties &prop, MWAWListManager &listManager)
     else
       *(para.m_margins[0])=-0.2;
 
-    auto &actList=listsList[line.m_listType];
+    auto const &actList=listsList[line.m_listType];
     if (!actList) // we have a pb....
       continue;
-    for (auto &type : listsList)
+    for (auto const &type : listsList)
       if (type) type->setLevel(level);
     if (line.m_listType==0)
       listsList[2]->setStartValueForNextElement(level==1 ? prop.m_startListIndex : 1);
@@ -666,7 +668,7 @@ void MindWrtParser::parse(librevenge::RVNGTextInterface *docInterface)
     }
 
     libmwaw::DebugStream f;
-    for (auto it : m_state->m_entryMap) {
+    for (auto const &it : m_state->m_entryMap) {
       MWAWEntry const &entry = it.second;
       if (entry.isParsed())
         continue;
@@ -872,7 +874,6 @@ bool MindWrtParser::sendZone(int id)
   else
     setProperty(para);
 
-  MWAWInputStreamPtr input = getInput();
   libmwaw::DebugStream f;
   if (id==0) m_state->m_actPage = 1;
   auto &zone = m_state->m_zones[id];
@@ -985,7 +986,7 @@ bool MindWrtParser::readRuler(MindWrtParserInternal::LineInfo &line)
   MWAWInputStreamPtr input = getInput();
   libmwaw::DebugStream f;
 
-  if (line.m_entry.length() < 10 || (line.m_entry.length()%2)) {
+  if (line.m_entry.length() < 10 || (line.m_entry.length()%2) || !input->checkPosition(line.m_entry.end())) {
     MWAW_DEBUG_MSG(("MindWrtParser::readRuler: zone size is two short or odd\n"));
     return false;
   }
@@ -1579,7 +1580,7 @@ bool MindWrtParser::readLinesInfo(MWAWEntry &entry)
 ////////////////////////////////////////////////////////////
 // read the last zone ( use ?)
 ////////////////////////////////////////////////////////////
-bool MindWrtParser::readLastZone(MWAWEntry &entry)
+bool MindWrtParser::readLastZone(MWAWEntry const &entry)
 {
   if (!entry.valid())
     return false;
@@ -1985,7 +1986,7 @@ bool MindWrtParser::readHeadingFields(MWAWEntry &entry)
 ////////////////////////////////////////////////////////////
 // read the print info
 ////////////////////////////////////////////////////////////
-bool MindWrtParser::readPrintInfo(MWAWEntry &entry)
+bool MindWrtParser::readPrintInfo(MWAWEntry const &entry)
 {
   if (!entry.valid())
     return false;
