@@ -525,9 +525,9 @@ bool PixelPaintParser::readBitmapV1(bool onlyCheck)
     f.str("");
     f << "Bitmap[R" << row << "C" << col << "]:";
     int nPixel=0;
-    while (input->tell()+2<=endPos) {
+    while (input->tell()+2<=endPos) { // UnpackBits
       auto n=static_cast<int>(input->readULong(1));
-      if (n>=0x81) { // checkme
+      if (n>=0x81) {
         auto color=static_cast<int>(input->readULong(1));
         if (color>=numColors) {
           static bool first=true;
@@ -545,7 +545,7 @@ bool PixelPaintParser::readBitmapV1(bool onlyCheck)
         }
         nPixel+=0x101-n;
       }
-      else {
+      else { // checkme normally 0x80 is reserved and almost nobody used it (for ending the compression)
         if (input->tell()+n+1>endPos) {
           input->seek(-1, librevenge::RVNG_SEEK_CUR);
           break;
@@ -761,9 +761,9 @@ bool PixelPaintParser::readBitmapV2(bool onlyCheck)
     f.str("");
     f << "Bitmap[R" << row << "]:";
     int col=0, nPixel=0;
-    while (input->tell()+2<=endPos) {
+    while (input->tell()+2<=endPos) { // UnpackBits
       auto n=static_cast<int>(input->readULong(1));
-      if (n>=0x81) { // checkme
+      if (n>=0x81) {
         auto color=static_cast<int>(input->readULong(1));
         if (color>=numColors) {
           static bool first=true;
@@ -781,7 +781,7 @@ bool PixelPaintParser::readBitmapV2(bool onlyCheck)
         }
         nPixel+=0x101-n;
       }
-      else {
+      else { // checkme normally 0x80 is reserved and almost nobody used it (for ending the compression)
         if (input->tell()+n+1>endPos) {
           input->seek(-1, librevenge::RVNG_SEEK_CUR);
           break;
