@@ -1661,7 +1661,7 @@ void RagTime5Zone::createAsciiFile()
   }
   m_localAsciiFile.reset(new libmwaw::DebugFile(m_input));
   m_asciiFile = m_localAsciiFile.get();
-  m_asciiFile->open(m_asciiName.c_str());
+  m_asciiFile->open(m_asciiName);
 }
 
 std::string RagTime5Zone::getZoneName() const
