@@ -310,7 +310,7 @@ bool RagTimeText::readFontNames(MWAWEntry &entry)
     ascFile.addNote(f.str().c_str());
   }
 
-  for (auto it : fontIdPosMap) {
+  for (auto const &it : fontIdPosMap) {
     pos=it.second;
     int fId=it.first;
     if (pos>=endPos) continue;
@@ -1145,7 +1145,7 @@ void RagTimeText::flushExtra()
     MWAW_DEBUG_MSG(("RagTimeText::flushExtra: can not find the listener\n"));
     return;
   }
-  for (auto it : m_state->m_idTextMap) {
+  for (auto const &it : m_state->m_idTextMap) {
     if (!it.second) continue;
     RagTimeTextInternal::TextZone const &zone=*it.second;
     if (zone.m_isSent) continue;
