@@ -50,6 +50,7 @@
 
 #include "STOFFCell.hxx"
 #include "STOFFFont.hxx"
+#include "STOFFFrameStyle.hxx"
 #include "STOFFGraphicStyle.hxx"
 #include "STOFFGraphicShape.hxx"
 #include "STOFFInputStream.hxx"
@@ -72,10 +73,22 @@ enum { PageBreakBit=0x1, ColumnBreakBit=0x2 };
 //! a class to store the document state of a STOFFTextListener
 struct TextState {
   //! constructor
-  explicit TextState(std::vector<STOFFPageSpan> const &pageList) :
-    m_pageList(pageList), m_pageSpan(), m_metaData(), m_footNoteNumber(0), m_endNoteNumber(0), m_smallPictureNumber(0),
-    m_isDocumentStarted(false), m_isHeaderFooterOpened(false), m_isHeaderFooterRegionOpened(false), m_sentListMarkers(), m_subDocuments(),
-    m_definedFontStyleSet(), m_definedGraphicStyleSet(), m_definedParagraphStyleSet()
+  explicit TextState(std::vector<STOFFPageSpan> const &pageList)
+    : m_pageList(pageList)
+    , m_pageSpan()
+    , m_metaData()
+    , m_footNoteNumber(0)
+    , m_endNoteNumber(0)
+    , m_smallPictureNumber(0)
+    , m_isDocumentStarted(false)
+    , m_isHeaderFooterOpened(false)
+    , m_isHeaderFooterRegionOpened(false)
+    , m_sentListMarkers()
+    , m_subDocuments()
+
+    , m_definedFontStyleSet()
+    , m_definedGraphicStyleSet()
+    , m_definedParagraphStyleSet()
   {
   }
   //! destructor
@@ -129,7 +142,7 @@ struct State {
   //! a sequence of bit used to know if we need page/column break
   int m_paragraphNeedBreak;
 
-  shared_ptr<STOFFList> m_list;
+  std::shared_ptr<STOFFList> m_list;
 
   bool m_isPageSpanOpened;
   bool m_isSectionOpened;
@@ -171,44 +184,58 @@ private:
   State &operator=(const State &);
 };
 
-State::State() :
-  m_textBuffer(""), m_numDeferredTabs(0),
+State::State()
+  : m_textBuffer("")
+  , m_numDeferredTabs(0)
 
-  m_font(),
+  , m_font()
 
-  m_paragraph(), m_paragraphNeedBreak(0),
+  , m_paragraph()
+  , m_paragraphNeedBreak(0)
 
-  m_list(),
+  , m_list()
 
-  m_isPageSpanOpened(false), m_isSectionOpened(false), m_isFrameOpened(false),
-  m_isPageSpanBreakDeferred(false),
-  m_isHeaderFooterWithoutParagraph(false),
-  m_isGroupOpened(false),
+  , m_isPageSpanOpened(false)
+  , m_isSectionOpened(false)
+  , m_isFrameOpened(false)
+  , m_isPageSpanBreakDeferred(false)
+  , m_isHeaderFooterWithoutParagraph(false)
+  , m_isGroupOpened(false)
 
-  m_isSpanOpened(false), m_isParagraphOpened(false), m_isListElementOpened(false),
+  , m_isSpanOpened(false)
+  , m_isParagraphOpened(false)
+  , m_isListElementOpened(false)
 
-  m_firstParagraphInPageSpan(true),
+  , m_firstParagraphInPageSpan(true)
 
-  m_isTableOpened(false), m_isTableRowOpened(false), m_isTableColumnOpened(false),
-  m_isTableCellOpened(false),
+  , m_isTableOpened(false)
+  , m_isTableRowOpened(false)
+  , m_isTableColumnOpened(false)
+  , m_isTableCellOpened(false)
 
-  m_currentPage(0), m_numPagesRemainingInSpan(0), m_currentPageNumber(1),
+  , m_currentPage(0)
+  , m_numPagesRemainingInSpan(0)
+  , m_currentPageNumber(1)
 
-  m_sectionAttributesChanged(false),
-  m_section(),
+  , m_sectionAttributesChanged(false)
+  , m_section()
 
-  m_listOrderedLevels(),
+  , m_listOrderedLevels()
 
-  m_inSubDocument(false),
-  m_isNote(false), m_inLink(false),
-  m_subDocumentType(libstoff::DOC_NONE)
+  , m_inSubDocument(false)
+  , m_isNote(false)
+  , m_inLink(false)
+  , m_subDocumentType(libstoff::DOC_NONE)
 {
 }
 }
 
-STOFFTextListener::STOFFTextListener(STOFFListManagerPtr listManager, std::vector<STOFFPageSpan> const &pageList, librevenge::RVNGTextInterface *documentInterface) : STOFFListener(listManager),
-  m_ds(new STOFFTextListenerInternal::TextState(pageList)), m_ps(new STOFFTextListenerInternal::State), m_psStack(),
-  m_documentInterface(documentInterface)
+STOFFTextListener::STOFFTextListener(STOFFListManagerPtr &listManager, std::vector<STOFFPageSpan> const &pageList, librevenge::RVNGTextInterface *documentInterface)
+  : STOFFListener(listManager)
+  , m_ds(new STOFFTextListenerInternal::TextState(pageList))
+  , m_ps(new STOFFTextListenerInternal::State)
+  , m_psStack()
+  , m_documentInterface(documentInterface)
 {
 }
 
@@ -557,7 +584,7 @@ void STOFFTextListener::_openPageSpan(bool sendHeaderFooters)
     throw libstoff::ParseException();
   }
   unsigned actPage = 0;
-  std::vector<STOFFPageSpan>::iterator it = m_ds->m_pageList.begin();
+  auto it = m_ds->m_pageList.begin();
   ++m_ps->m_currentPage;
   while (true) {
     actPage+=static_cast<unsigned>(it->m_pageSpan);
@@ -568,7 +595,7 @@ void STOFFTextListener::_openPageSpan(bool sendHeaderFooters)
       break;
     }
   }
-  STOFFPageSpan &currentPage = *it;
+  auto &currentPage = *it;
 
   librevenge::RVNGPropertyList propList;
   currentPage.getPageProperty(propList);
@@ -890,7 +917,7 @@ void STOFFTextListener::_closeListElement()
 
 int STOFFTextListener::_getListId() const
 {
-  size_t newLevel= size_t(m_ps->m_paragraph.m_listLevelIndex);
+  auto newLevel= size_t(m_ps->m_paragraph.m_listLevelIndex);
   if (newLevel == 0) return -1;
   int newListId = m_ps->m_paragraph.m_listId;
   if (newListId > 0) return newListId;
@@ -899,8 +926,7 @@ int STOFFTextListener::_getListId() const
     STOFF_DEBUG_MSG(("STOFFTextListener::_getListId: the list id is not set, try to find a new one\n"));
     first = false;
   }
-  shared_ptr<STOFFList> list=m_listManager->getNewList
-                             (m_ps->m_list, int(newLevel), m_ps->m_paragraph.m_listLevel);
+  auto list=m_listManager->getNewList(m_ps->m_list, int(newLevel), m_ps->m_paragraph.m_listLevel);
   if (!list) return -1;
   return list->getId();
 }
@@ -911,7 +937,7 @@ void STOFFTextListener::_changeList()
     _closeParagraph();
 
   size_t actualLevel = m_ps->m_listOrderedLevels.size();
-  size_t newLevel= size_t(m_ps->m_paragraph.m_listLevelIndex > 0 && !m_ps->m_paragraph.m_outline ? m_ps->m_paragraph.m_listLevelIndex : 0);
+  auto newLevel= size_t(m_ps->m_paragraph.m_listLevelIndex > 0 && !m_ps->m_paragraph.m_outline ? m_ps->m_paragraph.m_listLevelIndex : 0);
   if (newLevel>100) {
     STOFF_DEBUG_MSG(("STOFFTextListener::_changeList: find level=%d, set it to 100\n", static_cast<int>(newLevel)));
     newLevel=100;
@@ -932,7 +958,7 @@ void STOFFTextListener::_changeList()
   }
 
   if (newLevel) {
-    shared_ptr<STOFFList> theList;
+    std::shared_ptr<STOFFList> theList;
 
     theList=m_listManager->getList(newListId);
     if (!theList) {
@@ -1125,9 +1151,9 @@ void STOFFTextListener::insertComment(STOFFSubDocumentPtr &subDocument, libreven
 }
 
 void STOFFTextListener::insertTextBox
-(STOFFPosition const &pos, STOFFSubDocumentPtr subDocument, STOFFGraphicStyle const &frameStyle)
+(STOFFFrameStyle const &frame, STOFFSubDocumentPtr subDocument, STOFFGraphicStyle const &frameStyle)
 {
-  if (!openFrame(pos, frameStyle)) return;
+  if (!openFrame(frame, frameStyle)) return;
 
   librevenge::RVNGPropertyList propList;
   if (frameStyle.m_propertyList["librevenge:next-frame-name"])
@@ -1140,12 +1166,12 @@ void STOFFTextListener::insertTextBox
   closeFrame();
 }
 
-void STOFFTextListener::insertShape(STOFFGraphicShape const &shape, STOFFGraphicStyle const &style, STOFFPosition const &pos)
+void STOFFTextListener::insertShape(STOFFFrameStyle const &frame, STOFFGraphicShape const &shape, STOFFGraphicStyle const &style)
 {
   if (!m_ps->m_isPageSpanOpened)
     _openPageSpan();
   // now check that the anchor is coherent with the actual state
-  switch (pos.m_anchorTo) {
+  switch (frame.m_position.m_anchorTo) {
   case STOFFPosition::Page:
     if (m_ps->m_isParagraphOpened)
       _closeParagraph();
@@ -1161,7 +1187,7 @@ void STOFFTextListener::insertShape(STOFFGraphicShape const &shape, STOFFGraphic
   default:
 #endif
     STOFF_DEBUG_MSG(("STOFFTextListener::insertShape: UNKNOWN position, insert as char position\n"));
-  // fallthrough intended
+    STOFF_FALLTHROUGH;
   case STOFFPosition::CharBaseLine:
   case STOFFPosition::Char:
     if (m_ps->m_isSpanOpened)
@@ -1175,7 +1201,7 @@ void STOFFTextListener::insertShape(STOFFGraphicShape const &shape, STOFFGraphic
   }
 
   librevenge::RVNGPropertyList shapeProp, styleProp;
-  pos.addTo(shapeProp);
+  frame.addTo(shapeProp);
   shape.addTo(shapeProp);
   style.addTo(styleProp);
   STOFFGraphicStyle::checkForDefault(styleProp);
@@ -1210,9 +1236,9 @@ void STOFFTextListener::insertShape(STOFFGraphicShape const &shape, STOFFGraphic
   }
 }
 
-void STOFFTextListener::insertPicture(STOFFPosition const &pos, STOFFEmbeddedObject const &picture, STOFFGraphicStyle const &style)
+void STOFFTextListener::insertPicture(STOFFFrameStyle const &frame, STOFFEmbeddedObject const &picture, STOFFGraphicStyle const &style)
 {
-  if (!openFrame(pos, style)) return;
+  if (!openFrame(frame, style)) return;
 
   librevenge::RVNGPropertyList propList;
   if (picture.addTo(propList))
@@ -1220,10 +1246,26 @@ void STOFFTextListener::insertPicture(STOFFPosition const &pos, STOFFEmbeddedObj
   closeFrame();
 }
 
+void STOFFTextListener::insertEquation(STOFFFrameStyle const &frame, librevenge::RVNGString const &equation,
+                                       STOFFGraphicStyle const &style)
+{
+  if (equation.empty()) {
+    STOFF_DEBUG_MSG(("STOFFTextListener::insertEquation: oops the equation is empty\n"));
+    return;
+  }
+  if (!openFrame(frame, style)) return;
+
+  librevenge::RVNGPropertyList propList;
+  propList.insert("librevenge:mime-type", "application/mathml+xml");
+  propList.insert("librevenge:data", equation);
+  m_documentInterface->insertEquation(propList);
+  closeFrame();
+}
+
 ///////////////////
 // frame
 ///////////////////
-bool STOFFTextListener::openFrame(STOFFPosition const &pos, STOFFGraphicStyle const &style)
+bool STOFFTextListener::openFrame(STOFFFrameStyle const &frame, STOFFGraphicStyle const &style)
 {
   if (m_ps->m_isTableOpened && !m_ps->m_isTableCellOpened) {
     STOFF_DEBUG_MSG(("STOFFTextListener::openFrame: called in table but cell is not opened\n"));
@@ -1233,8 +1275,8 @@ bool STOFFTextListener::openFrame(STOFFPosition const &pos, STOFFGraphicStyle co
     STOFF_DEBUG_MSG(("STOFFTextListener::openFrame: called but a frame is already opened\n"));
     return false;
   }
-  STOFFPosition fPos(pos);
-  switch (pos.m_anchorTo) {
+  auto finalFrame=frame;
+  switch (finalFrame.m_position.m_anchorTo) {
   case STOFFPosition::Page:
     break;
   case STOFFPosition::Paragraph:
@@ -1245,7 +1287,7 @@ bool STOFFTextListener::openFrame(STOFFPosition const &pos, STOFFGraphicStyle co
     break;
   case STOFFPosition::Unknown:
     STOFF_DEBUG_MSG(("STOFFTextListener::openFrame: UNKNOWN position, insert as char position\n"));
-  // fallthrough intended
+    STOFF_FALLTHROUGH;
   case STOFFPosition::CharBaseLine:
   case STOFFPosition::Char:
     if (m_ps->m_isSpanOpened)
@@ -1264,7 +1306,7 @@ bool STOFFTextListener::openFrame(STOFFPosition const &pos, STOFFGraphicStyle co
         _flushText();
       else
         _openParagraph();
-      fPos.m_anchorTo=STOFFPosition::Paragraph;
+      finalFrame.m_position.m_anchorTo=STOFFPosition::Paragraph;
     }
     break;
   case STOFFPosition::Cell:
@@ -1272,7 +1314,7 @@ bool STOFFTextListener::openFrame(STOFFPosition const &pos, STOFFGraphicStyle co
       STOFF_DEBUG_MSG(("STOFFTextListener::openFrame: called with Cell position not in a table cell\n"));
       return false;
     }
-    if (!pos.m_propertyList["table:end-cell-address"]) {
+    if (!finalFrame.m_position.m_propertyList["table:end-cell-address"]) {
       STOFF_DEBUG_MSG(("STOFFTextListener::openFrame: can not find the cell name\n"));
       return false;
     }
@@ -1285,7 +1327,7 @@ bool STOFFTextListener::openFrame(STOFFPosition const &pos, STOFFGraphicStyle co
   }
 
   librevenge::RVNGPropertyList propList;
-  _handleFrameParameters(propList, fPos, style);
+  _handleFrameParameters(propList, finalFrame, style);
   m_documentInterface->openFrame(propList);
 
   m_ps->m_isFrameOpened = true;
@@ -1302,7 +1344,7 @@ void STOFFTextListener::closeFrame()
   m_ps->m_isFrameOpened = false;
 }
 
-bool STOFFTextListener::openGroup(STOFFPosition const &pos)
+bool STOFFTextListener::openGroup(STOFFFrameStyle const &frame)
 {
   if (!m_ds->m_isDocumentStarted) {
     STOFF_DEBUG_MSG(("STOFFTextListener::openGroup: the document is not started\n"));
@@ -1314,7 +1356,7 @@ bool STOFFTextListener::openGroup(STOFFPosition const &pos)
   }
 
   // now check that the anchor is coherent with the actual state
-  switch (pos.m_anchorTo) {
+  switch (frame.m_position.m_anchorTo) {
   case STOFFPosition::Page:
     break;
   case STOFFPosition::Paragraph:
@@ -1328,7 +1370,7 @@ bool STOFFTextListener::openGroup(STOFFPosition const &pos)
   default:
 #endif
     STOFF_DEBUG_MSG(("STOFFTextListener::openGroup: UNKNOWN position, insert as char position\n"));
-  // fallthrough intended
+    STOFF_FALLTHROUGH;
   case STOFFPosition::CharBaseLine:
   case STOFFPosition::Char:
     if (m_ps->m_isSpanOpened)
@@ -1346,7 +1388,7 @@ bool STOFFTextListener::openGroup(STOFFPosition const &pos)
   m_ps->m_isGroupOpened = true;
 
   librevenge::RVNGPropertyList propList;
-  pos.addTo(propList);
+  frame.addTo(propList);
   m_documentInterface->openGroup(propList);
 
   return true;
@@ -1364,11 +1406,11 @@ void  STOFFTextListener::closeGroup()
 }
 
 void STOFFTextListener::_handleFrameParameters
-(librevenge::RVNGPropertyList &propList, STOFFPosition const &pos, STOFFGraphicStyle const &style)
+(librevenge::RVNGPropertyList &propList, STOFFFrameStyle const &frame, STOFFGraphicStyle const &style)
 {
   if (!m_ds->m_isDocumentStarted)
     return;
-  pos.addTo(propList);
+  frame.addTo(propList);
   style.addTo(propList);
 }
 
@@ -1408,10 +1450,12 @@ void STOFFTextListener::handleSubDocument(STOFFSubDocumentPtr subDocument, libst
 
   // Check whether the document is calling itself
   bool sendDoc = true;
-  for (size_t i = 0; i < m_ds->m_subDocuments.size(); i++) {
+  for (auto const &doc : m_ds->m_subDocuments) {
     if (!subDocument)
       break;
-    if (subDocument == m_ds->m_subDocuments[i]) {
+    if (!doc)
+      continue;
+    if (*subDocument == *doc) {
       STOFF_DEBUG_MSG(("STOFFTextListener::handleSubDocument: recursif call, stop...\n"));
       sendDoc = false;
       break;
@@ -1420,7 +1464,7 @@ void STOFFTextListener::handleSubDocument(STOFFSubDocumentPtr subDocument, libst
   if (sendDoc) {
     if (subDocument) {
       m_ds->m_subDocuments.push_back(subDocument);
-      shared_ptr<STOFFListener> listen(this, STOFF_shared_ptr_noop_deleter<STOFFTextListener>());
+      std::shared_ptr<STOFFListener> listen(this, STOFF_shared_ptr_noop_deleter<STOFFTextListener>());
       try {
         subDocument->parse(listen, subDocumentType);
       }
@@ -1625,9 +1669,9 @@ void STOFFTextListener::closeTableCell()
 ///////////////////
 
 // ---------- state stack ------------------
-shared_ptr<STOFFTextListenerInternal::State> STOFFTextListener::_pushParsingState()
+std::shared_ptr<STOFFTextListenerInternal::State> STOFFTextListener::_pushParsingState()
 {
-  shared_ptr<STOFFTextListenerInternal::State> actual = m_ps;
+  auto actual = m_ps;
   m_psStack.push_back(actual);
   m_ps.reset(new STOFFTextListenerInternal::State);
 
