@@ -164,15 +164,15 @@ std::string numberingValueToString(NumberingType type, int value)
     return s;
   case LOWERCASE_ROMAN:
   case UPPERCASE_ROMAN: {
-    static char const *(romanS[]) = {"M", "CM", "D", "CD", "C", "XC", "L",
-                                     "XL", "X", "IX", "V", "IV", "I"
-                                    };
-    static char const *(romans[]) = {"m", "cm", "d", "cd", "c", "xc", "l",
-                                     "xl", "x", "ix", "v", "iv", "i"
-                                    };
-    static int const(romanV[]) = {1000, 900, 500, 400,  100, 90, 50,
-                                  40, 10, 9, 5, 4, 1
-                                 };
+    static char const *romanS[] = {"M", "CM", "D", "CD", "C", "XC", "L",
+                                   "XL", "X", "IX", "V", "IV", "I"
+                                  };
+    static char const *romans[] = {"m", "cm", "d", "cd", "c", "xc", "l",
+                                   "xl", "x", "ix", "v", "iv", "i"
+                                  };
+    static int const romanV[] = {1000, 900, 500, 400,  100, 90, 50,
+                                 40, 10, 9, 5, 4, 1
+                                };
     if (value <= 0 || value >= 4000) {
       STOFF_DEBUG_MSG(("libstoff::numberingValueToString: out of range value for type %d\n", int(type)));
       return (type == LOWERCASE_ROMAN) ? "i" : "I";
@@ -205,7 +205,7 @@ STOFFColor STOFFColor::barycenter(float alpha, STOFFColor const &colA,
     float val=alpha*float((colA.m_value>>depl)&0xFF)+beta*float((colB.m_value>>depl)&0xFF);
     if (val < 0) val=0;
     if (val > 256) val=256;
-    unsigned char comp= static_cast<unsigned char>(val);
+    auto comp= static_cast<unsigned char>(val);
     res+=uint32_t(comp<<depl);
   }
   return STOFFColor(res);
@@ -452,6 +452,21 @@ librevenge::RVNGString simplifyString(librevenge::RVNGString const &s)
   return res;
 }
 
+std::string getCellName(STOFFVec2i const &cellPos, STOFFVec2b const &relative)
+{
+  if (cellPos[0]<0 || cellPos[0]>=26*26*26 || cellPos[1]<0) {
+    STOFF_DEBUG_MSG(("libwps::getCellName: invalid cell position\n"));
+    return "";
+  }
+  std::stringstream o;
+  if (!relative[0]) o << "$";
+  if (cellPos[0]>=26*26) o << (char)(cellPos[0]/26/26-1 + 'A');
+  if (cellPos[0]>=26) o << (char)((cellPos[0]%(26*26))/26-1 + 'A');
+  o << (char)(cellPos[0]%26+'A');
+  if (!relative[1]) o << "$";
+  o << cellPos[1]+1;
+  return o.str();
+}
 // a little geometry
 STOFFVec2f rotatePointAroundCenter(STOFFVec2f const &point, STOFFVec2f const &center, float angle)
 {
