@@ -25,16 +25,22 @@
  * in which case the provisions of the GPLv2+ or the LGPLv2+ are applicable
  * instead of those above.
  */
+
+#include "http-session.hxx"
+
 #include <cctype>
+#include <memory>
 #include <string>
+#include <assert.h>
 
 #include <libxml/parser.h>
 #include <libxml/tree.h>
 #include <libxml/xpath.h>
 
+#include <libcmis/session-factory.hxx>
+#include <libcmis/xml-utils.hxx>
+
 #include "oauth2-handler.hxx"
-#include "session-factory.hxx"
-#include "xml-utils.hxx"
 
 using namespace std;
 
@@ -78,6 +84,7 @@ namespace
         return is.gcount( ) / size;
     }
 
+#if (LIBCURL_VERSION_MAJOR < 7) || (LIBCURL_VERSION_MAJOR == 7 && LIBCURL_VERSION_MINOR < 85)
     curlioerr lcl_ioctlStream( CURL* /*handle*/, int cmd, void* data )
     {
         curlioerr errCode = CURLIOE_OK;
@@ -104,6 +111,28 @@ namespace
         }
         return errCode;
     }
+#endif
+
+    int lcl_seekStream(void* data, curl_off_t offset, int origin)
+    {
+        std::ios_base::seekdir dir = {};
+        switch (origin)
+        {
+            case SEEK_SET: dir = std::ios_base::beg; break;
+            case SEEK_CUR: dir = std::ios_base::cur; break;
+            case SEEK_END: dir = std::ios_base::end; break;
+            default: assert(false); break;
+        }
+        istream& is = *(static_cast<istream*>(data));
+        is.clear();
+        is.seekg(offset, dir);
+        if (!is.good())
+        {
+            fprintf(stderr, "rewind failed\n");
+            return CURL_SEEKFUNC_FAIL;
+        }
+        return CURL_SEEKFUNC_OK;
+    }
 
     template<typename T>
     class ScopeGuard
@@ -128,9 +157,10 @@ namespace
 }
 
 HttpSession::HttpSession( string username, string password, bool noSslCheck,
-                          libcmis::OAuth2DataPtr oauth2, bool verbose )
-        throw ( libcmis::Exception ) :
+                          libcmis::OAuth2DataPtr oauth2, bool verbose,
+                          libcmis::CurlInitProtocolsFunction initProtocolsFunction) :
     m_curlHandle( NULL ),
+    m_CurlInitProtocolsFunction(initProtocolsFunction),
     m_no100Continue( false ),
     m_oauth2Handler( NULL ),
     m_username( username ),
@@ -221,19 +251,19 @@ HttpSession::~HttpSession( )
     delete( m_oauth2Handler );
 }
 
-string& HttpSession::getUsername( ) throw ( CurlException )
+string& HttpSession::getUsername( )
 {
     checkCredentials( );
     return m_username;
 }
 
-string& HttpSession::getPassword( ) throw ( CurlException )
+string& HttpSession::getPassword( )
 {
     checkCredentials( );
     return m_password;
 }
 
-libcmis::HttpResponsePtr HttpSession::httpGetRequest( string url ) throw ( CurlException )
+libcmis::HttpResponsePtr HttpSession::httpGetRequest( string url )
 {
     checkOAuth2( url );
 
@@ -252,14 +282,14 @@ libcmis::HttpResponsePtr HttpSession::httpGetRequest( string url ) throw ( CurlE
     // fix Cloudoku too many redirects error
     // note: though curl doc says -1 is the default for MAXREDIRS, the error i got
     // said it was 0
-    curl_easy_setopt( m_curlHandle, CURLOPT_MAXREDIRS, 100);
+    curl_easy_setopt( m_curlHandle, CURLOPT_MAXREDIRS, 20);
 
     try
     {
         httpRunRequest( url );
         response->getData( )->finish( );
     }
-    catch ( const CurlException& e )
+    catch ( const CurlException& )
     {
         // If the access token is expired, we get 401 error,
         // Need to use the refresh token to get a new one.
@@ -289,7 +319,7 @@ libcmis::HttpResponsePtr HttpSession::httpGetRequest( string url ) throw ( CurlE
     return response;
 }
 
-libcmis::HttpResponsePtr HttpSession::httpPutRequest( string url, istream& is, vector< string > headers ) throw ( CurlException )
+libcmis::HttpResponsePtr HttpSession::httpPatchRequest( string url, istream& is, vector< string > headers )
 {
     checkOAuth2( url );
 
@@ -310,6 +340,8 @@ libcmis::HttpResponsePtr HttpSession::httpPutRequest( string url, istream& is, v
     curl_easy_setopt( m_curlHandle, CURLOPT_HEADERFUNCTION, &lcl_getHeaders );
     curl_easy_setopt( m_curlHandle, CURLOPT_WRITEHEADER, response.get() );
 
+    curl_easy_setopt( m_curlHandle, CURLOPT_MAXREDIRS, 20);
+
     // Get the stream length
     is.seekg( 0, ios::end );
     long size = is.tellg( );
@@ -318,8 +350,14 @@ libcmis::HttpResponsePtr HttpSession::httpPutRequest( string url, istream& is, v
     curl_easy_setopt( m_curlHandle, CURLOPT_READDATA, &isOriginal );
     curl_easy_setopt( m_curlHandle, CURLOPT_READFUNCTION, lcl_readStream );
     curl_easy_setopt( m_curlHandle, CURLOPT_UPLOAD, 1 );
+    curl_easy_setopt( m_curlHandle, CURLOPT_CUSTOMREQUEST, "PATCH" );
+#if (LIBCURL_VERSION_MAJOR > 7) || (LIBCURL_VERSION_MAJOR == 7 && LIBCURL_VERSION_MINOR >= 85)
+    curl_easy_setopt( m_curlHandle, CURLOPT_SEEKFUNCTION, lcl_seekStream );
+    curl_easy_setopt( m_curlHandle, CURLOPT_SEEKDATA, &isOriginal );
+#else
     curl_easy_setopt( m_curlHandle, CURLOPT_IOCTLFUNCTION, lcl_ioctlStream );
     curl_easy_setopt( m_curlHandle, CURLOPT_IOCTLDATA, &isOriginal );
+#endif
 
     // If we know for sure that 100-Continue won't be accepted,
     // don't even try with it to save one HTTP request.
@@ -330,7 +368,99 @@ libcmis::HttpResponsePtr HttpSession::httpPutRequest( string url, istream& is, v
         httpRunRequest( url, headers );
         response->getData( )->finish();
     }
-    catch ( const CurlException& e )
+    catch ( const CurlException& )
+    {
+        long status = getHttpStatus( );
+        /** If we had a HTTP 417 response, this is likely to be due to some
+            HTTP 1.0 proxy / server not accepting the "Expect: 100-continue"
+            header. Try to disable this header and try again.
+        */
+        if ( status == 417 && !m_no100Continue)
+        {
+            // Remember that we don't want 100-Continue for the future requests
+            m_no100Continue = true;
+            response = httpPutRequest( url, isBackup, headers );
+        }
+
+        // If the access token is expired, we get 401 error,
+        // Need to use the refresh token to get a new one.
+        if ( status == 401 && !getRefreshToken( ).empty( ) && !m_refreshedToken )
+        {
+
+            // Refresh the token
+            oauth2Refresh();
+
+            // Resend the query
+            try
+            {
+                // Avoid infinite recursive call
+                m_refreshedToken = true;
+                response = httpPutRequest( url, isBackup, headers );
+                m_refreshedToken = false;
+            }
+            catch (const CurlException&)
+            {
+                m_refreshedToken = false;
+                throw;
+            }
+        }
+        // Has tried but failed
+        if ( ( status != 417 || m_no100Continue ) &&
+             ( status != 401 || getRefreshToken( ).empty( ) || m_refreshedToken ) ) throw;
+    }
+    m_refreshedToken = false;
+    return response;
+}
+
+libcmis::HttpResponsePtr HttpSession::httpPutRequest( string url, istream& is, vector< string > headers )
+{
+    checkOAuth2( url );
+
+    // Duplicate istream in case we need to retry
+    string isStr( static_cast< stringstream const&>( stringstream( ) << is.rdbuf( ) ).str( ) );
+
+    istringstream isOriginal( isStr ), isBackup( isStr );
+
+    // Reset the handle for the request
+    curl_easy_reset( m_curlHandle );
+    initProtocols( );
+
+    libcmis::HttpResponsePtr response( new libcmis::HttpResponse( ) );
+
+    curl_easy_setopt( m_curlHandle, CURLOPT_WRITEFUNCTION, lcl_bufferData );
+    curl_easy_setopt( m_curlHandle, CURLOPT_WRITEDATA, response->getData( ).get( ) );
+
+    curl_easy_setopt( m_curlHandle, CURLOPT_HEADERFUNCTION, &lcl_getHeaders );
+    curl_easy_setopt( m_curlHandle, CURLOPT_WRITEHEADER, response.get() );
+
+    curl_easy_setopt( m_curlHandle, CURLOPT_MAXREDIRS, 20);
+
+    // Get the stream length
+    is.seekg( 0, ios::end );
+    long size = is.tellg( );
+    is.seekg( 0, ios::beg );
+    curl_easy_setopt( m_curlHandle, CURLOPT_INFILESIZE, size );
+    curl_easy_setopt( m_curlHandle, CURLOPT_READDATA, &isOriginal );
+    curl_easy_setopt( m_curlHandle, CURLOPT_READFUNCTION, lcl_readStream );
+    curl_easy_setopt( m_curlHandle, CURLOPT_UPLOAD, 1 );
+#if (LIBCURL_VERSION_MAJOR > 7) || (LIBCURL_VERSION_MAJOR == 7 && LIBCURL_VERSION_MINOR >= 85)
+    curl_easy_setopt( m_curlHandle, CURLOPT_SEEKFUNCTION, lcl_seekStream );
+    curl_easy_setopt( m_curlHandle, CURLOPT_SEEKDATA, &isOriginal );
+#else
+    curl_easy_setopt( m_curlHandle, CURLOPT_IOCTLFUNCTION, lcl_ioctlStream );
+    curl_easy_setopt( m_curlHandle, CURLOPT_IOCTLDATA, &isOriginal );
+#endif
+
+    // If we know for sure that 100-Continue won't be accepted,
+    // don't even try with it to save one HTTP request.
+    if ( m_no100Continue )
+        headers.push_back( "Expect:" );
+    try
+    {
+        httpRunRequest( url, headers );
+        response->getData( )->finish();
+    }
+    catch ( const CurlException& )
     {
         long status = getHttpStatus( );
         /** If we had a HTTP 417 response, this is likely to be due to some
@@ -375,7 +505,7 @@ libcmis::HttpResponsePtr HttpSession::httpPutRequest( string url, istream& is, v
 }
 
 libcmis::HttpResponsePtr HttpSession::httpPostRequest( const string& url, istream& is,
-    const string& contentType, bool redirect ) throw ( CurlException )
+    const string& contentType, bool redirect )
 {
     checkOAuth2( url );
 
@@ -396,6 +526,8 @@ libcmis::HttpResponsePtr HttpSession::httpPostRequest( const string& url, istrea
     curl_easy_setopt( m_curlHandle, CURLOPT_HEADERFUNCTION, &lcl_getHeaders );
     curl_easy_setopt( m_curlHandle, CURLOPT_WRITEHEADER, response.get() );
 
+    curl_easy_setopt( m_curlHandle, CURLOPT_MAXREDIRS, 20);
+
     // Get the stream length
     is.seekg( 0, ios::end );
     long size = is.tellg( );
@@ -404,8 +536,13 @@ libcmis::HttpResponsePtr HttpSession::httpPostRequest( const string& url, istrea
     curl_easy_setopt( m_curlHandle, CURLOPT_READDATA, &isOriginal );
     curl_easy_setopt( m_curlHandle, CURLOPT_READFUNCTION, lcl_readStream );
     curl_easy_setopt( m_curlHandle, CURLOPT_POST, 1 );
+#if (LIBCURL_VERSION_MAJOR > 7) || (LIBCURL_VERSION_MAJOR == 7 && LIBCURL_VERSION_MINOR >= 85)
+    curl_easy_setopt( m_curlHandle, CURLOPT_SEEKFUNCTION, lcl_seekStream );
+    curl_easy_setopt( m_curlHandle, CURLOPT_SEEKDATA, &isOriginal );
+#else
     curl_easy_setopt( m_curlHandle, CURLOPT_IOCTLFUNCTION, lcl_ioctlStream );
     curl_easy_setopt( m_curlHandle, CURLOPT_IOCTLDATA, &isOriginal );
+#endif
 
     vector< string > headers;
     headers.push_back( string( "Content-Type:" ) + contentType );
@@ -419,7 +556,7 @@ libcmis::HttpResponsePtr HttpSession::httpPostRequest( const string& url, istrea
         httpRunRequest( url, headers, redirect );
         response->getData( )->finish();
     }
-    catch ( const CurlException& e )
+    catch ( const CurlException& )
     {
 
         long status = getHttpStatus( );
@@ -465,7 +602,7 @@ libcmis::HttpResponsePtr HttpSession::httpPostRequest( const string& url, istrea
     return response;
 }
 
-void HttpSession::httpDeleteRequest( string url ) throw ( CurlException )
+void HttpSession::httpDeleteRequest( string url )
 {
     checkOAuth2( url );
 
@@ -478,7 +615,7 @@ void HttpSession::httpDeleteRequest( string url ) throw ( CurlException )
     {
         httpRunRequest( url );
     }
-    catch ( const CurlException& e )
+    catch ( const CurlException& )
     {
         // If the access token is expired, we get 401 error,
         // Need to use the refresh token to get a new one.
@@ -507,7 +644,7 @@ void HttpSession::httpDeleteRequest( string url ) throw ( CurlException )
     m_refreshedToken = false;
 }
 
-void HttpSession::checkCredentials( ) throw ( CurlException )
+void HttpSession::checkCredentials( )
 {
     // Check that we have the complete credentials
     libcmis::AuthProviderPtr authProvider = libcmis::SessionFactory::getAuthenticationProvider();
@@ -521,7 +658,7 @@ void HttpSession::checkCredentials( ) throw ( CurlException )
     }
 }
 
-void HttpSession::httpRunRequest( string url, vector< string > headers, bool redirect ) throw ( CurlException )
+void HttpSession::httpRunRequest( string url, vector< string > headers, bool redirect )
 {
     // Redirect
     curl_easy_setopt( m_curlHandle, CURLOPT_FOLLOWLOCATION, redirect);
@@ -692,7 +829,6 @@ void HttpSession::httpRunRequest( string url, vector< string > headers, bool red
 
 
 void HttpSession::checkOAuth2( string url )
-    throw ( CurlException )
 try
 {
     if ( m_oauth2Handler )
@@ -715,12 +851,12 @@ long HttpSession::getHttpStatus( )
     return status;
 }
 
-void HttpSession::setOAuth2Data( libcmis::OAuth2DataPtr oauth2 ) throw ( libcmis::Exception )
+void HttpSession::setOAuth2Data( libcmis::OAuth2DataPtr oauth2 )
 {
     m_oauth2Handler = new OAuth2Handler( this, oauth2 );
 }
 
-void HttpSession::oauth2Authenticate( ) throw ( libcmis::Exception )
+void HttpSession::oauth2Authenticate( )
 {
     string authCode;
 
@@ -737,12 +873,11 @@ void HttpSession::oauth2Authenticate( ) throw ( libcmis::Exception )
             libcmis::OAuth2AuthCodeProvider fallbackProvider = libcmis::SessionFactory::getOAuth2AuthCodeProvider( );
             if ( fallbackProvider != NULL )
             {
-                char * code = fallbackProvider( m_oauth2Handler->getAuthURL().c_str(), getUsername().c_str(), getPassword().c_str() );
-                if ( code != NULL )
-                {
-                    authCode = string( code );
-                    free( code );
-                }
+                unique_ptr< char, void (*)( void * ) > code{
+                    fallbackProvider( m_oauth2Handler->getAuthURL().c_str(), getUsername().c_str(), getPassword().c_str() ),
+                    free };
+                if ( code )
+                    authCode = string( code.get() );
             }
         }
     }
@@ -764,7 +899,7 @@ void HttpSession::setNoSSLCertificateCheck( bool noCheck )
     m_noSSLCheck = noCheck;
 }
 
-string HttpSession::getRefreshToken( ) throw ( libcmis::Exception )
+string HttpSession::getRefreshToken( )
 {
     string refreshToken;
     if ( m_oauth2Handler )
@@ -773,7 +908,6 @@ string HttpSession::getRefreshToken( ) throw ( libcmis::Exception )
 }
 
 void HttpSession::oauth2Refresh( )
-    throw ( CurlException )
 try
 {
     const ScopeGuard<bool> inOauth2Guard(m_inOAuth2Authentication, true);
@@ -786,12 +920,22 @@ catch ( const libcmis::Exception& e )
 
 void HttpSession::initProtocols( )
 {
+#if (LIBCURL_VERSION_MAJOR > 7) || (LIBCURL_VERSION_MAJOR == 7 && LIBCURL_VERSION_MINOR >= 85)
+    auto const protocols = "https,http";
+    curl_easy_setopt(m_curlHandle, CURLOPT_PROTOCOLS_STR, protocols);
+    curl_easy_setopt(m_curlHandle, CURLOPT_REDIR_PROTOCOLS_STR, protocols);
+#else
     const unsigned long protocols = CURLPROTO_HTTP | CURLPROTO_HTTPS;
     curl_easy_setopt(m_curlHandle, CURLOPT_PROTOCOLS, protocols);
     curl_easy_setopt(m_curlHandle, CURLOPT_REDIR_PROTOCOLS, protocols);
+#endif
+    if (m_CurlInitProtocolsFunction)
+    {
+        (*m_CurlInitProtocolsFunction)(m_curlHandle);
+    }
 }
 
-const char* CurlException::what( ) const throw ()
+const char* CurlException::what( ) const noexcept
 {
     if ( !isCancelled( ) )
     {
