@@ -23,7 +23,7 @@ namespace {
 
 class mock_ref_resolver : public import_reference_resolver
 {
-    virtual src_address_t resolve_address(const char* p, size_t n) override
+    virtual src_address_t resolve_address(std::string_view) override
     {
         src_address_t ret;
         ret.sheet = 0;
@@ -33,7 +33,7 @@ class mock_ref_resolver : public import_reference_resolver
         return ret;
     }
 
-    virtual src_range_t resolve_range(const char* p, size_t n) override
+    virtual src_range_t resolve_range(std::string_view) override
     {
         src_range_t ret;
         ret.first.sheet = 0;
@@ -58,25 +58,25 @@ public:
         assert(range.last.column == 1);
     }
 
-    virtual void set_formula(formula_grammar_t grammar, const char* p, size_t n) override
+    virtual void set_formula(formula_grammar_t grammar, std::string_view formula) override
     {
         assert(grammar == formula_grammar_t::xlsx);
-        assert(string(p, n) == "A1:A2");
+        assert(formula == "A1:A2");
     }
 
-    virtual void set_result_bool(row_t row, col_t col, bool value) override
+    virtual void set_result_bool(row_t, col_t, bool) override
     {
     }
 
-    virtual void set_result_empty(row_t row, col_t col) override
+    virtual void set_result_empty(row_t, col_t) override
     {
     }
 
-    virtual void set_result_string(row_t row, col_t col, size_t sindex) override
+    virtual void set_result_string(row_t, col_t, std::string_view) override
     {
     }
 
-    virtual void set_result_value(row_t row, col_t col, double value) override
+    virtual void set_result_value(row_t, col_t, double) override
     {
     }
 
@@ -113,15 +113,17 @@ public:
 class mock_sheet_properties : public import_sheet_properties
 {
 public:
-    void set_column_hidden(col_t col, bool hidden)
+    void set_column_hidden(col_t col, col_t col_span, bool hidden)
     {
         assert(col == 1);
+        assert(col_span == 1);
         assert(hidden);
     }
 
-    void set_row_hidden(row_t row, bool hidden)
+    void set_row_hidden(row_t row, row_t row_span, bool hidden)
     {
         assert(row == 3);
+        assert(row_span == 1);
         assert(hidden);
     }
 };
@@ -142,7 +144,7 @@ void test_cell_value()
 {
     mock_sheet sheet;
     mock_ref_resolver resolver;
-    session_context cxt(new xlsx_session_data);
+    session_context cxt(std::make_unique<xlsx_session_data>());
     config opt(format_t::xlsx);
     opt.structure_check = false;
 
@@ -151,11 +153,11 @@ void test_cell_value()
 
     orcus::xmlns_id_t ns = NS_ooxml_xlsx;
     orcus::xml_token_t elem = XML_c;
-    orcus::xml_attrs_t attrs;
+    orcus::xml_token_attrs_t attrs;
     context.start_element(ns, elem, attrs);
 
     {
-        xml_attrs_t val_attrs;
+        xml_token_attrs_t val_attrs;
         context.start_element(ns, XML_v, val_attrs);
         context.characters("5", false);
         context.end_element(ns, XML_v);
@@ -168,7 +170,7 @@ void test_cell_bool()
 {
     mock_sheet sheet;
     mock_ref_resolver resolver;
-    session_context cxt(new xlsx_session_data);
+    session_context cxt(std::make_unique<xlsx_session_data>());
     config opt(format_t::xlsx);
     opt.structure_check = false;
 
@@ -177,12 +179,12 @@ void test_cell_bool()
 
     orcus::xmlns_id_t ns = NS_ooxml_xlsx;
     orcus::xml_token_t elem = XML_c;
-    orcus::xml_attrs_t attrs;
+    orcus::xml_token_attrs_t attrs;
     attrs.push_back(xml_token_attr_t(NS_ooxml_xlsx, XML_t, "b", false));
     context.start_element(ns, elem, attrs);
 
     {
-        xml_attrs_t val_attrs;
+        xml_token_attrs_t val_attrs;
         context.start_element(ns, XML_v, val_attrs);
         context.characters("1", false);
         context.end_element(ns, XML_v);
@@ -195,7 +197,7 @@ void test_array_formula()
 {
     mock_sheet sheet;
     mock_ref_resolver resolver;
-    session_context cxt(new xlsx_session_data);
+    session_context cxt(std::make_unique<xlsx_session_data>());
     config opt(format_t::xlsx);
     opt.structure_check = false;
 
@@ -204,11 +206,11 @@ void test_array_formula()
 
     orcus::xmlns_id_t ns = NS_ooxml_xlsx;
     orcus::xml_token_t elem = XML_c;
-    orcus::xml_attrs_t attrs;
+    orcus::xml_token_attrs_t attrs;
     context.start_element(ns, elem, attrs);
 
     {
-        xml_attrs_t formula_attrs;
+        xml_token_attrs_t formula_attrs;
         formula_attrs.push_back(xml_token_attr_t(NS_ooxml_xlsx, XML_t, "array", false));
         formula_attrs.push_back(xml_token_attr_t(NS_ooxml_xlsx, XML_ref, "B3:B4", false));
         context.start_element(ns, XML_f, formula_attrs);
@@ -216,7 +218,7 @@ void test_array_formula()
         context.end_element(ns, XML_f);
     }
     {
-        xml_attrs_t val_attrs;
+        xml_token_attrs_t val_attrs;
         context.start_element(ns, XML_v, val_attrs);
         context.characters("5", false);
         context.end_element(ns, XML_v);
@@ -229,7 +231,7 @@ void test_hidden_col()
 {
     mock_sheet2 sheet;
     mock_ref_resolver resolver;
-    session_context cxt(new xlsx_session_data);
+    session_context cxt(std::make_unique<xlsx_session_data>());
     config opt(format_t::xlsx);
     opt.structure_check = false;
 
@@ -238,7 +240,7 @@ void test_hidden_col()
 
     orcus::xmlns_id_t ns = NS_ooxml_xlsx;
     orcus::xml_token_t elem = XML_col;
-    orcus::xml_attrs_t attrs;
+    orcus::xml_token_attrs_t attrs;
     attrs.push_back(orcus::xml_token_attr_t(ns, XML_min, "2", false));
     attrs.push_back(orcus::xml_token_attr_t(ns, XML_max, "2", false));
     attrs.push_back(orcus::xml_token_attr_t(ns, XML_hidden, "1", false));
@@ -250,7 +252,7 @@ void test_hidden_row()
 {
     mock_sheet2 sheet;
     mock_ref_resolver resolver;
-    session_context cxt(new xlsx_session_data);
+    session_context cxt(std::make_unique<xlsx_session_data>());
     config opt(format_t::xlsx);
     opt.structure_check = false;
 
@@ -259,7 +261,7 @@ void test_hidden_row()
 
     orcus::xmlns_id_t ns = NS_ooxml_xlsx;
     orcus::xml_token_t elem = XML_row;
-    orcus::xml_attrs_t attrs;
+    orcus::xml_token_attrs_t attrs;
     attrs.push_back(orcus::xml_token_attr_t(ns, XML_r, "4", false));
     attrs.push_back(orcus::xml_token_attr_t(ns, XML_hidden, "1", false));
     context.start_element(ns, elem, attrs);
