@@ -12,8 +12,6 @@
 import re, sys
 
 def stdout():
-    if sys.version_info[0] < 3:
-        return sys.stdout
     class bytes_stdout(object):
         def write(self, data):
             if isinstance(data, bytes):
@@ -21,10 +19,7 @@ def stdout():
             sys.stdout.write(data)
     return bytes_stdout()
 
-try:
-    from StringIO import StringIO as BytesIO
-except ImportError:
-    from io import BytesIO
+from io import BytesIO
 
 from lxml import etree as ElementTree
 from lxml import _elementpath as ElementPath
@@ -48,16 +43,7 @@ def serialize(elem, **options):
     file = BytesIO()
     tree = ElementTree.ElementTree(elem)
     tree.write(file, **options)
-    if sys.version_info[0] < 3:
-        try:
-            encoding = options["encoding"]
-        except KeyError:
-            encoding = "utf-8"
-    else:
-        encoding = 'ISO8859-1'
-    result = fix_compatibility(file.getvalue().decode(encoding))
-    if sys.version_info[0] < 3:
-        result = result.encode(encoding)
+    result = fix_compatibility(file.getvalue().decode('ISO8859-1'))
     return result
 
 def summarize(elem):
@@ -574,7 +560,7 @@ def encoding():
     >>> serialize(elem, encoding="iso-8859-1").lower()
     '<?xml version=\'1.0\' encoding=\'iso-8859-1\'?>\n<tag key="&lt;&amp;&quot;\'&gt;"/>'
 
-    >>> elem.text = u'\xe5\xf6\xf6<>'
+    >>> elem.text = '\xe5\xf6\xf6<>'
     >>> elem.attrib.clear()
     >>> serialize(elem)
     '<tag>&#229;&#246;&#246;&lt;&gt;</tag>'
@@ -585,7 +571,7 @@ def encoding():
     >>> serialize(elem, encoding="iso-8859-1").lower()
     "<?xml version='1.0' encoding='iso-8859-1'?>\n<tag>\xe5\xf6\xf6&lt;&gt;</tag>"
 
-    >>> elem.attrib["key"] = u'\xe5\xf6\xf6<>'
+    >>> elem.attrib["key"] = '\xe5\xf6\xf6<>'
     >>> elem.text = None
     >>> serialize(elem)
     '<tag key="&#229;&#246;&#246;&lt;&gt;"/>'
@@ -597,8 +583,6 @@ def encoding():
     '<?xml version=\'1.0\' encoding=\'iso-8859-1\'?>\n<tag key="\xe5\xf6\xf6&lt;&gt;"/>'
     """
 
-if sys.version_info[0] >= 3:
-    encoding.__doc__ = encoding.__doc__.replace("u'", "'")
 
 def methods():
     r"""
@@ -622,6 +606,7 @@ def methods():
 # doesn't work with lxml.etree
 del methods
 
+
 def iterators():
     """
     Test iterators.
