@@ -26,6 +26,7 @@
 #include <xmlsec/keys.h>
 #include <xmlsec/transforms.h>
 #include <xmlsec/xmltree.h>
+#include <xmlsec/private.h>
 
 #include <xmlsec/mscng/certkeys.h>
 #include <xmlsec/mscng/crypto.h>
@@ -52,7 +53,7 @@ xmlSecMSCngKeyDataCertGetPubkey(PCCERT_CONTEXT cert, BCRYPT_KEY_HANDLE* key) {
     xmlSecAssert2(key != NULL, -1);
 
     if(!CryptImportPublicKeyInfoEx2(X509_ASN_ENCODING,
-            &cert->pCertInfo->SubjectPublicKeyInfo,
+            &(cert->pCertInfo->SubjectPublicKeyInfo),
             0,
             NULL,
             key)) {
@@ -932,7 +933,7 @@ done:
 static int
 xmlSecMSCngKeyDataDsaWrite(xmlSecKeyDataId id, xmlSecKeyDataPtr data,
                         xmlSecKeyValueDsaPtr dsaValue,
-                        int writePrivateKey ATTRIBUTE_UNUSED) {
+                        int writePrivateKey XMLSEC_ATTRIBUTE_UNUSED) {
     xmlSecMSCngKeyDataCtxPtr ctx;
     NTSTATUS status;
     xmlSecBuffer buf;
@@ -1374,7 +1375,7 @@ done:
 static int
 xmlSecMSCngKeyDataRsaWrite(xmlSecKeyDataId id, xmlSecKeyDataPtr data,
                            xmlSecKeyValueRsaPtr rsaValue,
-                           int writePrivateKey ATTRIBUTE_UNUSED) {
+                           int writePrivateKey XMLSEC_ATTRIBUTE_UNUSED) {
     xmlSecMSCngKeyDataCtxPtr ctx;
     NTSTATUS status;
     xmlSecBuffer buf;
@@ -2112,15 +2113,4 @@ xmlSecMSCngKeyDataEcGetKlass(void) {
     return(&xmlSecMSCngKeyDataEcKlass);
 }
 
-/**
- * xmlSecMSCngKeyDataEcdsaGetKlass:
- *
- * Deprecated. The MSCng EC CertKey data klass.
- *
- * Returns: pointer to MSCng EC key data klass.
- */
-xmlSecKeyDataId
-xmlSecMSCngKeyDataEcdsaGetKlass(void) {
-    return(xmlSecMSCngKeyDataEcGetKlass());
-}
 #endif /* XMLSEC_NO_EC */
