@@ -6,7 +6,7 @@
  * distribution for preciese wording.
  *
  * Copyright (C) 2003 Cordys R&D BV, All rights reserved.
- * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
  * SECTION:certkeys
@@ -18,6 +18,7 @@
 #include "globals.h"
 
 #include <string.h>
+#include <stdlib.h>
 
 #ifndef XMLSEC_NO_GOST
 #include "csp_oid.h"
@@ -31,6 +32,7 @@
 #include <xmlsec/transforms.h>
 #include <xmlsec/errors.h>
 #include <xmlsec/bn.h>
+#include <xmlsec/private.h>
 
 #include <xmlsec/mscrypto/certkeys.h>
 #include <xmlsec/mscrypto/crypto.h>
@@ -40,12 +42,10 @@
 #include "../cast_helpers.h"
 #include "../keysdata_helpers.h"
 
-// GOST CSP don't support keys duplicating, so we use NT4 analogs for these...
+// GOST CSP don't support keys duplicating, so we use custom refcounting instead
 #ifndef XMLSEC_NO_GOST
-#ifndef XMLSEC_MSCRYPTO_NT4
-#define XMLSEC_MSCRYPTO_NT4
-#endif
-#endif
+#define XMLSEC_MSCRYPTO_CUSTOM_REFCOUNT
+#endif  /* XMLSEC_NO_GOST */
 
 #define XMLSEC_MSCRYPTO_DSA_MAX_Q_SIZE     ((xmlSecSize)0x14U)
 
@@ -57,7 +57,7 @@
 typedef struct _xmlSecMSCryptoKeyDataCtx xmlSecMSCryptoKeyDataCtx,
                                                 *xmlSecMSCryptoKeyDataCtxPtr;
 
-#ifdef XMLSEC_MSCRYPTO_NT4
+#ifdef XMLSEC_MSCRYPTO_CUSTOM_REFCOUNT
 /*-
  * A wrapper of HCRYPTKEY, a reference counter is introduced, the function is
  * the same as CryptDuplicateKey. Because the CryptDuplicateKey is not support
@@ -78,7 +78,7 @@ struct _mscrypt_prov {
         BOOL fCallerFreeProv ;
         volatile LONG refcnt ;
 } ;
-#endif /* XMLSEC_MSCRYPTO_NT4 */
+#endif /* XMLSEC_MSCRYPTO_CUSTOM_REFCOUNT */
 
 /*
  * Since MSCrypto does not provide direct handles to private keys, we support
@@ -88,21 +88,21 @@ struct _mscrypt_prov {
  * now is however directed to certificates.  Wouter
  */
 struct _xmlSecMSCryptoKeyDataCtx {
-#ifndef XMLSEC_MSCRYPTO_NT4
+#ifndef XMLSEC_MSCRYPTO_CUSTOM_REFCOUNT
     HCRYPTPROV                          hProv;
     BOOL                                fCallerFreeProv;
     HCRYPTKEY                           hKey;
-#else /* XMLSEC_MSCRYPTO_NT4 */
+#else /* XMLSEC_MSCRYPTO_CUSTOM_REFCOUNT */
     struct _mscrypt_prov*               p_prov ;
     struct _mscrypt_key*                p_key ;
-#endif /* XMLSEC_MSCRYPTO_NT4 */
+#endif /* XMLSEC_MSCRYPTO_CUSTOM_REFCOUNT */
     PCCERT_CONTEXT                      pCert;
     const xmlSecMSCryptoProviderInfo  * providers;
     DWORD                               dwKeySpec;
     xmlSecKeyDataType   type;
 };
 
-#ifndef XMLSEC_MSCRYPTO_NT4
+#ifndef XMLSEC_MSCRYPTO_CUSTOM_REFCOUNT
 
 /******************************** Provider *****************************************/
 #define xmlSecMSCryptoKeyDataCtxGetProvider(ctx)            (ctx)->hProv
@@ -204,7 +204,7 @@ xmlSecMSCryptoKeyDataCtxDuplicateKey(xmlSecMSCryptoKeyDataCtxPtr ctxDst, xmlSecM
     return(0);
 }
 
-#else /* XMLSEC_MSCRYPTO_NT4 */
+#else /* XMLSEC_MSCRYPTO_CUSTOM_REFCOUNT */
 
 /******************************** Provider *****************************************/
 #define xmlSecMSCryptoKeyDataCtxGetProvider(ctx)            (((ctx)->p_prov) ? ((ctx)->p_prov->hProv) : 0)
@@ -351,7 +351,7 @@ xmlSecMSCryptoKeyDataCtxDuplicateKey(xmlSecMSCryptoKeyDataCtxPtr ctxDst, xmlSecM
     return(0);
 }
 
-#endif /* XMLSEC_MSCRYPTO_NT4 */
+#endif /* XMLSEC_MSCRYPTO_CUSTOM_REFCOUNT */
 
 /******************************** Cert *****************************************/
 #define xmlSecMSCryptoKeyDataCtxGetCert(ctx)            ((ctx)->pCert)
@@ -1135,7 +1135,7 @@ xmlSecMSCryptoKeyDataRsaXmlWrite(xmlSecKeyDataId id, xmlSecKeyPtr key,
 
 static int
 xmlSecMSCryptoKeyDataRsaGenerate(xmlSecKeyDataPtr data, xmlSecSize sizeBits,
-                                xmlSecKeyDataType type ATTRIBUTE_UNUSED) {
+                                xmlSecKeyDataType type XMLSEC_ATTRIBUTE_UNUSED) {
     xmlSecMSCryptoKeyDataCtxPtr ctx;
     HCRYPTPROV hProv = 0;
     HCRYPTKEY hKey = 0;
@@ -1376,7 +1376,7 @@ done:
 
 static int
 xmlSecMSCryptoKeyDataRsaWrite(xmlSecKeyDataId id, xmlSecKeyDataPtr data,
-    xmlSecKeyValueRsaPtr rsaValue, int writePrivateKey ATTRIBUTE_UNUSED) {
+    xmlSecKeyValueRsaPtr rsaValue, int writePrivateKey XMLSEC_ATTRIBUTE_UNUSED) {
 
     xmlSecMSCryptoKeyDataCtxPtr ctx;
     xmlSecBuffer buf;
@@ -1725,7 +1725,7 @@ xmlSecMSCryptoKeyDataDsaXmlWrite(xmlSecKeyDataId id, xmlSecKeyPtr key,
 }
 
 static int
-xmlSecMSCryptoKeyDataDsaGenerate(xmlSecKeyDataPtr data, xmlSecSize sizeBits, xmlSecKeyDataType type ATTRIBUTE_UNUSED) {
+xmlSecMSCryptoKeyDataDsaGenerate(xmlSecKeyDataPtr data, xmlSecSize sizeBits, xmlSecKeyDataType type XMLSEC_ATTRIBUTE_UNUSED) {
     xmlSecMSCryptoKeyDataCtxPtr ctx;
     HCRYPTPROV hProv = 0;
     HCRYPTKEY hKey = 0;
@@ -2006,7 +2006,7 @@ done:
 static int
 xmlSecMSCryptoKeyDataDsaWrite(xmlSecKeyDataId id, xmlSecKeyDataPtr data,
                               xmlSecKeyValueDsaPtr dsaValue,
-                              int writePrivateKey ATTRIBUTE_UNUSED)  {
+                              int writePrivateKey XMLSEC_ATTRIBUTE_UNUSED)  {
     xmlSecMSCryptoKeyDataCtxPtr ctx;
     xmlSecBuffer buf;
     int bufInitialized = 0;
