@@ -19,7 +19,7 @@
 #include <string.h>
 
 #include <nss.h>
-#include <pk11func.h>
+#include <pk11pub.h>
 #include <keyhi.h>
 #include <hasht.h>
 
@@ -80,12 +80,12 @@ static int
 xmlSecNssKeyTransportCheckId(xmlSecTransformPtr transform) {
 
 #ifndef XMLSEC_NO_RSA
+#ifndef XMLSEC_NO_RSA_PKCS15
     if(xmlSecTransformCheckId(transform, xmlSecNssTransformRsaPkcs1Id)) {
         return(1);
     }
-#endif /* XMLSEC_NO_RSA */
+#endif /* XMLSEC_NO_RSA_PKCS15 */
 
-#ifndef XMLSEC_NO_RSA
 #ifndef XMLSEC_NO_RSA_OAEP
     if(xmlSecTransformCheckId(transform, xmlSecNssTransformRsaOaepId)) {
         return (1);
@@ -118,13 +118,14 @@ xmlSecNssKeyTransportInitialize(xmlSecTransformPtr transform) {
     memset(context, 0, sizeof(xmlSecNssKeyTransportCtx));
 
 #ifndef XMLSEC_NO_RSA
+
+#ifndef XMLSEC_NO_RSA_PKCS15
     if(transform->id == xmlSecNssTransformRsaPkcs1Id) {
         context->cipher = CKM_RSA_PKCS;
         context->keyId = xmlSecNssKeyDataRsaId;
     } else
-#endif /* XMLSEC_NO_RSA */
+#endif /* XMLSEC_NO_RSA_PKCS15 */
 
-#ifndef XMLSEC_NO_RSA
 #ifndef XMLSEC_NO_RSA_OAEP
     if(transform->id == xmlSecNssTransformRsaOaepId) {
         context->cipher = CKM_RSA_PKCS_OAEP;
@@ -134,6 +135,7 @@ xmlSecNssKeyTransportInitialize(xmlSecTransformPtr transform) {
         context->keyId = xmlSecNssKeyDataRsaId;
     } else
 #endif /* XMLSEC_NO_RSA_OAEP */
+
 #endif /* XMLSEC_NO_RSA */
 
     /* not found */
@@ -345,6 +347,8 @@ xmlSecNssKeyTransportCtxUpdate(xmlSecNssKeyTransportCtxPtr ctx, xmlSecBufferPtr
 #ifndef XMLSEC_NO_RSA_OAEP
 static int
 xmlSecNssKeyTransportSetOaepParams(xmlSecNssKeyTransportCtxPtr ctx, CK_RSA_PKCS_OAEP_PARAMS* oaepParams) {
+    xmlSecSize size;
+
     xmlSecAssert2(ctx != NULL, -1);
     xmlSecAssert2(oaepParams != NULL, -1);
 
@@ -352,7 +356,9 @@ xmlSecNssKeyTransportSetOaepParams(xmlSecNssKeyTransportCtxPtr ctx, CK_RSA_PKCS_
     oaepParams->mgf     = ctx->oaepMgf ;
     oaepParams->source  = CKZ_DATA_SPECIFIED;
     oaepParams->pSourceData      = xmlSecBufferGetData(&(ctx->oaepParams));
-    oaepParams->ulSourceDataLen  = xmlSecBufferGetSize(&(ctx->oaepParams));
+
+    size = xmlSecBufferGetSize(&(ctx->oaepParams));
+    XMLSEC_SAFE_CAST_SIZE_TO_ULONG(size, oaepParams->ulSourceDataLen, return(-1), NULL);
 
     return(0);
 }
@@ -405,7 +411,7 @@ static int
 xmlSecNssKeyTransportCtxFinal(xmlSecNssKeyTransportCtxPtr ctx, xmlSecBufferPtr in, xmlSecBufferPtr out,
                               int encrypt, xmlSecTransformCtxPtr transformCtx) {
     PK11SymKey* symKey = NULL;
-    SECItem oriskv;
+    SECItem oriskv = { siBuffer, NULL, 0 };
     xmlSecSize blockSize, materialSize, resultSize;
     unsigned int resultLen;
     xmlSecBufferPtr result;
@@ -471,7 +477,7 @@ xmlSecNssKeyTransportCtxFinal(xmlSecNssKeyTransportCtxPtr ctx, xmlSecBufferPtr i
     XMLSEC_SAFE_CAST_SIZE_TO_UINT(materialSize, oriskv.len, goto done, NULL);
 
     if(encrypt != 0) {
-        SECItem wrpskv;
+        SECItem wrpskv = { siBuffer, NULL, 0 };
 
         /* Create template symmetric key from material if needed */
         symKey = xmlSecNssKeyTransportLoadSymKeyUsingPublicKeySlot(ctx, &oriskv);
@@ -671,6 +677,7 @@ xmlSecNssKeyTransportExecute(xmlSecTransformPtr transform, int last, xmlSecTrans
 
 
 #ifndef XMLSEC_NO_RSA
+#ifndef XMLSEC_NO_RSA_PKCS15
 
 static xmlSecTransformKlass xmlSecNssRsaPkcs1Klass = {
     /* klass/object sizes */
@@ -710,9 +717,7 @@ xmlSecTransformId
 xmlSecNssTransformRsaPkcs1GetKlass(void) {
     return(&xmlSecNssRsaPkcs1Klass);
 }
-#endif /* XMLSEC_NO_RSA */
-
-#ifndef XMLSEC_NO_RSA
+#endif /* XMLSEC_NO_RSA_PKCS15 */
 
 #ifndef XMLSEC_NO_RSA_OAEP
 
@@ -800,7 +805,7 @@ xmlSecNssTransformRsaOaepEnc11GetKlass(void) {
 
 static int
 xmlSecNssRsaOaepNodeRead(xmlSecTransformPtr transform, xmlNodePtr node,
-                         xmlSecTransformCtxPtr transformCtx ATTRIBUTE_UNUSED) {
+                         xmlSecTransformCtxPtr transformCtx XMLSEC_ATTRIBUTE_UNUSED) {
     xmlSecNssKeyTransportCtxPtr ctx;
     xmlSecTransformRsaOaepParams oaepParams;
     int ret;
