@@ -50,7 +50,7 @@
 #include <openssl/x509v3.h>
 #include <openssl/asn1.h>
 
-#ifdef OPENSSL_IS_BORINGSSL
+#if defined(OPENSSL_IS_BORINGSSL) || defined(OPENSSL_IS_AWSLC)
 #include <openssl/mem.h>
 #endif /* OPENSSL_IS_BORINGSSL */
 
