@@ -34,7 +34,6 @@
 #include <xmlsec/openssl/crypto.h>
 #include <xmlsec/openssl/evp.h>
 #include <xmlsec/openssl/x509.h>
-#include "openssl_compat.h"
 
 #include <openssl/evp.h>
 #include <openssl/x509.h>
@@ -182,7 +181,7 @@ xmlSecOpenSSLX509StoreFindCert_ex(xmlSecKeyDataStorePtr store,
     xmlChar *subjectName,
     xmlChar *issuerName, xmlChar *issuerSerial,
     xmlSecByte * ski, xmlSecSize skiSize,
-    xmlSecKeyInfoCtx* keyInfoCtx ATTRIBUTE_UNUSED
+    xmlSecKeyInfoCtx* keyInfoCtx XMLSEC_ATTRIBUTE_UNUSED
 ) {
     xmlSecOpenSSLX509StoreCtxPtr ctx;
     xmlSecOpenSSLX509FindCertCtx findCertCtx;
@@ -1673,6 +1672,36 @@ xmlSecOpenSSLX509GetIssuerHash(X509* x) {
 
 static STACK_OF(X509)*
 xmlSecOpenSSLX509StoreCombineCerts(STACK_OF(X509)* certs1, STACK_OF(X509)* certs2) {
+#if defined(XMLSEC_OPENSSL_API_300)
+    STACK_OF(X509)* res = NULL;
+    int ret;
+
+    res = sk_X509_new_null();
+    if (res == NULL) {
+        xmlSecOpenSSLError("sk_X509_new_null()", NULL);
+        return(NULL);
+    }
+
+    /* certs 1 */
+    ret = X509_add_certs(res, certs1, X509_ADD_FLAG_UP_REF);
+    if (ret != 1) {
+        xmlSecOpenSSLError("X509_add_certs(certs1)", NULL);
+        return(NULL);
+    }
+
+
+    /* certs 2 */
+    ret = X509_add_certs(res, certs2, X509_ADD_FLAG_UP_REF);
+    if (ret != 1) {
+        xmlSecOpenSSLError("X509_add_certs(certs2)", NULL);
+        return(NULL);
+    }
+
+    /* done
+    */
+    return (res);
+
+#else /* defined(XMLSEC_OPENSSL_API_300) */
     STACK_OF(X509)* res = NULL;
 
     /* certs1 */
@@ -1722,6 +1751,7 @@ xmlSecOpenSSLX509StoreCombineCerts(STACK_OF(X509)* certs1, STACK_OF(X509)* certs
 
     /* done */
     return(res);
+#endif /* defined(XMLSEC_OPENSSL_API_300) */
 }
 
 
