@@ -5,7 +5,7 @@
  * This is free software; see Copyright file in the source
  * distribution for preciese wording.
  *
- * Copyright (C) 2002-2022 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
+ * Copyright (C) 2002-2024 Aleksey Sanin <aleksey@aleksey.com>. All Rights Reserved.
  */
 /**
  * SECTION:parser
@@ -32,6 +32,11 @@
 
 #include "cast_helpers.h"
 
+/* inputPush was renamed into xmlCtxtPushInput in libxml2 2.15.0 */
+#if (LIBXML_VERSION < 21500)
+#define xmlCtxtPushInput    inputPush
+#endif /* (LIBXML_VERSION < 21500) */
+
 /**************************************************************************
  *
  * Internal parser
@@ -189,8 +194,7 @@ xmlSecParserPushBin(xmlSecTransformPtr transform, const xmlSecByte* data,
     if(final != 0) {
         ret = xmlParseChunk(ctx->parserCtx, NULL, 0, 1);
         if((ret != 0) || (ctx->parserCtx->myDoc == NULL)) {
-            xmlSecXmlParserError("xmlParseChunk", ctx->parserCtx,
-                xmlSecTransformGetName(transform));
+            xmlSecXmlParserError("xmlParseChunk", ctx->parserCtx, xmlSecTransformGetName(transform));
             return(-1);
         }
 
@@ -257,23 +261,19 @@ xmlSecParserPopXml(xmlSecTransformPtr transform, xmlSecNodeSetPtr* nodes,
 
     /* prepare parser context */
     if(transform->prev == NULL) {
-        xmlSecInvalidTransfromError2(transform,
-                                     "prev transform=\"%s\"",
-                                     xmlSecErrorsSafeString(transform->prev));
+        xmlSecInvalidTransfromError2(transform, "prev transform=\"%s\"", xmlSecErrorsSafeString(transform->prev));
         return(-1);
     }
 
     buf = xmlSecTransformCreateInputBuffer(transform->prev, transformCtx);
     if(buf == NULL) {
-        xmlSecInternalError("xmlSecTransformCreateInputBuffer",
-                            xmlSecTransformGetName(transform));
+        xmlSecInternalError("xmlSecTransformCreateInputBuffer", xmlSecTransformGetName(transform));
         return(-1);
     }
 
     ctxt = xmlNewParserCtxt();
     if (ctxt == NULL) {
-        xmlSecXmlError("xmlNewParserCtxt",
-                       xmlSecTransformGetName(transform));
+        xmlSecXmlError("xmlNewParserCtxt", xmlSecTransformGetName(transform));
         xmlFreeParserInputBuffer(buf);
         return(-1);
     }
@@ -281,17 +281,15 @@ xmlSecParserPopXml(xmlSecTransformPtr transform, xmlSecNodeSetPtr* nodes,
 
     input = xmlNewIOInputStream(ctxt, buf, XML_CHAR_ENCODING_NONE);
     if(input == NULL) {
-        xmlSecXmlParserError("xmlNewParserCtxt", ctxt,
-                             xmlSecTransformGetName(transform));
+        xmlSecXmlParserError("xmlNewParserCtxt", ctxt, xmlSecTransformGetName(transform));
         xmlFreeParserCtxt(ctxt);
         xmlFreeParserInputBuffer(buf);
         return(-1);
     }
 
-    ret = inputPush(ctxt, input);
+    ret = xmlCtxtPushInput(ctxt, input);
     if(ret < 0) {
-        xmlSecXmlParserError("inputPush", ctxt,
-                             xmlSecTransformGetName(transform));
+        xmlSecXmlParserError("xmlCtxtPushInput", ctxt, xmlSecTransformGetName(transform));
         xmlFreeInputStream(input);
         if(ctxt->myDoc != NULL) {
             xmlFreeDoc(ctxt->myDoc);
@@ -304,8 +302,7 @@ xmlSecParserPopXml(xmlSecTransformPtr transform, xmlSecNodeSetPtr* nodes,
     /* finaly do the parsing */
     ret = xmlParseDocument(ctxt);
     if(ret < 0) {
-        xmlSecXmlParserError("xmlParseDocument", ctxt,
-                             xmlSecTransformGetName(transform));
+        xmlSecXmlParserError("xmlParseDocument", ctxt, xmlSecTransformGetName(transform));
         if(ctxt->myDoc != NULL) {
             xmlFreeDoc(ctxt->myDoc);
             ctxt->myDoc = NULL;
@@ -322,8 +319,7 @@ xmlSecParserPopXml(xmlSecTransformPtr transform, xmlSecNodeSetPtr* nodes,
     /* return result to the caller */
     (*nodes) = xmlSecNodeSetCreate(doc, NULL, xmlSecNodeSetTree);
     if((*nodes) == NULL) {
-        xmlSecInternalError("xmlSecNodeSetCreate",
-                            xmlSecTransformGetName(transform));
+        xmlSecInternalError("xmlSecNodeSetCreate", xmlSecTransformGetName(transform));
         xmlFreeDoc(doc);
         return(-1);
     }
@@ -366,8 +362,7 @@ xmlSecParseFile(const char *filename) {
     xmlInitParser();
     ctxt = xmlCreateFileParserCtxt(filename);
     if (ctxt == NULL) {
-        xmlSecXmlError2("xmlCreateFileParserCtxt", NULL,
-                        "filename=%s", xmlSecErrorsSafeString(filename));
+        xmlSecXmlError2("xmlCreateFileParserCtxt", NULL, "filename=%s", xmlSecErrorsSafeString(filename));
         goto done;
     }
     xmlSecParsePrepareCtxt(ctxt);
@@ -376,17 +371,14 @@ xmlSecParseFile(const char *filename) {
     if (ctxt->directory == NULL) {
         ctxt->directory = xmlParserGetDirectory(filename);
         if(ctxt->directory == NULL) {
-            xmlSecXmlError2("xmlParserGetDirectory", NULL,
-                "filename=%s", xmlSecErrorsSafeString(filename));
+            xmlSecXmlError2("xmlParserGetDirectory", NULL, "filename=%s", xmlSecErrorsSafeString(filename));
             goto done;
         }
     }
 
     ret = xmlParseDocument(ctxt);
     if(ret < 0) {
-        xmlSecXmlParserError2("xmlParseDocument", ctxt, NULL,
-            "filename=%s",
-            xmlSecErrorsSafeString(filename));
+        xmlSecXmlParserError2("xmlParseDocument", ctxt, NULL, "filename=%s", xmlSecErrorsSafeString(filename));
         goto done;
     }
 
@@ -448,8 +440,7 @@ xmlSecParseMemoryExt(const xmlSecByte *prefix, xmlSecSize prefixSize,
         XMLSEC_SAFE_CAST_SIZE_TO_INT(prefixSize, prefixLen, goto done, NULL);
         ret = xmlParseChunk(ctxt, (const char*)prefix, prefixLen, 0);
         if(ret != 0) {
-            xmlSecXmlParserError2("xmlParseChunk", ctxt, NULL,
-                "chunkSize=%d", prefixLen);
+            xmlSecXmlParserError2("xmlParseChunk", ctxt, NULL, "chunkSize=%d", prefixLen);
             goto done;
         }
     }
@@ -461,8 +452,7 @@ xmlSecParseMemoryExt(const xmlSecByte *prefix, xmlSecSize prefixSize,
         XMLSEC_SAFE_CAST_SIZE_TO_INT(bufferSize, bufferLen, goto done, NULL);
         ret = xmlParseChunk(ctxt, (const char*)buffer, bufferLen, 0);
         if(ret != 0) {
-            xmlSecXmlParserError2("xmlParseChunk", ctxt, NULL,
-                "chunkSize=%d", bufferLen);
+            xmlSecXmlParserError2("xmlParseChunk", ctxt, NULL, "chunkSize=%d", bufferLen);
             goto done;
         }
     }
@@ -474,8 +464,7 @@ xmlSecParseMemoryExt(const xmlSecByte *prefix, xmlSecSize prefixSize,
         XMLSEC_SAFE_CAST_SIZE_TO_INT(postfixSize, postfixLen, goto done, NULL);
         ret = xmlParseChunk(ctxt, (const char*)postfix, postfixLen, 0);
         if(ret != 0) {
-            xmlSecXmlParserError2("xmlParseChunk", ctxt, NULL,
-                "chunkSize=%d", postfixLen);
+            xmlSecXmlParserError2("xmlParseChunk", ctxt, NULL, "chunkSize=%d", postfixLen);
             goto done;
         }
     }
@@ -567,20 +556,39 @@ void
 xmlSecParsePrepareCtxt(xmlParserCtxtPtr ctxt) {
     xmlSecAssert(ctxt != NULL);
 
+#if LIBXML_VERSION < 21300
     /* required for c14n! */
     ctxt->loadsubset = XML_DETECT_IDS | XML_COMPLETE_ATTRS;
     ctxt->replaceEntities = 1;
+#else  /* LIBXML_VERSION < 21300 */
+    xmlCtxtSetOptions(ctxt, xmlSecParserGetDefaultOptions());
+#endif /* LIBXML_VERSION < 21300 */
 
     xmlCtxtUseOptions(ctxt, xmlSecParserGetDefaultOptions());
 }
 
-/*
- * XML_PARSE_NONET  to support c14n
- * XML_PARSE_NODICT to avoid problems with moving nodes around
- * XML_PARSE_HUGE   to enable parsing of XML documents with large text nodes
- */
-static int g_xmlsec_parser_default_options = XML_PARSE_NONET | XML_PARSE_NODICT | XML_PARSE_HUGE;
 
+/*
+ * To block network access and loading of external entities:
+ * - XML_PARSE_NO_XXE: disable loading of external content (available >= 2.13.0),
+ *   it disables XML_PARSE_DTDLOAD | XML_PARSE_DTDATTR ut we keep those in defaults
+ *   to make it work if XML_PARSE_NO_XXE is disabled (e.g. with --xxe option)
+ * - XML_PARSE_NONET: forbid network access
+ *
+ * To support c14n:
+ * - XML_PARSE_NOENT: substitute entities
+ * - XML_PARSE_DTDLOAD: load the external subset (disabled with XML_PARSE_NO_XXE)
+ * - XML_PARSE_DTDATTR: default DTD attributes (disabled with XML_PARSE_NO_XXE)
+ *
+ * Misc:
+ * XML_PARSE_NODICT: do not reuse the context dictionary (to avoid problems with moving nodes around)
+ * XML_PARSE_HUGE: relax any hardcoded limit from the parser (to enable parsing of XML documents with large text nodes)
+ */
+#if LIBXML_VERSION < 21300
+static int g_xmlsec_parser_default_options = XML_PARSE_NONET | XML_PARSE_NOENT | XML_PARSE_DTDLOAD | XML_PARSE_DTDATTR | XML_PARSE_NODICT | XML_PARSE_HUGE;
+#else  /* LIBXML_VERSION < 21300 */
+static int g_xmlsec_parser_default_options = XML_PARSE_NO_XXE | XML_PARSE_NONET | XML_PARSE_NOENT | XML_PARSE_DTDLOAD | XML_PARSE_DTDATTR | XML_PARSE_NODICT | XML_PARSE_HUGE;
+#endif /* LIBXML_VERSION < 21300 */
 /**
  * xmlSecParserGetDefaultOptions:
  *
