@@ -6,28 +6,43 @@
  */
 
 #include "include/core/SkDataTable.h"
+#include "include/core/SkFontArguments.h"
 #include "include/core/SkFontMgr.h"
 #include "include/core/SkFontStyle.h"
-#include "include/core/SkMath.h"
+#include "include/core/SkMatrix.h"
 #include "include/core/SkRefCnt.h"
+#include "include/core/SkScalar.h"
 #include "include/core/SkStream.h"
 #include "include/core/SkString.h"
 #include "include/core/SkTypeface.h"
 #include "include/core/SkTypes.h"
-#include "include/private/SkFixed.h"
-#include "include/private/SkMutex.h"
-#include "include/private/SkTDArray.h"
-#include "include/private/SkTemplates.h"
+#include "include/ports/SkFontMgr_fontconfig.h"
+#include "include/ports/SkFontScanner_FreeType.h"
+#include "include/private/base/SkDebug.h"
+#include "include/private/base/SkMutex.h"
+#include "include/private/base/SkTArray.h"
+#include "include/private/base/SkTDArray.h"
+#include "include/private/base/SkTemplates.h"
+#include "include/private/base/SkThreadAnnotations.h"
+#include "src/base/SkTSort.h"
 #include "src/core/SkAdvancedTypefaceMetrics.h"
 #include "src/core/SkFontDescriptor.h"
 #include "src/core/SkOSFile.h"
+#include "src/core/SkScalerContext.h"
 #include "src/core/SkTypefaceCache.h"
-#include "src/ports/SkFontHost_FreeType_common.h"
+#include "src/ports/SkTypeface_proxy.h"
 
 #include <fontconfig/fontconfig.h>
+
 #include <string.h>
+#include <array>
+#include <cstddef>
+#include <cstdint>
+#include <memory>
+#include <utility>
 
 class SkData;
+using namespace skia_private;
 
 // FC_POSTSCRIPT_NAME was added with b561ff20 which ended up in 2.10.92
 // Ubuntu 14.04 is on 2.11.0
@@ -220,7 +235,7 @@ static SkWeakReturn is_weak(FcPattern* pattern, const char object[], int id) {
     // However, there appears to be no way to match/sort without it.
     SkAutoFcConfig config;
     FcFontSet* fontSets[1] = { fontSet };
-    SkAutoFcPattern match(FcFontSetMatch(config, fontSets, SK_ARRAY_COUNT(fontSets),
+    SkAutoFcPattern match(FcFontSetMatch(config, fontSets, std::size(fontSets),
                                          minimal, &result));
 
     FcLangSet* matchLangSet;
@@ -321,7 +336,7 @@ static SkFontStyle skfontstyle_from_fcpattern(FcPattern* pattern) {
         { FC_WEIGHT_EXTRABLACK, SkFS::kExtraBlack_Weight },
     };
     SkScalar weight = map_ranges(get_int(pattern, FC_WEIGHT, FC_WEIGHT_REGULAR),
-                                 weightRanges, SK_ARRAY_COUNT(weightRanges));
+                                 weightRanges, std::size(weightRanges));
 
     static constexpr MapRanges widthRanges[] = {
         { FC_WIDTH_ULTRACONDENSED, SkFS::kUltraCondensed_Width },
@@ -335,7 +350,7 @@ static SkFontStyle skfontstyle_from_fcpattern(FcPattern* pattern) {
         { FC_WIDTH_ULTRAEXPANDED,  SkFS::kUltraExpanded_Width },
     };
     SkScalar width = map_ranges(get_int(pattern, FC_WIDTH, FC_WIDTH_NORMAL),
-                                widthRanges, SK_ARRAY_COUNT(widthRanges));
+                                widthRanges, std::size(widthRanges));
 
     SkFS::Slant slant = SkFS::kUpright_Slant;
     switch (get_int(pattern, FC_SLANT, FC_SLANT_ROMAN)) {
@@ -368,7 +383,7 @@ static void fcpattern_from_skfontstyle(SkFontStyle style, FcPattern* pattern) {
         { SkFS::kBlack_Weight,      FC_WEIGHT_BLACK },
         { SkFS::kExtraBlack_Weight, FC_WEIGHT_EXTRABLACK },
     };
-    int weight = map_ranges(style.weight(), weightRanges, SK_ARRAY_COUNT(weightRanges));
+    int weight = map_ranges(style.weight(), weightRanges, std::size(weightRanges));
 
     static constexpr MapRanges widthRanges[] = {
         { SkFS::kUltraCondensed_Width, FC_WIDTH_ULTRACONDENSED },
@@ -381,7 +396,7 @@ static void fcpattern_from_skfontstyle(SkFontStyle style, FcPattern* pattern) {
         { SkFS::kExtraExpanded_Width,  FC_WIDTH_EXTRAEXPANDED },
         { SkFS::kUltraExpanded_Width,  FC_WIDTH_ULTRAEXPANDED },
     };
-    int width = map_ranges(style.width(), widthRanges, SK_ARRAY_COUNT(widthRanges));
+    int width = map_ranges(style.width(), widthRanges, std::size(widthRanges));
 
     int slant = FC_SLANT_ROMAN;
     switch (style.slant()) {
@@ -396,86 +411,58 @@ static void fcpattern_from_skfontstyle(SkFontStyle style, FcPattern* pattern) {
     FcPatternAddInteger(pattern, FC_SLANT , slant);
 }
 
-class SkTypeface_stream : public SkTypeface_FreeType {
+class SkTypeface_fontconfig : public SkTypeface_proxy {
 public:
-    SkTypeface_stream(std::unique_ptr<SkFontData> data,
-                      SkString familyName, const SkFontStyle& style, bool fixedWidth)
-        : INHERITED(style, fixedWidth)
-        , fFamilyName(std::move(familyName))
-        , fData(std::move(data))
-    { }
-
-    void onGetFamilyName(SkString* familyName) const override {
-        *familyName = fFamilyName;
-    }
-
-    void onGetFontDescriptor(SkFontDescriptor* desc, bool* serialize) const override {
-        *serialize = true;
-    }
-
-    std::unique_ptr<SkStreamAsset> onOpenStream(int* ttcIndex) const override {
-        *ttcIndex = fData->getIndex();
-        return fData->getStream()->duplicate();
-    }
-
-    std::unique_ptr<SkFontData> onMakeFontData() const override {
-        return std::make_unique<SkFontData>(*fData);
-    }
-
-    sk_sp<SkTypeface> onMakeClone(const SkFontArguments& args) const override {
-        std::unique_ptr<SkFontData> data = this->cloneFontData(args);
-        if (!data) {
-            return nullptr;
-        }
-        return sk_make_sp<SkTypeface_stream>(std::move(data),
-                                             fFamilyName,
-                                             this->fontStyle(),
-                                             this->isFixedPitch());
-    }
-
-private:
-    SkString fFamilyName;
-    const std::unique_ptr<const SkFontData> fData;
-
-    using INHERITED = SkTypeface_FreeType;
-};
-
-class SkTypeface_fontconfig : public SkTypeface_FreeType {
-public:
-    static sk_sp<SkTypeface_fontconfig> Make(SkAutoFcPattern pattern, SkString sysroot) {
-        return sk_sp<SkTypeface_fontconfig>(new SkTypeface_fontconfig(std::move(pattern),
-                                                                      std::move(sysroot)));
-    }
-    mutable SkAutoFcPattern fPattern;  // Mutable for passing to FontConfig API.
-    const SkString fSysroot;
-
-    void onGetFamilyName(SkString* familyName) const override {
-        *familyName = get_string(fPattern, FC_FAMILY);
-    }
-
-    void onGetFontDescriptor(SkFontDescriptor* desc, bool* serialize) const override {
-        FCLocker lock;
-        desc->setFamilyName(get_string(fPattern, FC_FAMILY));
-        desc->setFullName(get_string(fPattern, FC_FULLNAME));
-        desc->setPostscriptName(get_string(fPattern, FC_POSTSCRIPT_NAME));
-        desc->setStyle(this->fontStyle());
-        *serialize = false;
-    }
-
-    std::unique_ptr<SkStreamAsset> onOpenStream(int* ttcIndex) const override {
-        FCLocker lock;
-        *ttcIndex = get_int(fPattern, FC_INDEX, 0);
-        const char* filename = get_string(fPattern, FC_FILE);
-        // See FontAccessible for note on searching sysroot then non-sysroot path.
+    static sk_sp<SkTypeface> Make(SkAutoFcPattern pattern,
+                                  SkString sysroot,
+                                  SkFontScanner* scanner) {
         SkString resolvedFilename;
-        if (!fSysroot.isEmpty()) {
-            resolvedFilename = fSysroot;
+        FCLocker lock;
+        const char* filename = get_string(pattern, FC_FILE);
+        // See FontAccessible for note on searching sysroot then non-sysroot path.
+        if (!sysroot.isEmpty()) {
+            resolvedFilename = sysroot;
             resolvedFilename += filename;
             if (sk_exists(resolvedFilename.c_str(), kRead_SkFILE_Flag)) {
                 filename = resolvedFilename.c_str();
             }
         }
-        return SkStream::MakeFromFile(filename);
+        // TODO: FC_VARIABLE and FC_FONT_VARIATIONS in arguments
+        int ttcIndex = get_int(pattern, FC_INDEX, 0);
+        auto realTypeface = scanner->MakeFromStream(SkStream::MakeFromFile(filename),
+                                                    SkFontArguments().setCollectionIndex(ttcIndex));
+        if (!realTypeface) {
+            // Unref the pattern while holding the lock.
+            pattern.reset();
+            return nullptr;
+        }
+
+        SkFontStyle proxyStyle = skfontstyle_from_fcpattern(pattern);
+        return sk_sp(new SkTypeface_fontconfig(std::move(realTypeface), std::move(pattern),
+                                               proxyStyle, nullptr));
+    }
+
+    mutable SkAutoFcPattern fPattern;  // Mutable for passing to FontConfig API.
+    SkFontStyle fOriginalRealStyle;
+
+protected:
+    void onGetFamilyName(SkString* familyName) const override {
+        *familyName = get_string(fPattern, FC_FAMILY);
+    }
+
+    SkFontStyle onGetFontStyle() const override {
+        return SkTypeface::onGetFontStyle();
+    }
+
+    void onGetFontDescriptor(SkFontDescriptor* desc, bool* serialize) const override {
+        // TODO: need to serialize FC_MATRIX and FC_EMBOLDEN
+        FCLocker lock;
+        SkTypeface_proxy::onGetFontDescriptor(desc, serialize);
+        desc->setFamilyName(get_string(fPattern, FC_FAMILY));
+        desc->setFullName(get_string(fPattern, FC_FULLNAME));
+        desc->setPostscriptName(get_string(fPattern, FC_POSTSCRIPT_NAME));
+        desc->setStyle(this->fontStyle());
+        *serialize = false;
     }
 
     void onFilterRec(SkScalerContextRec* rec) const override {
@@ -505,13 +492,11 @@ public:
         if (get_bool(fPattern, FC_EMBOLDEN)) {
             rec->fFlags |= SkScalerContext::kEmbolden_Flag;
         }
-        this->INHERITED::onFilterRec(rec);
+
+        SkTypeface_proxy::onFilterRec(rec);
     }
-
     std::unique_ptr<SkAdvancedTypefaceMetrics> onGetAdvancedMetrics() const override {
-        std::unique_ptr<SkAdvancedTypefaceMetrics> info =
-            this->INHERITED::onGetAdvancedMetrics();
-
+        std::unique_ptr<SkAdvancedTypefaceMetrics> info = SkTypeface_proxy::onGetAdvancedMetrics();
         // Simulated fonts shouldn't be considered to be of the type of their data.
         if (get_matrix(fPattern, FC_MATRIX) || get_bool(fPattern, FC_EMBOLDEN)) {
             info->fType = SkAdvancedTypefaceMetrics::kOther_Font;
@@ -519,58 +504,60 @@ public:
         return info;
     }
 
-    sk_sp<SkTypeface> onMakeClone(const SkFontArguments& args) const override {
-        std::unique_ptr<SkFontData> data = this->cloneFontData(args);
-        if (!data) {
-            return nullptr;
-        }
-
-        SkString familyName;
-        this->getFamilyName(&familyName);
-
-        return sk_make_sp<SkTypeface_stream>(std::move(data),
-                                             familyName,
-                                             this->fontStyle(),
-                                             this->isFixedPitch());
-    }
-
-    std::unique_ptr<SkFontData> onMakeFontData() const override {
-        int index;
-        std::unique_ptr<SkStreamAsset> stream(this->onOpenStream(&index));
-        if (!stream) {
-            return nullptr;
-        }
-        // TODO: FC_VARIABLE and FC_FONT_VARIATIONS
-        return std::make_unique<SkFontData>(std::move(stream), index, nullptr, 0);
-    }
-
     ~SkTypeface_fontconfig() override {
         // Hold the lock while unrefing the pattern.
         FCLocker lock;
         fPattern.reset();
     }
 
-private:
-    SkTypeface_fontconfig(SkAutoFcPattern pattern, SkString sysroot)
-        : INHERITED(skfontstyle_from_fcpattern(pattern),
-                    FC_PROPORTIONAL != get_int(pattern, FC_SPACING, FC_PROPORTIONAL))
-        , fPattern(std::move(pattern))
-        , fSysroot(std::move(sysroot))
-    { }
+    sk_sp<SkTypeface> onMakeClone(const SkFontArguments& args) const override {
+        sk_sp<SkTypeface> realTypeface = SkTypeface_proxy::onMakeClone(args);
+        if (!realTypeface) {
+            return nullptr;
+        }
+        SkAutoFcPattern pattern;
+        {
+            FCLocker lock;
+            FcPatternReference(fPattern);
+            pattern.reset(fPattern.get());
+        }
 
-    using INHERITED = SkTypeface_FreeType;
+        SkFontStyle newRealStyle = realTypeface->fontStyle();
+        SkFontStyle originalProxyStyle = skfontstyle_from_fcpattern(pattern);
+        SkFontStyle newProxyStyle(
+            // keep consistent weight and width offsets (though they will be clamped)
+            originalProxyStyle.weight() + (newRealStyle.weight() - fOriginalRealStyle.weight()),
+            originalProxyStyle.width() + (newRealStyle.width() - fOriginalRealStyle.width()),
+            // the originalProxyStyle's slant is an override for the originalRealStyle's slant
+            newRealStyle.slant() == fOriginalRealStyle.slant() ? originalProxyStyle.slant()
+                                                               : newRealStyle.slant());
+
+        return sk_sp(new SkTypeface_fontconfig(std::move(realTypeface), std::move(pattern),
+                                               newProxyStyle, &fOriginalRealStyle));
+    }
+
+private:
+    SkTypeface_fontconfig(sk_sp<SkTypeface> realTypeface, SkAutoFcPattern pattern,
+                          const SkFontStyle& proxyStyle, const SkFontStyle* originalRealStyle)
+        : SkTypeface_proxy(realTypeface, proxyStyle,
+                           FC_PROPORTIONAL != get_int(pattern, FC_SPACING, FC_PROPORTIONAL))
+        , fPattern(std::move(pattern))
+        , fOriginalRealStyle(originalRealStyle ? *originalRealStyle
+                                               : SkTypeface_proxy::onGetFontStyle())
+    {}
 };
 
 class SkFontMgr_fontconfig : public SkFontMgr {
     mutable SkAutoFcConfig fFC;  // Only mutable to avoid const cast when passed to FontConfig API.
     const SkString fSysroot;
     const sk_sp<SkDataTable> fFamilyNames;
-    const SkTypeface_FreeType::Scanner fScanner;
+    std::unique_ptr<SkFontScanner> fScanner;
 
     class StyleSet : public SkFontStyleSet {
     public:
         StyleSet(sk_sp<SkFontMgr_fontconfig> parent, SkAutoFcFontSet fontSet)
-            : fFontMgr(std::move(parent)), fFontSet(std::move(fontSet))
+            : fFontMgr(std::move(parent))
+            , fFontSet(std::move(fontSet))
         { }
 
         ~StyleSet() override {
@@ -595,7 +582,7 @@ class SkFontMgr_fontconfig : public SkFontMgr {
             }
         }
 
-        SkTypeface* createTypeface(int index) override {
+        sk_sp<SkTypeface> createTypeface(int index) override {
             if (index < 0 || fFontSet->nfont <= index) {
                 return nullptr;
             }
@@ -604,10 +591,10 @@ class SkFontMgr_fontconfig : public SkFontMgr {
                 FcPatternReference(fFontSet->fonts[index]);
                 return fFontSet->fonts[index];
             }());
-            return fFontMgr->createTypefaceFromFcPattern(std::move(match)).release();
+            return fFontMgr->createTypefaceFromFcPattern(std::move(match));
         }
 
-        SkTypeface* matchStyle(const SkFontStyle& style) override {
+        sk_sp<SkTypeface> matchStyle(const SkFontStyle& style) override {
             SkAutoFcPattern match([this, &style]() {
                 FCLocker lock;
 
@@ -619,11 +606,11 @@ class SkFontMgr_fontconfig : public SkFontMgr {
                 FcResult result;
                 FcFontSet* fontSets[1] = { fFontSet };
                 return FcFontSetMatch(fFontMgr->fFC,
-                                      fontSets, SK_ARRAY_COUNT(fontSets),
+                                      fontSets, std::size(fontSets),
                                       pattern, &result);
 
             }());
-            return fFontMgr->createTypefaceFromFcPattern(std::move(match)).release();
+            return fFontMgr->createTypefaceFromFcPattern(std::move(match));
         }
 
     private:
@@ -632,7 +619,7 @@ class SkFontMgr_fontconfig : public SkFontMgr {
     };
 
     static bool FindName(const SkTDArray<const char*>& list, const char* str) {
-        int count = list.count();
+        int count = list.size();
         for (int i = 0; i < count; ++i) {
             if (!strcmp(list[i], str)) {
                 return true;
@@ -648,7 +635,7 @@ class SkFontMgr_fontconfig : public SkFontMgr {
         SkTDArray<size_t> sizes;
 
         static const FcSetName fcNameSet[] = { FcSetSystem, FcSetApplication };
-        for (int setIndex = 0; setIndex < (int)SK_ARRAY_COUNT(fcNameSet); ++setIndex) {
+        for (int setIndex = 0; setIndex < (int)std::size(fcNameSet); ++setIndex) {
             // Return value of FcConfigGetFonts must not be destroyed.
             FcFontSet* allFonts(FcConfigGetFonts(fcconfig, fcNameSet[setIndex]));
             if (nullptr == allFonts) {
@@ -676,7 +663,7 @@ class SkFontMgr_fontconfig : public SkFontMgr {
         }
 
         return SkDataTable::MakeCopyArrays((void const *const *)names.begin(),
-                                           sizes.begin(), names.count());
+                                           sizes.begin(), names.size());
     }
 
     static bool FindByFcPattern(SkTypeface* cached, void* ctx) {
@@ -706,9 +693,10 @@ class SkFontMgr_fontconfig : public SkFontMgr {
             return face;
         }();
         if (!face) {
-            face = SkTypeface_fontconfig::Make(std::move(pattern), fSysroot);
+            // Cannot hold FCLocker around Make; may need to destory pattern.
+            face = SkTypeface_fontconfig::Make(std::move(pattern), fSysroot, fScanner.get());
             if (face) {
-                // Cannot hold FCLocker in fTFCache.add; evicted typefaces may need to lock.
+                // Cannot hold FCLocker around fTFCache.add; evicted typefaces may need to lock.
                 fTFCache.add(face);
             }
         }
@@ -717,10 +705,11 @@ class SkFontMgr_fontconfig : public SkFontMgr {
 
 public:
     /** Takes control of the reference to 'config'. */
-    explicit SkFontMgr_fontconfig(FcConfig* config)
+    SkFontMgr_fontconfig(FcConfig* config, std::unique_ptr<SkFontScanner> scanner)
         : fFC(config ? config : FcInitLoadConfigAndFonts())
         , fSysroot(reinterpret_cast<const char*>(FcConfigGetSysRoot(fFC)))
-        , fFamilyNames(GetFamilyNames(fFC)) { }
+        , fFamilyNames(GetFamilyNames(fFC))
+        , fScanner(std::move(scanner)) { }
 
     ~SkFontMgr_fontconfig() override {
         // Hold the lock while unrefing the config.
@@ -737,40 +726,51 @@ protected:
         familyName->set(fFamilyNames->atStr(index));
     }
 
-    SkFontStyleSet* onCreateStyleSet(int index) const override {
+    sk_sp<SkFontStyleSet> onCreateStyleSet(int index) const override {
         return this->onMatchFamily(fFamilyNames->atStr(index));
     }
 
     /** True if any string object value in the font is the same
      *         as a string object value in the pattern.
      */
-    static bool AnyMatching(FcPattern* font, FcPattern* pattern, const char* object) {
-        FcChar8* fontString;
-        FcChar8* patternString;
-        FcResult result;
-        // Set an arbitrary limit on the number of pattern object values to consider.
-        // TODO: re-write this to avoid N*M
-        static const int maxId = 16;
-        for (int patternId = 0; patternId < maxId; ++patternId) {
-            result = FcPatternGetString(pattern, object, patternId, &patternString);
-            if (FcResultNoId == result) {
-                break;
-            }
-            if (FcResultMatch != result) {
-                continue;
-            }
-            for (int fontId = 0; fontId < maxId; ++fontId) {
-                result = FcPatternGetString(font, object, fontId, &fontString);
-                if (FcResultNoId == result) {
+    static bool AnyStringMatching(FcPattern* font, FcPattern* pattern, const char* object) {
+        auto getStrings = [](FcPattern* p, const char* object, STArray<32, FcChar8*>& strings) {
+            // Set an arbitrary (but high) limit on the number of pattern object values to consider.
+            static constexpr const int maxId = 65536;
+            for (int patternId = 0; patternId < maxId; ++patternId) {
+                FcChar8* patternString;
+                FcResult result = FcPatternGetString(p, object, patternId, &patternString);
+                if (result == FcResultNoId) {
                     break;
                 }
-                if (FcResultMatch != result) {
-                    continue;
-                }
-                if (0 == FcStrCmpIgnoreCase(patternString, fontString)) {
-                    return true;
+                if (result == FcResultMatch) {
+                    strings.push_back(patternString);
                 }
             }
+        };
+        auto compareStrings = [](FcChar8* a, FcChar8* b) -> bool {
+            return FcStrCmpIgnoreCase(a, b) < 0;
+        };
+
+        STArray<32, FcChar8*> fontStrings;
+        STArray<32, FcChar8*> patternStrings;
+        getStrings(font, object, fontStrings);
+        getStrings(pattern, object, patternStrings);
+
+        SkTQSort(fontStrings.begin(), fontStrings.end(), compareStrings);
+        SkTQSort(patternStrings.begin(), patternStrings.end(), compareStrings);
+
+        FcChar8** fontString = fontStrings.begin();
+        FcChar8** patternString = patternStrings.begin();
+        while (fontString != fontStrings.end() && patternString != patternStrings.end()) {
+            int cmp = FcStrCmpIgnoreCase(*fontString, *patternString);
+            if (cmp < 0) {
+                ++fontString;
+            } else if (cmp > 0) {
+                ++patternString;
+            } else {
+                return true;
+            }
         }
         return false;
     }
@@ -796,14 +796,19 @@ protected:
             resolvedFilename = fSysroot;
             resolvedFilename += filename;
             if (sk_exists(resolvedFilename.c_str(), kRead_SkFILE_Flag)) {
-                return true;
+                auto&& file(SkData::MakeFromFileName(resolvedFilename.c_str()));
+                return file && fScanner->scanFile(SkMemoryStream::Make(file).get(), nullptr);
             }
         }
-        return sk_exists(filename, kRead_SkFILE_Flag);
+        if (sk_exists(filename, kRead_SkFILE_Flag)) {
+            auto&& file(SkData::MakeFromFileName(filename));
+            return file && fScanner->scanFile(SkMemoryStream::Make(file).get(), nullptr);
+        }
+        return false;
     }
 
     static bool FontFamilyNameMatches(FcPattern* font, FcPattern* pattern) {
-        return AnyMatching(font, pattern, FC_FAMILY);
+        return AnyStringMatching(font, pattern, FC_FAMILY);
     }
 
     static bool FontContainsCharacter(FcPattern* font, uint32_t character) {
@@ -824,14 +829,14 @@ protected:
         return false;
     }
 
-    SkFontStyleSet* onMatchFamily(const char familyName[]) const override {
+    sk_sp<SkFontStyleSet> onMatchFamily(const char familyName[]) const override {
         if (!familyName) {
             return nullptr;
         }
         FCLocker lock;
 
         SkAutoFcPattern pattern;
-        FcPatternAddString(pattern, FC_FAMILY, (FcChar8*)familyName);
+        FcPatternAddString(pattern, FC_FAMILY, (const FcChar8*)familyName);
         FcConfigSubstitute(fFC, pattern, FcMatchPattern);
         FcDefaultSubstitute(pattern);
 
@@ -850,7 +855,7 @@ protected:
         // The patterns are exactly the same except for the FC_FILE.
         // It should be possible to collapse these patterns by normalizing.
         static const FcSetName fcNameSet[] = { FcSetSystem, FcSetApplication };
-        for (int setIndex = 0; setIndex < (int)SK_ARRAY_COUNT(fcNameSet); ++setIndex) {
+        for (int setIndex = 0; setIndex < (int)std::size(fcNameSet); ++setIndex) {
             // Return value of FcConfigGetFonts must not be destroyed.
             FcFontSet* allFonts(FcConfigGetFonts(fFC, fcNameSet[setIndex]));
             if (nullptr == allFonts) {
@@ -859,23 +864,23 @@ protected:
 
             for (int fontIndex = 0; fontIndex < allFonts->nfont; ++fontIndex) {
                 FcPattern* font = allFonts->fonts[fontIndex];
-                if (FontAccessible(font) && FontFamilyNameMatches(font, matchPattern)) {
+                if (FontFamilyNameMatches(font, matchPattern) && FontAccessible(font)) {
                     FcFontSetAdd(matches, FcFontRenderPrepare(fFC, pattern, font));
                 }
             }
         }
 
-        return new StyleSet(sk_ref_sp(this), std::move(matches));
+        return sk_sp<SkFontStyleSet>(new StyleSet(sk_ref_sp(this), std::move(matches)));
     }
 
-    SkTypeface* onMatchFamilyStyle(const char familyName[],
-                                   const SkFontStyle& style) const override
+    sk_sp<SkTypeface> onMatchFamilyStyle(const char familyName[],
+                                         const SkFontStyle& style) const override
     {
         SkAutoFcPattern font([this, &familyName, &style]() {
             FCLocker lock;
 
             SkAutoFcPattern pattern;
-            FcPatternAddString(pattern, FC_FAMILY, (FcChar8*)familyName);
+            FcPatternAddString(pattern, FC_FAMILY, (const FcChar8*)familyName);
             fcpattern_from_skfontstyle(style, pattern);
             FcConfigSubstitute(fFC, pattern, FcMatchPattern);
             FcDefaultSubstitute(pattern);
@@ -900,19 +905,19 @@ protected:
 
             FcResult result;
             SkAutoFcPattern font(FcFontMatch(fFC, pattern, &result));
-            if (!font || !FontAccessible(font) || !FontFamilyNameMatches(font, matchPattern)) {
+            if (!font || !FontFamilyNameMatches(font, matchPattern) || !FontAccessible(font)) {
                 font.reset();
             }
             return font;
         }());
-        return createTypefaceFromFcPattern(std::move(font)).release();
+        return createTypefaceFromFcPattern(std::move(font));
     }
 
-    SkTypeface* onMatchFamilyStyleCharacter(const char familyName[],
-                                            const SkFontStyle& style,
-                                            const char* bcp47[],
-                                            int bcp47Count,
-                                            SkUnichar character) const override
+    sk_sp<SkTypeface> onMatchFamilyStyleCharacter(const char familyName[],
+                                                  const SkFontStyle& style,
+                                                  const char* bcp47[],
+                                                  int bcp47Count,
+                                                  SkUnichar character) const override
     {
         SkAutoFcPattern font([&](){
             FCLocker lock;
@@ -944,54 +949,28 @@ protected:
 
             FcResult result;
             SkAutoFcPattern font(FcFontMatch(fFC, pattern, &result));
-            if (!font || !FontAccessible(font) || !FontContainsCharacter(font, character)) {
+            if (!font || !FontContainsCharacter(font, character) || !FontAccessible(font)) {
                 font.reset();
             }
             return font;
         }());
-        return createTypefaceFromFcPattern(std::move(font)).release();
+        return this->createTypefaceFromFcPattern(std::move(font));
     }
 
     sk_sp<SkTypeface> onMakeFromStreamIndex(std::unique_ptr<SkStreamAsset> stream,
                                             int ttcIndex) const override {
+        return this->makeFromStream(std::move(stream),
+                                    SkFontArguments().setCollectionIndex(ttcIndex));
+    }
+
+    sk_sp<SkTypeface> onMakeFromStreamArgs(std::unique_ptr<SkStreamAsset> stream,
+                                           const SkFontArguments& args) const override {
         const size_t length = stream->getLength();
         if (length <= 0 || (1u << 30) < length) {
             return nullptr;
         }
 
-        SkString name;
-        SkFontStyle style;
-        bool isFixedWidth = false;
-        if (!fScanner.scanFont(stream.get(), ttcIndex, &name, &style, &isFixedWidth, nullptr)) {
-            return nullptr;
-        }
-
-        auto data = std::make_unique<SkFontData>(std::move(stream), ttcIndex, nullptr, 0);
-        return sk_sp<SkTypeface>(new SkTypeface_stream(std::move(data), std::move(name),
-                                                       style, isFixedWidth));
-    }
-
-    sk_sp<SkTypeface> onMakeFromStreamArgs(std::unique_ptr<SkStreamAsset> stream,
-                                           const SkFontArguments& args) const override {
-        using Scanner = SkTypeface_FreeType::Scanner;
-        bool isFixedPitch;
-        SkFontStyle style;
-        SkString name;
-        Scanner::AxisDefinitions axisDefinitions;
-        if (!fScanner.scanFont(stream.get(), args.getCollectionIndex(),
-                               &name, &style, &isFixedPitch, &axisDefinitions))
-        {
-            return nullptr;
-        }
-
-        SkAutoSTMalloc<4, SkFixed> axisValues(axisDefinitions.count());
-        Scanner::computeAxisValues(axisDefinitions, args.getVariationDesignPosition(),
-                                   axisValues, name);
-
-        auto data = std::make_unique<SkFontData>(std::move(stream), args.getCollectionIndex(),
-                                                   axisValues.get(), axisDefinitions.count());
-        return sk_sp<SkTypeface>(new SkTypeface_stream(std::move(data), std::move(name),
-                                                       style, isFixedPitch));
+        return fScanner->MakeFromStream(std::move(stream), args);
     }
 
     sk_sp<SkTypeface> onMakeFromData(sk_sp<SkData> data, int ttcIndex) const override {
@@ -1012,6 +991,10 @@ protected:
     }
 };
 
-SK_API sk_sp<SkFontMgr> SkFontMgr_New_FontConfig(FcConfig* fc) {
-    return sk_make_sp<SkFontMgr_fontconfig>(fc);
+sk_sp<SkFontMgr> SkFontMgr_New_FontConfig(FcConfig* fc, std::unique_ptr<SkFontScanner> scanner) {
+    return sk_make_sp<SkFontMgr_fontconfig>(fc, std::move(scanner));
+}
+
+sk_sp<SkFontMgr> SkFontMgr_New_FontConfig(FcConfig* fc) {
+    return sk_make_sp<SkFontMgr_fontconfig>(fc, SkFontScanner_Make_FreeType());
 }
