@@ -7,38 +7,24 @@
 
 #include "src/sksl/SkSLCompiler.h"
 
-#include "include/private/SkSLDefines.h"
-#include "include/private/SkSLIRNode.h"
-#include "include/private/SkSLProgramKind.h"
-#include "include/private/SkSLSymbol.h"
-#include "include/sksl/DSLCore.h"
-#include "include/sksl/DSLModifiers.h"
-#include "include/sksl/DSLType.h"
+#include "include/private/base/SkDebug.h"
 #include "src/core/SkTraceEvent.h"
 #include "src/sksl/SkSLAnalysis.h"
 #include "src/sksl/SkSLContext.h"
+#include "src/sksl/SkSLDefines.h"
 #include "src/sksl/SkSLInliner.h"
+#include "src/sksl/SkSLModule.h"
 #include "src/sksl/SkSLModuleLoader.h"
-#include "src/sksl/SkSLOutputStream.h"
 #include "src/sksl/SkSLParser.h"
+#include "src/sksl/SkSLPool.h"
+#include "src/sksl/SkSLProgramKind.h"
 #include "src/sksl/SkSLProgramSettings.h"
-#include "src/sksl/SkSLStringStream.h"
 #include "src/sksl/analysis/SkSLProgramUsage.h"
-#include "src/sksl/ir/SkSLExpression.h"
-#include "src/sksl/ir/SkSLExternalFunction.h"
-#include "src/sksl/ir/SkSLExternalFunctionReference.h"
-#include "src/sksl/ir/SkSLField.h"
-#include "src/sksl/ir/SkSLFieldAccess.h"
-#include "src/sksl/ir/SkSLFunctionDeclaration.h"
-#include "src/sksl/ir/SkSLFunctionReference.h"
 #include "src/sksl/ir/SkSLProgram.h"
-#include "src/sksl/ir/SkSLSymbolTable.h"  // IWYU pragma: keep
-#include "src/sksl/ir/SkSLTypeReference.h"
-#include "src/sksl/ir/SkSLVariable.h"
-#include "src/sksl/ir/SkSLVariableReference.h"
+#include "src/sksl/ir/SkSLProgramElement.h"  // IWYU pragma: keep
+#include "src/sksl/ir/SkSLSymbolTable.h"     // IWYU pragma: keep
 #include "src/sksl/transform/SkSLTransform.h"
 
-#include <atomic>
 #include <cstdint>
 #include <memory>
 #include <utility>
@@ -47,47 +33,12 @@
 #include <fstream>
 #endif
 
-#if defined(SKSL_STANDALONE) || SK_SUPPORT_GPU || defined(SK_GRAPHITE_ENABLED)
-#include "src/sksl/codegen/SkSLGLSLCodeGenerator.h"
-#include "src/sksl/codegen/SkSLMetalCodeGenerator.h"
-#include "src/sksl/codegen/SkSLSPIRVCodeGenerator.h"
-#include "src/sksl/codegen/SkSLSPIRVtoHLSL.h"
-#include "src/sksl/codegen/SkSLWGSLCodeGenerator.h"
-#endif
-
-#ifdef SK_ENABLE_SPIRV_VALIDATION
-#include "spirv-tools/libspirv.hpp"
-#endif
-
-#ifdef SK_ENABLE_WGSL_VALIDATION
-#include "tint/tint.h"
-#endif
-
 namespace SkSL {
 
-class ModifiersPool;
-
 // These flags allow tools like Viewer or Nanobench to override the compiler's ProgramSettings.
 Compiler::OverrideFlag Compiler::sOptimizer = OverrideFlag::kDefault;
 Compiler::OverrideFlag Compiler::sInliner = OverrideFlag::kDefault;
 
-using RefKind = VariableReference::RefKind;
-
-class AutoSource {
-public:
-    AutoSource(Compiler* compiler, std::string_view source)
-            : fCompiler(compiler) {
-        SkASSERT(!fCompiler->errorReporter().source().data());
-        fCompiler->errorReporter().setSource(source);
-    }
-
-    ~AutoSource() {
-        fCompiler->errorReporter().setSource(std::string_view());
-    }
-
-    Compiler* fCompiler;
-};
-
 class AutoProgramConfig {
 public:
     AutoProgramConfig(Context& context, ProgramConfig* config)
@@ -104,43 +55,9 @@ public:
     ProgramConfig* fOldConfig;
 };
 
-class AutoShaderCaps {
-public:
-    AutoShaderCaps(std::shared_ptr<Context>& context, const ShaderCaps* caps)
-            : fContext(context.get())
-            , fOldCaps(fContext->fCaps) {
-        fContext->fCaps = caps;
-    }
-
-    ~AutoShaderCaps() {
-        fContext->fCaps = fOldCaps;
-    }
-
-    Context* fContext;
-    const ShaderCaps* fOldCaps;
-};
-
-class AutoModifiersPool {
-public:
-    AutoModifiersPool(std::shared_ptr<Context>& context, ModifiersPool* modifiersPool)
-            : fContext(context.get()) {
-        SkASSERT(!fContext->fModifiersPool);
-        fContext->fModifiersPool = modifiersPool;
-    }
-
-    ~AutoModifiersPool() {
-        fContext->fModifiersPool = nullptr;
-    }
-
-    Context* fContext;
-};
-
-Compiler::Compiler(const ShaderCaps* caps) : fErrorReporter(this), fCaps(caps) {
-    SkASSERT(caps);
-
+Compiler::Compiler() : fErrorReporter(this) {
     auto moduleLoader = ModuleLoader::Get();
-    fContext = std::make_shared<Context>(moduleLoader.builtinTypes(), /*caps=*/nullptr,
-                                         fErrorReporter);
+    fContext = std::make_shared<Context>(moduleLoader.builtinTypes(), fErrorReporter);
 }
 
 Compiler::~Compiler() {}
@@ -148,20 +65,21 @@ Compiler::~Compiler() {}
 const Module* Compiler::moduleForProgramKind(ProgramKind kind) {
     auto m = ModuleLoader::Get();
     switch (kind) {
-        case ProgramKind::kVertex:                return m.loadVertexModule(this);
         case ProgramKind::kFragment:              return m.loadFragmentModule(this);
+        case ProgramKind::kVertex:                return m.loadVertexModule(this);
         case ProgramKind::kCompute:               return m.loadComputeModule(this);
-        case ProgramKind::kGraphiteVertex:        return m.loadGraphiteVertexModule(this);
         case ProgramKind::kGraphiteFragment:      return m.loadGraphiteFragmentModule(this);
+        case ProgramKind::kGraphiteVertex:        return m.loadGraphiteVertexModule(this);
+        case ProgramKind::kGraphiteFragmentES2:   return m.loadGraphiteFragmentES2Module(this);
+        case ProgramKind::kGraphiteVertexES2:     return m.loadGraphiteVertexES2Module(this);
+        case ProgramKind::kPrivateRuntimeBlender:
+        case ProgramKind::kPrivateRuntimeColorFilter:
         case ProgramKind::kPrivateRuntimeShader:  return m.loadPrivateRTShaderModule(this);
         case ProgramKind::kRuntimeColorFilter:
         case ProgramKind::kRuntimeShader:
         case ProgramKind::kRuntimeBlender:
-        case ProgramKind::kPrivateRuntimeColorFilter:
-        case ProgramKind::kPrivateRuntimeBlender:
         case ProgramKind::kMeshVertex:
-        case ProgramKind::kMeshFragment:
-        case ProgramKind::kGeneric:               return m.loadPublicModule(this);
+        case ProgramKind::kMeshFragment:          return m.loadPublicModule(this);
     }
     SkUNREACHABLE;
 }
@@ -197,42 +115,91 @@ void Compiler::FinalizeSettings(ProgramSettings* settings, ProgramKind kind) {
     settings->fRemoveDeadFunctions &= settings->fOptimize;
     settings->fRemoveDeadVariables &= settings->fOptimize;
 
-    if (kind == ProgramKind::kGeneric) {
-        // For "generic" interpreter programs, leave all functions intact. (The SkVM API supports
-        // calling any function, not just 'main').
-        settings->fRemoveDeadFunctions = false;
-    } else {
-        // Only generic programs (limited to CPU) are able to use external functions.
-        SkASSERT(!settings->fExternalFunctions);
-    }
-
     // Runtime effects always allow narrowing conversions.
     if (ProgramConfig::IsRuntimeEffect(kind)) {
         settings->fAllowNarrowingConversions = true;
     }
 }
 
+void Compiler::initializeContext(const SkSL::Module* module,
+                                 ProgramKind kind,
+                                 ProgramSettings settings,
+                                 std::string_view source,
+                                 ModuleType moduleType) {
+    SkASSERT(!fPool);
+    SkASSERT(!fConfig);
+    SkASSERT(!fContext->fSymbolTable);
+    SkASSERT(!fContext->fConfig);
+    SkASSERT(!fContext->fModule);
+
+    // Start the ErrorReporter with a clean slate.
+    this->resetErrors();
+
+    fConfig = std::make_unique<ProgramConfig>();
+    fConfig->fModuleType = moduleType;
+    fConfig->fSettings = settings;
+    fConfig->fKind = kind;
+
+    // Make sure the passed-in settings are valid.
+    FinalizeSettings(&fConfig->fSettings, kind);
+
+    if (settings.fUseMemoryPool) {
+        fPool = Pool::Create();
+        fPool->attachToThread();
+    }
+
+    fContext->fConfig = fConfig.get();
+    fContext->fModule = module;
+    fContext->fErrors->setSource(source);
+
+    // Set up a clean symbol table atop the parent module's symbols.
+    fGlobalSymbols = std::make_unique<SymbolTable>(module->fSymbols.get(),
+                                                   moduleType != ModuleType::program);
+    fGlobalSymbols->markModuleBoundary();
+    fContext->fSymbolTable = fGlobalSymbols.get();
+}
+
+void Compiler::cleanupContext() {
+    // Clear out the fields we initialized above.
+    fContext->fConfig = nullptr;
+    fContext->fModule = nullptr;
+    fContext->fErrors->setSource(std::string_view());
+    fContext->fSymbolTable = nullptr;
+
+    fConfig = nullptr;
+    fGlobalSymbols = nullptr;
+
+    if (fPool) {
+        fPool->detachFromThread();
+        fPool = nullptr;
+    }
+}
+
 std::unique_ptr<Module> Compiler::compileModule(ProgramKind kind,
-                                                const char* moduleName,
+                                                ModuleType moduleType,
                                                 std::string moduleSource,
-                                                const Module* parent,
-                                                ModifiersPool& modifiersPool,
+                                                const Module* parentModule,
                                                 bool shouldInline) {
-    SkASSERT(parent);
-    SkASSERT(!moduleSource.empty());
+    SkASSERT(parentModule);
     SkASSERT(this->errorCount() == 0);
 
-    // Modules are shared and cannot rely on shader caps.
-    AutoShaderCaps autoCaps(fContext, nullptr);
-    AutoModifiersPool autoPool(fContext, &modifiersPool);
+    // Wrap the program source in a pointer so it is guaranteed to be stable across moves.
+    auto sourcePtr = std::make_unique<std::string>(std::move(moduleSource));
 
-    // Compile the module from source, using default program settings.
+    // Compile the module from source, using default program settings (but no memory pooling).
     ProgramSettings settings;
-    FinalizeSettings(&settings, kind);
-    SkSL::Parser parser{this, settings, kind, std::move(moduleSource)};
-    std::unique_ptr<Module> module = parser.moduleInheritingFrom(parent);
+    settings.fUseMemoryPool = false;
+    this->initializeContext(parentModule, kind, settings, *sourcePtr, moduleType);
+
+    std::unique_ptr<Module> module = SkSL::Parser(this, settings, kind, std::move(sourcePtr))
+                                             .moduleInheritingFrom(parentModule);
+
+    this->cleanupContext();
+
     if (this->errorCount() != 0) {
-        SkDebugf("Unexpected errors compiling %s:\n\n%s\n", moduleName, this->errorText().c_str());
+        SkDebugf("Unexpected errors compiling %s:\n\n%s\n",
+                 ModuleTypeToString(moduleType),
+                 this->errorText().c_str());
         return nullptr;
     }
     if (shouldInline) {
@@ -242,78 +209,66 @@ std::unique_ptr<Module> Compiler::compileModule(ProgramKind kind,
 }
 
 std::unique_ptr<Program> Compiler::convertProgram(ProgramKind kind,
-                                                  std::string text,
-                                                  ProgramSettings settings) {
+                                                  std::string programSource,
+                                                  const ProgramSettings& settings) {
     TRACE_EVENT0("skia.shaders", "SkSL::Compiler::convertProgram");
 
-    // Make sure the passed-in settings are valid.
-    FinalizeSettings(&settings, kind);
+    // Wrap the program source in a pointer so it is guaranteed to be stable across moves.
+    auto sourcePtr = std::make_unique<std::string>(std::move(programSource));
 
-    // Put the ShaderCaps into the context while compiling a program.
-    AutoShaderCaps autoCaps(fContext, fCaps);
+    // Load the module used by this ProgramKind.
+    const SkSL::Module* module = this->moduleForProgramKind(kind);
 
-    this->resetErrors();
+    this->initializeContext(module, kind, settings, *sourcePtr, ModuleType::program);
 
-    return Parser(this, settings, kind, std::move(text)).program();
+    std::unique_ptr<Program> program = SkSL::Parser(this, settings, kind, std::move(sourcePtr))
+                                               .programInheritingFrom(module);
+
+    this->cleanupContext();
+    return program;
 }
 
-std::unique_ptr<Expression> Compiler::convertIdentifier(Position pos, std::string_view name) {
-    const Symbol* result = fSymbolTable->find(name);
-    if (!result) {
-        this->errorReporter().error(pos, "unknown identifier '" + std::string(name) + "'");
-        return nullptr;
-    }
-    switch (result->kind()) {
-        case Symbol::Kind::kFunctionDeclaration: {
-            return std::make_unique<FunctionReference>(*fContext, pos,
-                                                       &result->as<FunctionDeclaration>());
-        }
-        case Symbol::Kind::kVariable: {
-            const Variable* var = &result->as<Variable>();
-            // default to kRead_RefKind; this will be corrected later if the variable is written to
-            return VariableReference::Make(pos, var, VariableReference::RefKind::kRead);
-        }
-        case Symbol::Kind::kField: {
-            const Field* field = &result->as<Field>();
-            auto base = VariableReference::Make(pos, &field->owner(),
-                                                VariableReference::RefKind::kRead);
-            return FieldAccess::Make(*fContext, pos, std::move(base), field->fieldIndex(),
-                                     FieldAccess::OwnerKind::kAnonymousInterfaceBlock);
-        }
-        case Symbol::Kind::kType: {
-            // go through DSLType so we report errors on private types
-            dsl::DSLModifiers modifiers;
-            dsl::DSLType dslType(result->name(), &modifiers, pos);
-            return TypeReference::Convert(*fContext, pos, &dslType.skslType());
-        }
-        case Symbol::Kind::kExternal: {
-            const ExternalFunction* r = &result->as<ExternalFunction>();
-            return std::make_unique<ExternalFunctionReference>(pos, r);
-        }
-        default:
-            SK_ABORT("unsupported symbol type %d\n", (int) result->kind());
+std::unique_ptr<SkSL::Program> Compiler::releaseProgram(
+        std::unique_ptr<std::string> source,
+        std::vector<std::unique_ptr<SkSL::ProgramElement>> programElements) {
+    Pool* pool = fPool.get();
+    auto result = std::make_unique<SkSL::Program>(std::move(source),
+                                                  std::move(fConfig),
+                                                  fContext,
+                                                  std::move(programElements),
+                                                  std::move(fGlobalSymbols),
+                                                  std::move(fPool));
+    fContext->fSymbolTable = nullptr;
+
+    bool success = this->finalize(*result) &&
+                   this->optimize(*result);
+    if (pool) {
+        pool->detachFromThread();
     }
+    return success ? std::move(result) : nullptr;
 }
 
-bool Compiler::optimizeModuleBeforeMinifying(ProgramKind kind, Module& module) {
+bool Compiler::optimizeModuleBeforeMinifying(ProgramKind kind, Module& module, bool shrinkSymbols) {
     SkASSERT(this->errorCount() == 0);
 
     auto m = SkSL::ModuleLoader::Get();
 
     // Create a temporary program configuration with default settings.
     ProgramConfig config;
-    config.fIsBuiltinCode = true;
+    config.fModuleType = module.fModuleType;
     config.fKind = kind;
     AutoProgramConfig autoConfig(this->context(), &config);
-    AutoModifiersPool autoPool(fContext, &m.coreModifiers());
 
     std::unique_ptr<ProgramUsage> usage = Analysis::GetUsage(module);
 
-    // Assign shorter names to symbols as long as it won't change the external meaning of the code.
-    Transform::RenamePrivateSymbols(this->context(), module, usage.get(), kind);
+    if (shrinkSymbols) {
+        // Assign shorter names to symbols as long as it won't change the external meaning of the
+        // code.
+        Transform::RenamePrivateSymbols(this->context(), module, usage.get(), kind);
 
-    // Replace constant variables with their literal values to save space.
-    Transform::ReplaceConstVarsWithLiterals(module, usage.get());
+        // Replace constant variables with their literal values to save space.
+        Transform::ReplaceConstVarsWithLiterals(module, usage.get());
+    }
 
     // Remove any unreachable code.
     Transform::EliminateUnreachableCode(module, usage.get());
@@ -343,6 +298,12 @@ bool Compiler::optimizeModuleBeforeMinifying(ProgramKind kind, Module& module) {
     // We eliminate empty statements to avoid runs of `;;;;;;` caused by the previous passes.
     SkSL::Transform::EliminateEmptyStatements(module);
 
+    // We can eliminate `{}` around single-statement blocks.
+    SkSL::Transform::EliminateUnnecessaryBraces(this->context(), module);
+
+    // We can convert `float4(myFloat)` with `myFloat.xxxx` to save a few characters.
+    SkSL::Transform::ReplaceSplatCastsWithSwizzles(this->context(), module);
+
     // Make sure that program usage is still correct after the optimization pass is complete.
     SkASSERT(*usage == *Analysis::GetUsage(module));
 
@@ -355,7 +316,7 @@ bool Compiler::optimizeModuleAfterLoading(ProgramKind kind, Module& module) {
 #ifndef SK_ENABLE_OPTIMIZE_SIZE
     // Create a temporary program configuration with default settings.
     ProgramConfig config;
-    config.fIsBuiltinCode = true;
+    config.fModuleType = module.fModuleType;
     config.fKind = kind;
     AutoProgramConfig autoConfig(this->context(), &config);
 
@@ -364,7 +325,7 @@ bool Compiler::optimizeModuleAfterLoading(ProgramKind kind, Module& module) {
     // Perform inline-candidate analysis and inline any functions deemed suitable.
     Inliner inliner(fContext.get());
     while (this->errorCount() == 0) {
-        if (!this->runInliner(&inliner, module.fElements, module.fSymbols, usage.get())) {
+        if (!this->runInliner(&inliner, module.fElements, module.fSymbols.get(), usage.get())) {
             break;
         }
     }
@@ -381,15 +342,14 @@ bool Compiler::optimize(Program& program) {
         return true;
     }
 
-    AutoShaderCaps autoCaps(fContext, fCaps);
-
     SkASSERT(!this->errorCount());
     if (this->errorCount() == 0) {
 #ifndef SK_ENABLE_OPTIMIZE_SIZE
         // Run the inliner only once; it is expensive! Multiple passes can occasionally shake out
         // more wins, but it's diminishing returns.
         Inliner inliner(fContext.get());
-        this->runInliner(&inliner, program.fOwnedElements, program.fSymbols, program.fUsage.get());
+        this->runInliner(&inliner, program.fOwnedElements, program.fSymbols.get(),
+                         program.fUsage.get());
 #endif
 
         // Unreachable code can confuse some drivers, so it's worth removing. (skia:12012)
@@ -406,45 +366,52 @@ bool Compiler::optimize(Program& program) {
         }
         // Make sure that program usage is still correct after the optimization pass is complete.
         SkASSERT(*program.usage() == *Analysis::GetUsage(program));
+
+        // Make sure that variables are still declared in the correct symbol tables.
+        SkDEBUGCODE(Analysis::CheckSymbolTableCorrectness(program));
     }
 
     return this->errorCount() == 0;
 }
 
+void Compiler::runInliner(Program& program) {
+#ifndef SK_ENABLE_OPTIMIZE_SIZE
+    AutoProgramConfig autoConfig(this->context(), program.fConfig.get());
+    Inliner inliner(fContext.get());
+    this->runInliner(&inliner, program.fOwnedElements, program.fSymbols.get(),
+                     program.fUsage.get());
+#endif
+}
+
 bool Compiler::runInliner(Inliner* inliner,
                           const std::vector<std::unique_ptr<ProgramElement>>& elements,
-                          std::shared_ptr<SymbolTable> symbols,
+                          SymbolTable* symbols,
                           ProgramUsage* usage) {
 #ifdef SK_ENABLE_OPTIMIZE_SIZE
     return true;
 #else
-    // The program's SymbolTable was taken out of fSymbolTable when the program was bundled, but
-    // the inliner relies (indirectly) on having a valid SymbolTable.
-    // In particular, inlining can turn a non-optimizable expression like `normalize(myVec)` into
-    // `normalize(vec2(7))`, which is now optimizable. The optimizer can use DSL to simplify this
-    // expression--e.g., in the case of normalize, using DSL's Length(). The DSL relies on
-    // convertIdentifier() to look up `length`. convertIdentifier() needs a valid symbol table to
-    // find the declaration of `length`. To allow this chain of events to succeed, we re-insert the
-    // program's symbol table temporarily.
-    SkASSERT(!fSymbolTable);
-    fSymbolTable = symbols;
+    // The program's SymbolTable was taken out of the context when the program was bundled, but
+    // the inliner creates IR objects which may expect the context to hold a valid SymbolTable.
+    SkASSERT(!fContext->fSymbolTable);
+    fContext->fSymbolTable = symbols;
 
     bool result = inliner->analyze(elements, symbols, usage);
 
-    fSymbolTable = nullptr;
+    fContext->fSymbolTable = nullptr;
     return result;
 #endif
 }
 
 bool Compiler::finalize(Program& program) {
-    AutoShaderCaps autoCaps(fContext, fCaps);
-
     // Copy all referenced built-in functions into the Program.
     Transform::FindAndDeclareBuiltinFunctions(program);
 
-    // Variables defined in the pre-includes need their declaring elements added to the program.
+    // Variables defined in modules need their declaring elements added to the program.
     Transform::FindAndDeclareBuiltinVariables(program);
 
+    // Structs from module code need to be added to the program's shared elements.
+    Transform::FindAndDeclareBuiltinStructs(program);
+
     // Do one last correctness-check pass. This looks for dangling FunctionReference/TypeReference
     // expressions, and reports them as errors.
     Analysis::DoFinalizationChecks(program);
@@ -457,8 +424,10 @@ bool Compiler::finalize(Program& program) {
         }
     }
     if (this->errorCount() == 0) {
-        bool enforceSizeLimit = ProgramConfig::IsRuntimeEffect(program.fConfig->fKind);
-        Analysis::CheckProgramStructure(program, enforceSizeLimit);
+        Analysis::CheckProgramStructure(program);
+
+        // Make sure that variables are declared in the symbol tables that immediately enclose them.
+        SkDEBUGCODE(Analysis::CheckSymbolTableCorrectness(program));
     }
 
     // Make sure that program usage is still correct after finalization is complete.
@@ -467,178 +436,6 @@ bool Compiler::finalize(Program& program) {
     return this->errorCount() == 0;
 }
 
-#if defined(SKSL_STANDALONE) || SK_SUPPORT_GPU || defined(SK_GRAPHITE_ENABLED)
-
-#if defined(SK_ENABLE_SPIRV_VALIDATION)
-static bool validate_spirv(ErrorReporter& reporter, std::string_view program) {
-    SkASSERT(0 == program.size() % 4);
-    const uint32_t* programData = reinterpret_cast<const uint32_t*>(program.data());
-    size_t programSize = program.size() / 4;
-
-    spvtools::SpirvTools tools(SPV_ENV_VULKAN_1_0);
-    std::string errors;
-    auto msgFn = [&errors](spv_message_level_t, const char*, const spv_position_t&, const char* m) {
-        errors += "SPIR-V validation error: ";
-        errors += m;
-        errors += '\n';
-    };
-    tools.SetMessageConsumer(msgFn);
-
-    // Verify that the SPIR-V we produced is valid. At runtime, we will abort() with a message
-    // explaining the error. In standalone mode (skslc), we will send the message, plus the
-    // entire disassembled SPIR-V (for easier context & debugging) as *our* error message.
-    bool result = tools.Validate(programData, programSize);
-    if (!result) {
-#if defined(SKSL_STANDALONE)
-        // Convert the string-stream to a SPIR-V disassembly.
-        std::string disassembly;
-        if (tools.Disassemble(programData, programSize, &disassembly)) {
-            errors.append(disassembly);
-        }
-        reporter.error(Position(), errors);
-#else
-        SkDEBUGFAILF("%s", errors.c_str());
-#endif
-    }
-    return result;
-}
-#endif
-
-bool Compiler::toSPIRV(Program& program, OutputStream& out) {
-    TRACE_EVENT0("skia.shaders", "SkSL::Compiler::toSPIRV");
-    AutoSource as(this, *program.fSource);
-    AutoShaderCaps autoCaps(fContext, fCaps);
-    ProgramSettings settings;
-    settings.fUseMemoryPool = false;
-    dsl::Start(this, program.fConfig->fKind, settings);
-    dsl::SetErrorReporter(&fErrorReporter);
-    fSymbolTable = program.fSymbols;
-#ifdef SK_ENABLE_SPIRV_VALIDATION
-    StringStream buffer;
-    SPIRVCodeGenerator cg(fContext.get(), &program, &buffer);
-    bool result = cg.generateCode();
-
-    if (result && program.fConfig->fSettings.fValidateSPIRV) {
-        std::string_view binary = buffer.str();
-        result = validate_spirv(this->errorReporter(), binary);
-        out.write(binary.data(), binary.size());
-    }
-#else
-    SPIRVCodeGenerator cg(fContext.get(), &program, &out);
-    bool result = cg.generateCode();
-#endif
-    dsl::End();
-    return result;
-}
-
-bool Compiler::toSPIRV(Program& program, std::string* out) {
-    StringStream buffer;
-    bool result = this->toSPIRV(program, buffer);
-    if (result) {
-        *out = buffer.str();
-    }
-    return result;
-}
-
-bool Compiler::toGLSL(Program& program, OutputStream& out) {
-    TRACE_EVENT0("skia.shaders", "SkSL::Compiler::toGLSL");
-    AutoSource as(this, *program.fSource);
-    AutoShaderCaps autoCaps(fContext, fCaps);
-    GLSLCodeGenerator cg(fContext.get(), &program, &out);
-    bool result = cg.generateCode();
-    return result;
-}
-
-bool Compiler::toGLSL(Program& program, std::string* out) {
-    StringStream buffer;
-    bool result = this->toGLSL(program, buffer);
-    if (result) {
-        *out = buffer.str();
-    }
-    return result;
-}
-
-bool Compiler::toHLSL(Program& program, OutputStream& out) {
-    TRACE_EVENT0("skia.shaders", "SkSL::Compiler::toHLSL");
-    std::string hlsl;
-    if (!this->toHLSL(program, &hlsl)) {
-        return false;
-    }
-    out.writeString(hlsl);
-    return true;
-}
-
-bool Compiler::toHLSL(Program& program, std::string* out) {
-    std::string spirv;
-    if (!this->toSPIRV(program, &spirv)) {
-        return false;
-    }
-
-    if (!SPIRVtoHLSL(spirv, out)) {
-        fErrorText += "HLSL cross-compilation not enabled";
-        return false;
-    }
-
-    return true;
-}
-
-bool Compiler::toMetal(Program& program, OutputStream& out) {
-    TRACE_EVENT0("skia.shaders", "SkSL::Compiler::toMetal");
-    AutoSource as(this, *program.fSource);
-    AutoShaderCaps autoCaps(fContext, fCaps);
-    MetalCodeGenerator cg(fContext.get(), &program, &out);
-    bool result = cg.generateCode();
-    return result;
-}
-
-bool Compiler::toMetal(Program& program, std::string* out) {
-    StringStream buffer;
-    bool result = this->toMetal(program, buffer);
-    if (result) {
-        *out = buffer.str();
-    }
-    return result;
-}
-
-#if defined(SK_ENABLE_WGSL_VALIDATION)
-static bool validate_wgsl(ErrorReporter& reporter, const std::string& wgsl) {
-    tint::Source::File srcFile("", wgsl);
-    tint::Program program(tint::reader::wgsl::Parse(&srcFile));
-    if (program.Diagnostics().count() > 0) {
-        tint::diag::Formatter diagFormatter;
-        std::string diagOutput = diagFormatter.format(program.Diagnostics());
-#if defined(SKSL_STANDALONE)
-        reporter.error(Position(), diagOutput);
-#else
-        SkDEBUGFAILF("%s", diagOutput.c_str());
-#endif
-        return false;
-    }
-    return true;
-}
-#endif  // defined(SK_ENABLE_WGSL_VALIDATION)
-
-bool Compiler::toWGSL(Program& program, OutputStream& out) {
-    TRACE_EVENT0("skia.shaders", "SkSL::Compiler::toWGSL");
-    AutoSource as(this, *program.fSource);
-#ifdef SK_ENABLE_WGSL_VALIDATION
-    StringStream wgsl;
-    WGSLCodeGenerator cg(fContext.get(), &program, &wgsl);
-    bool result = cg.generateCode();
-    if (result) {
-        std::string wgslString = wgsl.str();
-        result = validate_wgsl(this->errorReporter(), wgslString);
-        out.writeString(wgslString);
-    }
-#else
-    WGSLCodeGenerator cg(fContext.get(), &program, &out);
-    bool result = cg.generateCode();
-#endif
-    return result;
-}
-
-#endif // defined(SKSL_STANDALONE) || SK_SUPPORT_GPU || defined(SK_GRAPHITE_ENABLED)
-
 void Compiler::handleError(std::string_view msg, Position pos) {
     fErrorText += "error: ";
     bool printLocation = false;
@@ -730,11 +527,8 @@ std::string Compiler::errorText(bool showCount) {
 void Compiler::writeErrorCount() {
     int count = this->errorCount();
     if (count) {
-        fErrorText += std::to_string(count) + " error";
-        if (count > 1) {
-            fErrorText += "s";
-        }
-        fErrorText += "\n";
+        fErrorText += std::to_string(count) +
+                      ((count == 1) ? " error\n" : " errors\n");
     }
 }
 
