@@ -15,6 +15,7 @@
 #include "flat_dumper.hpp"
 #include "html_dumper.hpp"
 #include "sheet_impl.hpp"
+#include "debug_state_context.hpp"
 #include "debug_state_dumper.hpp"
 
 #include <iostream>
@@ -263,7 +264,7 @@ void sheet::set_col_width(col_t col, col_t col_span, col_width_t width)
 col_width_t sheet::get_col_width(col_t col, col_t* col_start, col_t* col_end) const
 {
     detail::col_widths_store_type& col_widths = mp_impl->col_widths;
-    if (!col_widths.is_tree_valid())
+    if (!col_widths.valid_tree())
         col_widths.build_tree();
 
     col_width_t ret = 0;
@@ -282,7 +283,7 @@ void sheet::set_col_hidden(col_t col, col_t col_span, bool hidden)
 bool sheet::is_col_hidden(col_t col, col_t* col_start, col_t* col_end) const
 {
     detail::col_hidden_store_type& col_hidden = mp_impl->col_hidden;
-    if (!col_hidden.is_tree_valid())
+    if (!col_hidden.valid_tree())
         col_hidden.build_tree();
 
     bool hidden = false;
@@ -292,16 +293,16 @@ bool sheet::is_col_hidden(col_t col, col_t* col_start, col_t* col_end) const
     return hidden;
 }
 
-void sheet::set_row_height(row_t row, row_height_t height)
+void sheet::set_row_height(row_t row, row_t row_span, row_height_t height)
 {
     mp_impl->row_height_pos =
-        mp_impl->row_heights.insert(mp_impl->row_height_pos, row, row+1, height).first;
+        mp_impl->row_heights.insert(mp_impl->row_height_pos, row, row+row_span, height).first;
 }
 
 row_height_t sheet::get_row_height(row_t row, row_t* row_start, row_t* row_end) const
 {
     detail::row_heights_store_type& row_heights = mp_impl->row_heights;
-    if (!row_heights.is_tree_valid())
+    if (!row_heights.valid_tree())
         row_heights.build_tree();
 
     row_height_t ret = 0;
@@ -311,16 +312,16 @@ row_height_t sheet::get_row_height(row_t row, row_t* row_start, row_t* row_end)
     return ret;
 }
 
-void sheet::set_row_hidden(row_t row, bool hidden)
+void sheet::set_row_hidden(row_t row, row_t row_span, bool hidden)
 {
     mp_impl->row_hidden_pos =
-        mp_impl->row_hidden.insert(mp_impl->row_hidden_pos, row, row+1, hidden).first;
+        mp_impl->row_hidden.insert(mp_impl->row_hidden_pos, row, row+row_span, hidden).first;
 }
 
 bool sheet::is_row_hidden(row_t row, row_t* row_start, row_t* row_end) const
 {
     detail::row_hidden_store_type& row_hidden = mp_impl->row_hidden;
-    if (!row_hidden.is_tree_valid())
+    if (!row_hidden.valid_tree())
         row_hidden.build_tree();
 
     bool hidden = false;
@@ -390,19 +391,14 @@ size_t sheet::get_string_identifier(row_t row, col_t col) const
     return cxt.get_string_identifier(ixion::abs_address_t(mp_impl->sheet_id, row, col));
 }
 
-auto_filter_t* sheet::get_auto_filter_data()
+void sheet::set_auto_filter(std::unique_ptr<auto_filter_t> filter)
 {
-    return mp_impl->auto_filter_data.get();
+    mp_impl->auto_filter = std::move(filter);
 }
 
-const auto_filter_t* sheet::get_auto_filter_data() const
+const auto_filter_t* sheet::get_auto_filter() const
 {
-    return mp_impl->auto_filter_data.get();
-}
-
-void sheet::set_auto_filter_data(auto_filter_t* p)
-{
-    mp_impl->auto_filter_data.reset(p);
+    return mp_impl->auto_filter.get();
 }
 
 ixion::abs_range_t sheet::get_data_range() const
@@ -486,10 +482,10 @@ void sheet::dump_check(ostream& os, std::string_view sheet_name) const
 
 void sheet::dump_html(std::ostream& os) const
 {
-    if (!mp_impl->col_widths.is_tree_valid())
+    if (!mp_impl->col_widths.valid_tree())
         mp_impl->col_widths.build_tree();
 
-    if (!mp_impl->row_heights.is_tree_valid())
+    if (!mp_impl->row_heights.valid_tree())
         mp_impl->row_heights.build_tree();
 
     detail::html_dumper dumper(mp_impl->doc, mp_impl->merge_ranges, mp_impl->sheet_id);
@@ -511,7 +507,8 @@ void sheet::dump_csv(std::ostream& os) const
 void sheet::dump_debug_state(const std::string& output_dir, std::string_view sheet_name) const
 {
     fs::path outdir{output_dir};
-    detail::sheet_debug_state_dumper dumper(*mp_impl, sheet_name);
+    detail::debug_state_context cxt;
+    detail::sheet_debug_state_dumper dumper(cxt, *mp_impl, sheet_name);
     dumper.dump(outdir);
 }
 
@@ -522,7 +519,7 @@ size_t sheet::get_cell_format(row_t row, col_t col) const
     if (it != mp_impl->cell_formats.end())
     {
         detail::segment_row_index_type& con = *it->second;
-        if (!con.is_tree_valid())
+        if (!con.valid_tree())
             con.build_tree();
 
         // Return only if the index is not a default index
@@ -532,7 +529,7 @@ size_t sheet::get_cell_format(row_t row, col_t col) const
     }
 
     // Not found in the cell format store. Check the row store.
-    if (!mp_impl->row_formats.is_tree_valid())
+    if (!mp_impl->row_formats.valid_tree())
         mp_impl->row_formats.build_tree();
 
     std::size_t index;
@@ -540,7 +537,7 @@ size_t sheet::get_cell_format(row_t row, col_t col) const
         return index;
 
     // Not found in the row store. Check the column store.
-    if (!mp_impl->column_formats.is_tree_valid())
+    if (!mp_impl->column_formats.valid_tree())
         mp_impl->column_formats.build_tree();
 
     if (mp_impl->column_formats.search_tree(col, index).second && index)
