@@ -57,8 +57,9 @@ typedef struct
 } fixture_t;
 
 static void
-fixture_init (fixture_t *fixture, gconstpointer user_data)
+fixture_init (gpointer fixture_, gconstpointer user_data)
 {
+  fixture_t *fixture = fixture_;
   hb_buffer_t *b;
   unsigned int i;
 
@@ -71,7 +72,7 @@ fixture_init (fixture_t *fixture, gconstpointer user_data)
 
     case BUFFER_ONE_BY_ONE:
       for (i = 1; i < G_N_ELEMENTS (utf32) - 1; i++)
-	hb_buffer_add (b, utf32[i], i);
+      hb_buffer_add (b, utf32[i], i);
       break;
 
     case BUFFER_UTF32:
@@ -92,93 +93,96 @@ fixture_init (fixture_t *fixture, gconstpointer user_data)
 }
 
 static void
-fixture_finish (fixture_t *fixture, gconstpointer user_data HB_UNUSED)
+fixture_finish (gpointer fixture_, gconstpointer user_data HB_UNUSED)
 {
+  fixture_t *fixture = fixture_;
   hb_buffer_destroy (fixture->buffer);
 }
 
 
 static void
-test_buffer_properties (fixture_t *fixture, gconstpointer user_data HB_UNUSED)
+test_buffer_properties (gpointer fixture_, gconstpointer user_data HB_UNUSED)
 {
+  fixture_t *fixture = fixture_;
   hb_buffer_t *b = fixture->buffer;
   hb_unicode_funcs_t *ufuncs;
 
   /* test default properties */
 
-  g_assert (hb_buffer_get_unicode_funcs (b) == hb_unicode_funcs_get_default ());
-  g_assert (hb_buffer_get_direction (b) == HB_DIRECTION_INVALID);
-  g_assert (hb_buffer_get_script (b) == HB_SCRIPT_INVALID);
-  g_assert (hb_buffer_get_language (b) == NULL);
+  g_assert_true (hb_buffer_get_unicode_funcs (b) == hb_unicode_funcs_get_default ());
+  g_assert_true (hb_buffer_get_direction (b) == HB_DIRECTION_INVALID);
+  g_assert_true (hb_buffer_get_script (b) == HB_SCRIPT_INVALID);
+  g_assert_true (hb_buffer_get_language (b) == NULL);
 
 
   /* test property changes are retained */
   ufuncs = hb_unicode_funcs_create (NULL);
   hb_buffer_set_unicode_funcs (b, ufuncs);
   hb_unicode_funcs_destroy (ufuncs);
-  g_assert (hb_buffer_get_unicode_funcs (b) == ufuncs);
+  g_assert_true (hb_buffer_get_unicode_funcs (b) == ufuncs);
 
   hb_buffer_set_direction (b, HB_DIRECTION_RTL);
-  g_assert (hb_buffer_get_direction (b) == HB_DIRECTION_RTL);
+  g_assert_true (hb_buffer_get_direction (b) == HB_DIRECTION_RTL);
 
   hb_buffer_set_script (b, HB_SCRIPT_ARABIC);
-  g_assert (hb_buffer_get_script (b) == HB_SCRIPT_ARABIC);
+  g_assert_true (hb_buffer_get_script (b) == HB_SCRIPT_ARABIC);
 
   hb_buffer_set_language (b, hb_language_from_string ("fa", -1));
-  g_assert (hb_buffer_get_language (b) == hb_language_from_string ("Fa", -1));
+  g_assert_true (hb_buffer_get_language (b) == hb_language_from_string ("Fa", -1));
 
   hb_buffer_set_flags (b, HB_BUFFER_FLAG_BOT);
-  g_assert (hb_buffer_get_flags (b) == HB_BUFFER_FLAG_BOT);
+  g_assert_true (hb_buffer_get_flags (b) == HB_BUFFER_FLAG_BOT);
 
   hb_buffer_set_replacement_codepoint (b, (unsigned int) -1);
-  g_assert (hb_buffer_get_replacement_codepoint (b) == (unsigned int) -1);
+  g_assert_true (hb_buffer_get_replacement_codepoint (b) == (unsigned int) -1);
 
 
   /* test clear_contents clears all these properties: */
 
   hb_buffer_clear_contents (b);
 
-  g_assert (hb_buffer_get_unicode_funcs (b) == ufuncs);
-  g_assert (hb_buffer_get_direction (b) == HB_DIRECTION_INVALID);
-  g_assert (hb_buffer_get_script (b) == HB_SCRIPT_INVALID);
-  g_assert (hb_buffer_get_language (b) == NULL);
+  g_assert_true (hb_buffer_get_unicode_funcs (b) == ufuncs);
+  g_assert_true (hb_buffer_get_direction (b) == HB_DIRECTION_INVALID);
+  g_assert_true (hb_buffer_get_script (b) == HB_SCRIPT_INVALID);
+  g_assert_true (hb_buffer_get_language (b) == NULL);
 
   /* but not these: */
 
-  g_assert (hb_buffer_get_flags (b) != HB_BUFFER_FLAGS_DEFAULT);
-  g_assert (hb_buffer_get_replacement_codepoint (b) != HB_BUFFER_REPLACEMENT_CODEPOINT_DEFAULT);
+  g_assert_true (hb_buffer_get_flags (b) != HB_BUFFER_FLAG_DEFAULT);
+  g_assert_true (hb_buffer_get_replacement_codepoint (b) != HB_BUFFER_REPLACEMENT_CODEPOINT_DEFAULT);
 
 
   /* test reset clears all properties */
 
   hb_buffer_set_direction (b, HB_DIRECTION_RTL);
-  g_assert (hb_buffer_get_direction (b) == HB_DIRECTION_RTL);
+  g_assert_true (hb_buffer_get_direction (b) == HB_DIRECTION_RTL);
 
   hb_buffer_set_script (b, HB_SCRIPT_ARABIC);
-  g_assert (hb_buffer_get_script (b) == HB_SCRIPT_ARABIC);
+  g_assert_true (hb_buffer_get_script (b) == HB_SCRIPT_ARABIC);
 
   hb_buffer_set_language (b, hb_language_from_string ("fa", -1));
-  g_assert (hb_buffer_get_language (b) == hb_language_from_string ("Fa", -1));
+  g_assert_true (hb_buffer_get_language (b) == hb_language_from_string ("Fa", -1));
 
   hb_buffer_set_flags (b, HB_BUFFER_FLAG_BOT);
-  g_assert (hb_buffer_get_flags (b) == HB_BUFFER_FLAG_BOT);
+  g_assert_true (hb_buffer_get_flags (b) == HB_BUFFER_FLAG_BOT);
 
   hb_buffer_set_replacement_codepoint (b, (unsigned int) -1);
-  g_assert (hb_buffer_get_replacement_codepoint (b) == (unsigned int) -1);
+  g_assert_true (hb_buffer_get_replacement_codepoint (b) == (unsigned int) -1);
 
   hb_buffer_reset (b);
 
-  g_assert (hb_buffer_get_unicode_funcs (b) == hb_unicode_funcs_get_default ());
-  g_assert (hb_buffer_get_direction (b) == HB_DIRECTION_INVALID);
-  g_assert (hb_buffer_get_script (b) == HB_SCRIPT_INVALID);
-  g_assert (hb_buffer_get_language (b) == NULL);
-  g_assert (hb_buffer_get_flags (b) == HB_BUFFER_FLAGS_DEFAULT);
-  g_assert (hb_buffer_get_replacement_codepoint (b) == HB_BUFFER_REPLACEMENT_CODEPOINT_DEFAULT);
+  g_assert_true (hb_buffer_get_unicode_funcs (b) == hb_unicode_funcs_get_default ());
+  g_assert_true (hb_buffer_get_direction (b) == HB_DIRECTION_INVALID);
+  g_assert_true (hb_buffer_get_script (b) == HB_SCRIPT_INVALID);
+  g_assert_true (hb_buffer_get_language (b) == NULL);
+  g_assert_true (hb_buffer_get_flags (b) == HB_BUFFER_FLAG_DEFAULT);
+  g_assert_true (hb_buffer_get_replacement_codepoint (b) == HB_BUFFER_REPLACEMENT_CODEPOINT_DEFAULT);
 }
 
 static void
-test_buffer_contents (fixture_t *fixture, gconstpointer user_data)
+test_buffer_contents (gpointer fixture_, gconstpointer user_data)
 {
+  fixture_t *fixture = fixture_;
   hb_buffer_t *b = fixture->buffer;
   unsigned int i, len, len2;
   buffer_type_t buffer_type = GPOINTER_TO_INT (user_data);
@@ -208,7 +212,7 @@ test_buffer_contents (fixture_t *fixture, gconstpointer user_data)
       if (buffer_type == BUFFER_UTF16)
 	cluster++;
       else if (buffer_type == BUFFER_UTF8)
-        cluster += 3;
+	cluster += 3;
     }
     g_assert_cmphex (glyphs[i].codepoint, ==, utf32[1+i]);
     g_assert_cmphex (glyphs[i].cluster,   ==, cluster);
@@ -269,7 +273,7 @@ test_buffer_contents (fixture_t *fixture, gconstpointer user_data)
   /* test setting length */
 
   /* enlarge */
-  g_assert (hb_buffer_set_length (b, 10));
+  g_assert_true (hb_buffer_set_length (b, 10));
   glyphs = hb_buffer_get_glyph_infos (b, NULL);
   g_assert_cmpint (hb_buffer_get_length (b), ==, 10);
   for (i = 0; i < 5; i++)
@@ -277,14 +281,14 @@ test_buffer_contents (fixture_t *fixture, gconstpointer user_data)
   for (i = 5; i < 10; i++)
     g_assert_cmphex (glyphs[i].codepoint, ==, 0);
   /* shrink */
-  g_assert (hb_buffer_set_length (b, 3));
+  g_assert_true (hb_buffer_set_length (b, 3));
   glyphs = hb_buffer_get_glyph_infos (b, NULL);
   g_assert_cmpint (hb_buffer_get_length (b), ==, 3);
   for (i = 0; i < 3; i++)
     g_assert_cmphex (glyphs[i].codepoint, ==, utf32[1+i]);
 
 
-  g_assert (hb_buffer_allocation_successful (b));
+  g_assert_true (hb_buffer_allocation_successful (b));
 
 
   /* test reset clears content */
@@ -294,8 +298,9 @@ test_buffer_contents (fixture_t *fixture, gconstpointer user_data)
 }
 
 static void
-test_buffer_positions (fixture_t *fixture, gconstpointer user_data HB_UNUSED)
+test_buffer_positions (gpointer fixture_, gconstpointer user_data HB_UNUSED)
 {
+  fixture_t *fixture = fixture_;
   hb_buffer_t *b = fixture->buffer;
   unsigned int i, len, len2;
   hb_glyph_position_t *positions;
@@ -319,46 +324,47 @@ test_buffer_positions (fixture_t *fixture, gconstpointer user_data HB_UNUSED)
 }
 
 static void
-test_buffer_allocation (fixture_t *fixture, gconstpointer user_data HB_UNUSED)
+test_buffer_allocation (gpointer fixture_, gconstpointer user_data HB_UNUSED)
 {
+  fixture_t *fixture = fixture_;
   hb_buffer_t *b = fixture->buffer;
 
   g_assert_cmpint (hb_buffer_get_length (b), ==, 0);
 
-  g_assert (hb_buffer_pre_allocate (b, 100));
+  g_assert_true (hb_buffer_pre_allocate (b, 100));
   g_assert_cmpint (hb_buffer_get_length (b), ==, 0);
-  g_assert (hb_buffer_allocation_successful (b));
+  g_assert_true (hb_buffer_allocation_successful (b));
 
   /* lets try a huge allocation, make sure it fails */
-  g_assert (!hb_buffer_pre_allocate (b, (unsigned int) -1));
+  g_assert_true (!hb_buffer_pre_allocate (b, (unsigned int) -1));
   g_assert_cmpint (hb_buffer_get_length (b), ==, 0);
-  g_assert (!hb_buffer_allocation_successful (b));
+  g_assert_true (!hb_buffer_allocation_successful (b));
 
   /* small one again */
-  g_assert (hb_buffer_pre_allocate (b, 50));
+  g_assert_true (hb_buffer_pre_allocate (b, 50));
   g_assert_cmpint (hb_buffer_get_length (b), ==, 0);
-  g_assert (!hb_buffer_allocation_successful (b));
+  g_assert_true (!hb_buffer_allocation_successful (b));
 
   hb_buffer_reset (b);
-  g_assert (hb_buffer_allocation_successful (b));
+  g_assert_true (hb_buffer_allocation_successful (b));
 
   /* all allocation and size  */
-  g_assert (!hb_buffer_pre_allocate (b, ((unsigned int) -1) / 20 + 1));
-  g_assert (!hb_buffer_allocation_successful (b));
+  g_assert_true (!hb_buffer_pre_allocate (b, ((unsigned int) -1) / 20 + 1));
+  g_assert_true (!hb_buffer_allocation_successful (b));
 
   hb_buffer_reset (b);
-  g_assert (hb_buffer_allocation_successful (b));
+  g_assert_true (hb_buffer_allocation_successful (b));
 
   /* technically, this one can actually pass on 64bit machines, but
    * I'm doubtful that any malloc allows 4GB allocations at a time.
    * But let's only enable it on a 32-bit machine. */
   if (sizeof (long) == 4) {
-    g_assert (!hb_buffer_pre_allocate (b, ((unsigned int) -1) / 20 - 1));
-    g_assert (!hb_buffer_allocation_successful (b));
+    g_assert_true (!hb_buffer_pre_allocate (b, ((unsigned int) -1) / 20 - 1));
+    g_assert_true (!hb_buffer_allocation_successful (b));
   }
 
   hb_buffer_reset (b);
-  g_assert (hb_buffer_allocation_successful (b));
+  g_assert_true (hb_buffer_allocation_successful (b));
 }
 
 
@@ -698,9 +704,9 @@ test_buffer_utf8_validity (void)
       if (glyphs[j].codepoint == (hb_codepoint_t) -1)
 	break;
 
-    g_assert (test->valid ? j == len : j < len);
+    g_assert_true (test->valid ? j == len : j < len);
     if (!test->valid)
-      g_assert (glyphs[j].cluster == test->offset);
+      g_assert_true (glyphs[j].cluster == test->offset);
   }
 
   hb_buffer_destroy (b);
@@ -816,8 +822,8 @@ static void
 test_empty (hb_buffer_t *b)
 {
   g_assert_cmpint (hb_buffer_get_length (b), ==, 0);
-  g_assert (!hb_buffer_get_glyph_infos (b, NULL));
-  g_assert (!hb_buffer_get_glyph_positions (b, NULL));
+  g_assert_true (!hb_buffer_get_glyph_infos (b, NULL));
+  g_assert_true (!hb_buffer_get_glyph_positions (b, NULL));
 }
 
 static void
@@ -825,10 +831,10 @@ test_buffer_empty (void)
 {
   hb_buffer_t *b = hb_buffer_get_empty ();
 
-  g_assert (hb_buffer_get_empty ());
-  g_assert (hb_buffer_get_empty () == b);
+  g_assert_true (hb_buffer_get_empty ());
+  g_assert_true (hb_buffer_get_empty () == b);
 
-  g_assert (!hb_buffer_allocation_successful (b));
+  g_assert_true (!hb_buffer_allocation_successful (b));
 
   test_empty (b);
 
@@ -839,21 +845,105 @@ test_buffer_empty (void)
   hb_buffer_reverse (b);
   hb_buffer_reverse_clusters (b);
 
-  g_assert (!hb_buffer_set_length (b, 10));
+  g_assert_true (!hb_buffer_set_length (b, 10));
 
   test_empty (b);
 
-  g_assert (hb_buffer_set_length (b, 0));
+  g_assert_true (hb_buffer_set_length (b, 0));
 
   test_empty (b);
 
-  g_assert (!hb_buffer_allocation_successful (b));
+  g_assert_true (!hb_buffer_allocation_successful (b));
 
   hb_buffer_reset (b);
 
   test_empty (b);
 
-  g_assert (!hb_buffer_allocation_successful (b));
+  g_assert_true (!hb_buffer_allocation_successful (b));
+}
+
+typedef struct {
+  const char *contents;
+  hb_buffer_serialize_format_t format;
+  int parsed_length;
+  unsigned int num_items;
+  hb_bool_t success;
+} serialization_test_t;
+
+static const serialization_test_t serialization_tests[] = {
+  { "<U+0640=0|U+0635=1>", HB_BUFFER_SERIALIZE_FORMAT_TEXT, -1, 2, TRUE },
+  { "[{\"u\":1600,\"cl\":0},{\"u\":1589,\"cl\":1}]", HB_BUFFER_SERIALIZE_FORMAT_JSON, -1, 2, TRUE },
+
+  /* Unfinished */
+  { "[{\"u\":1600,\"cl\":0},{\"u\":1589,\"cl\":1}", HB_BUFFER_SERIALIZE_FORMAT_JSON, 36, 1, TRUE },
+
+  /* Mixed glyphs/Unicodes -> parse fail */
+  { "[{\"u\":1600,\"cl\":0},{\"g\":1589,\"cl\":1}]", HB_BUFFER_SERIALIZE_FORMAT_JSON, -1, 0, FALSE },
+  { "<U+0640=0|uni0635=1>", HB_BUFFER_SERIALIZE_FORMAT_TEXT, -1, 0, FALSE },
+};
+
+static void
+test_buffer_serialize_deserialize (void)
+{
+  hb_buffer_t *b;
+  unsigned int i;
+
+  for (i = 0; i < G_N_ELEMENTS (serialization_tests); i++)
+  {
+    unsigned int consumed;
+    char round_trip[1024];
+    const char *end_ptr;
+
+    b = hb_buffer_create ();
+    hb_buffer_set_replacement_codepoint (b, (hb_codepoint_t) -1);
+
+    const serialization_test_t *test = &serialization_tests[i];
+    g_test_message ("serialize test #%d", i);
+
+    bool ret = hb_buffer_deserialize_unicode (b, test->contents, -1, &end_ptr, test->format);
+
+    // Expected parse failure, got one, don't round-trip
+    if (test->success)
+    {
+      unsigned int num_glyphs = hb_buffer_get_length (b);
+      g_assert_cmpint (num_glyphs, ==, test->num_items);
+
+      if (test->parsed_length != -1)
+      {
+	consumed = end_ptr - test->contents;
+	g_assert_cmpint (consumed, ==, test->parsed_length);
+      }
+      else
+      {
+	consumed = 0;
+	hb_buffer_serialize (b, 0, num_glyphs, round_trip,
+			     sizeof(round_trip), &consumed, NULL, test->format,
+			     HB_BUFFER_SERIALIZE_FLAG_DEFAULT);
+	g_assert_cmpstr (round_trip, ==, test->contents);
+      }
+    }
+    else
+      g_assert_true (!ret);
+
+    hb_buffer_destroy (b);
+
+  }
+
+  char test[1024];
+  unsigned int consumed;
+  hb_buffer_t *indeterminate = hb_buffer_get_empty ();
+  hb_buffer_serialize (indeterminate, 0, (unsigned) -1,
+		       test, sizeof(test), &consumed, NULL,
+		       HB_BUFFER_SERIALIZE_FORMAT_JSON,
+		       HB_BUFFER_SERIALIZE_FLAG_DEFAULT);
+  g_assert_cmpstr ( test, ==, "[]");
+
+  hb_buffer_serialize (indeterminate, 0, (unsigned) - 1,
+		       test, sizeof(test), &consumed, NULL,
+		       HB_BUFFER_SERIALIZE_FORMAT_TEXT,
+		       HB_BUFFER_SERIALIZE_FLAG_DEFAULT);
+  g_assert_cmpstr ( test, ==, "!!");
+
 }
 
 int
@@ -880,6 +970,7 @@ main (int argc, char **argv)
   hb_test_add (test_buffer_utf16_conversion);
   hb_test_add (test_buffer_utf32_conversion);
   hb_test_add (test_buffer_empty);
+  hb_test_add (test_buffer_serialize_deserialize);
 
   return hb_test_run();
 }
