@@ -223,6 +223,11 @@ namespace bitfieldReverse
 		}
 	};
 
+#		if GLM_COMPILER & GLM_COMPILER_VC
+#			pragma warning(push)
+#			pragma warning(disable : 4309)
+#		endif
+
 	template<glm::length_t L, typename T, glm::qualifier Q>
 	GLM_FUNC_QUALIFIER glm::vec<L, T, Q> bitfieldReverseOps(glm::vec<L, T, Q> const& v)
 	{
@@ -236,12 +241,21 @@ namespace bitfieldReverse
 		return x;
 	}
 
+#		if GLM_COMPILER & GLM_COMPILER_VC
+#			pragma warning(pop)
+#		endif
+
 	template<typename genType>
 	GLM_FUNC_QUALIFIER genType bitfieldReverseOps(genType x)
 	{
 		return bitfieldReverseOps(glm::vec<1, genType, glm::defaultp>(x)).x;
 	}
 
+#if GLM_COMPILER & GLM_COMPILER_CLANG
+#	pragma clang diagnostic push
+#	pragma clang diagnostic ignored "-Wpadded"
+#endif
+
 	template<typename genType>
 	struct type
 	{
@@ -250,6 +264,10 @@ namespace bitfieldReverse
 		result		Result;
 	};
 
+#if GLM_COMPILER & GLM_COMPILER_CLANG
+#	pragma clang diagnostic pop
+#endif
+
 	typedef type<glm::uint> typeU32;
 
 	typeU32 const Data32[] =
@@ -1181,7 +1199,7 @@ namespace usubBorrow
 			glm::uint Result = glm::usubBorrow(x, y, Borrow);
 
 			Error += Borrow == 1 ? 0 : 1;
-			Error += Result == 1 ? 0 : 1;
+			Error += Result == glm::uint(x-y) ? 0 : 1;
 		}
 
 		{
@@ -1191,7 +1209,7 @@ namespace usubBorrow
 			glm::uvec1 Result(glm::usubBorrow(x, y, Borrow));
 
 			Error += glm::all(glm::equal(Borrow, glm::uvec1(1))) ? 0 : 1;
-			Error += glm::all(glm::equal(Result, glm::uvec1(1))) ? 0 : 1;
+			Error += glm::all(glm::equal(Result, glm::uvec1(x-y))) ? 0 : 1;
 		}
 
 		{
@@ -1201,7 +1219,7 @@ namespace usubBorrow
 			glm::uvec2 Result(glm::usubBorrow(x, y, Borrow));
 
 			Error += glm::all(glm::equal(Borrow, glm::uvec2(1))) ? 0 : 1;
-			Error += glm::all(glm::equal(Result, glm::uvec2(1))) ? 0 : 1;
+			Error += glm::all(glm::equal(Result, glm::uvec2(x-y))) ? 0 : 1;
 		}
 
 		{
@@ -1211,7 +1229,7 @@ namespace usubBorrow
 			glm::uvec3 Result(glm::usubBorrow(x, y, Borrow));
 
 			Error += glm::all(glm::equal(Borrow, glm::uvec3(1))) ? 0 : 1;
-			Error += glm::all(glm::equal(Result, glm::uvec3(1))) ? 0 : 1;
+			Error += glm::all(glm::equal(Result, glm::uvec3(x-y))) ? 0 : 1;
 		}
 
 		{
@@ -1221,7 +1239,7 @@ namespace usubBorrow
 			glm::uvec4 Result(glm::usubBorrow(x, y, Borrow));
 
 			Error += glm::all(glm::equal(Borrow, glm::uvec4(1))) ? 0 : 1;
-			Error += glm::all(glm::equal(Result, glm::uvec4(1))) ? 0 : 1;
+			Error += glm::all(glm::equal(Result, glm::uvec4(x-y))) ? 0 : 1;
 		}
 
 		return Error;
@@ -1423,10 +1441,15 @@ namespace bitCount
 		}
 	};
 
+#		if GLM_COMPILER & GLM_COMPILER_VC
+#			pragma warning(push)
+#			pragma warning(disable : 4309)
+#		endif
+
 	template<glm::length_t L, typename T, glm::qualifier Q>
 	static glm::vec<L, int, Q> bitCount_bitfield(glm::vec<L, T, Q> const& v)
 	{
-		glm::vec<L, typename glm::detail::make_unsigned<T>::type, Q> x(*reinterpret_cast<glm::vec<L, typename glm::detail::make_unsigned<T>::type, Q> const *>(&v));
+		glm::vec<L, typename glm::detail::make_unsigned<T>::type, Q> x(v);
 		x = compute_bitfieldBitCountStep<sizeof(T) * 8 >=  2>::call(x, static_cast<typename glm::detail::make_unsigned<T>::type>(0x5555555555555555ull), static_cast<typename glm::detail::make_unsigned<T>::type>( 1));
 		x = compute_bitfieldBitCountStep<sizeof(T) * 8 >=  4>::call(x, static_cast<typename glm::detail::make_unsigned<T>::type>(0x3333333333333333ull), static_cast<typename glm::detail::make_unsigned<T>::type>( 2));
 		x = compute_bitfieldBitCountStep<sizeof(T) * 8 >=  8>::call(x, static_cast<typename glm::detail::make_unsigned<T>::type>(0x0F0F0F0F0F0F0F0Full), static_cast<typename glm::detail::make_unsigned<T>::type>( 4));
@@ -1436,6 +1459,10 @@ namespace bitCount
 		return glm::vec<L, int, Q>(x);
 	}
 
+#		if GLM_COMPILER & GLM_COMPILER_VC
+#			pragma warning(pop)
+#		endif
+
 	template<typename genType>
 	static int bitCount_bitfield(genType x)
 	{
@@ -1510,15 +1537,19 @@ namespace bitCount
 		for(std::size_t i = 0, n = sizeof(DataI32) / sizeof(type<int>); i < n; ++i)
 		{
 			int ResultA = glm::bitCount(DataI32[i].Value);
-			int ResultB = bitCount_if(DataI32[i].Value);
-			int ResultC = bitCount_vec(DataI32[i].Value);
-			int ResultE = bitCount_bitfield(DataI32[i].Value);
-
 			Error += DataI32[i].Return == ResultA ? 0 : 1;
-			Error += DataI32[i].Return == ResultB ? 0 : 1;
-			Error += DataI32[i].Return == ResultC ? 0 : 1;
-			Error += DataI32[i].Return == ResultE ? 0 : 1;
+			assert(!Error);
 
+			int ResultB = bitCount_if(DataI32[i].Value);
+			Error += DataI32[i].Return == ResultB ? 0 : 1;
+			assert(!Error);
+
+			int ResultC = bitCount_vec(DataI32[i].Value);
+			Error += DataI32[i].Return == ResultC ? 0 : 1;
+			assert(!Error);
+
+			int ResultE = bitCount_bitfield(DataI32[i].Value);
+			Error += DataI32[i].Return == ResultE ? 0 : 1;
 			assert(!Error);
 		}
 
