@@ -44,7 +44,6 @@ BEGIN {
 }
 
 use Memoize;
-use MIME::Base64;
 
 my @xml;      # test data file contents
 my $xmlfile;  # test data file name
@@ -132,7 +131,6 @@ sub getpart {
 
     my @this;
     my $inside=0;
-    my $base64=0;
     my $hex=0;
     my $line;
 
@@ -145,10 +143,6 @@ sub getpart {
             if($inside > 1) {
                 push @this, $_;
             }
-            elsif($_ =~ /$part [^>]*base64=/) {
-                # attempt to detect our base64 encoded part
-                $base64=1;
-            }
             elsif($_ =~ /$part [^>]*hex=/) {
                 # attempt to detect a hex-encoded part
                 $hex=1;
@@ -172,14 +166,7 @@ sub getpart {
             if($warning && !@this) {
                 print STDERR "*** getpart.pm: $section/$part returned empty!\n";
             }
-            if($base64) {
-                # decode the whole array before returning it!
-                for(@this) {
-                    my $decoded = decode_base64($_);
-                    $_ = $decoded;
-                }
-            }
-            elsif($hex) {
+            if($hex) {
                 # decode the whole array before returning it!
                 for(@this) {
                     my $decoded = decode_hex($_);
@@ -306,56 +293,56 @@ sub striparray {
 # pass array *REFERENCES* !
 #
 sub compareparts {
- my ($firstref, $secondref)=@_;
+    my ($firstref, $secondref)=@_;
 
- # we cannot compare arrays index per index since with the base64 chunks,
- # they may not be "evenly" distributed
- my $first = join("", @$firstref);
- my $second = join("", @$secondref);
+    # we cannot compare arrays index per index since with data chunks,
+    # they may not be "evenly" distributed
+    my $first = join("", @$firstref);
+    my $second = join("", @$secondref);
 
- if($first =~ /%alternatives\[/) {
-     die "bad use of compareparts\n";
- }
+    if($first =~ /%alternatives\[/) {
+        die "bad use of compareparts\n";
+    }
 
- if($second =~ /%alternatives\[([^,]*),([^\]]*)\]/) {
-     # there can be many %alternatives in this chunk, so we call
-     # this function recursively
-     my $alt = $second;
-     $alt =~ s/%alternatives\[([^,]*),([^\]]*)\]/$1/;
+    if($second =~ /%alternatives\[([^,]*),([^\]]*)\]/) {
+        # there can be many %alternatives in this chunk, so we call
+        # this function recursively
+        my $alt = $second;
+        $alt =~ s/%alternatives\[([^,]*),([^\]]*)\]/$1/;
 
-     # check first alternative
-     {
-         my @f;
-         my @s;
-         push @f, $first;
-         push @s, $alt;
-         if(!compareparts(\@f, \@s)) {
-             return 0;
-         }
-     }
+        # check first alternative
+        {
+            my @f;
+            my @s;
+            push @f, $first;
+            push @s, $alt;
+            if(!compareparts(\@f, \@s)) {
+                return 0;
+            }
+        }
 
-     $alt = $second;
-     $alt =~ s/%alternatives\[([^,]*),([^\]]*)\]/$2/;
-     # check second alternative
-     {
-         my @f;
-         my @s;
-         push @f, $first;
-         push @s, $alt;
-         if(!compareparts(\@f, \@s)) {
-             return 0;
-         }
-     }
+        $alt = $second;
+        $alt =~ s/%alternatives\[([^,]*),([^\]]*)\]/$2/;
+        # check second alternative
+        {
+            my @f;
+            my @s;
+            push @f, $first;
+            push @s, $alt;
+            if(!compareparts(\@f, \@s)) {
+                return 0;
+            }
+        }
 
-     # neither matched
-     return 1;
- }
+        # neither matched
+        return 1;
+    }
 
- if($first ne $second) {
-     return 1;
- }
+    if($first ne $second) {
+        return 1;
+    }
 
- return 0;
+    return 0;
 }
 
 #
