@@ -19,6 +19,7 @@
 #include "src/gpu/graphite/KeyContext.h"
 #include "src/gpu/graphite/PipelineData.h"
 #include "src/gpu/graphite/PrecompileInternal.h"
+#include "src/gpu/graphite/RenderPassDesc.h"
 #include "src/gpu/graphite/Renderer.h"
 #include "src/gpu/graphite/RuntimeEffectDictionary.h"
 #include "src/gpu/graphite/precompile/PaintOptionsPriv.h"
@@ -41,16 +42,17 @@ static constexpr int kExpectedTableCFCombos = 1;
 
 // shaders
 static constexpr int kExpectedGradientCombos = 3;
-static constexpr int kExpectedImageCombos = 6;
+static constexpr int kExpectedImageCombos = 16;
 static constexpr int kExpectedPerlinNoiseCombos = 1;
-static constexpr int kExpectedPictureCombos = 12;
-static constexpr int kExpectedRawImageCombos = 3;
+static constexpr int kExpectedPictureCombos = 32;
+static constexpr int kExpectedRawImageCombos = 6;
 static constexpr int kExpectedSolidColorCombos = 1;
 
 
 // A default kSrcOver blend mode will be supplied if no other blend options are added
 void no_blend_mode_option_test(const KeyContext& keyContext,
                                PipelineDataGatherer* gatherer,
+                               const RenderPassDesc& renderPassDesc,
                                skiatest::Reporter* reporter) {
     PaintOptions paintOptions;
     paintOptions.setShaders({ PrecompileShaders::Color() });
@@ -63,10 +65,12 @@ void no_blend_mode_option_test(const KeyContext& keyContext,
                                           DrawTypeFlags::kNone,
                                           /* withPrimitiveBlender= */ false,
                                           Coverage::kNone,
+                                          renderPassDesc,
                                           [&precompileIDs](UniquePaintParamsID id,
                                                            DrawTypeFlags,
                                                            bool /* withPrimitiveBlender */,
-                                                           Coverage) {
+                                                           Coverage,
+                                                           const RenderPassDesc&) {
                                                                precompileIDs.push_back(id);
                                                            });
 
@@ -77,6 +81,7 @@ void no_blend_mode_option_test(const KeyContext& keyContext,
 // generated by 'buildCombinations' agree with the expected number of combinations.
 void run_test(const KeyContext& keyContext,
               PipelineDataGatherer* gatherer,
+              const RenderPassDesc& renderPassDesc,
               skiatest::Reporter* reporter,
               const PaintOptions& paintOptions,
               int expectedNumOptions) {
@@ -90,10 +95,12 @@ void run_test(const KeyContext& keyContext,
                                           DrawTypeFlags::kNone,
                                           /* withPrimitiveBlender= */ false,
                                           Coverage::kNone,
+                                          renderPassDesc,
                                           [&precompileIDs](UniquePaintParamsID id,
                                                            DrawTypeFlags,
                                                            bool /* withPrimitiveBlender */,
-                                                           Coverage) {
+                                                           Coverage,
+                                                           const RenderPassDesc&) {
                                               precompileIDs.push_back(id);
                                           });
 
@@ -102,19 +109,20 @@ void run_test(const KeyContext& keyContext,
 
 void big_test(const KeyContext& keyContext,
               PipelineDataGatherer* gatherer,
+              const RenderPassDesc& renderPassDesc,
               skiatest::Reporter* reporter) {
 
-    static constexpr int kNumExpected = 444;
-    // paintOptions (444 = 4*111)
-    //  |- (111 = 3+108) sweepGrad_0 (3) |
-    //  |                blendShader_0 (108 = 1*4*27)
+    static constexpr int kNumExpected = 1084;
+    // paintOptions (1084 = 4*271)
+    //  |- (271 = 3+268) sweepGrad_0 (3) |
+    //  |                blendShader_0 (268 = 1*4*67)
     //  |                 |- 0: (1)       kSrc (1)
     //  |                 |- 1: (4=3+1)   (dsts) linearGrad_0 (3) | solid_0 (1)
-    //  |                 |- 2: (27=3+24) (srcs) linearGrad_1 (3) |
-    //  |                                        blendShader_1 (24=1*4*6)
+    //  |                 |- 2: (67=3+64) (srcs) linearGrad_1 (3) |
+    //  |                                        blendShader_1 (64=1*4*16)
     //  |                                         |- 0: (1) kDst (1)
     //  |                                         |- 1: (4=3+1) (dsts) radGrad_0 (3) | solid_1 (1)
-    //  |                                         |- 2: (6) (srcs) imageShader_0 (6)
+    //  |                                         |- 2: (16) (srcs) imageShader_0 (16)
     //  |
     //  |- (4) 4-built-in-blend-modes
 
@@ -168,10 +176,12 @@ void big_test(const KeyContext& keyContext,
                                           DrawTypeFlags::kNone,
                                           /* withPrimitiveBlender= */ false,
                                           Coverage::kNone,
+                                          renderPassDesc,
                                           [&precompileIDs](UniquePaintParamsID id,
                                                            DrawTypeFlags,
                                                            bool /* withPrimitiveBlender */,
-                                                           Coverage) {
+                                                           Coverage,
+                                                           const RenderPassDesc&) {
                                                                precompileIDs.push_back(id);
                                                            });
 
@@ -215,6 +225,7 @@ std::vector<sk_sp<T>> create_runtime_combos(
 
 void runtime_effect_test(const KeyContext& keyContext,
                          PipelineDataGatherer* gatherer,
+                         const RenderPassDesc& renderPassDesc,
                          skiatest::Reporter* reporter) {
     // paintOptions (64 = 4*4*4)
     //  |- combineShader (4)
@@ -332,10 +343,12 @@ void runtime_effect_test(const KeyContext& keyContext,
                                           DrawTypeFlags::kNone,
                                           /* withPrimitiveBlender= */ false,
                                           Coverage::kNone,
+                                          renderPassDesc,
                                           [&precompileIDs](UniquePaintParamsID id,
                                                            DrawTypeFlags,
                                                            bool /* withPrimitiveBlender */,
-                                                           Coverage) {
+                                                           Coverage,
+                                                           const RenderPassDesc&) {
                                                                precompileIDs.push_back(id);
                                                            });
 
@@ -345,13 +358,15 @@ void runtime_effect_test(const KeyContext& keyContext,
 // Exercise all the PrecompileBlenders factories
 void blend_subtest(const KeyContext& keyContext,
                    PipelineDataGatherer* gatherer,
+                   const RenderPassDesc& renderPassDesc,
                    skiatest::Reporter* reporter) {
     // The BlendMode PrecompileBlender only ever has 1 combination
     {
         PaintOptions paintOptions;
         paintOptions.setBlenders({ PrecompileBlenders::Mode(SkBlendMode::kColorDodge) });
 
-        run_test(keyContext, gatherer, reporter, paintOptions, /* expectedNumOptions= */ 1);
+        run_test(keyContext, gatherer, renderPassDesc,
+                 reporter, paintOptions, /* expectedNumOptions= */ 1);
     }
 
     // Specifying the BlendMode PrecompileBlender by SkBlendMode should also only ever
@@ -360,7 +375,8 @@ void blend_subtest(const KeyContext& keyContext,
         PaintOptions paintOptions;
         paintOptions.setBlendModes({ SkBlendMode::kSrcOver });
 
-        run_test(keyContext, gatherer, reporter, paintOptions, /* expectedNumOptions= */ 1);
+        run_test(keyContext, gatherer, renderPassDesc,
+                 reporter, paintOptions, /* expectedNumOptions= */ 1);
     }
 
     // The Arithmetic PrecompileBlender only ever has 1 combination
@@ -368,19 +384,22 @@ void blend_subtest(const KeyContext& keyContext,
         PaintOptions paintOptions;
         paintOptions.setBlenders({ PrecompileBlenders::Arithmetic() });
 
-        run_test(keyContext, gatherer, reporter, paintOptions, /* expectedNumOptions= */ 1);
+        run_test(keyContext, gatherer, renderPassDesc,
+                 reporter, paintOptions, /* expectedNumOptions= */ 1);
     }
 }
 
 // Exercise all the PrecompileShaders factories
 void shader_subtest(const KeyContext& keyContext,
                     PipelineDataGatherer* gatherer,
+                    const RenderPassDesc& renderPassDesc,
                     skiatest::Reporter* reporter) {
     {
         PaintOptions paintOptions;
         paintOptions.setShaders({ PrecompileShaders::Empty() });
 
-        run_test(keyContext, gatherer, reporter, paintOptions, /* expectedNumOptions= */ 1);
+        run_test(keyContext, gatherer, renderPassDesc,
+                 reporter, paintOptions, /* expectedNumOptions= */ 1);
     }
 
     // The solid color shader only ever generates one combination. Because it is constant
@@ -390,7 +409,7 @@ void shader_subtest(const KeyContext& keyContext,
         PaintOptions paintOptions;
         paintOptions.setShaders({ PrecompileShaders::Color() });
 
-        run_test(keyContext, gatherer, reporter, paintOptions,
+        run_test(keyContext, gatherer, renderPassDesc, reporter, paintOptions,
                  /* expectedNumOptions= */ kExpectedSolidColorCombos);
     }
 
@@ -414,28 +433,30 @@ void shader_subtest(const KeyContext& keyContext,
                                            { PrecompileShaders::Color() },
                                            { PrecompileShaders::MakeFractalNoise() }) });
 
-        run_test(keyContext, gatherer, reporter, paintOptions,
+        run_test(keyContext, gatherer, renderPassDesc, reporter, paintOptions,
                  /* expectedNumOptions= */ 4 *  // Porter-Duff, HSLC, Screen, Darken
                                            kExpectedSolidColorCombos *
                                            kExpectedPerlinNoiseCombos);
     }
 
-    // The ImageShaders have 6 combinations (3 sampling/tiling x 2 alpha/non-alpha)
+    // The ImageShaders have 16 combinations
+    // (4 sampling/tiling x 4 color space xform)
     // The CoordClamp shader doesn't add any additional combinations to its wrapped shader.
     {
         PaintOptions paintOptions;
         paintOptions.setShaders({ PrecompileShaders::CoordClamp({ PrecompileShaders::Image() }) });
 
-        run_test(keyContext, gatherer, reporter, paintOptions,
+        run_test(keyContext, gatherer, renderPassDesc, reporter, paintOptions,
                  /* expectedNumOptions= */ kExpectedImageCombos);
     }
 
-    // RawImageShaders only have 3 combinations (since they never incorporate alpha)
+    // RawImageShaders only have 6 combinations since they never do cubic filtering and only have
+    // two possible color space xform variants.
     {
         PaintOptions paintOptions;
         paintOptions.setShaders({ PrecompileShaders::RawImage() });
 
-        run_test(keyContext, gatherer, reporter, paintOptions,
+        run_test(keyContext, gatherer, renderPassDesc, reporter, paintOptions,
                  /* expectedNumOptions= */ kExpectedRawImageCombos);
     }
 
@@ -445,7 +466,7 @@ void shader_subtest(const KeyContext& keyContext,
         paintOptions.setShaders({ PrecompileShaders::MakeFractalNoise(),
                                   PrecompileShaders::MakeTurbulence() });
 
-        run_test(keyContext, gatherer, reporter, paintOptions,
+        run_test(keyContext, gatherer, renderPassDesc, reporter, paintOptions,
                  /* expectedNumOptions= */ kExpectedPerlinNoiseCombos + kExpectedPerlinNoiseCombos);
     }
 
@@ -457,18 +478,18 @@ void shader_subtest(const KeyContext& keyContext,
                                   PrecompileShaders::TwoPointConicalGradient(),
                                   PrecompileShaders::SweepGradient() });
 
-        run_test(keyContext, gatherer, reporter, paintOptions,
+        run_test(keyContext, gatherer, renderPassDesc, reporter, paintOptions,
                  /* expectedNumOptions= */ kExpectedGradientCombos + kExpectedGradientCombos +
                                            kExpectedGradientCombos + kExpectedGradientCombos);
     }
 
-    // Each picture shader generates 12 combinations:
-    //    2 (pictureShader LM) x 6 (imageShader variations)
+    // Each picture shader generates 32 combinations:
+    //    2 (pictureShader LM) x 16 (imageShader variations)
     {
         PaintOptions paintOptions;
         paintOptions.setShaders({ PrecompileShaders::Picture() });
 
-        run_test(keyContext, gatherer, reporter, paintOptions,
+        run_test(keyContext, gatherer, renderPassDesc, reporter, paintOptions,
                  /* expectedNumOptions= */ kExpectedPictureCombos);
     }
 
@@ -479,7 +500,7 @@ void shader_subtest(const KeyContext& keyContext,
         paintOptions.setShaders(
                 { PrecompileShaders::LocalMatrix({ PrecompileShaders::LinearGradient() }) });
 
-        run_test(keyContext, gatherer, reporter, paintOptions,
+        run_test(keyContext, gatherer, renderPassDesc, reporter, paintOptions,
                  /* expectedNumOptions= */ kExpectedGradientCombos);
     }
 
@@ -490,7 +511,7 @@ void shader_subtest(const KeyContext& keyContext,
                 { PrecompileShaders::ColorFilter({ PrecompileShaders::LinearGradient() },
                                                  { PrecompileColorFilters::Blend() }) });
 
-        run_test(keyContext, gatherer, reporter, paintOptions,
+        run_test(keyContext, gatherer, renderPassDesc, reporter, paintOptions,
                  /* expectedNumOptions= */ kExpectedGradientCombos * kExpectedBlendCFCombos);
     }
 
@@ -500,7 +521,7 @@ void shader_subtest(const KeyContext& keyContext,
                 { PrecompileShaders::WorkingColorSpace({ PrecompileShaders::LinearGradient() },
                                                        { SkColorSpace::MakeSRGBLinear() }) });
 
-        run_test(keyContext, gatherer, reporter, paintOptions,
+        run_test(keyContext, gatherer, renderPassDesc, reporter, paintOptions,
                  /* expectedNumOptions= */ kExpectedGradientCombos *
                                            1 /* only one colorSpace */);
     }
@@ -512,6 +533,7 @@ void shader_subtest(const KeyContext& keyContext,
 // simply generate the cross product of their children.
 void colorfilter_subtest(const KeyContext& keyContext,
                          PipelineDataGatherer* gatherer,
+                         const RenderPassDesc& renderPassDesc,
                          skiatest::Reporter* reporter) {
 
 
@@ -527,14 +549,15 @@ void colorfilter_subtest(const KeyContext& keyContext,
                     { PrecompileColorFilters::Table(), PrecompileColorFilters::Lighting() },
                     { PrecompileColorFilters::HighContrast(), PrecompileColorFilters::Luma() }) });
 
-        run_test(keyContext, gatherer, reporter, paintOptions, kExpectedNumOptions);
+        run_test(keyContext, gatherer, renderPassDesc, reporter, paintOptions, kExpectedNumOptions);
     }
 
     {
         PaintOptions paintOptions;
         paintOptions.setColorFilters({ PrecompileColorFilters::Blend() });
 
-        run_test(keyContext, gatherer, reporter, paintOptions, kExpectedBlendCFCombos);
+        run_test(keyContext, gatherer, renderPassDesc, reporter, paintOptions,
+                 kExpectedBlendCFCombos);
     }
 
     {
@@ -543,7 +566,7 @@ void colorfilter_subtest(const KeyContext& keyContext,
                                        PrecompileColorFilters::HSLAMatrix() });
 
         // HSLAMatrix and Matrix map to the same color filter
-        run_test(keyContext, gatherer, reporter, paintOptions,
+        run_test(keyContext, gatherer, renderPassDesc, reporter, paintOptions,
                  kExpectedMatrixCFCombos + kExpectedMatrixCFCombos);
     }
 
@@ -553,7 +576,7 @@ void colorfilter_subtest(const KeyContext& keyContext,
                                        PrecompileColorFilters::SRGBToLinearGamma() });
 
         // LinearToSRGB and SRGBToLinear both map to the colorspace colorfilter
-        run_test(keyContext, gatherer, reporter, paintOptions,
+        run_test(keyContext, gatherer, renderPassDesc, reporter, paintOptions,
                  kExpectedColorSpaceCFCombos + kExpectedColorSpaceCFCombos);
     }
 
@@ -569,43 +592,41 @@ void colorfilter_subtest(const KeyContext& keyContext,
                     { PrecompileColorFilters::Matrix(), PrecompileColorFilters::Luma() },
                     { PrecompileColorFilters::Blend(), PrecompileColorFilters::Overdraw() }) });
 
-        run_test(keyContext, gatherer, reporter, paintOptions, kExpectedNumOptions);
+        run_test(keyContext, gatherer, renderPassDesc, reporter, paintOptions, kExpectedNumOptions);
     }
 }
 
 } // anonymous namespace
 
 DEF_GRAPHITE_TEST_FOR_ALL_CONTEXTS(CombinationBuilderTest, reporter, context,
-                                   CtsEnforcement::kNextRelease) {
+                                   CtsEnforcement::kNever) {
     ShaderCodeDictionary* dict = context->priv().shaderCodeDictionary();
 
     auto rtEffectDict = std::make_unique<RuntimeEffectDictionary>();
 
     SkColorInfo ci(kRGBA_8888_SkColorType, kPremul_SkAlphaType, nullptr);
-    KeyContext keyContext(context->priv().caps(),
-                          dict,
-                          rtEffectDict.get(),
-                          ci,
-                          /* dstTexture= */ nullptr,
-                          /* dstOffset= */ {0, 0});
+    KeyContext keyContext(context->priv().caps(), dict, rtEffectDict.get(), ci);
 
     PipelineDataGatherer gatherer(Layout::kMetal);
 
+    RenderPassDesc unusedRenderPassDesc;
+
     // The default PaintOptions should create a single combination with a solid color shader and
     // kSrcOver blending
     {
         PaintOptions paintOptions;
 
-        run_test(keyContext, &gatherer, reporter, paintOptions, /* expectedNumOptions= */ 1);
+        run_test(keyContext, &gatherer, unusedRenderPassDesc, reporter, paintOptions,
+                 /* expectedNumOptions= */ 1);
     }
 
-    blend_subtest(keyContext, &gatherer, reporter);
-    shader_subtest(keyContext, &gatherer, reporter);
-    colorfilter_subtest(keyContext, &gatherer, reporter);
+    blend_subtest(keyContext, &gatherer, unusedRenderPassDesc, reporter);
+    shader_subtest(keyContext, &gatherer, unusedRenderPassDesc, reporter);
+    colorfilter_subtest(keyContext, &gatherer, unusedRenderPassDesc, reporter);
 
-    no_blend_mode_option_test(keyContext, &gatherer, reporter);
-    big_test(keyContext, &gatherer, reporter);
-    runtime_effect_test(keyContext, &gatherer, reporter);
+    no_blend_mode_option_test(keyContext, &gatherer, unusedRenderPassDesc, reporter);
+    big_test(keyContext, &gatherer, unusedRenderPassDesc, reporter);
+    runtime_effect_test(keyContext, &gatherer, unusedRenderPassDesc, reporter);
 }
 
 #endif // SK_GRAPHITE
