@@ -1,6 +1,6 @@
-#!/bin/sh
+#!/bin/sh -x
 
-OS_ARCH=`uname -o`
+OS_ARCH=`uname -o 2>/dev/null || echo ""`
 OS_KERNEL=`uname -s`
 
 #
@@ -29,16 +29,19 @@ if [ "z$TMPFOLDER" = "z" ] ; then
     TMPFOLDER=/tmp
 fi
 testname=`basename $testfile`
+testfolder=$TMPFOLDER/xmlsec-$testname-$crypto-$timestamp
+mkdir -p $testfolder
+
 if test "z$OS_ARCH" = "zCygwin" || test "z$OS_ARCH" = "zMsys" ; then
-    tmpfile=`cygpath -wa $TMPFOLDER/$testname.$timestamp-$$.tmp`
-    logfile=`cygpath -wa $TMPFOLDER/$testname.$timestamp-$$.log`
-    curlogfile=`cygpath -wa $TMPFOLDER/$testname.$timestamp-$$.cur.log`
-    failedlogfile=`cygpath -wa $TMPFOLDER/$testname.$timestamp-$$.failed.log`
+    tmpfile=`cygpath -wa $testfolder/tmp.tmp`
+    logfile=`cygpath -wa $testfolder/full.log`
+    curlogfile=`cygpath -wa $testfolder/cur.log`
+    failedlogfile=`cygpath -wa $testfolder/failed.log`
 else
-    tmpfile=$TMPFOLDER/$testname.$timestamp-$$.tmp
-    logfile=$TMPFOLDER/$testname.$timestamp-$$.log
-    curlogfile=$TMPFOLDER/$testname.$timestamp-$$.cur.log
-    failedlogfile=$TMPFOLDER/$testname.$timestamp-$$.failed.log
+    tmpfile=$testfolder/tmp.tmp
+    logfile=$testfolder/full.log
+    curlogfile=$testfolder/cur.log
+    failedlogfile=$testfolder/failed.log
 fi
 
 #
@@ -63,12 +66,6 @@ if [ -n "$DEBUG_MEMORY" ] ; then
     xmlsec_params="$xmlsec_params --repeat $REPEAT"
 fi
 
-#
-# Setup crypto config folder
-#
-crypto_config_folder=$TMPFOLDER/xmlsec-crypto-config
-keysfile=$crypto_config_folder/keys.xml
-mkdir -p $crypto_config_folder
 
 #
 # Setup crypto engine
@@ -79,27 +76,6 @@ elif [ "z$crypto" != "z" ] ; then
     xmlsec_params="$xmlsec_params --crypto $crypto"
 fi
 
-# see https://learn.microsoft.com/en-us/windows/win32/api/wincrypt/nf-wincrypt-certopensystemstorea
-if [ "z$crypto" = "zmscng" ] ; then
-    crypto_config="MY"
-else
-    crypto_config="$crypto_config_folder"
-fi
-xmlsec_params="$xmlsec_params --crypto-config $crypto_config"
-
-# What flavour of OpenSSL do we have?
-case $XMLSEC_OPENSSL_VERSION in
-*LibreSSL*)
-    xmlsec_openssl_flavor="libressl"
-    ;;
-*BoringSSL*)
-    xmlsec_openssl_flavor="boringssl"
-    ;;
-*)
-    xmlsec_openssl_flavor="openssl"
-    ;;
-esac
-
 #
 # Setup extra vars
 #
@@ -114,12 +90,178 @@ if [ "z$crypto" = "zopenssl" -a "z$XMLSEC_OPENSSL_TEST_CONFIG" != "z" ] ; then
     export OPENSSL_CONF="$opensslconf"
 fi
 
-if [ "z$crypto" = "zopenssl" ] ; then
-    # phaos certs use RSA-MD5 which might be disabled
+#
+#  Configure supported features
+#
+case $XMLSEC_OPENSSL_VERSION in
+*LibreSSL*)
+    xmlsec_openssl_flavor="libressl"
+    ;;
+*BoringSSL*)
+    xmlsec_openssl_flavor="boringssl"
+    ;;
+*AWSLC*)
+    xmlsec_openssl_flavor="aws-lc"
+    ;;
+*)
+    xmlsec_openssl_flavor="openssl"
+    ;;
+esac
+
+# only original openssl supports --privkey-openssl-store
+if [ "z$crypto" = "zopenssl" -a "z$xmlsec_openssl_flavor" = "zopenssl" ] ; then
+    xmlsec_feature_openssl_store="yes"
+else
+    xmlsec_feature_openssl_store="no"
+fi
+
+# phaos certs use RSA-MD5 which might be disabled
+if [ "z$crypto" = "zopenssl" -a "z$xmlsec_openssl_flavor" != "zaws-lc" ] ; then
     extra_vars="$extra_vars OPENSSL_ENABLE_MD5_VERIFY=1"
     export OPENSSL_ENABLE_MD5_VERIFY=1
+
+    xmlsec_feature_md5_certs="yes"
+else
+    xmlsec_feature_md5_certs="no"
 fi
 
+# gcrypt doesn't support pkcs12
+if [ "z$crypto" != "zgcrypt" ] ; then
+    xmlsec_feature_pkcs12="yes"
+else
+    xmlsec_feature_pkcs12="no"
+fi
+
+# gcrypt and mscrypto don't support keynames in pkcs12
+if [ "z$crypto" != "zgcrypt" -a "z$crypto" != "zmscrypto" ] ; then
+    xmlsec_feature_pkcs12_keyname="yes"
+else
+    xmlsec_feature_pkcs12_keyname="no"
+fi
+
+# gcrypt, mscrypto, mscng, nss don't support pkcs8
+if [ "z$crypto" != "zgcrypt" -a "z$crypto" != "znss" -a "z$crypto" != "zmscrypto" -a "z$crypto" != "zmscng" ] ; then
+    xmlsec_feature_pkcs8="yes"
+else
+    xmlsec_feature_pkcs8="no"
+fi
+
+# gcrypt doesn't support pem
+# nss, mscrypto, mscng don't like private keys in pem / der
+if [ "z$crypto" != "zgcrypt" -a "z$crypto" != "znss" -a "z$crypto" != "zmscrypto" -a "z$crypto" != "zmscng" ] ; then
+    xmlsec_feature_privkey_pem="yes"
+else
+    xmlsec_feature_privkey_pem="no"
+fi
+
+# nss, mscrypto, mscng don't like private keys in pem / der
+if [ "z$crypto" != "znss"  -a "z$crypto" != "zmscrypto" -a "z$crypto" != "zmscng" ] ; then
+    xmlsec_feature_privkey_der="yes"
+else
+    xmlsec_feature_privkey_der="no"
+fi
+
+# nss, gcrypt don't support pem
+# mscrypto, mscng don't support standalong pubkeys
+if [ "z$crypto" != "zgcrypt" -a "z$crypto" != "znss" -a "z$crypto" != "zmscrypto" -a "z$crypto" != "zmscng" ] ; then
+    xmlsec_feature_pubkey_pem="yes"
+else
+    xmlsec_feature_pubkey_pem="no"
+fi
+
+# mscrypto, mscng don't support standalong pubkeys
+if [ "z$crypto" != "zmscrypto" -a "z$crypto" != "zmscng" ] ; then
+    xmlsec_feature_pubkey_der="yes"
+else
+    xmlsec_feature_pubkey_der="no"
+fi
+
+# gcrypt doesn't support certs
+# mscrypto, mscng don't support pem
+if [ "z$crypto" != "zgcrypt" -a "z$crypto" != "zmscrypto" -a "z$crypto" != "zmscng" ] ; then
+    xmlsec_feature_cert_pem="yes"
+else
+    xmlsec_feature_cert_pem="no"
+fi
+
+# gcrypt doesn't support certs
+if [ "z$crypto" != "zgcrypt" ] ; then
+    xmlsec_feature_cert_der="yes"
+else
+    xmlsec_feature_cert_der="no"
+fi
+
+# Only OpenSSL / NSS / GnuTLS currently has capability to lookup the certs/keys using X509 data
+if [ "z$crypto" = "zopenssl" -o "z$crypto" = "znss" -o "z$crypto" = "zgnutls" ] ; then
+    xmlsec_feature_x509_data_lookup="yes"
+else
+    xmlsec_feature_x509_data_lookup="no"
+fi
+
+# Only NSS can lookup certs in NSS DB, skip certs verification for signatures
+if [ "z$crypto" = "znss"  ] ; then
+    xmlsec_feature_nssdb_lookup="yes"
+else
+    xmlsec_feature_nssdb_lookup="no"
+fi
+
+# MSCng only supports SHA1 as cert digests and cannot lookup the key
+if [ "z$crypto" = "zmscng" ] ; then
+    xmlsec_feature_x509_data_lookup_digest="yes"
+else
+    xmlsec_feature_x509_data_lookup_digest="no"
+fi
+
+# currently only openssl and gnutls support skipping time checks
+# https://github.com/lsh123/xmlsec/issues/852
+if [ "z$crypto" = "zopenssl" -o "z$crypto" = "zgnutls" -o "z$crypto" = "zmscng"  ] ; then
+    xmlsec_feature_cert_check_skip_time="yes"
+else
+    xmlsec_feature_cert_check_skip_time="no"
+fi
+
+# currently only openssl/gnutls/nss support loading CRL from the command line
+# https://github.com/lsh123/xmlsec/issues/583
+if [ "z$crypto" = "zopenssl" -o  "z$crypto" = "zgnutls" -o "z$crypto" = "znss" ] ; then
+    xmlsec_feature_crl_load="yes"
+else
+    xmlsec_feature_crl_load="no"
+fi
+
+# currently only openssl/nss support CRL verification by time
+# https://github.com/lsh123/xmlsec/issues/579
+if [ "z$crypto" = "zopenssl" -o "z$crypto" = "znss"  ] ; then
+    xmlsec_feature_crl_check_skip_time="yes"
+else
+    xmlsec_feature_crl_check_skip_time="no"
+fi
+
+# only openssl, gnutls, nss, and mcng supports key verification
+# https://github.com/lsh123/xmlsec/issues/587
+if [ "z$crypto" = "zopenssl" -o  "z$crypto" = "zgnutls" -o "z$crypto" = "znss" -o "z$crypto" = "zmscng" ] ; then
+    xmlsec_feature_key_check="yes"
+else
+    xmlsec_feature_key_check="no"
+fi
+
+# Advanced RSA OAEP modes:
+# - MSCrypto only supports SHA1 for digest and mgf1
+if [ "z$crypto" != "zmscrypto" ] ; then
+    xmlsec_feature_rsa_oaep_non_sha1="yes"
+else
+    xmlsec_feature_rsa_oaep_non_sha1="no"
+fi
+
+# Advanced RSA OAEP modes:
+# - MSCrypto only supports SHA1 for digest and mgf1
+# - GCrypt/GnuTLS and MSCng only supoprts the *same* algorithm for *both* digest and mgf1
+if [ "z$crypto" != "zmscrypto" -a "z$crypto" != "zmscng" -a "z$crypto" != "zgcrypt" ] ; then
+    xmlsec_feature_rsa_oaep_different_digest_and_mgf1="yes"
+else
+    xmlsec_feature_rsa_oaep_different_digest_and_mgf1="no"
+fi
+
+
 #
 # Setup keys config
 #
@@ -191,14 +333,50 @@ else
     diff_param=-u
 fi
 
-NSS_TEST_CERT_NICKNAME="NSS Certificate DB:Aleksey Sanin - XML Security Library (http://www.aleksey.com/xmlsec)"
 
-cleanupNssCerts() {
-    echo "certutil  -D -n \"$NSS_TEST_CERT_NICKNAME\" -d \"$crypto_config_folder\""  >> $logfile
-    certutil  -D -n "$NSS_TEST_CERT_NICKNAME" -d "$crypto_config_folder"   >> $logfile 2>> $logfile
-    if [ $? -ne 0 ]; then
-        echo "--- FAILED TO DELETE TRUSTED TEST CERTIFICATE FROM NSS CERT DB. THE NEXT TEST MIGHT FAIL" >> $logfile
+#
+# Setup crypto config folder
+#
+config_number=0
+setupCryptoConfig() {
+    config_number=$((config_number + 1))
+    if test "z$OS_ARCH" = "zCygwin" || test "z$OS_ARCH" = "zMsys" ; then
+        crypto_config_folder=`cygpath -wa $testfolder/crypto-config-$config_number`
+    else
+        crypto_config_folder=$testfolder/crypto-config-$config_number
     fi
+    mkdir $crypto_config_folder
+
+    # see https://learn.microsoft.com/en-us/windows/win32/api/wincrypt/nf-wincrypt-certopensystemstorea
+    if [ "z$crypto" = "zmscng" ] ; then
+        default_crypto_config="MY"
+    else
+        default_crypto_config="$crypto_config_folder"
+    fi
+}
+
+tearDownCryptoConfig() {
+    if [ -n "$crypto_config_folder" ]; then
+        rm -rf $crypto_config_folder
+    fi
+    unset crypto_config_folder
+    unset default_crypto_config
+}
+
+setupTest() {
+    # prepare
+    old_pwd=`pwd`
+    setupCryptoConfig
+}
+
+tearDownTest() {
+    # cleanup
+    tearDownCryptoConfig
+    rm -f $tmpfile $tmpfile.2 $tmpfile.3
+    if [ -n "$old_pwd" ]; then
+        cd $old_pwd
+    fi
+    unset old_pwd
 }
 
 #
@@ -256,6 +434,10 @@ extra_message=""
 # Keys test function
 #
 execKeysTest() {
+    execKeysTestWithCryptoConfig "$1" "$2" "$3" "$4" "$5" "$6" "$7" "$8" "$9" ""
+}
+
+execKeysTestWithCryptoConfig() {
     expected_res="$1"
     req_key_data="$2"
     key_name="$3"
@@ -265,6 +447,7 @@ execKeysTest() {
     certkey_file="$7"
     asym_key_test="$8"
     key_test_options="$9"
+    crypto_config="${10}"
     failures=0
 
     if [ -n "$XMLSEC_TEST_NAME" -a "$XMLSEC_TEST_NAME" != "$key_name" ]; then
@@ -272,15 +455,23 @@ execKeysTest() {
     fi
 
     # prepare
-    rm -f $tmpfile
-    old_pwd=`pwd`
+    setupTest
+
+    if test "z$OS_ARCH" = "zCygwin" || test "z$OS_ARCH" = "zMsys" ; then
+        keysfile=`cygpath -wa $crypto_config_folder/keys.xml`
+    else
+        keysfile=$crypto_config_folder/keys.xml
+    fi
 
     # check params
     if [ "z$expected_res" != "z$res_success" -a "z$expected_res" != "z$res_fail" ] ; then
         echo " Bad parameter: expected_res=$expected_res"
-        cd $old_pwd
+        tearDownTest
         return
     fi
+    if [ "z$crypto_config" = "z" ] ; then
+        crypto_config="$default_crypto_config"
+    fi
 
     # starting test
     echo "Test: $alg_name $extra_message"
@@ -290,13 +481,13 @@ execKeysTest() {
     # check key data
     if [ -n "$req_key_data" ] ; then
         printf "    Checking required key data                            "
-        echo "$extra_vars $xmlsec_app check-key-data $xmlsec_params $req_key_data" >> $curlogfile
-        $xmlsec_app check-key-data $xmlsec_params $req_key_data >> $curlogfile 2>> $curlogfile
+        echo "$extra_vars $xmlsec_app check-key-data $xmlsec_params --crypto-config $crypto_config $req_key_data" >> $curlogfile
+        $xmlsec_app check-key-data $xmlsec_params --crypto-config $crypto_config $req_key_data >> $curlogfile 2>> $curlogfile
         printCheckStatus $?
         res=$?
         if [ $res -ne 0 ]; then
 	        cat $curlogfile >> $logfile
-	        cd $old_pwd
+	        tearDownTest
             return
         fi
     fi
@@ -310,8 +501,8 @@ execKeysTest() {
         if [ -f $keysfile ] ; then
             params="$params --keys-file $keysfile"
         fi
-        echo "$extra_vars $VALGRIND $xmlsec_app keys $params $xmlsec_params $keysfile" >>  $curlogfile
-        $VALGRIND $xmlsec_app keys $params $xmlsec_params $keysfile >> $curlogfile 2>> $curlogfile
+        echo "$extra_vars $VALGRIND $xmlsec_app keys $params $xmlsec_params  --crypto-config $crypto_config $keysfile" >>  $curlogfile
+        $VALGRIND $xmlsec_app keys $params $xmlsec_params  --crypto-config $crypto_config $keysfile >> $curlogfile 2>> $curlogfile
         printRes $expected_res $?
         if [ $? -ne 0 ]; then
             failures=`expr $failures + 1`
@@ -320,91 +511,83 @@ execKeysTest() {
 
     # test reading private keys
     if [ -n "$privkey_file" -a -n "$asym_key_test" ]; then
-        # gcrypt doesn't support pkcs12
-        if [ "z$crypto" != "zgcrypt" ] ; then
+        if [ "z$xmlsec_feature_pkcs12" = "zyes" ] ; then
             printf "    Reading private key from pkcs12 file                  "
             rm -f $tmpfile
             params="--lax-key-search --pkcs12 $privkey_file.p12 $pkcs12_key_extra_options $key_test_options --output $tmpfile $asym_key_test.tmpl"
-            echo "$extra_vars $VALGRIND $xmlsec_app sign $xmlsec_params $params" >>  $curlogfile
-            $VALGRIND $xmlsec_app sign $xmlsec_params $params >> $curlogfile 2>> $curlogfile
+            echo "$extra_vars $VALGRIND $xmlsec_app sign $xmlsec_params --crypto-config $crypto_config $params" >>  $curlogfile
+            $VALGRIND $xmlsec_app sign $xmlsec_params --crypto-config $crypto_config $params >> $curlogfile 2>> $curlogfile
             printRes $expected_res $?
             if [ $? -ne 0 ]; then
                 failures=`expr $failures + 1`
             fi
         fi
-        # only gcrypt and mscrypto doesn't support keynames in pkcs12
-        if [ "z$crypto" != "zgcrypt" -a "z$crypto" != "zmscrypto" ] ; then
+        if [ "zckcs12_keyname" = "zyes" ] ; then
             printf "    Reading private key name from pkcs12 file             "
             rm -f $tmpfile
             params="--pkcs12 $privkey_file.p12 $pkcs12_key_extra_options $key_test_options --output $tmpfile $asym_key_test.tmpl"
-            echo "$extra_vars $VALGRIND $xmlsec_app sign $xmlsec_params $params" >>  $curlogfile
-            $VALGRIND $xmlsec_app sign $xmlsec_params $params >> $curlogfile 2>> $curlogfile
+            echo "$extra_vars $VALGRIND $xmlsec_app sign $xmlsec_params --crypto-config $crypto_config $params" >>  $curlogfile
+            $VALGRIND $xmlsec_app sign $xmlsec_params --crypto-config $crypto_config $params >> $curlogfile 2>> $curlogfile
             printRes $expected_res $?
             if [ $? -ne 0 ]; then
                 failures=`expr $failures + 1`
             fi
         fi
 
-        # only openssl supports --privkey-openssl-store
-        if [ "z$crypto" = "zopenssl" -a "z$xmlsec_openssl_flavor" != "zlibressl" ] ; then
+        if [ "z$xmlsec_feature_openssl_store" = "zyes" ] ; then
             printf "    Reading private key from pkcs12 file using ossl-store "
             rm -f $tmpfile
             params="--lax-key-search --privkey-openssl-store $privkey_file.p12 $pkcs12_key_extra_options $key_test_options --output $tmpfile $asym_key_test.tmpl"
-            echo "$extra_vars $VALGRIND $xmlsec_app sign $xmlsec_params $params" >>  $curlogfile
-            $VALGRIND $xmlsec_app sign $xmlsec_params $params >> $curlogfile 2>> $curlogfile
+            echo "$extra_vars $VALGRIND $xmlsec_app sign $xmlsec_params --crypto-config $crypto_config $params" >>  $curlogfile
+            $VALGRIND $xmlsec_app sign $xmlsec_params --crypto-config $crypto_config $params >> $curlogfile 2>> $curlogfile
             printRes $expected_res $?
             if [ $? -ne 0 ]; then
                 failures=`expr $failures + 1`
             fi
         fi
 
-        # gcrypt, mscrypto, mscng, nss don't support pkcs8
-        if [ "z$crypto" != "zgcrypt" -a "z$crypto" != "znss" -a "z$crypto" != "zmscrypto" -a "z$crypto" != "zmscng" ] ; then
+        if [ "z$xmlsec_feature_pkcs8" = "zyes" ] ; then
             printf "    Reading private key from pkcs8 pem file               "
             rm -f $tmpfile
             params="--lax-key-search --pkcs8-pem $privkey_file.p8-pem $key_test_options --output $tmpfile $asym_key_test.tmpl"
-            echo "$extra_vars $VALGRIND $xmlsec_app sign $xmlsec_params $params" >>  $curlogfile
-            $VALGRIND $xmlsec_app  sign $xmlsec_params $params >> $curlogfile 2>> $curlogfile
+            echo "$extra_vars $VALGRIND $xmlsec_app sign $xmlsec_params --crypto-config $v $params" >>  $curlogfile
+            $VALGRIND $xmlsec_app  sign $xmlsec_params --crypto-config $crypto_config $params >> $curlogfile 2>> $curlogfile
             printRes $expected_res $?
             if [ $? -ne 0 ]; then
                 failures=`expr $failures + 1`
             fi
         fi
 
-        # gcrypt, mscrypto, mscng, nss don't support pkcs8
-        if [ "z$crypto" != "zgcrypt" -a "z$crypto" != "znss" -a "z$crypto" != "zmscrypto" -a "z$crypto" != "zmscng" ] ; then
+        if [ "z$xmlsec_feature_pkcs8" = "zyes" ] ; then
             printf "    Reading private key from pkcs8 der file               "
             rm -f $tmpfile
             params="--lax-key-search --pkcs8-der $privkey_file.p8-der $key_test_options --output $tmpfile $asym_key_test.tmpl"
-            echo "$extra_vars $VALGRIND $xmlsec_app sign $xmlsec_params $params" >>  $curlogfile
-            $VALGRIND $xmlsec_app  sign $xmlsec_params $params >> $curlogfile 2>> $curlogfile
+            echo "$extra_vars $VALGRIND $xmlsec_app sign $xmlsec_params --crypto-config $crypto_config $params" >>  $curlogfile
+            $VALGRIND $xmlsec_app  sign $xmlsec_params --crypto-config $crypto_config $params >> $curlogfile 2>> $curlogfile
             printRes $expected_res $?
             if [ $? -ne 0 ]; then
                 failures=`expr $failures + 1`
             fi
         fi
 
-        # gcrypt doesn't support pem
-        # nss, mscrypto, mscng don't like private keys in pem / der
-        if [ "z$crypto" != "zgcrypt" -a "z$crypto" != "znss" -a "z$crypto" != "zmscrypto" -a "z$crypto" != "zmscng" ] ; then
+        if [ "z$xmlsec_feature_privkey_pem" = "zyes" ] ; then
             printf "    Reading private key from pem file                     "
             rm -f $tmpfile
             params="--lax-key-search --privkey-pem $privkey_file.pem $key_test_options --output $tmpfile $asym_key_test.tmpl"
-            echo "$extra_vars $VALGRIND $xmlsec_app sign $xmlsec_params $params" >>  $curlogfile
-            $VALGRIND $xmlsec_app  sign $xmlsec_params $params >> $curlogfile 2>> $curlogfile
+            echo "$extra_vars $VALGRIND $xmlsec_app sign $xmlsec_params --crypto-config $crypto_config $params" >>  $curlogfile
+            $VALGRIND $xmlsec_app  sign $xmlsec_params --crypto-config $crypto_config $params >> $curlogfile 2>> $curlogfile
             printRes $expected_res $?
             if [ $? -ne 0 ]; then
                 failures=`expr $failures + 1`
             fi
         fi
 
-        # nss, mscrypto, mscng don't like private keys in pem / der
-        if [ "z$crypto" != "znss"  -a "z$crypto" != "zmscrypto" -a "z$crypto" != "zmscng" ] ; then
+        if [ "z$xmlsec_feature_privkey_der" = "zyes" ] ; then
             printf "    Reading private key from der file                     "
             rm -f $tmpfile
             params="--lax-key-search --privkey-der $privkey_file.der $key_test_options --output $tmpfile $asym_key_test.tmpl"
-            echo "$extra_vars $VALGRIND $xmlsec_app sign $xmlsec_params $params" >>  $curlogfile
-            $VALGRIND $xmlsec_app  sign $xmlsec_params $params >> $curlogfile 2>> $curlogfile
+            echo "$extra_vars $VALGRIND $xmlsec_app sign $xmlsec_params --crypto-config $crypto_config $params" >>  $curlogfile
+            $VALGRIND $xmlsec_app  sign $xmlsec_params --crypto-config $crypto_config $params >> $curlogfile 2>> $curlogfile
             printRes $expected_res $?
             if [ $? -ne 0 ]; then
                 failures=`expr $failures + 1`
@@ -414,13 +597,12 @@ execKeysTest() {
 
     # test reading public keys
     if [ -n "$pubkey_file" -a -n "$asym_key_test" ]; then
-        # only openssl supports --pubkey-openssl-store
-        if [ "z$crypto" = "zopenssl" -a "z$xmlsec_openssl_flavor" != "zlibressl" ] ; then
+        if [ "z$xmlsec_feature_openssl_store" = "zyes" ] ; then
             printf "    Reading public key from pem file using ossl-store     "
             rm -f $tmpfile
             params="--lax-key-search --pubkey-openssl-store $pubkey_file.pem $key_test_options $asym_key_test.xml"
-            echo "$extra_vars $VALGRIND $xmlsec_app verify $xmlsec_params $params" >>  $curlogfile
-            $VALGRIND $xmlsec_app verify $xmlsec_params $params >> $curlogfile 2>> $curlogfile
+            echo "$extra_vars $VALGRIND $xmlsec_app verify $xmlsec_params --crypto-config $crypto_config $params" >>  $curlogfile
+            $VALGRIND $xmlsec_app verify $xmlsec_params --crypto-config $crypto_config $params >> $curlogfile 2>> $curlogfile
             printRes $expected_res $?
             if [ $? -ne 0 ]; then
                 failures=`expr $failures + 1`
@@ -428,14 +610,12 @@ execKeysTest() {
 
         fi
 
-        # nss, gcrypt don't support pem
-        # mscrypto, mscng don't support standalong pubkeys
-        if [ "z$crypto" != "zgcrypt" -a "z$crypto" != "znss" -a "z$crypto" != "zmscrypto" -a "z$crypto" != "zmscng" ] ; then
+        if [ "z$xmlsec_feature_pubkey_pem" = "zyes" ] ; then
             printf "    Reading public key from pem file                      "
             rm -f $tmpfile
             params="--lax-key-search --pubkey-pem $pubkey_file.pem $key_test_options $asym_key_test.xml"
-            echo "$extra_vars $VALGRIND $xmlsec_app verify $xmlsec_params $params" >>  $curlogfile
-            $VALGRIND $xmlsec_app verify $xmlsec_params $params >> $curlogfile 2>> $curlogfile
+            echo "$extra_vars $VALGRIND $xmlsec_app verify $xmlsec_params --crypto-config $crypto_config $params" >>  $curlogfile
+            $VALGRIND $xmlsec_app verify $xmlsec_params --crypto-config $crypto_config $params >> $curlogfile 2>> $curlogfile
             printRes $expected_res $?
             if [ $? -ne 0 ]; then
                 failures=`expr $failures + 1`
@@ -446,13 +626,12 @@ execKeysTest() {
         if [ "z$crypto" = "zgcrypt" -a "z$req_key_data" = "zrsa" ] ; then
             pubkey_file="$pubkey_file-gcrypt"
         fi
-        # mscrypto, mscng don't support standalong pubkeys
-        if [ "z$crypto" != "zmscrypto" -a "z$crypto" != "zmscng" ] ; then
+        if [ "z$xmlsec_feature_pubkey_der" = "zyes" ] ; then
             printf "    Reading public key from der file                      "
             rm -f $tmpfile
             params="--lax-key-search --pubkey-der $pubkey_file.der $key_test_options $asym_key_test.xml"
-            echo "$extra_vars $VALGRIND $xmlsec_app verify $xmlsec_params $params" >>  $curlogfile
-            $VALGRIND $xmlsec_app verify $xmlsec_params $params >> $curlogfile 2>> $curlogfile
+            echo "$extra_vars $VALGRIND $xmlsec_app verify $xmlsec_params --crypto-config $crypto_config $params" >>  $curlogfile
+            $VALGRIND $xmlsec_app verify $xmlsec_params --crypto-config $crypto_config $params >> $curlogfile 2>> $curlogfile
             printRes $expected_res $?
             if [ $? -ne 0 ]; then
                 failures=`expr $failures + 1`
@@ -461,27 +640,24 @@ execKeysTest() {
     fi
 
     if [ -n "$certkey_file" -a -n "$asym_key_test" ]; then
-        # gcrypt doesn't support cert
-        # mscrypto, mscng don't support pem
-        if [ "z$crypto" != "zgcrypt" -a "z$crypto" != "zmscrypto" -a "z$crypto" != "zmscng" ] ; then
+        if [ "z$xmlsec_feature_cert_pem" = "zyes" ] ; then
             printf "    Reading public key from pem cert file                 "
             rm -f $tmpfile
             params="--lax-key-search --pubkey-cert-pem $certkey_file.pem $key_test_options $asym_key_test.xml"
-            echo "$extra_vars $VALGRIND $xmlsec_app verify $xmlsec_params $params" >>  $curlogfile
-            $VALGRIND $xmlsec_app verify $xmlsec_params $params >> $curlogfile 2>> $curlogfile
+            echo "$extra_vars $VALGRIND $xmlsec_app verify $xmlsec_params --crypto-config $crypto_config $params" >>  $curlogfile
+            $VALGRIND $xmlsec_app verify $xmlsec_params --crypto-config $crypto_config $params >> $curlogfile 2>> $curlogfile
             printRes $expected_res $?
             if [ $? -ne 0 ]; then
                 failures=`expr $failures + 1`
             fi
         fi
 
-        # gcrypt doesn't support cert
-        if [ "z$crypto" != "zgcrypt" ] ; then
+        if [ "z$xmlsec_feature_cert_der" = "zyes" ] ; then
             printf "    Reading public key from der cert file                 "
             rm -f $tmpfile
             params="--lax-key-search --pubkey-cert-der $certkey_file.der $key_test_options $asym_key_test.xml"
-            echo "$extra_vars $VALGRIND $xmlsec_app verify $xmlsec_params $params" >>  $curlogfile
-            $VALGRIND $xmlsec_app verify $xmlsec_params $params >> $curlogfile 2>> $curlogfile
+            echo "$extra_vars $VALGRIND $xmlsec_app verify $xmlsec_params --crypto-config $crypto_config $params" >>  $curlogfile
+            $VALGRIND $xmlsec_app verify $xmlsec_params --crypto-config $crypto_config $params >> $curlogfile 2>> $curlogfile
             printRes $expected_res $?
             if [ $? -ne 0 ]; then
                 failures=`expr $failures + 1`
@@ -496,14 +672,17 @@ execKeysTest() {
     fi
 
     # cleanup
-    cd $old_pwd
-    rm -f $tmpfile
+    tearDownTest
 }
 
 #
 # DSig test function
 #
 execDSigTest() {
+    execDSigTestWithCryptoConfig "$1" "$2" "$3" "$4" "$5" "$6" "$7" "$8" ""
+}
+
+execDSigTestWithCryptoConfig() {
     expected_res="$1"
     folder="$2"
     filename="$3"
@@ -512,6 +691,7 @@ execDSigTest() {
     params1="$6"
     params2="$7"
     params3="$8"
+    crypto_config="$9"
     failures=0
 
     if [ -n "$XMLSEC_TEST_NAME" -a "$XMLSEC_TEST_NAME" != "$filename" ]; then
@@ -519,19 +699,17 @@ execDSigTest() {
     fi
 
     # prepare
-    rm -f $tmpfile
-    old_pwd=`pwd`
-
-    if [ "z$crypto" = "znss" ] ; then
-        cleanupNssCerts
-    fi
+    setupTest
 
     # check params
     if [ "z$expected_res" != "z$res_success" -a "z$expected_res" != "z$res_fail" ] ; then
         echo " Bad parameter: expected_res=$expected_res"
-        cd $old_pwd
+        tearDownTest
         return
     fi
+    if [ "z$crypto_config" = "z" ] ; then
+        crypto_config="$default_crypto_config"
+    fi
 
     # starting test
     if [ -n "$folder" ] ; then
@@ -549,13 +727,13 @@ execDSigTest() {
     # check transforms
     if [ -n "$req_transforms" ] ; then
         printf "    Checking required transforms                         "
-        echo "$extra_vars $xmlsec_app check-transforms $xmlsec_params $req_transforms" >> $curlogfile
-        $xmlsec_app check-transforms $xmlsec_params $req_transforms >> $curlogfile 2>> $curlogfile
+        echo "$extra_vars $xmlsec_app check-transforms  --crypto-config $crypto_config $xmlsec_params $req_transforms" >> $curlogfile
+        $xmlsec_app check-transforms $xmlsec_params  --crypto-config $crypto_config $req_transforms >> $curlogfile 2>> $curlogfile
         printCheckStatus $?
         res=$?
         if [ $res -ne 0 ]; then
             cat $curlogfile >> $logfile
-	    cd $old_pwd
+	        tearDownTest
             return
         fi
     fi
@@ -563,13 +741,13 @@ execDSigTest() {
     # check key data
     if [ -n "$req_key_data" ] ; then
         printf "    Checking required key data                           "
-        echo "$extra_vars $xmlsec_app check-key-data $xmlsec_params $req_key_data" >> $curlogfile
-        $xmlsec_app check-key-data $xmlsec_params $req_key_data >> $curlogfile 2>> $curlogfile
+        echo "$extra_vars $xmlsec_app check-key-data $xmlsec_params --crypto-config $crypto_config $req_key_data" >> $curlogfile
+        $xmlsec_app check-key-data $xmlsec_params --crypto-config $crypto_config $req_key_data >> $curlogfile 2>> $curlogfile
         printCheckStatus $?
         res=$?
         if [ $res -ne 0 ]; then
             cat $curlogfile >> $logfile
-	    cd $old_pwd
+	        tearDownTest
             return
         fi
     fi
@@ -577,8 +755,8 @@ execDSigTest() {
     # run tests
     if [ -n "$params1" ] ; then
         printf "    Verify existing signature                            "
-        echo "$extra_vars $VALGRIND $xmlsec_app verify --X509-skip-strict-checks $xmlsec_params $params1 $full_file.xml" >> $curlogfile
-        $VALGRIND $xmlsec_app verify --X509-skip-strict-checks $xmlsec_params $params1 $full_file.xml >> $curlogfile 2>> $curlogfile
+        echo "$extra_vars $VALGRIND $xmlsec_app verify --X509-skip-strict-checks $xmlsec_params  --crypto-config $crypto_config $params1 $full_file.xml" >> $curlogfile
+        $VALGRIND $xmlsec_app verify --X509-skip-strict-checks $xmlsec_params --crypto-config $crypto_config $params1 $full_file.xml >> $curlogfile 2>> $curlogfile
         printRes $expected_res $?
         if [ $? -ne 0 ]; then
             failures=`expr $failures + 1`
@@ -587,8 +765,8 @@ execDSigTest() {
 
     if [ -n "$params2" -a -z "$PERF_TEST" ] ; then
         printf "    Create new signature                                 "
-        echo "$extra_vars $VALGRIND $xmlsec_app sign $xmlsec_params $params2 --output $tmpfile $full_file.tmpl" >> $curlogfile
-        $VALGRIND $xmlsec_app sign $xmlsec_params $params2 --output $tmpfile $full_file.tmpl >> $curlogfile 2>> $curlogfile
+        echo "$extra_vars $VALGRIND $xmlsec_app sign $xmlsec_params --crypto-config $crypto_config $params2 --output $tmpfile $full_file.tmpl" >> $curlogfile
+        $VALGRIND $xmlsec_app sign $xmlsec_params --crypto-config $crypto_config $params2 --output $tmpfile $full_file.tmpl >> $curlogfile 2>> $curlogfile
         printRes $res_success $?
         if [ $? -ne 0 ]; then
             failures=`expr $failures + 1`
@@ -597,8 +775,8 @@ execDSigTest() {
 
     if [ -n "$params3" -a -z "$PERF_TEST" ] ; then
         printf "    Verify new signature                                 "
-        echo "$extra_vars $VALGRIND $xmlsec_app verify --X509-skip-strict-checks $xmlsec_params $params3 $tmpfile" >> $curlogfile
-        $VALGRIND $xmlsec_app verify --X509-skip-strict-checks $xmlsec_params $params3 $tmpfile >> $curlogfile 2>> $curlogfile
+        echo "$extra_vars $VALGRIND $xmlsec_app verify --X509-skip-strict-checks $xmlsec_params --crypto-config $crypto_config $params3 $tmpfile" >> $curlogfile
+        $VALGRIND $xmlsec_app verify --X509-skip-strict-checks $xmlsec_params --crypto-config $crypto_config $params3 $tmpfile >> $curlogfile 2>> $curlogfile
         printRes $res_success $?
         if [ $? -ne  0 ]; then
             failures=`expr $failures + 1`
@@ -612,14 +790,17 @@ execDSigTest() {
     fi
 
     # cleanup
-    cd $old_pwd
-    rm -f $tmpfile
+    tearDownTest
 }
 
 #
 # Enc test function
 #
 execEncTest() {
+    execEncTestWithCryptoConfig "$1" "$2" "$3" "$4" "$5" "$6" "$7" "$8" "$9" ""
+}
+
+execEncTestWithCryptoConfig() {
     expected_res="$1"
     folder="$2"
     filename="$3"
@@ -629,6 +810,7 @@ execEncTest() {
     params2="$7"
     params3="$8"
     outputTransform="$9"
+    crypto_config="${10}"
     failures=0
 
     if [ -n "$XMLSEC_TEST_NAME" -a "$XMLSEC_TEST_NAME" != "$filename" ]; then
@@ -636,20 +818,17 @@ execEncTest() {
     fi
 
     # prepare
-    rm -f $tmpfile $tmpfile.2
-    old_pwd=`pwd`
-
-
-    if [ "z$crypto" = "znss" ] ; then
-        cleanupNssCerts
-    fi
+    setupTest
 
     # check params
     if [ "z$expected_res" != "z$res_success" -a "z$expected_res" != "z$res_fail" ] ; then
         echo " Bad parameter: expected_res=$expected_res"
-        cd $old_pwd
+        tearDownTest
         return
     fi
+    if [ "z$crypto_config" = "z" ] ; then
+        crypto_config="$default_crypto_config"
+    fi
 
     # starting test
     if [ -n "$folder" ] ; then
@@ -667,13 +846,13 @@ execEncTest() {
     # check transforms
     if [ -n "$req_transforms" ] ; then
         printf "    Checking required transforms                         "
-        echo "$extra_vars $xmlsec_app check-transforms $xmlsec_params $req_transforms" >> $curlogfile
-        $xmlsec_app check-transforms $xmlsec_params $req_transforms >> $curlogfile 2>> $curlogfile
+        echo "$extra_vars $xmlsec_app check-transforms $xmlsec_params  --crypto-config $crypto_config $req_transforms" >> $curlogfile
+        $xmlsec_app check-transforms $xmlsec_params  --crypto-config $crypto_config $req_transforms >> $curlogfile 2>> $curlogfile
         printCheckStatus $?
         res=$?
         if [ $res -ne 0 ]; then
-	    cat $curlogfile >> $logfile
-	    cd $old_pwd
+	        cat $curlogfile >> $logfile
+	        tearDownTest
             return
         fi
     fi
@@ -681,13 +860,13 @@ execEncTest() {
     # check key data
     if [ -n "$req_key_data" ] ; then
         printf "    Checking required key data                           "
-        echo "$extra_vars $xmlsec_app check-key-data $xmlsec_params $req_key_data" >> $curlogfile
-        $xmlsec_app check-key-data $xmlsec_params $req_key_data >> $curlogfile 2>> $curlogfile
+        echo "$extra_vars $xmlsec_app check-key-data $xmlsec_params --crypto-config $crypto_config $req_key_data" >> $curlogfile
+        $xmlsec_app check-key-data $xmlsec_params --crypto-config $crypto_config $req_key_data >> $curlogfile 2>> $curlogfile
         printCheckStatus $?
         res=$?
         if [ $res -ne 0 ]; then
             cat $curlogfile >> $logfile
-	    cd $old_pwd
+	        tearDownTest
             return
         fi
     fi
@@ -696,8 +875,8 @@ execEncTest() {
     if [ -n "$params1" ] ; then
         rm -f $tmpfile
         printf "    Decrypt existing document                            "
-        echo "$extra_vars $VALGRIND $xmlsec_app decrypt $xmlsec_params $params1 $full_file.xml" >>  $curlogfile
-        $VALGRIND $xmlsec_app decrypt $xmlsec_params $params1 --output $tmpfile $full_file.xml >> $curlogfile  2>> $curlogfile
+        echo "$extra_vars $VALGRIND $xmlsec_app decrypt $xmlsec_params --crypto-config $crypto_config $params1 $full_file.xml" >>  $curlogfile
+        $VALGRIND $xmlsec_app decrypt $xmlsec_params --crypto-config $crypto_config $params1 --output $tmpfile $full_file.xml >> $curlogfile  2>> $curlogfile
         res=$?
         echo "=== TEST RESULT: $res; expected: $expected_res" >> $curlogfile
         if [ $res -eq 0 -a "$expected_res" = "$res_success" ]; then
@@ -718,8 +897,8 @@ execEncTest() {
     if [ -n "$params2" -a -z "$PERF_TEST" ] ; then
         rm -f $tmpfile
         printf "    Encrypt document                                     "
-        echo "$extra_vars $VALGRIND $xmlsec_app encrypt $xmlsec_params $params2 --output $tmpfile $full_file.tmpl" >>  $curlogfile
-        $VALGRIND $xmlsec_app encrypt $xmlsec_params $params2 --output $tmpfile $full_file.tmpl >> $curlogfile 2>> $curlogfile
+        echo "$extra_vars $VALGRIND $xmlsec_app encrypt $xmlsec_params --crypto-config $crypto_config $params2 --output $tmpfile $full_file.tmpl" >>  $curlogfile
+        $VALGRIND $xmlsec_app encrypt $xmlsec_params --crypto-config $crypto_config $params2 --output $tmpfile $full_file.tmpl >> $curlogfile 2>> $curlogfile
         printRes $res_success $?
         if [ $? -ne 0 ]; then
             failures=`expr $failures + 1`
@@ -729,8 +908,8 @@ execEncTest() {
     if [ -n "$params3" -a -z "$PERF_TEST" ] ; then
         rm -f $tmpfile.2
         printf "    Decrypt new document                                 "
-        echo "$extra_vars $VALGRIND $xmlsec_app decrypt $xmlsec_params $params3 --output $tmpfile.2 $tmpfile" >>  $curlogfile
-        $VALGRIND $xmlsec_app decrypt $xmlsec_params $params3 --output $tmpfile.2 $tmpfile >> $curlogfile 2>> $curlogfile
+        echo "$extra_vars $VALGRIND $xmlsec_app decrypt $xmlsec_params --crypto-config $crypto_config $params3 --output $tmpfile.2 $tmpfile" >>  $curlogfile
+        $VALGRIND $xmlsec_app decrypt $xmlsec_params --crypto-config $crypto_config $params3 --output $tmpfile.2 $tmpfile >> $curlogfile 2>> $curlogfile
         res=$?
         if [ $res -eq 0 ]; then
             if [ "z$outputTransform" != "z" ] ; then
@@ -754,12 +933,11 @@ execEncTest() {
     fi
 
     # cleanup
-    cd $old_pwd
-    rm -f $tmpfile $tmpfile.2
+    tearDownTest
 }
 
 # prepare
-rm -rf $tmpfile $tmpfile.2 tmpfile.3
+rm -rf $tmpfile $tmpfile.2 $tmpfile.3
 
 # run tests
 source "$testfile"
@@ -771,8 +949,10 @@ if [ $count_total -gt 0 ] ; then
     percent_success=`expr 100 \* $count_success / $count_total`
 fi
 
-if [ "z$crypto" = "zopenssl" ] ; then
+if [ "z$crypto" = "zopenssl" -a "z$xmlsec_openssl_flavor" != "zaws-lc" ] ; then
     min_percent_success=90
+elif [ "z$crypto" = "zopenssl" -a "z$xmlsec_openssl_flavor" = "zaws-lc" ] ; then
+    min_percent_success=80
 elif [ "z$crypto" = "znss" ] ; then
     min_percent_success=90
 elif [ "z$crypto" = "zgnutls" ] ; then
