@@ -5,30 +5,27 @@
  * found in the LICENSE file.
  */
 
-#define SK_OPTS_NS sksl_minify_standalone
-#include "include/core/SkStream.h"
 #include "src/base/SkStringView.h"
-#include "src/core/SkCpu.h"
 #include "src/core/SkOpts.h"
-#include "src/opts/SkVM_opts.h"
 #include "src/sksl/SkSLCompiler.h"
 #include "src/sksl/SkSLFileOutputStream.h"
 #include "src/sksl/SkSLLexer.h"
+#include "src/sksl/SkSLModule.h"
 #include "src/sksl/SkSLModuleLoader.h"
 #include "src/sksl/SkSLProgramKind.h"
 #include "src/sksl/SkSLProgramSettings.h"
-#include "src/sksl/SkSLStringStream.h"
 #include "src/sksl/SkSLUtil.h"
+#include "src/sksl/ir/SkSLStructDefinition.h"
+#include "src/sksl/ir/SkSLSymbolTable.h"
 #include "src/sksl/transform/SkSLTransform.h"
+#include "src/utils/SkGetExecutablePath.h"
 #include "src/utils/SkOSPath.h"
-#include "tools/SkGetExecutablePath.h"
 #include "tools/skslc/ProcessWorklist.h"
 
 #include <cctype>
 #include <forward_list>
 #include <fstream>
 #include <limits.h>
-#include <optional>
 #include <stdarg.h>
 #include <stdio.h>
 
@@ -45,9 +42,6 @@ void SkDebugf(const char format[], ...) {
 
 namespace SkOpts {
     size_t raster_pipeline_highp_stride = 1;
-#if defined(SK_ENABLE_SKVM)
-    decltype(interpret_skvm) interpret_skvm = SK_OPTS_NS::interpret_skvm;
-#endif
 }
 
 static std::string base_name(const std::string& path) {
@@ -65,7 +59,7 @@ static std::string remove_extension(const std::string& path) {
  */
 static void show_usage() {
     printf("usage: sksl-minify <output> <input> [--frag|--vert|--compute|--shader|"
-           "--colorfilter|--blender] [dependencies...]\n");
+           "--colorfilter|--blender|--meshfrag|--meshvert] [dependencies...]\n");
 }
 
 static std::string_view stringize(const SkSL::Token& token, std::string_view text) {
@@ -76,6 +70,20 @@ static bool maybe_identifier(char c) {
     return std::isalnum(c) || c == '$' || c == '_';
 }
 
+static bool is_plus_or_minus(char c) {
+    return c == '+' || c == '-';
+}
+
+static SkSL::ModuleType module_type_for_path(const char* path) {
+    SkString filename = SkOSPath::Basename(path);
+
+#define M(type) if (filename.equals(#type ".sksl")) { return SkSL::ModuleType::type; }
+    SKSL_MODULE_LIST(M)
+#undef M
+
+    return SkSL::ModuleType::unknown;
+}
+
 static std::forward_list<std::unique_ptr<const SkSL::Module>> compile_module_list(
         SkSpan<const std::string> paths, SkSL::ProgramKind kind) {
     std::forward_list<std::unique_ptr<const SkSL::Module>> modules;
@@ -105,7 +113,7 @@ static std::forward_list<std::unique_ptr<const SkSL::Module>> compile_module_lis
 
     // Load in each input as a module, from right to left.
     // Each module inherits the symbols from its parent module.
-    SkSL::Compiler compiler(SkSL::ShaderCapsFactory::Standalone());
+    SkSL::Compiler compiler;
     for (auto modulePath = paths.rbegin(); modulePath != paths.rend(); ++modulePath) {
         std::ifstream in(*modulePath);
         std::string moduleSource{std::istreambuf_iterator<char>(in),
@@ -119,22 +127,19 @@ static std::forward_list<std::unique_ptr<const SkSL::Module>> compile_module_lis
                                                      : modules.front().get();
         std::unique_ptr<SkSL::Module> m =
                 compiler.compileModule(kind,
-                                       modulePath->c_str(),
+                                       module_type_for_path(modulePath->c_str()),
                                        std::move(moduleSource),
                                        parent,
-                                       SkSL::ModuleLoader::Get().coreModifiers(),
                                        /*shouldInline=*/false);
         if (!m) {
             return {};
         }
-        if (!gUnoptimized) {
-            // We need to optimize every module in the chain. We rename private functions at global
-            // scope, and we need to make sure there are no name collisions between nested modules.
-            // (i.e., if module A claims names `$a` and `$b` at global scope, module B will need to
-            // start at `$c`. The most straightforward way to handle this is to actually perform the
-            // renames.)
-            compiler.optimizeModuleBeforeMinifying(kind, *m);
-        }
+        // We need to optimize every module in the chain. We rename private functions at global
+        // scope, and we need to make sure there are no name collisions between nested modules.
+        // (i.e., if module A claims names `$a` and `$b` at global scope, module B will need to
+        // start at `$c`. The most straightforward way to handle this is to actually perform the
+        // renames.)
+        compiler.optimizeModuleBeforeMinifying(kind, *m, /*shrinkSymbols=*/!gUnoptimized);
         modules.push_front(std::move(m));
     }
     // Return all of the modules to transfer their ownership to the caller.
@@ -191,9 +196,18 @@ static bool generate_minified_text(std::string_view inputPath,
             out.writeText("\"\n\"");
             lineWidth = 1;
         }
-        if (maybe_identifier(lastTokenText.back()) && maybe_identifier(thisTokenText.front())) {
-            // We are about to put two alphanumeric characters side-by-side; add whitespace between
-            // the tokens.
+
+        // Detect tokens with abutting alphanumeric characters side-by-side.
+        bool adjacentIdentifiers =
+                maybe_identifier(lastTokenText.back()) && maybe_identifier(thisTokenText.front());
+
+        // Detect potentially ambiguous preincrement/postincrement operators.
+        // For instance, `x + ++y` and `x++ + y` require whitespace for differentiation.
+        bool adjacentPlusOrMinus =
+                is_plus_or_minus(lastTokenText.back()) && is_plus_or_minus(thisTokenText.front());
+
+        // Insert whitespace when it is necessary for program correctness.
+        if (adjacentIdentifiers || adjacentPlusOrMinus) {
             out.writeText(" ");
             lineWidth++;
         }
@@ -230,9 +244,13 @@ static ResultCode process_command(SkSpan<std::string> args) {
     bool isVert = find_boolean_flag(&args, "--vert");
     bool isCompute = find_boolean_flag(&args, "--compute");
     bool isShader = find_boolean_flag(&args, "--shader");
+    bool isPrivateShader = find_boolean_flag(&args, "--privshader");
     bool isColorFilter = find_boolean_flag(&args, "--colorfilter");
     bool isBlender = find_boolean_flag(&args, "--blender");
-    if (has_overlapping_flags({isFrag, isVert, isCompute, isShader, isColorFilter, isBlender})) {
+    bool isMeshFrag = find_boolean_flag(&args, "--meshfrag");
+    bool isMeshVert = find_boolean_flag(&args, "--meshvert");
+    if (has_overlapping_flags({isFrag, isVert, isCompute, isShader, isColorFilter,
+                               isBlender, isMeshFrag, isMeshVert})) {
         show_usage();
         return ResultCode::kInputError;
     }
@@ -246,6 +264,12 @@ static ResultCode process_command(SkSpan<std::string> args) {
         gProgramKind = SkSL::ProgramKind::kRuntimeColorFilter;
     } else if (isBlender) {
         gProgramKind = SkSL::ProgramKind::kRuntimeBlender;
+    } else if (isMeshFrag) {
+        gProgramKind = SkSL::ProgramKind::kMeshFragment;
+    } else if (isMeshVert) {
+        gProgramKind = SkSL::ProgramKind::kMeshVertex;
+    } else if (isPrivateShader) {
+        gProgramKind = SkSL::ProgramKind::kPrivateRuntimeShader;
     } else {
         // Default case, if no option is specified.
         gProgramKind = SkSL::ProgramKind::kRuntimeShader;
@@ -282,6 +306,14 @@ static ResultCode process_command(SkSpan<std::string> args) {
     // Generate the program text by getting the program's description.
     std::string text;
     for (const std::unique_ptr<SkSL::ProgramElement>& element : module->fElements) {
+        if ((isMeshFrag || isMeshVert) && element->is<SkSL::StructDefinition>()) {
+            std::string_view name = element->as<SkSL::StructDefinition>().type().name();
+            if (name == "Attributes" || name == "Varyings") {
+                // Don't emit the Attributes or Varyings structs from a mesh program into the
+                // minified output; those are synthesized via the SkMeshSpecification.
+                continue;
+            }
+        }
         text += element->description();
     }
 
