@@ -25,6 +25,7 @@
 #include "libport.h"
 #include "tif_config.h"
 
+#include <math.h> /* for isfinite() */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -360,7 +361,7 @@ static uint64_t ReadDirectory(int fd, unsigned int ix, uint64_t off)
     {
         n /= direntrysize;
         Error("Could only read %" PRIu32 " of %" PRIu16
-              " entries in directory at offset %#" PRIu64,
+              " entries in directory at offset %" PRIu64,
               n, dircount, off);
         dircount = n;
         nextdiroff = 0;
@@ -481,7 +482,7 @@ static uint64_t ReadDirectory(int fd, unsigned int ix, uint64_t off)
         }
         if (!datafits)
         {
-            datamem = _TIFFmalloc(datasize);
+            datamem = _TIFFmalloc((tmsize_t)datasize);
             if (datamem)
             {
                 if (_TIFF_lseek_f(fd, (_TIFF_off_t)dataoffset, 0) !=
@@ -491,7 +492,7 @@ static uint64_t ReadDirectory(int fd, unsigned int ix, uint64_t off)
                     _TIFFfree(datamem);
                     datamem = NULL;
                 }
-                else if (read(fd, datamem, (size_t)datasize) !=
+                else if (read(fd, datamem, (unsigned int)datasize) !=
                          (tmsize_t)datasize)
                 {
                     Error("Read error accessing tag %u value", tag);
@@ -798,7 +799,7 @@ static void PrintData(FILE *fd, uint16_t type, uint32_t count,
             uint32_t *lp = (uint32_t *)data;
             while (count-- > 0)
             {
-                if (lp[1] == 0)
+                if (lp[1] == 0 || !isfinite((double)lp[1]))
                     fprintf(fd, "%sNan (%" PRIu32 "/%" PRIu32 ")", sep, lp[0],
                             lp[1]);
                 else
