@@ -1521,6 +1521,7 @@ static int test_conc327(MYSQL *unused __attribute__((unused)))
   const char *env= getenv("MYSQL_TMP_DIR");
   char cnf_file1[FN_REFLEN + 1];
   char cnf_file2[FN_REFLEN + 1];
+  my_bool failed_opening_files;
 
   SKIP_SKYSQL;
 
@@ -1539,7 +1540,19 @@ static int test_conc327(MYSQL *unused __attribute__((unused)))
 
   fp1= fopen(cnf_file1, "w");
   fp2= fopen(cnf_file2, "w");
-  FAIL_IF(!fp1 || !fp2, "fopen failed");
+  if((failed_opening_files = !fp1 || !fp2))
+  {
+    if(fp1)
+    {
+      fclose(fp1);
+    }
+    if(fp2)
+    {
+      fclose(fp2);
+    }
+  }
+  
+  FAIL_IF(failed_opening_files, "fopen failed");
 
   fprintf(fp1, "!include %s\n", cnf_file2);
   
@@ -1564,7 +1577,18 @@ static int test_conc327(MYSQL *unused __attribute__((unused)))
   snprintf(cnf_file1, FN_REFLEN, "%s%cmy.cnf", env, FN_LIBCHAR);
   fp1= fopen(cnf_file1, "w");
   fp2= fopen(cnf_file2, "w");
-  FAIL_IF(!fp1 || !fp2, "fopen failed");
+  if((failed_opening_files = !fp1 || !fp2))
+  {
+    if(fp1)
+    {
+      fclose(fp1);
+    }
+    if(fp2)
+    {
+      fclose(fp2);
+    }
+  }
+  FAIL_IF(failed_opening_files, "fopen failed");
 
   fprintf(fp2, "!includedir %s\n", env);
   
@@ -2339,6 +2363,7 @@ static int test_conc748(MYSQL *my __attribute__((unused)))
   MYSQL *mysql;
   int i;
   const char *ciphers[3]= {"TLS_AES_128_GCM_SHA256", "TLS_AES_256_GCM_SHA384", "TLS_CHACHA20_POLY1305_SHA256"};
+  my_bool verify= 0;
 
   SKIP_MAXSCALE;
 
@@ -2348,6 +2373,7 @@ static int test_conc748(MYSQL *my __attribute__((unused)))
     mysql= mysql_init(NULL);
 
     mysql_ssl_set(mysql, NULL, NULL, NULL, NULL, NULL);
+    mysql_optionsv(mysql, MYSQL_OPT_SSL_VERIFY_SERVER_CERT, &verify);
     mysql_optionsv(mysql, MYSQL_OPT_SSL_CIPHER, ciphers[i]);
 
     if (!my_test_connect(mysql, hostname, username,
@@ -2370,7 +2396,6 @@ static int test_conc748(MYSQL *my __attribute__((unused)))
 static int test_conc589(MYSQL *my)
 {
   MYSQL *mysql= mysql_init(NULL);
-  MYSQL_RES *result;
   int rc;
   my_bool reconnect= 1, verify= 0;
   unsigned long last_thread_id= 0;
@@ -2390,6 +2415,75 @@ static int test_conc589(MYSQL *my)
   rc= mysql_query(mysql, "SET SESSION wait_timeout=5");
   check_mysql_rc(rc, mysql);
 
+  last_thread_id= mysql_thread_id(mysql);
+  rc= mysql_query(mysql, "SET @a:=1");
+  check_mysql_rc(rc, mysql);
+
+  sleep(10);
+
+  rc= mysql_query(mysql, "SET @a:=2");
+  check_mysql_rc(rc, mysql);
+  FAIL_IF(mysql_thread_id(mysql) == last_thread_id, "Expected new connection id");
+  last_thread_id= mysql_thread_id(mysql);
+
+  mysql_kill(my, last_thread_id);
+
+  sleep(10);
+
+  rc= mysql_query(mysql, "SET @a:=3");
+  check_mysql_rc(rc, mysql);
+  FAIL_IF(mysql_thread_id(mysql) == last_thread_id, "Expected new connection id");
+  mysql_close(mysql);
+  return OK;
+}
+
+#ifdef WIN32
+static int test_conc760(MYSQL *my)
+{
+  MYSQL *mysql= mysql_init(NULL);
+  MYSQL_RES *result;
+  MYSQL_ROW row;
+  int rc;
+  char named_pipe_name[128];
+  my_bool reconnect= 1, verify= 0;
+  unsigned long last_thread_id= 0;
+  unsigned int protocol= MYSQL_PROTOCOL_PIPE;
+  my_bool have_named_pipe= 0;
+
+  SKIP_MAXSCALE;
+
+  rc= mysql_query(my, "select @@named_pipe, @@socket");
+  check_mysql_rc(rc, mysql);
+
+  if ((result= mysql_store_result(my)))
+  {
+    if((row= mysql_fetch_row(result)))
+      have_named_pipe= atoi(row[0]);
+    strncpy(named_pipe_name, row[1], sizeof(named_pipe_name)-1);
+    named_pipe_name[sizeof(named_pipe_name)-1]= '\0';
+    mysql_free_result(result);
+  }
+
+  if (!have_named_pipe)
+  {
+    diag("Server doesn't support named pipes");
+    return SKIP;
+  }
+
+  mysql_options(mysql, MYSQL_OPT_RECONNECT, &reconnect);
+  mysql_options(mysql, MYSQL_OPT_SSL_VERIFY_SERVER_CERT, &verify);
+  mysql_options(mysql, MYSQL_OPT_PROTOCOL, &protocol);
+
+  if (!my_test_connect(mysql, hostname, username,
+                       password, schema, port, named_pipe_name, CLIENT_REMEMBER_OPTIONS))
+  {
+    diag("error: %s", mysql_error(mysql));
+    return FAIL;
+  }
+
+  rc= mysql_query(mysql, "SET SESSION wait_timeout=5");
+  check_mysql_rc(rc, mysql);
+
   last_thread_id= mysql_thread_id(mysql);
   if ((rc= mysql_query(mysql, "SELECT 1")) || (result= mysql_store_result(mysql)) == NULL)
     check_mysql_rc(rc, mysql);
@@ -2397,9 +2491,10 @@ static int test_conc589(MYSQL *my)
   mysql_free_result(result);
   sleep(10);
 
-  if ((rc= mysql_query(mysql, "SELECT 2")) || (result= mysql_store_result(mysql)) == NULL)
-    check_mysql_rc(rc, mysql);
-  mysql_free_result(result);
+  rc= mysql_query(mysql, "SELECT 2");
+  check_mysql_rc(rc, mysql);
+  if (result= mysql_store_result(mysql))
+    mysql_free_result(result);
   FAIL_IF(mysql_thread_id(mysql) == last_thread_id, "Expected new connection id");
   last_thread_id= mysql_thread_id(mysql);
 
@@ -2414,8 +2509,12 @@ static int test_conc589(MYSQL *my)
   mysql_close(mysql);
   return OK;
 }
+#endif
 
 struct my_tests_st my_tests[] = {
+#ifdef WIN32
+  {"test_conc760", test_conc760, TEST_CONNECTION_DEFAULT, 0, NULL, NULL},
+#endif
   {"test_conc589", test_conc589, TEST_CONNECTION_DEFAULT, 0, NULL, NULL},
 #ifdef HAVE_test_conc748
   {"test_conc748", test_conc748, TEST_CONNECTION_NONE, 0, NULL, NULL},
